/*!
 * Native Javascript Library
 * v1.0.2 (2019-11-13)
 * 新增登录和分享接口
 * v1.0.3 (2019-11-14)
 * 新增rsa加密算法
 * v1.0.4 (2019-11-16)
 * 新增打开文件接口
 * v1.0.5 (2019-11-16)
 * 新增退出登录接口
 * v1.0.6 (2019-11-19)
 * 新增打开网页接口
 * v1.0.7 (2019-11-20)
 * 修改判断android手机和iOS手机的逻辑
 */
(function(window) {
	
	/**
	 * android系统
	 */
	function Android() {
		this.toast = function(text) {
			android.toast(text);
		}
		
		this.finish = function() {
			android.finish();
		}
		this.showInputMethod = function() {
			android.showInputMethod();
		}
		this.hiddenInputMethod = function() {
			android.hiddenInputMethod();
		}
		this.hasNetWork = function() {
			return android.hasNetWork();
		}
		this.href = function(tag, path) {
			android.href(tag, path);
		}
        this.open = function(url) {
            android.open(url);
        }
		this.html = function(url, title) {
        	android.html(url, title);
        }
		this.callPhone = function(phone) {
			android.callPhone(phone);
		}
		this.copy = function(text) {
			android.copy(text);
		}
		this.setItem = function(key, value) {
			android.setItem(key,value);
		}
		this.getItem = function(key) {
			return android.getItem(key);
		}
		this.removeItem = function(key) {
            android.removeItem(key);
        }
		this.login = function(callback) {
        	android.login(callback);
        }
        this.logout = function() {
            android.logout();
        }
        this.share = function(url, title, description) {
            android.share(url, title, description);
        }
        this.clearCache = function() {
            android.clearCache();
        }
        this.rsa = function(text) {
            return android.rsa(text);
        }
	}

	/**
	 * iOS系统
	 */
	function IOS() {
		this.toast = function(text) {
			iOSToast(text);
		}
		this.finish = function() {
			iOSBack();
		}		
		this.showInputMethod = function() {
			// do nothing
		}		
		this.hiddenInputMethod = function() {
			// do nothing
		}
		this.hasNetWork = function() {
			return iOSHasNetWork();
		}		
		this.href = function(tag, path) {
			iOSHref(tag, path);
		}
		this.open = function(url) {
            iOSOpen(url);
        }
		this.html = function(url, title) {
        	iOSHtml(url, title);
        }
		this.callPhone = function(phone) {
			iOSCall(phone);
		}
		this.copy = function(text) {
			iOSCopy(text);
		}
		this.setItem = function(key, value) {
		    iOSSetItem(key, value);
        }
        this.getItem = function(key) {
        	return iOSGetItem(key);
        }
        this.removeItem = function(key) {
            iOSRemoveItem(key);
        }
        this.login = function(callback) {
            iOSLogin(callback);
        }
        this.logout = function() {
            iOSLogout();
        }
        this.share = function(url, title, description) {
            iOSShare(url, title, description);
        }
        this.clearCache = function() {
            iOSClearCache();
        }
        this.rsa = function(text) {
            return iOSRsa(text);
        }
	}

	/**
	 * 开发使用的调试数据
	 */
	function Developer() {
		this.toast = function(text) {
			console.log("toast:" + text);
		}
		this.finish = function() {
			window.close();
		}
		this.showInputMethod = function() {
			console.log("showInputMethod");
		}
		this.hiddenInputMethod = function() {
			console.log("hiddenInputMethod");
		}
		this.hasNetWork = function() {		
			return true;
		}
		this.href = function(tag, path) {
			console.log("href " + tag + " " + path);
		}
		this.open = function(url) {
            window.open(url);
        }
		this.html = function(url, title) {
		    window.open(url);
		}
		this.callPhone = function(phone) {
			console.log("callPhone:" + phone);
		}
		this.copy = function(text) {
			console.log("copy:" + text);
		}
		this.setItem = function(key, value) {
			localStorage.setItem(key, value);
		}
		this.getItem = function(key) {
			return localStorage.getItem(key);
		}
		this.removeItem = function(key) {
			localStorage.removeItem(key);
        }
		this.login = function(callback) {
			var res = {
				"userId": "SR0062710300",
				"infor": {
					"uid": "SR0062710300",
					"mobile": "18711078492",
					"sn": "SR0062710300",
					"email": "SR0062710300@szse.com",
					"loa": "40"
				},
				"deviceToken": "3daef4ae-5fd3-458f-8561-5b1274bc7fbe",
				"tgtToken": "TGT-40-1996-Yq2Eb4dFcAIHcVMUNsLQjI0OZBpTq0DcTx7m7KNoX6VeHKpG49-cas01.example.org",
				"userInfo": {
					"userId": "SR0062710300",
					"enterpriseId": "szsesprweb",
					"classType": null,
					"loginName": "SR0062710300",
					"userType": 0,
					"displayName": "小A"
				}
			};
			native.storage.setItem('keyLoginData', JSON.stringify(res));
			window[callback](0, {data:res});
        }
        this.logout = function() {
			native.storage.removeItem('keyLoginData');
        }
        this.share = function() {
          	console.log("share");
        }
        this.clearCache = function() {
        	console.log("clearCache");
        }
        this.rsa = function(text) {
            return text;
		}
		this.log = function(text) {
            console.log(text);
        }
	}

	/**
	 * UI组件
	 */
	function UI(wrapper) {

		/**
		 * 显示toast
		 */
		this.toast = function(text) {
			wrapper.toast(text);
		}

		/**
		 * 关闭当前界面
		 */
		this.finish = function() {
			wrapper.finish();
		}

		/**
		 * 显示输入法，iOS不支持
		 */
		this.showInputMethod = function() {
			wrapper.showInputMethod();
		}
		
		/**
		 * 隐藏输入法，iOS不支持
		 */
		this.hiddenInputMethod = function() {
			wrapper.hiddenInputMethod();
		}
	}

	/**
	 * 网络组件
	 */
	function Net(wrapper) {
		
		/**
		 * 是否有网络
		 */
		this.hasNetWork = function() {
			return wrapper.hasNetWork();
		}
	}
	
	/**
	 * 文件组件
	 */
	function File(wrapper) {
		
		/**
		 * 跳转轻应用指定页面
		 */
		this.href = function(tag, path) {
			wrapper.href(tag, path);
		}

		/**
         * 打开文件
         * url 文件网络地址
         */
        this.open = function(url) {
            wrapper.open(url);
        }

        /**
         * 打开html的网页
         */
        this.html = function(url, title) {
        	wrapper.html(url, title);
        }
	}
	
	/**
	 * 存储
	 */
	function Storage(wrapper) {
		
		/**
		 * 保存数据
		 */
		this.setItem = function(key, value) {
			wrapper.setItem(key, value);
		}
		
		/**
		 * 获取数据
		 */
		this.getItem = function(key) {
			return wrapper.getItem(key);
		}

		/**
         * 移除数据
         */
         this.removeItem = function(key) {
            wrapper.removeItem(key);
         }

		/**
		 * 清除缓存
		 */
		this.clearCache = function() {
		    wrapper.clearCache();
		}
	}
	
	/**
	 * 其他组件
	 */
	function Util(wrapper) {
		
		/**
		 * 拨打电话
		 */
		this.callPhone = function(phone) {
			wrapper.callPhone(phone);
		}
		
		/**
		 * 复制文本
		 */
		this.copy = function(text) {
			wrapper.copy(text);
		}

		/**
         * 记录日志
         */
        this.log = function(text) {
            wrapper.log(text);
        }
	}

	/**
     * 业务相关功能
     */
    function Business(wrapper) {

    	/**
    	 * 登录
    	 * 回调的code，0：成功；1：取消
    	 * json：后台返回的数据，json对象
    	 */
    	this.login = function(callback) {
		    window.nativeLoginCallback = function(code, json) {
		        window.loginCallback = null;
			    callback(code, json);
		    };
    		wrapper.login("nativeLoginCallback");
    	}

    	/**
         * 退出登录
         */
        this.logout = function() {
            wrapper.logout();
        }

    	/**
         * 分享
         */
    	this.share = function(url, title, description) {
            wrapper.share(url, title, description);
    	}

        /**
         * RSA加密
         */
    	this.rsa = function(text) {
    	    return wrapper.rsa(text);
    	}
    }

	function Native() {
		var wrapper;
		if (typeof iOS != 'undefined') {
			wrapper = new IOS();
		} else if (typeof android != 'undefined') {
			wrapper = new Android();
		} else {
			wrapper = new Developer();
		}

		this.ui = new UI(wrapper);
		this.net = new Net(wrapper);
		this.file = new File(wrapper);
		this.util = new Util(wrapper);
		this.storage = new Storage(wrapper);
		this.business = new Business(wrapper);
	}

	window.native = new Native();

})(window);