/*!
 * Native Javascript Library
 * v1.0.2 (2019-11-13)
 * 新增登录和分享接口
 * v1.0.3 (2019-11-14)
 * 新增rsa加密算法
 * v1.0.4 (2019-11-16)
 * 新增打开文件接口
 * v1.0.5 (2019-11-16)
 * 新增退出登录接口
 * v1.0.6 (2019-11-19)
 * 新增打开网页接口
 * v1.0.7 (2019-11-20)
 * 修改判断android手机和iOS手机的逻辑
 * v1.0.8 (2019-12-12)
 * iOS适配WKWebView
 * v1.0.9 (2020-05-06)
 * 默认使用WKWebView
 * v1.0.10 (2020-05-14)
 * 新增记录日志方法
 * v1.0.11 (2020-07-14)
 * 新增打开文件和分享功能
 * v1.0.12（2020-07-15）
 * 新增打开超链接的分享功能
 * v1.0.13（2020-08-12）
 * 新增APP版本号接口
 * v1.0.14（2022-01-11）
 * 新增权限撤回接口
 * v1.0.15（2022-08-23）
 * 新增页面获取原生传递参数的接口
 */
(function(window) {

    /**
     * android系统
     */
    function Android() {
        this.toast = function(text) {
            android.toast(text);
        }
        this.finish = function() {
            android.finish();
        }
        this.showInputMethod = function() {
            android.showInputMethod();
        }
        this.hiddenInputMethod = function() {
            android.hiddenInputMethod();
        }
        this.hasNetWork = function() {
            return android.hasNetWork();
        }
        this.href = function(tag, path) {
            android.href(tag, path);
        }
        this.open = function(url, title, shareEnabled) {
            android.open(url, title, shareEnabled);
        }
        this.html = function(url, title, shareEnabled) {
            try {
                android.html(url, title, shareEnabled);
            } catch (e) {
                android.html(url, title);
            }
        }
        this.callPhone = function(phone) {
            android.callPhone(phone);
        }
        this.copy = function(text) {
            android.copy(text);
        }
        this.setItem = function(key, value) {
            android.setItem(key,value);
        }
        this.getItem = function(key) {
            return android.getItem(key);
        }
        this.removeItem = function(key) {
            android.removeItem(key);
        }
        this.login = function(callback) {
            android.login(callback);
        }
        this.logout = function() {
            android.logout();
        }
        this.share = function(url, title, description) {
            android.share(url, title, description);
        }
        this.clearCache = function() {
            android.clearCache();
        }
        this.rsa = function(text) {
            return android.rsa(text);
        }
        this.log = function(text) {
            android.log(text);
        }
        this.version = function() {
            try {
                return android.version();
            } catch (e) {
                return "1.0.0";
            }
        }
        this.openAppSetting = function() {
            android.openAppSetting();
        }
        this.getJson = function() {
            return android.getJson();
        }
    }

    /**
     * iOS系统WKWebView
     */
    function iOSWKWebView() {
        this.toast = function(text) {
            prompt("xcode-oc:toast", text);
        }
        this.finish = function() {
            prompt("xcode-oc:finish");
        }
        this.showInputMethod = function() {
            // do nothing
        }
        this.hiddenInputMethod = function() {
            // do nothing
        }
        this.hasNetWork = function() {
            return prompt("xcode-oc:hasNetWork") == "YES";
        }
        this.href = function(tag, path) {
            var json = {};
            json.tag = tag;
            json.path = path;
            prompt("xcode-oc:href", JSON.stringify(json));
        }
        this.open = function(url, title, shareEnabled) {
            var json = {};
            json.url = url;
            json.title = title;
            json.shareEnabled = shareEnabled;
            var r = prompt("xcode-oc:open2", JSON.stringify(json));
            if (r == "") {
                prompt("xcode-oc:open", url);
            }
        }
        this.html = function(url, title, shareEnabled) {
            var json = {};
            json.url = url;
            json.title = title;
            json.shareEnabled = shareEnabled;
            prompt("xcode-oc:html", JSON.stringify(json));
        }
        this.callPhone = function(phone) {
            prompt("xcode-oc:callPhone", phone);
        }
        this.copy = function(text) {
            prompt("xcode-oc:copy", text);
        }
        this.setItem = function(key, value) {
            var json = {};
            json.key = key;
            json.value = value;
            prompt("xcode-oc:setItem", JSON.stringify(json));
        }
        this.getItem = function(key) {
            return prompt("xcode-oc:getItem", key);
        }
        this.removeItem = function(key) {
            prompt("xcode-oc:removeItem", key);
        }
        this.login = function(callback) {
            prompt("xcode-oc:login", callback);
        }
        this.logout = function() {
            prompt("xcode-oc:logout");
        }
        this.share = function(url, title, description) {
            var json = {};
            json.url = url;
            json.title = title;
            json.description = description;
            prompt("xcode-oc:share", JSON.stringify(json));
        }
        this.clearCache = function() {
            prompt("xcode-oc:clearCache");
        }
        this.rsa = function(text) {
            return prompt("xcode-oc:rsa", text);
        }
        this.log = function(text) {
            prompt("xcode-oc:log", text);
        }
        this.version = function() {
            var r = prompt("xcode-oc:version");
            if (r == "") {
                return "1.0.0";
            } else {
                return r;
            }
        }
        this.openAppSetting = function() {
            // do nothing
        }
        this.getJson = function() {
            return prompt("xcode-oc:getJson");
        }
    }

    /**
     * UI组件
     */
    function UI(wrapper) {

        /**
         * 显示toast
         */
        this.toast = function(text) {
            wrapper.toast(text);
        }

        /**
         * 关闭当前界面
         */
        this.finish = function() {
            wrapper.finish();
        }

        /**
         * 显示输入法，iOS不支持
         */
        this.showInputMethod = function() {
            wrapper.showInputMethod();
        }

        /**
         * 隐藏输入法，iOS不支持
         */
        this.hiddenInputMethod = function() {
            wrapper.hiddenInputMethod();
        }
    }

    /**
     * 网络组件
     */
    function Net(wrapper) {

        /**
         * 是否有网络
         */
        this.hasNetWork = function() {
            return wrapper.hasNetWork();
        }
    }

    /**
     * 文件组件
     */
    function File(wrapper) {

        /**
         * 跳转轻应用指定页面
         */
        this.href = function(tag, path) {
            wrapper.href(tag, path);
        }

        /**
         * 打开文件
         * url 文件网络地址
         */
        this.open = function(url, title, shareEnabled) {
            wrapper.open(url, title, shareEnabled);
        }

        /**
          * 打开html的网页
          */
        this.html = function(url, title, shareEnabled) {
            wrapper.html(url, title, shareEnabled);
        }
    }

    /**
     * 存储
     */
    function Storage(wrapper) {

        /**
         * 保存数据
         */
        this.setItem = function(key, value) {
            wrapper.setItem(key, value);
        }

        /**
         * 获取数据
         */
        this.getItem = function(key) {
            return wrapper.getItem(key);
        }

        /**
         * 移除数据
         */
         this.removeItem = function(key) {
            wrapper.removeItem(key);
         }

        /**
         * 清除缓存
         */
        this.clearCache = function() {
            wrapper.clearCache();
        }
    }

    /**
     * 其他组件
     */
    function Util(wrapper) {

        /**
         * 拨打电话
         */
        this.callPhone = function(phone) {
            wrapper.callPhone(phone);
        }

        /**
         * 复制文本
         */
        this.copy = function(text) {
            wrapper.copy(text);
        }

        /**
         * 记录日志
         */
        this.log = function(text) {
            wrapper.log(text);
        }
        this.openAppSetting = function() {
            wrapper.openAppSetting();
        }
    }

    /**
     * 业务相关功能
     */
    function Business(wrapper) {

        /**
         * 登录
         * 回调的code，0：成功；1：取消
         * json：后台返回的数据，json对象
         */
        this.login = function(callback) {
            window.nativeLoginCallback = function(code, json) {
                window.nativeLoginCallback = null;
                callback(code, json);
            };
            wrapper.login("nativeLoginCallback");
        }

        /**
         * 退出登录
         */
        this.logout = function() {
            wrapper.logout();
        }

        /**
         * 分享
         */
        this.share = function(url, title, description) {
            wrapper.share(url, title, description);
        }

        /**
         * RSA加密
         */
        this.rsa = function(text) {
            return wrapper.rsa(text);
        }

        /**
         * 获取APP的版本号
         */
        this.version = function() {
            return wrapper.version();
        }
        this.getJson = function() {
            return wrapper.getJson();
        }
    }

    function Native() {
        var wrapper;
        if (typeof android != 'undefined') {
            wrapper = new Android();
        } else {
            wrapper = new iOSWKWebView();
        }

        this.ui = new UI(wrapper);
        this.net = new Net(wrapper);
        this.file = new File(wrapper);
        this.util = new Util(wrapper);
        this.storage = new Storage(wrapper);
        this.business = new Business(wrapper);
    }
    window.native = new Native();

})(window);
