/*!
 * Native Javascript Library
 * v1.0.2 (2019-11-13)
 * 新增登录和分享接口
 * v1.0.3 (2019-11-14)
 * 新增rsa加密算法
 * v1.0.4 (2019-11-16)
 * 新增打开文件接口
 * v1.0.5 (2019-11-16)
 * 新增退出登录接口
 * v1.0.6 (2019-11-19)
 * 新增打开网页接口
 * v1.0.7 (2019-11-20)
 * 修改判断android手机和iOS手机的逻辑
 * v1.0.8 (2019-12-12)
 * iOS适配WKWebView
 * v1.0.9 (2020-05-06)
 * 默认使用WKWebView
 * v1.0.10 (2020-05-14)
 * 新增记录日志方法
 * v1.0.11 (2020-07-14)
 * 新增打开文件和分享功能
 * v1.0.12（2020-07-15）
 * 新增打开超链接的分享功能
 * v1.0.13（2020-08-12）
 * 新增APP版本号接口
 * v1.0.14（2022-01-11）
 * 新增权限撤回接口
 * v1.0.15（2022-08-23）
 * 新增页面获取原生传递参数的接口
 * v1.0.16（2022-11-1）
 * 新增是否切换备用域名的接口
 * v1.0.17（2025-3-27）
 * 适配纯血鸿蒙
 * v1.0.18（2025-4-1）
 * 新增原生commonAjax方法，请求可以跨域
 */
(function(window) {

    /**
     * android系统
     */
    function Android() {
        this.toast = function(text) {
            android.toast(text);
        }
        this.finish = function() {
            android.finish();
        }
        this.showInputMethod = function() {
            android.showInputMethod();
        }
        this.hiddenInputMethod = function() {
            android.hiddenInputMethod();
        }
        this.hasNetWork = function() {
            return android.hasNetWork();
        }
        this.href = function(tag, path) {
            android.href(tag, path);
        }
        this.open = function(url, title, shareEnabled) {
            android.open(url, title, shareEnabled);
        }
        this.html = function(url, title, shareEnabled) {
            try {
                android.html(url, title, shareEnabled);
            } catch (e) {
                android.html(url, title);
            }
        }
        this.callPhone = function(phone) {
            android.callPhone(phone);
        }
        this.copy = function(text) {
            android.copy(text);
        }
        this.setItem = function(key, value) {
            android.setItem(key,value);
        }
        this.getItem = function(key) {
            return android.getItem(key);
        }
        this.removeItem = function(key) {
            android.removeItem(key);
        }
        this.login = function(callback) {
            android.login(callback);
        }
        this.logout = function() {
            android.logout();
        }
        this.share = function(url, title, description) {
            android.share(url, title, description);
        }
        this.clearCache = function() {
            android.clearCache();
        }
        this.rsa = function(text) {
            return android.rsa(text);
        }
        this.log = function(text) {
            android.log(text);
        }
        this.version = function() {
            try {
                return android.version();
            } catch (e) {
                return "1.0.0";
            }
        }
        this.openAppSetting = function() {
            android.openAppSetting();
        }
        this.getJson = function() {
            return android.getJson();
        }
        this.isUsingBackServer = function() {
            return android.isUsingBackServer();
        }
        this.commonAjax = function(successFunName, errorFunName, url, type, contentType, data, tag) {
            android.http(successFunName, errorFunName, url, type, contentType, data, tag);
        }
    }

    /**
     * iOS系统WKWebView
     */
    function iOSWKWebView() {
        this.toast = function(text) {
            prompt("xcode-oc:toast", text);
        }
        this.finish = function() {
            prompt("xcode-oc:finish");
        }
        this.showInputMethod = function() {
            // do nothing
        }
        this.hiddenInputMethod = function() {
            // do nothing
        }
        this.hasNetWork = function() {
            return prompt("xcode-oc:hasNetWork") == "YES";
        }
        this.href = function(tag, path) {
            var json = {};
            json.tag = tag;
            json.path = path;
            prompt("xcode-oc:href", JSON.stringify(json));
        }
        this.open = function(url, title, shareEnabled) {
            var json = {};
            json.url = url;
            json.title = title;
            json.shareEnabled = shareEnabled;
            var r = prompt("xcode-oc:open2", JSON.stringify(json));
            if (r == "") {
                prompt("xcode-oc:open", url);
            }
        }
        this.html = function(url, title, shareEnabled) {
            var json = {};
            json.url = url;
            json.title = title;
            json.shareEnabled = shareEnabled;
            prompt("xcode-oc:html", JSON.stringify(json));
        }
        this.callPhone = function(phone) {
            prompt("xcode-oc:callPhone", phone);
        }
        this.copy = function(text) {
            prompt("xcode-oc:copy", text);
        }
        this.setItem = function(key, value) {
            var json = {};
            json.key = key;
            json.value = value;
            prompt("xcode-oc:setItem", JSON.stringify(json));
        }
        this.getItem = function(key) {
            return prompt("xcode-oc:getItem", key);
        }
        this.removeItem = function(key) {
            prompt("xcode-oc:removeItem", key);
        }
        this.login = function(callback) {
            prompt("xcode-oc:login", callback);
        }
        this.logout = function() {
            prompt("xcode-oc:logout");
        }
        this.share = function(url, title, description) {
            var json = {};
            json.url = url;
            json.title = title;
            json.description = description;
            prompt("xcode-oc:share", JSON.stringify(json));
        }
        this.clearCache = function() {
            prompt("xcode-oc:clearCache");
        }
        this.rsa = function(text) {
            return prompt("xcode-oc:rsa", text);
        }
        this.log = function(text) {
            prompt("xcode-oc:log", text);
        }
        this.version = function() {
            var r = prompt("xcode-oc:version");
            if (r == "") {
                return "1.0.0";
            } else {
                return r;
            }
        }
        this.openAppSetting = function() {
            // do nothing
        }
        this.getJson = function() {
            return prompt("xcode-oc:getJson");
        }
        this.isUsingBackServer = function() {
            return prompt("xcode-oc:isUsingBackServer") == "YES";
        }
        this.commonAjax = function(successFunName, errorFunName, url, type, contentType, data, tag) {
            var json = {}
            json.successFunName = successFunName
            json.errorFunName = errorFunName
            json.url = url
            json.type = type
            json.contentType = contentType
            json.data = data
            json.tag = tag
            prompt("xcode-oc:iosHttpTask",JSON.stringify(json))
        }
    }

    /**
     * 纯血鸿蒙系统
     */
    function HarmonyOS() {
        this.toast = function(text) {
            harmonyOSJSB.toast(text);
        }
        this.finish = function() {
            harmonyOSJSB.finish();
        }
        this.showInputMethod = function() {
            // do nothing
        }
        this.hiddenInputMethod = function() {
            // do nothing
        }
        this.hasNetWork = function() {
            return harmonyOSJSB.hasNetWork();
        }
        this.href = function(tag, path) {
            harmonyOSJSB.href(tag, path);
        }
        this.open = function(url, title, shareEnabled) {
            harmonyOSJSB.openDoc(url, title);
        }
        this.html = function(url, title, shareEnabled) {
            harmonyOSJSB.html(url, title, shareEnabled);
        }
        this.callPhone = function(phone) {
            harmonyOSJSB.callPhone(phone);
        }
        this.copy = function(text) {
            harmonyOSJSB.copy(text);
        }
        this.setItem = function(key, value) {
            harmonyOSJSB.setItem(key,value);
        }
        this.getItem = function(key) {
            return harmonyOSJSB.getItem(key);
        }
        this.removeItem = function(key) {
            harmonyOSJSB.removeItem(key);
        }
        this.login = function(callback) {
            harmonyOSJSB.login(callback);
        }
        this.logout = function() {
            harmonyOSJSB.logout();
        }
        this.share = function(url, title, description) {
            harmonyOSJSB.share(url, title, description);
        }
        this.clearCache = function() {
            harmonyOSJSB.clearCache();
        }
        this.rsa = function(text) {
            return harmonyOSJSB.rsa(text);
        }
        this.log = function(text) {
            // do nothing
        }
        this.version = function() {
            return harmonyOSJSB.version();
        }
        this.openAppSetting = function() {
            harmonyOSJSB.openAppSetting();
        }
        this.getJson = function() {
            return harmonyOSJSB.getJson();
        }
        this.isUsingBackServer = function() {
            return harmonyOSJSB.isUsingBackServer();
        }
        this.commonAjax = function(successFunName, errorFunName, url, type, contentType, data, tag) {
            harmonyOSJSB.http(successFunName, errorFunName, url, type, contentType, data, tag);
        }
    }

    /**
     * UI组件
     */
    function UI(wrapper) {

        /**
         * 显示toast
         */
        this.toast = function(text) {
            wrapper.toast(text);
        }

        /**
         * 关闭当前界面
         */
        this.finish = function() {
            wrapper.finish();
        }

        /**
         * 显示输入法，iOS不支持
         */
        this.showInputMethod = function() {
            wrapper.showInputMethod();
        }

        /**
         * 隐藏输入法，iOS不支持
         */
        this.hiddenInputMethod = function() {
            wrapper.hiddenInputMethod();
        }
    }

    /**
     * 网络组件
     */
    function Net(wrapper) {

        /**
         * 是否有网络
         */
        this.hasNetWork = function() {
            return wrapper.hasNetWork();
        }
    }

    /**
     * 文件组件
     */
    function File(wrapper) {

        /**
         * 跳转轻应用指定页面
         */
        this.href = function(tag, path) {
            wrapper.href(tag, path);
        }

        /**
         * 打开文件
         * url 文件网络地址
         */
        this.open = function(url, title, shareEnabled) {
            wrapper.open(url, title, shareEnabled);
        }

        /**
          * 打开html的网页
          */
        this.html = function(url, title, shareEnabled) {
            wrapper.html(url, title, shareEnabled);
        }
    }

    /**
     * 存储
     */
    function Storage(wrapper) {

        /**
         * 保存数据
         */
        this.setItem = function(key, value) {
            wrapper.setItem(key, value);
        }

        /**
         * 获取数据
         */
        this.getItem = function(key) {
            return wrapper.getItem(key);
        }

        /**
         * 移除数据
         */
         this.removeItem = function(key) {
            wrapper.removeItem(key);
         }

        /**
         * 清除缓存
         */
        this.clearCache = function() {
            wrapper.clearCache();
        }
    }

    /**
     * 其他组件
     */
    function Util(wrapper) {

        /**
         * 拨打电话
         */
        this.callPhone = function(phone) {
            wrapper.callPhone(phone);
        }

        /**
         * 复制文本
         */
        this.copy = function(text) {
            wrapper.copy(text);
        }

        /**
         * 记录日志
         */
        this.log = function(text) {
            wrapper.log(text);
        }

        /**
         * 打开系统设置
         */
        this.openAppSetting = function() {
            wrapper.openAppSetting();
        }
    }

    /**
     * 业务相关功能
     */
    function Business(wrapper) {

        /**
         * 登录
         * 回调的code，0：成功；1：取消
         * json：后台返回的数据，json对象
         */
        this.login = function(callback) {
            window.nativeLoginCallback = function(code, json) {
                window.nativeLoginCallback = null;
                callback(code, json);
            };
            wrapper.login("nativeLoginCallback");
        }

        /**
         * 退出登录
         */
        this.logout = function() {
            wrapper.logout();
        }

        /**
         * 分享
         */
        this.share = function(url, title, description) {
            wrapper.share(url, title, description);
        }

        /**
         * RSA加密
         */
        this.rsa = function(text) {
            return wrapper.rsa(text);
        }

        /**
         * 获取APP的版本号
         */
        this.version = function() {
            return wrapper.version();
        }

        /**
         * 获取native的传值
         */
        this.getJson = function() {
            return wrapper.getJson();
        }

        /**
         * 是否使用备份服务
         */
         this.isUsingBackServer = function() {
             return wrapper.isUsingBackServer();
         }
        /**
         * 通过原生发送网络请求
         */        
        this.commonAjax = function(options) {
            var tag = options.tag ? options.tag : "";
            var url = options.url,
                type = options.type,
                contentType = options.contentType,
                data = JSON.stringify(options.data);
        
            //原生需要数据
            var time = (new Date()).getTime();
            var successFunName = 'AjaxSuccess' + time + tag;
            var errorFunName = 'AjaxError' + time + tag;
        
            //原生调用方法，挂载在Windows
            var delectFun = function() {
                delete window[successFunName];
                delete window[errorFunName];
            };
            window[successFunName] = function(data) {
                if (typeof data == "string") {
                    data = JSON.parse(data);
                }
                options.success(data);
                delectFun();
            };
            window[errorFunName] = function(data) {
                options.error(data);
                delectFun();
            };
            wrapper.commonAjax(successFunName, errorFunName, url, type, contentType, data, tag);
        }
    }

    function Native() {
        var wrapper;
        if (typeof android != 'undefined') {
            wrapper = new Android();
        } else if (typeof harmonyOSJSB != 'undefined') {
            wrapper = new HarmonyOS();
        } else {
            wrapper = new iOSWKWebView();
        }
        this.ui = new UI(wrapper);
        this.net = new Net(wrapper);
        this.file = new File(wrapper);
        this.util = new Util(wrapper);
        this.storage = new Storage(wrapper);
        this.business = new Business(wrapper);
        this.isAndroid = function () {
            return typeof android != 'undefined'
        }
        this.isHarmonyOS = function () {
            return typeof harmonyOSJSB != "undefined"
        }
    }
    window.native = new Native();

})(window);
