Jx().package('s.ui.controls', function(J) {
    // 严格模式
    'use strict';
    var _currentPluginId = 0;
    var defaults = {
        color: {
            MA5: '#1b1b1b',
            MA10: '#ff00ff',
            MA20: '#2f89ca',
            upColor: '#db3338',
            downColor: '#319e26',
            lineColor: '#e1e1e1',
            closeLineColor: '#a0a0a0',
            fontColor: '#7f7f7f',
            pointerColor: '#5573aa',
            toolBgColor: '#fff',
            volUpColor: '#f29b76',
            volDownColr: '#5573aa'
        },
        grid: [
            ['4%', 0, '48%', 0],
			['65%',0, '0', 0]
        ],
        noDataGrid: [
            ['4%', 0, '48%', 0],
            ['65%', 0, '0', 0]
        ]
    };
    var attributeMap = {};

    this.kLineChart = new J.Class({
        extend: s.ui.BaseControl
    }, {
        defaults: defaults,
        attributeMap: attributeMap,
        // 构造函数
        init: function(element, options) {
            this.element = $(element);
            this.container = this.element;

            // 防止多次初始化
            if (this.element.data('initialized')) {
                return;
            }
            this.element.data('initialized', true);
            // 区分一个页面中存在多个控件实例
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);

            this.parseData(this.settings.data);
            this.render(this.chartData);
            this.dataZoomAction();
            this.setXAxisValue(this.chartData.xAxisMinMaxObj);
        },

        parseData: function(data) {
            data = data.data;
            if ($.isEmptyObject(data)) {
                console.warn('实例未获取到无数据!');
                return;
            }


            if (!data.picupdata || $.isEmptyObject(data.picupdata)) { //钱龙服务器挂掉或无历史数据
                this.chartData = {
                    isInit: true, //初始化
                    dataStart: 0,
                    categoryData: ['1', '2', '3'],
                    values: [],
                    volumes: [],
                    initY1Data: ['1', '2', '3', '4', '5', '6'],
                    initY2Data: ['1', '2', '3']
                };
                return;
            }
            var fistLastClose = '';
            if (data.picupdata.length > 200) {
                //后台多传一条数据用于k线图第一条数据的悬浮框字段颜色标示判断，实际不再线图中显示，删除这条数据
                fistLastClose = data.picupdata.splice(0, 1)[0][2]; //k线图悬浮框一条数据颜色判断基准

                data.picdowndata.splice(0, 1); //成交量多传了一条数据，删除第一条
            }
            // data.picupdata = data.picupdata.splice(0, 10);
            // data.picdowndata = data.picdowndata.splice(0, 10);
            var category = [];
            var dataStart, //k线图data-zoom显示数据的起始百分比
                dataZoomLenth = 50; //k线图界面渲染条数
            var len = data.picupdata.length; //k线数据长度

            for (var i = 0; i < len; i++) {
                category.push(data.picupdata[i].splice(0, 1)[0]);
            }

            if (len < dataZoomLenth) { //当数据小于50条，固定x轴长度
                for (var j = 0; j < (dataZoomLenth - len); j++) {
                    category.push(null);
                }
                dataStart = 0;
            } else {
                dataStart = 100 - Math.floor(dataZoomLenth / len * 100);
            }
            var values = data.picupdata;
            var volumes = data.picdowndata;
            var decimal = this.calcuDecimal(values);

            var xAxisMinI = parseInt(category.length * (dataStart/100))-1;
            var xAxisMiddI = parseInt(xAxisMinI + (category.length-xAxisMinI)/2)-1;
            var xAxisMaxI = category.length-1;
            var xAxisMinMax =[category[xAxisMinI],category[xAxisMiddI],category[xAxisMaxI]]

            var xAxisMinMaxObj = {
                startValue:category[xAxisMinI],
                middleValue:category[xAxisMiddI+1],
                endValue:category[xAxisMaxI]
            }

            // console.log(xAxisMinMax);

            this.chartData = {
                dataStart: dataStart,
                categoryData: category,
                values: values,
                volumes: volumes,
                decimal: decimal,
                fistLastClose: fistLastClose,
                xAxisMinMax:xAxisMinMax,
                xAxisMinMaxObj:xAxisMinMaxObj
            };
        },
        calcuDecimal: function(data) {
            var len = data.length;

            if (len < 1)
                return;
            var random = [];
            var decimal = [];
            for (var i = 0; i < 5; i++) {
                random.push(data[Math.round(Math.random() * (len - 1))][0]); //取开盘价的值
            }
            random.forEach(function(item) {
                item = item + '';
                if (/\./g.test(item)) {
                    decimal.push(item.split('.')[1].length);
                }
            });
            return decimal.length < 1 ? 0 : Math.max.apply(null, decimal); //最新价小数位数
        },
        render: function(data) {
            this.kLine(data);
            this.buildMATip();
            this.initElements();
            this.buildXaxisValue();
            this.XaxisInitElements();
            this.bindEvents();
        },
        initElements: function() {
            this.elements = {
                MAWrap: $('.MA-wrap', this.container),
                MA5: $('.MA-wrap .MA5 .value', this.container),
                MA10: $('.MA-wrap .MA10 .value', this.container),
                MA20: $('.MA-wrap .MA20 .value', this.container)
            };
        },
        bindEvents: function() {
            // this.element.on({
            //     'mouseenter': $.proxy(this.showMAWrap, this),
            //     mouseleave: $.proxy(this.hideMAWrap, this)
            // });
        },
        showMAWrap: function() {
            this.elements.MAWrap.css('visibility', 'visible');
        },
        hideMAWrap: function() {
            this.elements.MAWrap.css('visibility', 'hidden');
        },
        changeMA: function(obj) {
            var MA5, MA10, MA20;
            var context = this;

            obj.map(function(item) {
                if (item.seriesName === 'MA5')
                    MA5 = (typeof item.value === 'string') ? item.value : item.value.toFixed(context.chartData.decimal);
                if (item.seriesName === 'MA10')
                    MA10 = (typeof item.value === 'string') ? item.value : item.value.toFixed(context.chartData.decimal);
                if (item.seriesName === 'MA20')
                    MA20 = (typeof item.value === 'string') ? item.value : item.value.toFixed(context.chartData.decimal);
            });
            this.elements.MA5.text(MA5);
            this.elements.MA10.text(MA10);
            this.elements.MA20.text(MA20);
        },
        buildMATip: function() {
            var html = '' +
                '<ul class=MA-wrap>' +
                '   <li class="MA5">MA5：<span class="value">--</span></li>' +
                '   <li class="MA10">MA10：<span class="value">--</span></li>' +
                '   <li class="MA20">MA20：<span class="value">--</span></li>' +
                '</ul>';

            this.element.append($(html));
        },
        calculateMA: function(dayCount, data, flag) {
            var result = [];
            for (var i = 0, len = data.values.length; i < len; i++) {
                if (i < dayCount - 1) {
                    result.push('-');
                    continue;
                }
                var sum = 0;
                for (var j = 0; j < dayCount; j++) {
                    if (flag)
                        sum += +data.volumes[i - j][1];
                    sum += +data.values[i - j][1];
                }
                result.push(+(sum / dayCount).toFixed(this.chartData.decimal));
            }
            return result;
        },
        buildTooltip: function(seriesData,falg) {
            var weekly = ['周日', '周一', '周二', '周三', '周四', '周五', '周六'];
            var wrap = $('<div>');
            var i = 0;
            var html = '<div class="sectooltip-container">';
            if(!falg){

                html += '<span class="time">' + seriesData.axisValue.replace(/\//g, '.') + '&nbsp;&nbsp;' + weekly[new Date(seriesData.axisValue).getDay()] + '</span><br/>' +
                '<span class="name"><i>：</i>开盘</span><span class="value ' + (seriesData.openIsDown === true ? 'open-down' : (seriesData.openIsDown === null ? '' : 'open-up')) + '">' + this.formatToolTipsData(seriesData.data[1]) + '</span><br/>' +
                '<span class="name"><i>：</i>最高</span><span class="value ' + (seriesData.highIsDown === true ? 'high-down' : (seriesData.highIsDown === null ? '' : 'high-up')) + '">' + this.formatToolTipsData(seriesData.data[4]) + '</span><br/>' +
                '<span class="name"><i>：</i>最低</span><span class="value ' + (seriesData.lowIsDown === true ? 'low-down' : (seriesData.lowIsDown === null ? '' : 'low-up')) + '">' + this.formatToolTipsData(seriesData.data[3]) + '</span><br/>' +
                '<span class="name"><i>：</i>收盘</span><span class="value ' + (seriesData.closeIsDown === true ? 'close-down' : (seriesData.closeIsDown === null ? '' : 'close-up')) + '">' + this.formatToolTipsData(seriesData.data[2]) + '</span><br/>' +
                '<span class="name"><i>：</i>涨跌</span><span class="value ' + (seriesData.deltaIsDown === true ? 'delta-down' : (seriesData.deltaIsDown === null ? '' : 'delta-up')) + '">' + this.formatToolTipsData(seriesData.data[5]) + '</span><br/>' +
                '<span class="name"><i>：</i>涨幅</span><span class="value ' + (seriesData.deltaIsDown === true ? 'delta-down' : (seriesData.deltaIsDown === null ? '' : 'delta-up')) + '">' + this.formatToolTipsData(seriesData.data[6]) + '%</span><br/>' +
                '<span class="name"><i>：</i>成交量</span><span class="value">' + this.calcuUnit(seriesData.data[7], 2) + '手</span><br/>' +
                '<span class="name"><i>：</i>成交额</span><span class="value">' + this.calcuUnit(seriesData.data[8], 2) + '元</span><br/>' +
                '</div>';
            }
            var template = _.template(config.template.trendTemStr);
            var topInfoObj = {
                now:this.formatToolTipsData(seriesData.data[2]),
                newIsDown:seriesData.closeIsDown,
                delta:this.formatToolTipsData(seriesData.data[5]),
                deltaPercent:this.formatToolTipsData(seriesData.data[6]),
                high:this.formatToolTipsData(seriesData.data[4]),
                low:this.formatToolTipsData(seriesData.data[3]),
                highIsDown:seriesData.highIsDown,
                lowIsDown:seriesData.lowIsDown,
                volume:this.calcuUnit(seriesData.data[7], 2),
                amount:this.calcuUnit(seriesData.data[8], 2),
                openIsDown:false
            };
            calcuUnit(topInfoObj);

            topInfoObj.delta = Math.abs(topInfoObj.delta);
            topInfoObj.deltaPercent = Math.abs(topInfoObj.deltaPercent);
            
            this.toolTipCallback(topInfoObj);

            var htmlStr = template(topInfoObj);
            config.recroedMinData = htmlStr;
            config.elements.trendDetailWrap.empty().append(htmlStr);
            
            
            return wrap.html(html);
        },

        //点击图点的时候可以重写此方法。
        toolTipCallback:function(data){

        },

        XaxisInitElements:function(){
            this.elements["xAxisEle"] = $('.xAis-wrap', this.container);
            this.elements["startValue"] = $('.xAis-wrap .startValue .value', this.container);
            this.elements["middleValue"] = $('.xAis-wrap .middleValue .value', this.container);
            this.elements["endValue"] = $('.xAis-wrap .endValue .value', this.container);
        },
        buildXaxisValue: function() {
            var html = '' +
                '<ul class="xAis-wrap">' +
                '   <li class="startValue"><span class="value"></span></li>' +
                '   <li class="middleValue"><span class="value"></span></li>' +
                '   <li class="endValue"><span class="value"></span></li>' +
                '</ul>';
            this.element.append($(html));
        },
        //设置X轴 坐标值
        setXAxisValue: function(obj){
            this.elements.startValue.text(obj.startValue);
            this.elements.middleValue.text(obj.middleValue);
            this.elements.endValue.text(obj.endValue);
        },
        formatToolTipsData: function(value) {
            if (value === null)
                return '--';
            else
                return value;
        },
        parseTooltipsData: function(data) {
            var i = data.dataIndex - 1;
            var reg = /^\-/;
            var lastCloseData = ''; //前一个交易日收盘价

            if (i < 0) { //第一条数据的前一个交易日的收盘价   若总条数小于200条则取当日收盘价
                lastCloseData = this.chartData.fistLastClose ? +this.chartData.fistLastClose : this.chartData.values[i + 1][1];
            } else {
                lastCloseData = +this.chartData.values[i][1];
            }
            data.openIsDown = +data.data[1] < lastCloseData ? true : false;
            data.closeIsDown = +data.data[2] < lastCloseData ? true : false;
            data.highIsDown = +data.data[4] < lastCloseData ? true : false;
            data.lowIsDown = +data.data[3] < lastCloseData ? true : false;
            data.deltaIsDown = reg.test(data.data[5]); //false 涨   true 跌  今日的涨跌
        },
        calcuUnit: function(num, decimal) {
            var hundredMillion = 100000000;
            var tenThousand = 10000;
            var value = '';
            var r = '';

            if (num === null) {
                value = '--';
            } else if (num / hundredMillion >= 1) {
                r = num / hundredMillion;
                value = r.toString().length < 4 ? (r + '亿') : (r.toFixed(decimal) + '亿');
                // value = (num / hundredMillion).toFixed(decimal) + '亿';
            } else if (num / tenThousand >= 1) {
                r = num / tenThousand;
                value = r.toString().length < 4 ? (r + '万') : (r.toFixed(decimal) + '万');
                // value = (num / tenThousand).toFixed(decimal) + '万';
            } else {
                value = num;
            }
            return value;
        },
        dataZoomOption: function() {
            var dataZoomOption = [{
                type: 'inside',
                xAxisIndex: [0, 1],
                start: this.chartData.dataStart,
                throttle: 0,
                // filterMode: 'empty',
                zoomLock: this.chartData.dataStart === 0 ? true : false,
                // zoomLock:false,
                throttle:0,
                show: false,   
                end: 100
            }, {
                // show: this.chartData.dataStart === 0 ? false : true,
                show: false,                
                xAxisIndex: [0, 1],
                type: 'slider',
                bottom: '3%',
                throttle: 0,
                start: this.chartData.dataStart,
                end: 100,
                height: '8%',
                zoomLock: this.chartData.dataStart === 0 ? true : false,
                // filterMode: 'empty',
                showDetail: false,
                handleIcon: 'M10.7,11.9v-1.3H9.3v1.3c-4.9,0.3-8.8,4.4-8.8,9.4c0,5,3.9,9.1,8.8,9.4v1.3h1.3v-1.3c4.9-0.3,8.8-4.4,8.8-9.4C19.5,16.3,15.6,12.2,10.7,11.9z M13.3,24.4H6.7V23h6.6V24.4z M13.3,19.6H6.7v-1.4h6.6V19.6z',
                handleSize: '80%',
                handleStyle: {
                    color: '#fff',
                    shadowBlur: 3,
                    shadowColor: 'rgba(0, 0, 0, 0.6)',
                    shadowOffsetX: 2,
                    shadowOffsetY: 2
                }
            }];
            return dataZoomOption;
        },
        dataZoomAction:function(){//滑动改变数据时。X轴坐标值 最大最小变化。
            var context = this;
            context.settings.chart.on('datazoom',function(params){
                var opt = context.settings.chart.getOption();
                var dz = opt.dataZoom[0];
                var start = opt.xAxis[0].data[dz.startValue];
                var end = opt.xAxis[0].data[dz.endValue];
                var midd = opt.xAxis[0].data[dz.startValue + parseInt((dz.endValue - dz.startValue)/2)];
                context.chartData.xAxisMinMax[0] = start;
                context.chartData.xAxisMinMax[1] = midd;
                context.chartData.xAxisMinMax[2] = end;

                context.chartData.xAxisMinMaxObj.startValue = start,
                context.chartData.xAxisMinMaxObj.middleValue = midd,
                context.chartData.xAxisMinMaxObj.endValue = end,

                context.setXAxisValue(context.chartData.xAxisMinMaxObj);


            });
        },
        gridOption: function() {
            var gridOption = [{
                show: true,
                borderColor: this.settings.color.lineColor,
                borderWidth: 0.5,
                zlevel: 10,
                top: this.chartData.isInit ? this.settings.noDataGrid[0][0] : this.settings.grid[0][0],
                right: this.chartData.isInit ? this.settings.noDataGrid[0][1] : this.settings.grid[0][1],
                bottom: this.chartData.isInit ? this.settings.noDataGrid[0][2] : this.settings.grid[0][2],
                left: this.chartData.isInit ? this.settings.noDataGrid[0][3] : this.settings.grid[0][3]
            }, {
                show: true,
                borderColor: this.settings.color.lineColor,
                borderWidth: 0.5,
                zlevel: 10,
                top: this.chartData.isInit ? this.settings.noDataGrid[1][0] : this.settings.grid[1][0],
                right: this.chartData.isInit ? this.settings.noDataGrid[1][1] : this.settings.grid[1][1],
                bottom: this.chartData.isInit ? this.settings.noDataGrid[1][2] : this.settings.grid[1][2],
                left: this.chartData.isInit ? this.settings.noDataGrid[1][3] : this.settings.grid[1][3]
            }];
            return gridOption;
        },
        yAxisOption: function() {
            var context = this;
            var yAxisOption = [{
                type: this.chartData.isInit ? 'category' : 'value',
                boundaryGap: this.chartData.isInit ? false : true,
                scale: true,
                position: 'left',
                axisLabel: {
                    show: this.chartData.isInit ? false : true,
                    color: this.settings.color.fontColor,
                    verticalAlign:"bottom",
                    inside:true,
                    formatter: function(value) {
                        if (context.chartData.isInit)
                            return value;
                        return value.toFixed(context.chartData.decimal);
                    }
                   

                },
                axisTick: {
                    show: false
                },
                splitLine: {
                    show: true,
                    lineStyle: {
                        color: this.settings.color.lineColor,
                        type: 'dashed'
                    }
                },
                axisLine: {
                    show: false
                },
                axisPointer: {
                    // show: this.chartData.isInit ? false : true,
                    show: this.chartData.isInit ? false : true,                    
                    label: {
                        show: true,
                        precision:2
                    },
                    snap: false,
                    lineStyle: {
                        type: 'solid',
                        color: this.settings.color.pointerColor //坐标指示器的线条颜色
                    }
                }
            }, {
                type: this.chartData.isInit ? 'category' : 'value',
                boundaryGap: this.chartData.isInit ? false : true,
                // scale: true,
                gridIndex: 1,
                splitNumber: 2,
                position: 'left',
                axisLabel: {
                    color: this.settings.color.fontColor,
                    verticalAlign:"bottom",
                    inside:true,
                    formatter: function(value) {
                        if (context.chartData.isInit)
                            return;
                        return context.calcuUnit(value, 0);
                    }
                },
                axisTick: {
                    show: false
                },
                splitLine: {
                    show: true,
                    lineStyle: {
                        color: this.settings.color.lineColor,
                        type: 'dashed'
                    }
                },
                axisLine: {
                    lineStyle: {
                        color: this.settings.color.lineColor
                    }
                },
                axisPointer: {
                    show: this.chartData.isInit ? false : true,
                    label: {
                        show: false
                    },
                    snap: true,
                    lineStyle: {
                        type: 'solid',
                        color: this.settings.color.pointerColor //坐标指示器的线条颜色
                    }
                }
            }];
            return yAxisOption;
        },
        kLineOption: function() {
            var context = this;
            var option = {
                animation: false,
                tooltip: {
                    trigger: 'axis',
                    triggerOn:"click",
                    backgroundColor: context.settings.color.toolBgColor,
                    padding: [10, 20, 10, 14],
                    formatter: function(params) {
                        var html = '';
                        context.changeMA(params);
                        params.every(function(item) {
                            if (item.seriesIndex === 0) {
                                context.parseTooltipsData(item);
                                html = context.buildTooltip(item);
                                return false;
                            }
                            return true;
                        });
                        // return html.html();
                        return "";

                    },
                    axisPointer: {
                        type: 'cross',
                        // link: {xAxisIndex: 'all'},
                        label: {
                            color:"#fff",
                            backgroundColor: '#5573aa'
                        },
                       
                        lineStyle: {
                            color: context.settings.color.pointerColor, //坐标指示器的线条颜色
                            type: 'solid'
                        }
                    },
                    textStyle: {
                        color: context.settings.color.fontColor,
                        fontSize: 12
                    },
                    position: function(point, params, dom, rect, size) {
                        // 鼠标在左侧时 tooltip 显示到右侧，鼠标在右侧时 tooltip 显示到左侧。
                        var obj = {
                            top: '5%'
                        };
                        // obj[['left', 'right'][+(point[0] < size.viewSize[0] / 2)]] = 60;
                        if (point[0] < size.viewSize[0] / 2) {
                            obj.right = 65;
                        } else {
                            obj.left = 40;
                        }
                        return obj;
                    },
                    extraCssText: 'box-shadow: 0 0 6px rgba(0, 0, 0, .075)',
                    z:220
                    // extraCssText: 'width: 170px'
                },
                axisPointer: {
                    label: {
                        show: false
                    },
                    link: {
                        xAxisIndex: [0, 1]
                    }
                },
                visualMap: [{
                    show: false,
                    seriesIndex: 4,
                    dimension: 2,
                    pieces: [{
                        value: 'plus',
                        color: context.settings.color.upColor
                    }, {
                        value: 'minus',
                        color: context.settings.color.downColor
                    }]
                }],
                grid: this.gridOption(),
                dataZoom: this.dataZoomOption(),
                xAxis: [{
                    type: 'category',
                    scale: true,
                    boundaryGap: this.chartData.isInit ? false : true,
                    axisLine: {
                        onZero: false,
                        show: false
                    },
                    axisTick: {
                        show: false
                    },
                    splitLine: {
                        show: true,
                        lineStyle: {
                            color: this.settings.color.lineColor,
                            type: 'dashed'
                        },
                        interval:function(){
                            $.map(context.chartData.xAxisMinMaxObj,function(item){
                                return item;
                            });
                        }
                    },
                    axisLabel: {
                        show:false,
                        color: context.settings.color.fontColor
                    },
                    splitNumber: 20,
                    axisPointer: {
                        show: this.chartData.isInit ? false : true,
                        // label: {
                        //     show: false
                        // }
                    },
                    data: context.chartData.categoryData
                }, {
                    type: 'category',
                    gridIndex: 1,
                    boundaryGap: this.chartData.isInit ? false : true,
                    scale: true,
                    axisLine: {
                        onZero: false,
                        lineStyle: {
                            color: context.settings.color.lineColor,
                            type: 'dashed'
                        }
                    },
                    axisTick: {
                        show: false
                    },
                    splitLine: {
                        show: this.chartData.isInit ? true : false,
                        lineStyle: {
                            color: this.settings.color.lineColor,
                            type: 'dashed'
                        }
                    },
                    axisLabel: {
                        show: false
                    },
                    splitNumber: 20,
                    data: context.chartData.categoryData,
                    axisPointer: {
                        show: this.chartData.isInit ? false : true,
                        label: {
                            show: false
                        }
                    }
                }],
                yAxis: this.yAxisOption(),
                series: [{
                    name: 'Dow-Jones index',
                    type: 'candlestick',
                    itemStyle: {
                        normal: {
                            color: context.settings.color.upColor,
                            color0: context.settings.color.downColor,
                            borderColor: null,
                            borderColor0: null
                        }
                    },
                    barWidth: '60%',
                    data: []
                }, {
                    name: 'MA5',
                    type: 'line',
                    smooth: true,
                    symbol: 'circle',
                    symbolSize: 8,
                    showSymbol: false,
                    lineStyle: {
                        normal: {
                            // opacity: 0.5,
                            width: 1.2,
                            color: context.settings.color.MA5
                        }
                    },
                    z: 300,
                    data: []
                }, {
                    name: 'MA10',
                    type: 'line',
                    smooth: true,
                    symbol: 'circle',
                    symbolSize: 8,
                    showSymbol: false,
                    lineStyle: {
                        normal: {
                            // opacity: 0.5,
                            width: 1.2,
                            color: context.settings.color.MA10
                        }
                    },
                    z: 300,
                    data: []
                }, {
                    name: 'MA20',
                    type: 'line',
                    smooth: true,
                    symbol: 'circle',
                    symbolSize: 8,
                    showSymbol: false,
                    lineStyle: {
                        normal: {
                            // opacity: 0.5,
                            width: 1.2,
                            color: context.settings.color.MA20
                        }
                    },
                    z: 300,
                    data: []
                }, {
                    name: 'Volume',
                    type: 'bar',
                    xAxisIndex: 1,
                    yAxisIndex: 1,
                    barWidth: '60%',
                    data: []
                }, {
                    name: 'VMA5',
                    type: 'line',
                    xAxisIndex: 1,
                    yAxisIndex: 1,
                    smooth: true,
                    symbol: 'circle',
                    symbolSize: 0,
                    showSymbol: false,
                    connectNulls: true,
                    lineStyle: {
                        normal: {
                            // opacity: 0.5,
                            width: 1.2,
                            color: context.settings.color.MA5
                        }
                    },
                    data: []
                }, {
                    name: 'VMA10',
                    type: 'line',
                    xAxisIndex: 1,
                    yAxisIndex: 1,
                    smooth: true,
                    symbol: 'circle',
                    symbolSize: 0,
                    showSymbol: false,
                    connectNulls: true,
                    lineStyle: {
                        normal: {
                            // opacity: 0.5,
                            width: 1.2,
                            color: context.settings.color.MA10
                        }
                    },
                    data: []
                }, {
                    name: 'VMA20',
                    type: 'line',
                    xAxisIndex: 1,
                    yAxisIndex: 1,
                    smooth: true,
                    symbol: 'circle',
                    symbolSize: 0,
                    showSymbol: false,
                    connectNulls: true,
                    lineStyle: {
                        normal: {
                            // opacity: 0.5,
                            width: 1.2,
                            color: context.settings.color.MA20
                        }
                    },
                    data: []
                }]
            };
            return option;
        },
        kLine: function(data) {
            var context = this;
            var myChart = echarts.init(this.element[0]);
            var option = this.kLineOption();

            myChart.setOption(option);

            //echarts实例
            this.settings.chart = myChart;

            this.setKLineData();
        },
        updateOption: function(data) {
            this.parseData(data.data);
            var yAxisOption = this.yAxisOption();
            var gridOption = this.gridOption();
            var dataZoomOption = this.dataZoomOption();
            var series = this.resetData();

            this.settings.chart.setOption({
                grid: gridOption,
                dataZoom: dataZoomOption,
                yAxis: yAxisOption,
                series: series
            });
            this.setKLineData();
        },
        setKLineData: function() {
            var option = {
                xAxis: [{
                    axisPointer: {
                        show: this.chartData.isInit ? false : true,
                    },
                    data: this.chartData.categoryData
                }, {
                    axisPointer: {
                        show: this.chartData.isInit ? false : true,
                    },
                    data: this.chartData.categoryData
                }],
                series: [{
                    name: 'Dow-Jones index',
                    data: this.chartData.values
                }, {
                    name: 'MA5',
                    data: this.calculateMA(5, this.chartData),
                }, {
                    name: 'MA10',
                    data: this.calculateMA(10, this.chartData),
                }, {
                    name: 'MA20',
                    data: this.calculateMA(20, this.chartData),
                }, {
                    name: 'Volume',
                    data: this.chartData.volumes
                }, {
                    name: 'VMA5',
                    data: this.calculateMA(5, this.chartData, true),
                }, {
                    name: 'VMA10',
                    data: this.calculateMA(10, this.chartData, true),
                }, {
                    name: 'VMA20',
                    data: this.calculateMA(20, this.chartData, true),
                }]
            };

            if (this.chartData.isInit) { //初始化
                option.yAxis = [{
                    data: this.chartData.initY1Data
                }, {
                    data: this.chartData.initY2Data
                }];
            }
            this.settings.chart.resize(); //解决有些数据渲染K错乱的问题
            this.settings.chart.setOption(option);
            this.resize();
        },
        resetData: function() {
            var series = [{
                data: [] //k线
            }, {
                data: [] //MA5
            }, {
                data: [] //MA10
            }, {
                data: [] //MA20
            }, {
                data: [] //成交量
            }, {
                data: [] //VMA5
            }, {
                data: [] //VM10
            }, {
                data: [] //VM20
            }];
            return series;
        },
        resize: function() {
            var context = this;

            $(window).on('resize', function() {
                context.settings.chart.resize();
            });
        },
        getEchartsInstance: function() {
            return this.settings.chart;
        },
        //显示最后一个值
        thisLastData:function(){
            var data =  this.settings.chart.getOption().series[0].data;
            var dataLen = data.length-1;
            var unitData = {
                data:data[dataLen],
                dataIndex:dataLen
            };
            this.parseTooltipsData(unitData);
            this.buildTooltip(unitData,true);
        }
       
    });
});