import DETAIL_ART from "../template/detail.art";
import RUNTIME from 'art-template/lib/runtime';
import '../scss/detail.scss';

"use strict";

// 页面入口
window.initial = function () {
    new _currentObject().init();
}

// 页面主对象
var _currentObject = function () {
    var str = szse.getDetailJson(); // 获取原生传入的数据
    if (!str) {
        var cacheData = JSON.parse(window.localStorage.getItem('iporeportNewsCache')); 
        this.url = cacheData.jsonPathUrl;
    } else {
        this.url = domain.cmsweb + str;
    };
    this.cacheTag = this.url.split('/').pop().split('.')[0];
}

// 初始化
_currentObject.prototype.init = function () {

    // art-template日期格式注册
    RUNTIME.dateFormat = function (date, format) {
        var format = szse.formatDate(date, format);
        format = format.split("00:00:00")[0];
        return format;
    };

    // 判断是否有缓存，无则开启load页面，有则关闭load页面并渲染缓存
    if (!szse.judgeCache(this.cacheTag)) {
        szse.load(true);
        this.fetch(this.url);
    } else {
        szse.load(false);
        this.render(szse.getStorage(this.cacheTag));
    }
};

// 获取数据
_currentObject.prototype.fetch = function (params) {
    var _self = this;
    szse.ajax({
        url: params,
        contentType: "json",
        type: "GET",
        data: "",
        tag: _self.tag,
        success: function (data) {
            // 取消loading页面
            szse.load(false);
            if (data.code == 0) {
                if (JSON.stringify(data.data) == '{}') {
                    document.body.style.backgroundColor = '#fff';
                    szse.noData('main');
                } else {
                    szse.setStorage(_self.cacheTag, data); // 设置缓存
                    _self.render(data); // 渲染
                }
            } else {
                document.body.style.backgroundColor = '#fff';
                szse.noData('main');
            }
        },
        error: function (data) {
            szse.load(false);
            if (!szse.judgeCache(_self.cacheTag) && !szse.hasWeb()) {
                document.body.style.backgroundColor = '#fff';
                szse.error(true);
                mui("body").on('tap', '#errorBox', function () {
                    window.location.reload();
                });
            } else {
                document.body.style.backgroundColor = '#fff';
                szse.noData('main');
            }
        }
    });
};

// 渲染
_currentObject.prototype.render = function (data) {
    document.querySelector('#main').innerHTML = DETAIL_ART({
        'data': data.data,
        'url': domain.cmsweb
    });
    
    if (document.querySelector('#videoPlay')) {
        this.bindEvents();
    }
};

// 绑定事件
_currentObject.prototype.bindEvents = function () {
    // 初始化videojs
    var player = videojs('videoPlay', {}, function () {
        var play,
            isMuted = true,
            _this = this,
            isPlay = true,
            video = document.querySelector('#videoPlay_html5_api');

        // 点击视频画面播放/暂停视频
        mui('#main').on('tap', '#videoPlay', function (e) {
            isPlay = !isPlay;
            play = isPlay ? _this.pause() : _this.play();
        });

        // 点击播放、暂停按钮
        mui('#main').on('tap', '.vjs-control-bar .vjs-play-control', function (e) {
            e.stopPropagation();
            play = (this.getAttribute('title') == 'Pause') ? _this.pause() : _this.play();
        });

        // 点击静音按钮
        mui('#main').on('tap', '.vjs-volume-panel-horizontal>button', function (e) {
            e.stopPropagation();
            isMuted = !isMuted; 
            video.muted = isMuted ? false : true;
            if (video.muted) {
                this.querySelector('span').classList.remove('icon-unmuted');
                this.querySelector('span').classList.add('icon-muted');
            } else {
                this.querySelector('span').classList.remove('icon-muted');
                this.querySelector('span').classList.add('icon-unmuted');
            }
        });
    });
};