/**    
 * JET (Javascript Extension Tools) 
 * Copyright (c) 2009, KDV.cn, All rights reserved.
 * http://code.google.com/p/j-et/
 *
 * @version    1.0
 * @author    Kinvix(<a href="mailto:Kinvix@gmail.com">Kinvix@gmail.com</a>)
 * 
 */

/**    
 * @description
 * Package: Jx
 *
 * Need package:
 * no.
 * 
 */

/**
 * 1.[JET core]: JET 微内核
 */
;(function(tn){

    // 严格模式
    'use strict';

    var version = "1.0.0",
        mark = "JxMark",
        topNamespace = tn,
        undefinedValue,
        
        Jx = topNamespace.Jx,
        
        PACKAGES = {};
        

    if(typeof Jx === "undefined" || (Jx.mark && Jx.mark === mark)){
        
        // 如果已经有Jx对象则记录已有的信息
        if(Jx){
            PACKAGES = Jx.PACKAGES;
        }
        
        /**
         * 【Jx 对象原型】
         * Jx
         * @class 
         * @constructor Jx
         * @global
         * 
         * @since version 1.0
         * @description Jx 对象原型的描述
         * 
         * @param {Number} ver 要使用的 Jx 的版本号，当前是1.0
         * @param {Boolean} isCreateNew 是否创建一个新的 Jx 实例，默认为 false 不创建新的 Jx 实例，只返回同一版本在全局中的唯一一个实例，注意：除非特殊需要，否则一般不要创建新的 Jx 实例
         * @return {Object} 返回对应版本的 Jx 对象
         * 
         * @example
         * //代码组织方式一(传统)：
         * var J = new Jx();
         * J.out(J.version);    //输出当前Jx的版本
         * 
         * @example
         * //代码组织方式二(推荐)：
         * Jx().$package(function(J){
         *     J.out(J.version);    //输出当前Jx的版本
         * };
         * //注：此种方式可以利用匿名函数来防止变量污染全局命名空间，尤其适合大型WebApp的构建！
         * 
         * @example
         * //范例：
         * Jx().$package("tencent.alloy", function(J){
         *     var $ = J.dom.id,
         *     $D = J.dom,
         *     $E = J.event,
         *     $H = J.http;
         *     this.name = "腾讯Q+ Web";
         *     J.out(this.name);
         * };
         * 
         */
        Jx = function(isCreateNew){
            var J = this;

            var instanceOf = function(o, type) {
                return (o && o.hasOwnProperty && (o instanceof type));
            };

            if(isCreateNew){
                // 如果是第一次执行则初始化对象
                if ( !( instanceOf(J, Jx) ) ) {
                    J = new Jx();
                } else {
                    J._init();
                }
            }else{
                J = Jx.Root;
            }
            return J;
        };
        
        Jx.prototype = {
            version: version,

            _init: function(){
                this.constructor = Jx;
                //return true;
            },

            namespace: function(name) {
                // Handle "", null, undefined, false
                if ( !name ) {
                    return topNamespace;
                }

                name = String(name);

                var i,
                    ni,
                    nis = name.split("."),
                    ns = topNamespace;

                for(i = 0; i < nis.length; i=i+1){
                    ni = nis[i];
                    ns[ni] = ns[ni] || {};
                    ns = ns[nis[i]];
                }

                return ns;
            },

            package: function(){
                var name = arguments[0],
                    func = arguments[arguments.length-1],
                    ns = topNamespace,
                    returnValue;
                    if(typeof func === "function"){
                        // handle name as ""
                        if(typeof name === "string"){
                            ns = this.namespace(name);
                            if( !Jx.PACKAGES[name] ){
                                Jx.PACKAGES[name] = {
                                    isLoaded: true,
                                    returnValue: returnValue    // undefined as default
                                };
                            }
                            // else{
                            //     throw new Error("Package name [" + name + "] is exist!");    
                            // }
                            ns.packageName = name;
                        }else if(typeof name === "object"){
                            ns = name;
                        }
                        
                        returnValue = func.call(ns, this);
                        typeof name === "string" && (Jx.PACKAGES[name].returnValue = returnValue);
                    }else{
                        throw new Error("Function required");
                    }

            },
            

            checkPackage: function(name){
                return Jx.PACKAGES[name];
            },

            startTime: +new Date(),
            
            /**
             * Jx 对象转化为字符串的方法
             * 
             * @ignore
             * @return {String} 返回 Jx 对象串行化后的信息
             */
            toString: function(){
                return "JET version " + this.version + " !";
            }
        };

        
        /**
         * 记录加载的包的对象
         * 
         * @ignore
         * @type Object
         */
        Jx.PACKAGES = PACKAGES;

        /**
         * 创建一个当前版本 Jx 的实例
         * 
         * @ignore
         * @type Object
         */
        Jx.Root = new Jx(true);
    
        Jx.mark = mark;
        
        topNamespace.Jx = Jx;
    }else{
        throw new Error("\"Jx\" name is defined in other javascript code !!!");
    }
})(this);


/**
 * 2.[Javascript core]: 常用工具函数扩展
 */
Jx().package(function(J){
    
    // 严格模式
    'use strict';
    var isUndefined,
        isNull,
        isNumber,
        isString,
        isBoolean,
        isObject,
        isArray,
        isArguments,
        isFunction,
        $typeof,
        
        
        emptyFunc,
        
        random,

        extend,
        Class;

    /**
     * 判断变量的值是否是 undefined
     * Determines whether or not the provided object is undefined
     * 
     * @method isUndefined
     * @memberOf Jx.prototype
     * 
     * @param {Mixed} o 传入被检测变量的名称
     * @return {Boolean} 当 o 的值是 undefined 时返回 true
     */
    isUndefined = function(o) {
        return typeof(o) === "undefined";
    };
        
    /**
     * 判断变量的值是否是 null
     * Determines whether or not the provided object is null
     * 
     * @method isNull
     * @memberOf Jx.prototype
     * 
     * @param {Mixed} o 传入被检测变量的名称
     * @return {Boolean} 当 o 的值是 null 时返回 true
     */
    isNull = function(o) {
        return o === null;
    };
    
    /**
     * 判断变量的类型是否是 Number
     * Determines whether or not the provided object is a number
     * 
     * @memberOf Jx.prototype
     * @name isNumber
     * @function
     * @param {Mixed} o 传入被检测变量的名称
     * @return {Boolean} 当 o 的类型是 number 时返回 true
     */
    isNumber = function(o) {
        return (o === 0 || o) && o.constructor === Number;
    };
    
    /**
     * 判断变量的类型是否是 Boolean
     * Determines whether or not the provided object is a boolean
     * 
     * 
     * @method isBoolean
     * @memberOf Jx.prototype
     * 
     * @static
     * @param {Mixed} o 传入被检测变量的名称
     * @return {Boolean} 当 o 的类型是 boolean 时返回 true
     */
    isBoolean = function(o) {
        return (o === false || o) && (o.constructor === Boolean);
    };
    
    /**
     * 判断变量的类型是否是 String
     * Determines whether or not the provided object is a string
     * 
     * 
     * @method isString
     * @memberOf Jx.prototype
     * 
     * @static
     * @param {Mixed} o 传入被检测变量的名称
     * @return {Boolean} 当 o 的类型是 string 时返回 true
     */
    isString = function(o) {
        return (o === "" || o) && (o.constructor === String);
    };
    
    /**
     * 判断变量的类型是否是 Object
     * Determines whether or not the provided object is a object
     * 
     * 
     * @method isObject
     * @memberOf Jx.prototype
     * 
     * @param {Mixed} o 传入被检测变量的名称
     * @return {Boolean} 当 o 的类型是 object 时返回 true
     */
    isObject = function(o) {
        return o && (o.constructor === Object || Object.prototype.toString.call(o) === "[object Object]");
    };
    
    /**
     * 判断变量的类型是否是 Array
     * Determines whether or not the provided object is a array
     * 
     * 
     * @method isArray
     * @memberOf Jx.prototype
     * 
     * @param {Mixed} o 传入被检测变量的名称
     * @return {Boolean} 当 o 的类型是 array 时返回 true
     */
    isArray = function(o) {
        return o && (o.constructor === Array || Object.prototype.toString.call(o) === "[object Array]");
    };
    
    /**
     * 判断变量的类型是否是 Arguments
     * Determines whether or not the provided object is a arguments
     * 
     * 
     * @method isArguments
     * @memberOf Jx.prototype
     * 
     * @param {Mixed} o 传入被检测变量的名称
     * @return {Boolean} 当 o 的类型是 arguments 时返回 true
     */
    isArguments = function(o) {
        return o && o.callee && isNumber(o.length) ? true : false;
    };
    
    /**
     * 判断变量的类型是否是 Function
     * Determines whether or not the provided object is a function
     * 
     * 
     * @method isFunction
     * @memberOf Jx.prototype
     * 
     * @param {Mixed} o 传入被检测变量的名称
     * @return {Boolean} 当 o 的类型是 function 时返回 true
     */
    isFunction = function(o) {
        return o && (o.constructor === Function);
    };
    
    /**
     * 判断变量类型的方法
     * Determines the type of object
     * 
     * 
     * @method $typeof
     * @memberOf Jx.prototype
     * 
     * @param {Mixed} o 传入被检测变量的名称
     * @return {String} 返回变量的类型，如果不识别则返回 other
     */
    $typeof = function(o) {
        if(isUndefined(o)){
            return "undefined";
        }else if(isNull(o)){
            return "null";
        }else if(isNumber(o)){
            return "number";
        }else if(isBoolean(o)){
            return "boolean";
        }else if(isString(o)){
            return "string";
        }else if(isObject(o)){
            return "object";
        }else if(isArray(o)){
            return "array";
        }else if(isArguments(o)){
            return "arguments";
        }else if(isFunction(o)){
            return "function";
        }else{
            return "other";
        }
        
    };
    
    /**
     * 生成随机数的方法
     * 
     * @method random
     * @memberOf Jx.prototype
     * 
     * @param {Number} min 生成随机数的最小值
     * @param {Number} max 生成随机数的最大值
     * @return {Number} 返回生成的随机数
     */
    random = function(min, max){
        return Math.floor(Math.random() * (max - min + 1) + min);
    };
    
    /**
     * 对一个对象或数组进行扩展
     * 
     * @method extend
     * @memberOf Jx.prototype
     * 
     * @param {Mixed} beExtendObj 被扩展的对象或数组
     * @param {Mixed} extendObj1, extendObj2, .... 用来参照扩展的对象或数组
     * @return {Mixed} 返回被扩展后的对象或数组
     * 
     * @example
     * Jx().$package(function(J){
     *     // 用 objB 和objC 扩展 objA 对象；
     *     J.extend(objA, objB, objC);
     * };
     * 
     */
    extend = function(beExtendObj, extendObj1, extendObj2){
        var a = arguments,
            i,
            p,
            beExtendObj,
            extendObj;
            
        if(a.length === 1){
            beExtendObj = this;
            i=0;
        }else{
            beExtendObj = a[0] || {};
            i=1;
        }
        
        for(; i<arguments.length; i++){
            extendObj = arguments[i];
            for(p in extendObj){
                var src = beExtendObj[p],
                    obj = extendObj[p];
                if ( src === obj ){
                    continue;
                }
                
                if ( obj && isObject(obj) && !isArray(obj) && !obj.nodeType && !isFunction(obj)){
                    src = beExtendObj[p] || {};//2010-12-28
                    beExtendObj[p] = extend( src, 
                        // Never move original objects, clone them
                        obj || ( obj.length != null ? [ ] : { } ));

                // Don't bring in undefined values
                }else if ( obj !== undefined ){
                    beExtendObj[p] = obj;
                }
            }
        }

        return beExtendObj;
    };    
    
    /**
     * 一个空函数函数
     * 
     * @memberOf Jx.prototype
     */
    emptyFunc = function(){};

    /**
     * 创建Class类的类
     * Class
     * @class 
     * @memberOf Jx
     * @param {Object} option = {extend: superClass} 在option对象的extend属性中指定要继承的对象，可以不写
     * @param {Object} object 扩展的对象
     * @return {Object} 返回生成的日期时间字符串
     * 
     * @example
     * Jx().$package(function(J){
     *     var Person = new J.Class({
     *      init : function(name){
     *          this.name = name;
     *          alert("init");
     *      },
     *      showName : function(){
     *          alert(this.name);
     *  
     *      }
     *  
     *  });
     *  
     *  // 继承Person
     *     var Person2 = new J.Class({extend : Person}, {
     *      init : function(name){
     *          this.name = name;
     *          alert("init");
     *      },
     *      showName : function(){
     *          alert(this.name);
     *  
     *      }
     *  
     *  });
     *     
     * };
     * 
     */
    Class = function(){
        var length = arguments.length;
        var option = arguments[length-1];
        
        option.init = option.init || function(){};
        
        // 如果参数中有要继承的父类
        if(length === 2){
            /**
             * @ignore
             */
            var superClass = arguments[0].extend;
            
            /**
             * @ignore
             */
            var tempClass = function() {};
            tempClass.prototype = superClass.prototype;
            
            /**
             * @ignore
             */
            var subClass = function() {
                this.init.apply(this, arguments);
            }
            
            // 加一个对父类原型引用的静态属性
            subClass.superClass = superClass.prototype;
            //subClass.superClass = superClass;
            /**
             * @ignore
             */
            subClass.callSuper = function(context,funcName){
                var slice = Array.prototype.slice;
                var a = slice.call(arguments, 2);
                var func = subClass.superClass[funcName];
                //var func = subClass.superClass.prototype[func];
                if(func){
                    func.apply(context, a.concat(slice.call(arguments)));
                }
            };
            
            // 指定原型
            subClass.prototype = new tempClass();
            
            // 重新指定构造函数
            subClass.prototype.constructor = subClass;
            
            J.extend(subClass.prototype, option);
            
            /**
             * @ignore
             */
            subClass.prototype.init = function(){
                // 调用父类的构造函数
                // subClass.superClass.init.apply(this, arguments);
                // 调用此类自身的构造函数
                option.init.apply(this, arguments);
            };
            
            return subClass;
            
        // 如果参数中没有父类，则单纯构建一个类
        }else if(length === 1){
            /**
             * @ignore
             */
            var newClass = function() {
                // 加了return，否则init返回的对象不生效
                return this.init.apply(this, arguments);
            }
            newClass.prototype = option;
            return newClass;
        }
        
        
    };



    J.isUndefined = isUndefined;
    J.isNull = isNull;
    J.isNumber = isNumber;
    J.isString = isString;
    J.isBoolean = isBoolean;
    J.isObject = isObject;
    J.isArray = isArray;
    J.isArguments = isArguments;
    J.isFunction = isFunction;
    J.$typeof = $typeof;
    
    J.emptyFunc = emptyFunc;
    
    J.random = random;

    J.extend = extend;
    J.Class = Class;
});

//! moment.js
//! version : 2.10.6
//! authors : Tim Wood, Iskren Chernev, Moment.js contributors
//! license : MIT
//! momentjs.com

(function() {

    // 'use strict';

    var hookCallback;

    function utils_hooks__hooks () {
        return hookCallback.apply(null, arguments);
    }

    // This is done to register the method called with moment()
    // without creating circular dependencies.
    function setHookCallback (callback) {
        hookCallback = callback;
    }

    function isArray(input) {
        return Object.prototype.toString.call(input) === '[object Array]';
    }

    function isDate(input) {
        return input instanceof Date || Object.prototype.toString.call(input) === '[object Date]';
    }

    function map(arr, fn) {
        var res = [], i;
        for (i = 0; i < arr.length; ++i) {
            res.push(fn(arr[i], i));
        }
        return res;
    }

    function hasOwnProp(a, b) {
        return Object.prototype.hasOwnProperty.call(a, b);
    }

    function extend(a, b) {
        for (var i in b) {
            if (hasOwnProp(b, i)) {
                a[i] = b[i];
            }
        }

        if (hasOwnProp(b, 'toString')) {
            a.toString = b.toString;
        }

        if (hasOwnProp(b, 'valueOf')) {
            a.valueOf = b.valueOf;
        }

        return a;
    }

    function create_utc__createUTC (input, format, locale, strict) {
        return createLocalOrUTC(input, format, locale, strict, true).utc();
    }

    function defaultParsingFlags() {
        // We need to deep clone this object.
        return {
            empty           : false,
            unusedTokens    : [],
            unusedInput     : [],
            overflow        : -2,
            charsLeftOver   : 0,
            nullInput       : false,
            invalidMonth    : null,
            invalidFormat   : false,
            userInvalidated : false,
            iso             : false
        };
    }

    function getParsingFlags(m) {
        if (m._pf == null) {
            m._pf = defaultParsingFlags();
        }
        return m._pf;
    }

    function valid__isValid(m) {
        if (m._isValid == null) {
            var flags = getParsingFlags(m);
            m._isValid = !isNaN(m._d.getTime()) &&
                flags.overflow < 0 &&
                !flags.empty &&
                !flags.invalidMonth &&
                !flags.invalidWeekday &&
                !flags.nullInput &&
                !flags.invalidFormat &&
                !flags.userInvalidated;

            if (m._strict) {
                m._isValid = m._isValid &&
                    flags.charsLeftOver === 0 &&
                    flags.unusedTokens.length === 0 &&
                    flags.bigHour === undefined;
            }
        }
        return m._isValid;
    }

    function valid__createInvalid (flags) {
        var m = create_utc__createUTC(NaN);
        if (flags != null) {
            extend(getParsingFlags(m), flags);
        }
        else {
            getParsingFlags(m).userInvalidated = true;
        }

        return m;
    }

    var momentProperties = utils_hooks__hooks.momentProperties = [];

    function copyConfig(to, from) {
        var i, prop, val;

        if (typeof from._isAMomentObject !== 'undefined') {
            to._isAMomentObject = from._isAMomentObject;
        }
        if (typeof from._i !== 'undefined') {
            to._i = from._i;
        }
        if (typeof from._f !== 'undefined') {
            to._f = from._f;
        }
        if (typeof from._l !== 'undefined') {
            to._l = from._l;
        }
        if (typeof from._strict !== 'undefined') {
            to._strict = from._strict;
        }
        if (typeof from._tzm !== 'undefined') {
            to._tzm = from._tzm;
        }
        if (typeof from._isUTC !== 'undefined') {
            to._isUTC = from._isUTC;
        }
        if (typeof from._offset !== 'undefined') {
            to._offset = from._offset;
        }
        if (typeof from._pf !== 'undefined') {
            to._pf = getParsingFlags(from);
        }
        if (typeof from._locale !== 'undefined') {
            to._locale = from._locale;
        }

        if (momentProperties.length > 0) {
            for (i in momentProperties) {
                prop = momentProperties[i];
                val = from[prop];
                if (typeof val !== 'undefined') {
                    to[prop] = val;
                }
            }
        }

        return to;
    }

    var updateInProgress = false;

    // Moment prototype object
    function Moment(config) {
        copyConfig(this, config);
        this._d = new Date(config._d != null ? config._d.getTime() : NaN);
        // Prevent infinite loop in case updateOffset creates new moment
        // objects.
        if (updateInProgress === false) {
            updateInProgress = true;
            utils_hooks__hooks.updateOffset(this);
            updateInProgress = false;
        }
    }

    function isMoment (obj) {
        return obj instanceof Moment || (obj != null && obj._isAMomentObject != null);
    }

    function absFloor (number) {
        if (number < 0) {
            return Math.ceil(number);
        } else {
            return Math.floor(number);
        }
    }

    function toInt(argumentForCoercion) {
        var coercedNumber = +argumentForCoercion,
            value = 0;

        if (coercedNumber !== 0 && isFinite(coercedNumber)) {
            value = absFloor(coercedNumber);
        }

        return value;
    }

    function compareArrays(array1, array2, dontConvert) {
        var len = Math.min(array1.length, array2.length),
            lengthDiff = Math.abs(array1.length - array2.length),
            diffs = 0,
            i;
        for (i = 0; i < len; i++) {
            if ((dontConvert && array1[i] !== array2[i]) ||
                (!dontConvert && toInt(array1[i]) !== toInt(array2[i]))) {
                diffs++;
            }
        }
        return diffs + lengthDiff;
    }

    function Locale() {
    }

    var locales = {};
    var globalLocale;

    function normalizeLocale(key) {
        return key ? key.toLowerCase().replace('_', '-') : key;
    }

    // pick the locale from the array
    // try ['en-au', 'en-gb'] as 'en-au', 'en-gb', 'en', as in move through the list trying each
    // substring from most specific to least, but move to the next array item if it's a more specific variant than the current root
    function chooseLocale(names) {
        var i = 0, j, next, locale, split;

        while (i < names.length) {
            split = normalizeLocale(names[i]).split('-');
            j = split.length;
            next = normalizeLocale(names[i + 1]);
            next = next ? next.split('-') : null;
            while (j > 0) {
                locale = loadLocale(split.slice(0, j).join('-'));
                if (locale) {
                    return locale;
                }
                if (next && next.length >= j && compareArrays(split, next, true) >= j - 1) {
                    //the next array item is better than a shallower substring of this one
                    break;
                }
                j--;
            }
            i++;
        }
        return null;
    }

    function loadLocale(name) {
        var oldLocale = null;
        // TODO: Find a better way to register and load all the locales in Node
        if (!locales[name] && typeof module !== 'undefined' &&
                module && module.exports) {
            try {
                oldLocale = globalLocale._abbr;
                require('./locale/' + name);
                // because defineLocale currently also sets the global locale, we
                // want to undo that for lazy loaded locales
                locale_locales__getSetGlobalLocale(oldLocale);
            } catch (e) { }
        }
        return locales[name];
    }

    // This function will load locale and then set the global locale.  If
    // no arguments are passed in, it will simply return the current global
    // locale key.
    function locale_locales__getSetGlobalLocale (key, values) {
        var data;
        if (key) {
            if (typeof values === 'undefined') {
                data = locale_locales__getLocale(key);
            }
            else {
                data = defineLocale(key, values);
            }

            if (data) {
                // moment.duration._locale = moment._locale = data;
                globalLocale = data;
            }
        }

        return globalLocale._abbr;
    }

    function defineLocale (name, values) {
        if (values !== null) {
            values.abbr = name;
            locales[name] = locales[name] || new Locale();
            locales[name].set(values);

            // backwards compat for now: also set the locale
            locale_locales__getSetGlobalLocale(name);

            return locales[name];
        } else {
            // useful for testing
            delete locales[name];
            return null;
        }
    }

    // returns locale data
    function locale_locales__getLocale (key) {
        var locale;

        if (key && key._locale && key._locale._abbr) {
            key = key._locale._abbr;
        }

        if (!key) {
            return globalLocale;
        }

        if (!isArray(key)) {
            //short-circuit everything else
            locale = loadLocale(key);
            if (locale) {
                return locale;
            }
            key = [key];
        }

        return chooseLocale(key);
    }

    var aliases = {};

    function addUnitAlias (unit, shorthand) {
        var lowerCase = unit.toLowerCase();
        aliases[lowerCase] = aliases[lowerCase + 's'] = aliases[shorthand] = unit;
    }

    function normalizeUnits(units) {
        return typeof units === 'string' ? aliases[units] || aliases[units.toLowerCase()] : undefined;
    }

    function normalizeObjectUnits(inputObject) {
        var normalizedInput = {},
            normalizedProp,
            prop;

        for (prop in inputObject) {
            if (hasOwnProp(inputObject, prop)) {
                normalizedProp = normalizeUnits(prop);
                if (normalizedProp) {
                    normalizedInput[normalizedProp] = inputObject[prop];
                }
            }
        }

        return normalizedInput;
    }

    function makeGetSet (unit, keepTime) {
        return function (value) {
            if (value != null) {
                get_set__set(this, unit, value);
                utils_hooks__hooks.updateOffset(this, keepTime);
                return this;
            } else {
                return get_set__get(this, unit);
            }
        };
    }

    function get_set__get (mom, unit) {
        return mom._d['get' + (mom._isUTC ? 'UTC' : '') + unit]();
    }

    function get_set__set (mom, unit, value) {
        return mom._d['set' + (mom._isUTC ? 'UTC' : '') + unit](value);
    }

    // MOMENTS

    function getSet (units, value) {
        var unit;
        if (typeof units === 'object') {
            for (unit in units) {
                this.set(unit, units[unit]);
            }
        } else {
            units = normalizeUnits(units);
            if (typeof this[units] === 'function') {
                return this[units](value);
            }
        }
        return this;
    }

    function zeroFill(number, targetLength, forceSign) {
        var absNumber = '' + Math.abs(number),
            zerosToFill = targetLength - absNumber.length,
            sign = number >= 0;
        return (sign ? (forceSign ? '+' : '') : '-') +
            Math.pow(10, Math.max(0, zerosToFill)).toString().substr(1) + absNumber;
    }

    var formattingTokens = /(\[[^\[]*\])|(\\)?(Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Q|YYYYYY|YYYYY|YYYY|YY|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g;

    var localFormattingTokens = /(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g;

    var formatFunctions = {};

    var formatTokenFunctions = {};

    // token:    'M'
    // padded:   ['MM', 2]
    // ordinal:  'Mo'
    // callback: function () { this.month() + 1 }
    function addFormatToken (token, padded, ordinal, callback) {
        var func = callback;
        if (typeof callback === 'string') {
            func = function () {
                return this[callback]();
            };
        }
        if (token) {
            formatTokenFunctions[token] = func;
        }
        if (padded) {
            formatTokenFunctions[padded[0]] = function () {
                return zeroFill(func.apply(this, arguments), padded[1], padded[2]);
            };
        }
        if (ordinal) {
            formatTokenFunctions[ordinal] = function () {
                return this.localeData().ordinal(func.apply(this, arguments), token);
            };
        }
    }

    function removeFormattingTokens(input) {
        if (input.match(/\[[\s\S]/)) {
            return input.replace(/^\[|\]$/g, '');
        }
        return input.replace(/\\/g, '');
    }

    function makeFormatFunction(format) {
        var array = format.match(formattingTokens), i, length;

        for (i = 0, length = array.length; i < length; i++) {
            if (formatTokenFunctions[array[i]]) {
                array[i] = formatTokenFunctions[array[i]];
            } else {
                array[i] = removeFormattingTokens(array[i]);
            }
        }

        return function (mom) {
            var output = '';
            for (i = 0; i < length; i++) {
                output += array[i] instanceof Function ? array[i].call(mom, format) : array[i];
            }
            return output;
        };
    }

    // format date using native date object
    function formatMoment(m, format) {
        if (!m.isValid()) {
            return m.localeData().invalidDate();
        }

        format = expandFormat(format, m.localeData());
        formatFunctions[format] = formatFunctions[format] || makeFormatFunction(format);

        return formatFunctions[format](m);
    }

    function expandFormat(format, locale) {
        var i = 5;

        function replaceLongDateFormatTokens(input) {
            return locale.longDateFormat(input) || input;
        }

        localFormattingTokens.lastIndex = 0;
        while (i >= 0 && localFormattingTokens.test(format)) {
            format = format.replace(localFormattingTokens, replaceLongDateFormatTokens);
            localFormattingTokens.lastIndex = 0;
            i -= 1;
        }

        return format;
    }

    var match1         = /\d/;            //       0 - 9
    var match2         = /\d\d/;          //      00 - 99
    var match3         = /\d{3}/;         //     000 - 999
    var match4         = /\d{4}/;         //    0000 - 9999
    var match6         = /[+-]?\d{6}/;    // -999999 - 999999
    var match1to2      = /\d\d?/;         //       0 - 99
    var match1to3      = /\d{1,3}/;       //       0 - 999
    var match1to4      = /\d{1,4}/;       //       0 - 9999
    var match1to6      = /[+-]?\d{1,6}/;  // -999999 - 999999

    var matchUnsigned  = /\d+/;           //       0 - inf
    var matchSigned    = /[+-]?\d+/;      //    -inf - inf

    var matchOffset    = /Z|[+-]\d\d:?\d\d/gi; // +00:00 -00:00 +0000 -0000 or Z

    var matchTimestamp = /[+-]?\d+(\.\d{1,3})?/; // 123456789 123456789.123

    // any word (or two) characters or numbers including two/three word month in arabic.
    var matchWord = /[0-9]*['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+|[\u0600-\u06FF\/]+(\s*?[\u0600-\u06FF]+){1,2}/i;

    var regexes = {};

    function isFunction (sth) {
        // https://github.com/moment/moment/issues/2325
        return typeof sth === 'function' &&
            Object.prototype.toString.call(sth) === '[object Function]';
    }


    function addRegexToken (token, regex, strictRegex) {
        regexes[token] = isFunction(regex) ? regex : function (isStrict) {
            return (isStrict && strictRegex) ? strictRegex : regex;
        };
    }

    function getParseRegexForToken (token, config) {
        if (!hasOwnProp(regexes, token)) {
            return new RegExp(unescapeFormat(token));
        }

        return regexes[token](config._strict, config._locale);
    }

    // Code from http://stackoverflow.com/questions/3561493/is-there-a-regexp-escape-function-in-javascript
    function unescapeFormat(s) {
        return s.replace('\\', '').replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g, function (matched, p1, p2, p3, p4) {
            return p1 || p2 || p3 || p4;
        }).replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
    }

    var tokens = {};

    function addParseToken (token, callback) {
        var i, func = callback;
        if (typeof token === 'string') {
            token = [token];
        }
        if (typeof callback === 'number') {
            func = function (input, array) {
                array[callback] = toInt(input);
            };
        }
        for (i = 0; i < token.length; i++) {
            tokens[token[i]] = func;
        }
    }

    function addWeekParseToken (token, callback) {
        addParseToken(token, function (input, array, config, token) {
            config._w = config._w || {};
            callback(input, config._w, config, token);
        });
    }

    function addTimeToArrayFromToken(token, input, config) {
        if (input != null && hasOwnProp(tokens, token)) {
            tokens[token](input, config._a, config, token);
        }
    }

    var YEAR = 0;
    var MONTH = 1;
    var DATE = 2;
    var HOUR = 3;
    var MINUTE = 4;
    var SECOND = 5;
    var MILLISECOND = 6;

    function daysInMonth(year, month) {
        return new Date(Date.UTC(year, month + 1, 0)).getUTCDate();
    }

    // FORMATTING

    addFormatToken('M', ['MM', 2], 'Mo', function () {
        return this.month() + 1;
    });

    addFormatToken('MMM', 0, 0, function (format) {
        return this.localeData().monthsShort(this, format);
    });

    addFormatToken('MMMM', 0, 0, function (format) {
        return this.localeData().months(this, format);
    });

    // ALIASES

    addUnitAlias('month', 'M');

    // PARSING

    addRegexToken('M',    match1to2);
    addRegexToken('MM',   match1to2, match2);
    addRegexToken('MMM',  matchWord);
    addRegexToken('MMMM', matchWord);

    addParseToken(['M', 'MM'], function (input, array) {
        array[MONTH] = toInt(input) - 1;
    });

    addParseToken(['MMM', 'MMMM'], function (input, array, config, token) {
        var month = config._locale.monthsParse(input, token, config._strict);
        // if we didn't find a month name, mark the date as invalid.
        if (month != null) {
            array[MONTH] = month;
        } else {
            getParsingFlags(config).invalidMonth = input;
        }
    });

    // LOCALES

    var defaultLocaleMonths = 'January_February_March_April_May_June_July_August_September_October_November_December'.split('_');
    function localeMonths (m) {
        return this._months[m.month()];
    }

    var defaultLocaleMonthsShort = 'Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec'.split('_');
    function localeMonthsShort (m) {
        return this._monthsShort[m.month()];
    }

    function localeMonthsParse (monthName, format, strict) {
        var i, mom, regex;

        if (!this._monthsParse) {
            this._monthsParse = [];
            this._longMonthsParse = [];
            this._shortMonthsParse = [];
        }

        for (i = 0; i < 12; i++) {
            // make the regex if we don't have it already
            mom = create_utc__createUTC([2000, i]);
            if (strict && !this._longMonthsParse[i]) {
                this._longMonthsParse[i] = new RegExp('^' + this.months(mom, '').replace('.', '') + '$', 'i');
                this._shortMonthsParse[i] = new RegExp('^' + this.monthsShort(mom, '').replace('.', '') + '$', 'i');
            }
            if (!strict && !this._monthsParse[i]) {
                regex = '^' + this.months(mom, '') + '|^' + this.monthsShort(mom, '');
                this._monthsParse[i] = new RegExp(regex.replace('.', ''), 'i');
            }
            // test the regex
            if (strict && format === 'MMMM' && this._longMonthsParse[i].test(monthName)) {
                return i;
            } else if (strict && format === 'MMM' && this._shortMonthsParse[i].test(monthName)) {
                return i;
            } else if (!strict && this._monthsParse[i].test(monthName)) {
                return i;
            }
        }
    }

    // MOMENTS

    function setMonth (mom, value) {
        var dayOfMonth;

        // TODO: Move this out of here!
        if (typeof value === 'string') {
            value = mom.localeData().monthsParse(value);
            // TODO: Another silent failure?
            if (typeof value !== 'number') {
                return mom;
            }
        }

        dayOfMonth = Math.min(mom.date(), daysInMonth(mom.year(), value));
        mom._d['set' + (mom._isUTC ? 'UTC' : '') + 'Month'](value, dayOfMonth);
        return mom;
    }

    function getSetMonth (value) {
        if (value != null) {
            setMonth(this, value);
            utils_hooks__hooks.updateOffset(this, true);
            return this;
        } else {
            return get_set__get(this, 'Month');
        }
    }

    function getDaysInMonth () {
        return daysInMonth(this.year(), this.month());
    }

    function checkOverflow (m) {
        var overflow;
        var a = m._a;

        if (a && getParsingFlags(m).overflow === -2) {
            overflow =
                a[MONTH]       < 0 || a[MONTH]       > 11  ? MONTH :
                a[DATE]        < 1 || a[DATE]        > daysInMonth(a[YEAR], a[MONTH]) ? DATE :
                a[HOUR]        < 0 || a[HOUR]        > 24 || (a[HOUR] === 24 && (a[MINUTE] !== 0 || a[SECOND] !== 0 || a[MILLISECOND] !== 0)) ? HOUR :
                a[MINUTE]      < 0 || a[MINUTE]      > 59  ? MINUTE :
                a[SECOND]      < 0 || a[SECOND]      > 59  ? SECOND :
                a[MILLISECOND] < 0 || a[MILLISECOND] > 999 ? MILLISECOND :
                -1;

            if (getParsingFlags(m)._overflowDayOfYear && (overflow < YEAR || overflow > DATE)) {
                overflow = DATE;
            }

            getParsingFlags(m).overflow = overflow;
        }

        return m;
    }

    function warn(msg) {
        if (utils_hooks__hooks.suppressDeprecationWarnings === false && typeof console !== 'undefined' && console.warn) {
            console.warn('Deprecation warning: ' + msg);
        }
    }

    function deprecate(msg, fn) {
        var firstTime = true;

        return extend(function () {
            if (firstTime) {
                warn(msg + '\n' + (new Error()).stack);
                firstTime = false;
            }
            return fn.apply(this, arguments);
        }, fn);
    }

    var deprecations = {};

    function deprecateSimple(name, msg) {
        if (!deprecations[name]) {
            warn(msg);
            deprecations[name] = true;
        }
    }

    utils_hooks__hooks.suppressDeprecationWarnings = false;

    var from_string__isoRegex = /^\s*(?:[+-]\d{6}|\d{4})-(?:(\d\d-\d\d)|(W\d\d$)|(W\d\d-\d)|(\d\d\d))((T| )(\d\d(:\d\d(:\d\d(\.\d+)?)?)?)?([\+\-]\d\d(?::?\d\d)?|\s*Z)?)?$/;

    var isoDates = [
        ['YYYYYY-MM-DD', /[+-]\d{6}-\d{2}-\d{2}/],
        ['YYYY-MM-DD', /\d{4}-\d{2}-\d{2}/],
        ['GGGG-[W]WW-E', /\d{4}-W\d{2}-\d/],
        ['GGGG-[W]WW', /\d{4}-W\d{2}/],
        ['YYYY-DDD', /\d{4}-\d{3}/]
    ];

    // iso time formats and regexes
    var isoTimes = [
        ['HH:mm:ss.SSSS', /(T| )\d\d:\d\d:\d\d\.\d+/],
        ['HH:mm:ss', /(T| )\d\d:\d\d:\d\d/],
        ['HH:mm', /(T| )\d\d:\d\d/],
        ['HH', /(T| )\d\d/]
    ];

    var aspNetJsonRegex = /^\/?Date\((\-?\d+)/i;

    // date from iso format
    function configFromISO(config) {
        var i, l,
            string = config._i,
            match = from_string__isoRegex.exec(string);

        if (match) {
            getParsingFlags(config).iso = true;
            for (i = 0, l = isoDates.length; i < l; i++) {
                if (isoDates[i][1].exec(string)) {
                    config._f = isoDates[i][0];
                    break;
                }
            }
            for (i = 0, l = isoTimes.length; i < l; i++) {
                if (isoTimes[i][1].exec(string)) {
                    // match[6] should be 'T' or space
                    config._f += (match[6] || ' ') + isoTimes[i][0];
                    break;
                }
            }
            if (string.match(matchOffset)) {
                config._f += 'Z';
            }
            configFromStringAndFormat(config);
        } else {
            config._isValid = false;
        }
    }

    // date from iso format or fallback
    function configFromString(config) {
        var matched = aspNetJsonRegex.exec(config._i);

        if (matched !== null) {
            config._d = new Date(+matched[1]);
            return;
        }

        configFromISO(config);
        if (config._isValid === false) {
            delete config._isValid;
            utils_hooks__hooks.createFromInputFallback(config);
        }
    }

    utils_hooks__hooks.createFromInputFallback = deprecate(
        'moment construction falls back to js Date. This is ' +
        'discouraged and will be removed in upcoming major ' +
        'release. Please refer to ' +
        'https://github.com/moment/moment/issues/1407 for more info.',
        function (config) {
            config._d = new Date(config._i + (config._useUTC ? ' UTC' : ''));
        }
    );

    function createDate (y, m, d, h, M, s, ms) {
        //can't just apply() to create a date:
        //http://stackoverflow.com/questions/181348/instantiating-a-javascript-object-by-calling-prototype-constructor-apply
        var date = new Date(y, m, d, h, M, s, ms);

        //the date constructor doesn't accept years < 1970
        if (y < 1970) {
            date.setFullYear(y);
        }
        return date;
    }

    function createUTCDate (y) {
        var date = new Date(Date.UTC.apply(null, arguments));
        if (y < 1970) {
            date.setUTCFullYear(y);
        }
        return date;
    }

    addFormatToken(0, ['YY', 2], 0, function () {
        return this.year() % 100;
    });

    addFormatToken(0, ['YYYY',   4],       0, 'year');
    addFormatToken(0, ['YYYYY',  5],       0, 'year');
    addFormatToken(0, ['YYYYYY', 6, true], 0, 'year');

    // ALIASES

    addUnitAlias('year', 'y');

    // PARSING

    addRegexToken('Y',      matchSigned);
    addRegexToken('YY',     match1to2, match2);
    addRegexToken('YYYY',   match1to4, match4);
    addRegexToken('YYYYY',  match1to6, match6);
    addRegexToken('YYYYYY', match1to6, match6);

    addParseToken(['YYYYY', 'YYYYYY'], YEAR);
    addParseToken('YYYY', function (input, array) {
        array[YEAR] = input.length === 2 ? utils_hooks__hooks.parseTwoDigitYear(input) : toInt(input);
    });
    addParseToken('YY', function (input, array) {
        array[YEAR] = utils_hooks__hooks.parseTwoDigitYear(input);
    });

    // HELPERS

    function daysInYear(year) {
        return isLeapYear(year) ? 366 : 365;
    }

    function isLeapYear(year) {
        return (year % 4 === 0 && year % 100 !== 0) || year % 400 === 0;
    }

    // HOOKS

    utils_hooks__hooks.parseTwoDigitYear = function (input) {
        return toInt(input) + (toInt(input) > 68 ? 1900 : 2000);
    };

    // MOMENTS

    var getSetYear = makeGetSet('FullYear', false);

    function getIsLeapYear () {
        return isLeapYear(this.year());
    }

    addFormatToken('w', ['ww', 2], 'wo', 'week');
    addFormatToken('W', ['WW', 2], 'Wo', 'isoWeek');

    // ALIASES

    addUnitAlias('week', 'w');
    addUnitAlias('isoWeek', 'W');

    // PARSING

    addRegexToken('w',  match1to2);
    addRegexToken('ww', match1to2, match2);
    addRegexToken('W',  match1to2);
    addRegexToken('WW', match1to2, match2);

    addWeekParseToken(['w', 'ww', 'W', 'WW'], function (input, week, config, token) {
        week[token.substr(0, 1)] = toInt(input);
    });

    // HELPERS

    // firstDayOfWeek       0 = sun, 6 = sat
    //                      the day of the week that starts the week
    //                      (usually sunday or monday)
    // firstDayOfWeekOfYear 0 = sun, 6 = sat
    //                      the first week is the week that contains the first
    //                      of this day of the week
    //                      (eg. ISO weeks use thursday (4))
    function weekOfYear(mom, firstDayOfWeek, firstDayOfWeekOfYear) {
        var end = firstDayOfWeekOfYear - firstDayOfWeek,
            daysToDayOfWeek = firstDayOfWeekOfYear - mom.day(),
            adjustedMoment;


        if (daysToDayOfWeek > end) {
            daysToDayOfWeek -= 7;
        }

        if (daysToDayOfWeek < end - 7) {
            daysToDayOfWeek += 7;
        }

        adjustedMoment = local__createLocal(mom).add(daysToDayOfWeek, 'd');
        return {
            week: Math.ceil(adjustedMoment.dayOfYear() / 7),
            year: adjustedMoment.year()
        };
    }

    // LOCALES

    function localeWeek (mom) {
        return weekOfYear(mom, this._week.dow, this._week.doy).week;
    }

    var defaultLocaleWeek = {
        dow : 0, // Sunday is the first day of the week.
        doy : 6  // The week that contains Jan 1st is the first week of the year.
    };

    function localeFirstDayOfWeek () {
        return this._week.dow;
    }

    function localeFirstDayOfYear () {
        return this._week.doy;
    }

    // MOMENTS

    function getSetWeek (input) {
        var week = this.localeData().week(this);
        return input == null ? week : this.add((input - week) * 7, 'd');
    }

    function getSetISOWeek (input) {
        var week = weekOfYear(this, 1, 4).week;
        return input == null ? week : this.add((input - week) * 7, 'd');
    }

    addFormatToken('DDD', ['DDDD', 3], 'DDDo', 'dayOfYear');

    // ALIASES

    addUnitAlias('dayOfYear', 'DDD');

    // PARSING

    addRegexToken('DDD',  match1to3);
    addRegexToken('DDDD', match3);
    addParseToken(['DDD', 'DDDD'], function (input, array, config) {
        config._dayOfYear = toInt(input);
    });

    // HELPERS

    //http://en.wikipedia.org/wiki/ISO_week_date#Calculating_a_date_given_the_year.2C_week_number_and_weekday
    function dayOfYearFromWeeks(year, week, weekday, firstDayOfWeekOfYear, firstDayOfWeek) {
        var week1Jan = 6 + firstDayOfWeek - firstDayOfWeekOfYear, janX = createUTCDate(year, 0, 1 + week1Jan), d = janX.getUTCDay(), dayOfYear;
        if (d < firstDayOfWeek) {
            d += 7;
        }

        weekday = weekday != null ? 1 * weekday : firstDayOfWeek;

        dayOfYear = 1 + week1Jan + 7 * (week - 1) - d + weekday;

        return {
            year: dayOfYear > 0 ? year : year - 1,
            dayOfYear: dayOfYear > 0 ?  dayOfYear : daysInYear(year - 1) + dayOfYear
        };
    }

    // MOMENTS

    function getSetDayOfYear (input) {
        var dayOfYear = Math.round((this.clone().startOf('day') - this.clone().startOf('year')) / 864e5) + 1;
        return input == null ? dayOfYear : this.add((input - dayOfYear), 'd');
    }

    // Pick the first defined of two or three arguments.
    function defaults(a, b, c) {
        if (a != null) {
            return a;
        }
        if (b != null) {
            return b;
        }
        return c;
    }

    function currentDateArray(config) {
        var now = new Date();
        if (config._useUTC) {
            return [now.getUTCFullYear(), now.getUTCMonth(), now.getUTCDate()];
        }
        return [now.getFullYear(), now.getMonth(), now.getDate()];
    }

    // convert an array to a date.
    // the array should mirror the parameters below
    // note: all values past the year are optional and will default to the lowest possible value.
    // [year, month, day , hour, minute, second, millisecond]
    function configFromArray (config) {
        var i, date, input = [], currentDate, yearToUse;

        if (config._d) {
            return;
        }

        currentDate = currentDateArray(config);

        //compute day of the year from weeks and weekdays
        if (config._w && config._a[DATE] == null && config._a[MONTH] == null) {
            dayOfYearFromWeekInfo(config);
        }

        //if the day of the year is set, figure out what it is
        if (config._dayOfYear) {
            yearToUse = defaults(config._a[YEAR], currentDate[YEAR]);

            if (config._dayOfYear > daysInYear(yearToUse)) {
                getParsingFlags(config)._overflowDayOfYear = true;
            }

            date = createUTCDate(yearToUse, 0, config._dayOfYear);
            config._a[MONTH] = date.getUTCMonth();
            config._a[DATE] = date.getUTCDate();
        }

        // Default to current date.
        // * if no year, month, day of month are given, default to today
        // * if day of month is given, default month and year
        // * if month is given, default only year
        // * if year is given, don't default anything
        for (i = 0; i < 3 && config._a[i] == null; ++i) {
            config._a[i] = input[i] = currentDate[i];
        }

        // Zero out whatever was not defaulted, including time
        for (; i < 7; i++) {
            config._a[i] = input[i] = (config._a[i] == null) ? (i === 2 ? 1 : 0) : config._a[i];
        }

        // Check for 24:00:00.000
        if (config._a[HOUR] === 24 &&
                config._a[MINUTE] === 0 &&
                config._a[SECOND] === 0 &&
                config._a[MILLISECOND] === 0) {
            config._nextDay = true;
            config._a[HOUR] = 0;
        }

        config._d = (config._useUTC ? createUTCDate : createDate).apply(null, input);
        // Apply timezone offset from input. The actual utcOffset can be changed
        // with parseZone.
        if (config._tzm != null) {
            config._d.setUTCMinutes(config._d.getUTCMinutes() - config._tzm);
        }

        if (config._nextDay) {
            config._a[HOUR] = 24;
        }
    }

    function dayOfYearFromWeekInfo(config) {
        var w, weekYear, week, weekday, dow, doy, temp;

        w = config._w;
        if (w.GG != null || w.W != null || w.E != null) {
            dow = 1;
            doy = 4;

            // TODO: We need to take the current isoWeekYear, but that depends on
            // how we interpret now (local, utc, fixed offset). So create
            // a now version of current config (take local/utc/offset flags, and
            // create now).
            weekYear = defaults(w.GG, config._a[YEAR], weekOfYear(local__createLocal(), 1, 4).year);
            week = defaults(w.W, 1);
            weekday = defaults(w.E, 1);
        } else {
            dow = config._locale._week.dow;
            doy = config._locale._week.doy;

            weekYear = defaults(w.gg, config._a[YEAR], weekOfYear(local__createLocal(), dow, doy).year);
            week = defaults(w.w, 1);

            if (w.d != null) {
                // weekday -- low day numbers are considered next week
                weekday = w.d;
                if (weekday < dow) {
                    ++week;
                }
            } else if (w.e != null) {
                // local weekday -- counting starts from begining of week
                weekday = w.e + dow;
            } else {
                // default to begining of week
                weekday = dow;
            }
        }
        temp = dayOfYearFromWeeks(weekYear, week, weekday, doy, dow);

        config._a[YEAR] = temp.year;
        config._dayOfYear = temp.dayOfYear;
    }

    utils_hooks__hooks.ISO_8601 = function () {};

    // date from string and format string
    function configFromStringAndFormat(config) {
        // TODO: Move this to another part of the creation flow to prevent circular deps
        if (config._f === utils_hooks__hooks.ISO_8601) {
            configFromISO(config);
            return;
        }

        config._a = [];
        getParsingFlags(config).empty = true;

        // This array is used to make a Date, either with `new Date` or `Date.UTC`
        var string = '' + config._i,
            i, parsedInput, tokens, token, skipped,
            stringLength = string.length,
            totalParsedInputLength = 0;

        tokens = expandFormat(config._f, config._locale).match(formattingTokens) || [];

        for (i = 0; i < tokens.length; i++) {
            token = tokens[i];
            parsedInput = (string.match(getParseRegexForToken(token, config)) || [])[0];
            if (parsedInput) {
                skipped = string.substr(0, string.indexOf(parsedInput));
                if (skipped.length > 0) {
                    getParsingFlags(config).unusedInput.push(skipped);
                }
                string = string.slice(string.indexOf(parsedInput) + parsedInput.length);
                totalParsedInputLength += parsedInput.length;
            }
            // don't parse if it's not a known token
            if (formatTokenFunctions[token]) {
                if (parsedInput) {
                    getParsingFlags(config).empty = false;
                }
                else {
                    getParsingFlags(config).unusedTokens.push(token);
                }
                addTimeToArrayFromToken(token, parsedInput, config);
            }
            else if (config._strict && !parsedInput) {
                getParsingFlags(config).unusedTokens.push(token);
            }
        }

        // add remaining unparsed input length to the string
        getParsingFlags(config).charsLeftOver = stringLength - totalParsedInputLength;
        if (string.length > 0) {
            getParsingFlags(config).unusedInput.push(string);
        }

        // clear _12h flag if hour is <= 12
        if (getParsingFlags(config).bigHour === true &&
                config._a[HOUR] <= 12 &&
                config._a[HOUR] > 0) {
            getParsingFlags(config).bigHour = undefined;
        }
        // handle meridiem
        config._a[HOUR] = meridiemFixWrap(config._locale, config._a[HOUR], config._meridiem);

        configFromArray(config);
        checkOverflow(config);
    }


    function meridiemFixWrap (locale, hour, meridiem) {
        var isPm;

        if (meridiem == null) {
            // nothing to do
            return hour;
        }
        if (locale.meridiemHour != null) {
            return locale.meridiemHour(hour, meridiem);
        } else if (locale.isPM != null) {
            // Fallback
            isPm = locale.isPM(meridiem);
            if (isPm && hour < 12) {
                hour += 12;
            }
            if (!isPm && hour === 12) {
                hour = 0;
            }
            return hour;
        } else {
            // this is not supposed to happen
            return hour;
        }
    }

    function configFromStringAndArray(config) {
        var tempConfig,
            bestMoment,

            scoreToBeat,
            i,
            currentScore;

        if (config._f.length === 0) {
            getParsingFlags(config).invalidFormat = true;
            config._d = new Date(NaN);
            return;
        }

        for (i = 0; i < config._f.length; i++) {
            currentScore = 0;
            tempConfig = copyConfig({}, config);
            if (config._useUTC != null) {
                tempConfig._useUTC = config._useUTC;
            }
            tempConfig._f = config._f[i];
            configFromStringAndFormat(tempConfig);

            if (!valid__isValid(tempConfig)) {
                continue;
            }

            // if there is any input that was not parsed add a penalty for that format
            currentScore += getParsingFlags(tempConfig).charsLeftOver;

            //or tokens
            currentScore += getParsingFlags(tempConfig).unusedTokens.length * 10;

            getParsingFlags(tempConfig).score = currentScore;

            if (scoreToBeat == null || currentScore < scoreToBeat) {
                scoreToBeat = currentScore;
                bestMoment = tempConfig;
            }
        }

        extend(config, bestMoment || tempConfig);
    }

    function configFromObject(config) {
        if (config._d) {
            return;
        }

        var i = normalizeObjectUnits(config._i);
        config._a = [i.year, i.month, i.day || i.date, i.hour, i.minute, i.second, i.millisecond];

        configFromArray(config);
    }

    function createFromConfig (config) {
        var res = new Moment(checkOverflow(prepareConfig(config)));
        if (res._nextDay) {
            // Adding is smart enough around DST
            res.add(1, 'd');
            res._nextDay = undefined;
        }

        return res;
    }

    function prepareConfig (config) {
        var input = config._i,
            format = config._f;

        config._locale = config._locale || locale_locales__getLocale(config._l);

        if (input === null || (format === undefined && input === '')) {
            return valid__createInvalid({nullInput: true});
        }

        if (typeof input === 'string') {
            config._i = input = config._locale.preparse(input);
        }

        if (isMoment(input)) {
            return new Moment(checkOverflow(input));
        } else if (isArray(format)) {
            configFromStringAndArray(config);
        } else if (format) {
            configFromStringAndFormat(config);
        } else if (isDate(input)) {
            config._d = input;
        } else {
            configFromInput(config);
        }

        return config;
    }

    function configFromInput(config) {
        var input = config._i;
        if (input === undefined) {
            config._d = new Date();
        } else if (isDate(input)) {
            config._d = new Date(+input);
        } else if (typeof input === 'string') {
            configFromString(config);
        } else if (isArray(input)) {
            config._a = map(input.slice(0), function (obj) {
                return parseInt(obj, 10);
            });
            configFromArray(config);
        } else if (typeof(input) === 'object') {
            configFromObject(config);
        } else if (typeof(input) === 'number') {
            // from milliseconds
            config._d = new Date(input);
        } else {
            utils_hooks__hooks.createFromInputFallback(config);
        }
    }

    function createLocalOrUTC (input, format, locale, strict, isUTC) {
        var c = {};

        if (typeof(locale) === 'boolean') {
            strict = locale;
            locale = undefined;
        }
        // object construction must be done this way.
        // https://github.com/moment/moment/issues/1423
        c._isAMomentObject = true;
        c._useUTC = c._isUTC = isUTC;
        c._l = locale;
        c._i = input;
        c._f = format;
        c._strict = strict;

        return createFromConfig(c);
    }

    function local__createLocal (input, format, locale, strict) {
        return createLocalOrUTC(input, format, locale, strict, false);
    }

    var prototypeMin = deprecate(
         'moment().min is deprecated, use moment.min instead. https://github.com/moment/moment/issues/1548',
         function () {
             var other = local__createLocal.apply(null, arguments);
             return other < this ? this : other;
         }
     );

    var prototypeMax = deprecate(
        'moment().max is deprecated, use moment.max instead. https://github.com/moment/moment/issues/1548',
        function () {
            var other = local__createLocal.apply(null, arguments);
            return other > this ? this : other;
        }
    );

    // Pick a moment m from moments so that m[fn](other) is true for all
    // other. This relies on the function fn to be transitive.
    //
    // moments should either be an array of moment objects or an array, whose
    // first element is an array of moment objects.
    function pickBy(fn, moments) {
        var res, i;
        if (moments.length === 1 && isArray(moments[0])) {
            moments = moments[0];
        }
        if (!moments.length) {
            return local__createLocal();
        }
        res = moments[0];
        for (i = 1; i < moments.length; ++i) {
            if (!moments[i].isValid() || moments[i][fn](res)) {
                res = moments[i];
            }
        }
        return res;
    }

    // TODO: Use [].sort instead?
    function min () {
        var args = [].slice.call(arguments, 0);

        return pickBy('isBefore', args);
    }

    function max () {
        var args = [].slice.call(arguments, 0);

        return pickBy('isAfter', args);
    }

    function Duration (duration) {
        var normalizedInput = normalizeObjectUnits(duration),
            years = normalizedInput.year || 0,
            quarters = normalizedInput.quarter || 0,
            months = normalizedInput.month || 0,
            weeks = normalizedInput.week || 0,
            days = normalizedInput.day || 0,
            hours = normalizedInput.hour || 0,
            minutes = normalizedInput.minute || 0,
            seconds = normalizedInput.second || 0,
            milliseconds = normalizedInput.millisecond || 0;

        // representation for dateAddRemove
        this._milliseconds = +milliseconds +
            seconds * 1e3 + // 1000
            minutes * 6e4 + // 1000 * 60
            hours * 36e5; // 1000 * 60 * 60
        // Because of dateAddRemove treats 24 hours as different from a
        // day when working around DST, we need to store them separately
        this._days = +days +
            weeks * 7;
        // It is impossible translate months into days without knowing
        // which months you are are talking about, so we have to store
        // it separately.
        this._months = +months +
            quarters * 3 +
            years * 12;

        this._data = {};

        this._locale = locale_locales__getLocale();

        this._bubble();
    }

    function isDuration (obj) {
        return obj instanceof Duration;
    }

    function offset (token, separator) {
        addFormatToken(token, 0, 0, function () {
            var offset = this.utcOffset();
            var sign = '+';
            if (offset < 0) {
                offset = -offset;
                sign = '-';
            }
            return sign + zeroFill(~~(offset / 60), 2) + separator + zeroFill(~~(offset) % 60, 2);
        });
    }

    offset('Z', ':');
    offset('ZZ', '');

    // PARSING

    addRegexToken('Z',  matchOffset);
    addRegexToken('ZZ', matchOffset);
    addParseToken(['Z', 'ZZ'], function (input, array, config) {
        config._useUTC = true;
        config._tzm = offsetFromString(input);
    });

    // HELPERS

    // timezone chunker
    // '+10:00' > ['10',  '00']
    // '-1530'  > ['-15', '30']
    var chunkOffset = /([\+\-]|\d\d)/gi;

    function offsetFromString(string) {
        var matches = ((string || '').match(matchOffset) || []);
        var chunk   = matches[matches.length - 1] || [];
        var parts   = (chunk + '').match(chunkOffset) || ['-', 0, 0];
        var minutes = +(parts[1] * 60) + toInt(parts[2]);

        return parts[0] === '+' ? minutes : -minutes;
    }

    // Return a moment from input, that is local/utc/zone equivalent to model.
    function cloneWithOffset(input, model) {
        var res, diff;
        if (model._isUTC) {
            res = model.clone();
            diff = (isMoment(input) || isDate(input) ? +input : +local__createLocal(input)) - (+res);
            // Use low-level api, because this fn is low-level api.
            res._d.setTime(+res._d + diff);
            utils_hooks__hooks.updateOffset(res, false);
            return res;
        } else {
            return local__createLocal(input).local();
        }
    }

    function getDateOffset (m) {
        // On Firefox.24 Date#getTimezoneOffset returns a floating point.
        // https://github.com/moment/moment/pull/1871
        return -Math.round(m._d.getTimezoneOffset() / 15) * 15;
    }

    // HOOKS

    // This function will be called whenever a moment is mutated.
    // It is intended to keep the offset in sync with the timezone.
    utils_hooks__hooks.updateOffset = function () {};

    // MOMENTS

    // keepLocalTime = true means only change the timezone, without
    // affecting the local hour. So 5:31:26 +0300 --[utcOffset(2, true)]-->
    // 5:31:26 +0200 It is possible that 5:31:26 doesn't exist with offset
    // +0200, so we adjust the time as needed, to be valid.
    //
    // Keeping the time actually adds/subtracts (one hour)
    // from the actual represented time. That is why we call updateOffset
    // a second time. In case it wants us to change the offset again
    // _changeInProgress == true case, then we have to adjust, because
    // there is no such time in the given timezone.
    function getSetOffset (input, keepLocalTime) {
        var offset = this._offset || 0,
            localAdjust;
        if (input != null) {
            if (typeof input === 'string') {
                input = offsetFromString(input);
            }
            if (Math.abs(input) < 16) {
                input = input * 60;
            }
            if (!this._isUTC && keepLocalTime) {
                localAdjust = getDateOffset(this);
            }
            this._offset = input;
            this._isUTC = true;
            if (localAdjust != null) {
                this.add(localAdjust, 'm');
            }
            if (offset !== input) {
                if (!keepLocalTime || this._changeInProgress) {
                    add_subtract__addSubtract(this, create__createDuration(input - offset, 'm'), 1, false);
                } else if (!this._changeInProgress) {
                    this._changeInProgress = true;
                    utils_hooks__hooks.updateOffset(this, true);
                    this._changeInProgress = null;
                }
            }
            return this;
        } else {
            return this._isUTC ? offset : getDateOffset(this);
        }
    }

    function getSetZone (input, keepLocalTime) {
        if (input != null) {
            if (typeof input !== 'string') {
                input = -input;
            }

            this.utcOffset(input, keepLocalTime);

            return this;
        } else {
            return -this.utcOffset();
        }
    }

    function setOffsetToUTC (keepLocalTime) {
        return this.utcOffset(0, keepLocalTime);
    }

    function setOffsetToLocal (keepLocalTime) {
        if (this._isUTC) {
            this.utcOffset(0, keepLocalTime);
            this._isUTC = false;

            if (keepLocalTime) {
                this.subtract(getDateOffset(this), 'm');
            }
        }
        return this;
    }

    function setOffsetToParsedOffset () {
        if (this._tzm) {
            this.utcOffset(this._tzm);
        } else if (typeof this._i === 'string') {
            this.utcOffset(offsetFromString(this._i));
        }
        return this;
    }

    function hasAlignedHourOffset (input) {
        input = input ? local__createLocal(input).utcOffset() : 0;

        return (this.utcOffset() - input) % 60 === 0;
    }

    function isDaylightSavingTime () {
        return (
            this.utcOffset() > this.clone().month(0).utcOffset() ||
            this.utcOffset() > this.clone().month(5).utcOffset()
        );
    }

    function isDaylightSavingTimeShifted () {
        if (typeof this._isDSTShifted !== 'undefined') {
            return this._isDSTShifted;
        }

        var c = {};

        copyConfig(c, this);
        c = prepareConfig(c);

        if (c._a) {
            var other = c._isUTC ? create_utc__createUTC(c._a) : local__createLocal(c._a);
            this._isDSTShifted = this.isValid() &&
                compareArrays(c._a, other.toArray()) > 0;
        } else {
            this._isDSTShifted = false;
        }

        return this._isDSTShifted;
    }

    function isLocal () {
        return !this._isUTC;
    }

    function isUtcOffset () {
        return this._isUTC;
    }

    function isUtc () {
        return this._isUTC && this._offset === 0;
    }

    var aspNetRegex = /(\-)?(?:(\d*)\.)?(\d+)\:(\d+)(?:\:(\d+)\.?(\d{3})?)?/;

    // from http://docs.closure-library.googlecode.com/git/closure_goog_date_date.js.source.html
    // somewhat more in line with 4.4.3.2 2004 spec, but allows decimal anywhere
    var create__isoRegex = /^(-)?P(?:(?:([0-9,.]*)Y)?(?:([0-9,.]*)M)?(?:([0-9,.]*)D)?(?:T(?:([0-9,.]*)H)?(?:([0-9,.]*)M)?(?:([0-9,.]*)S)?)?|([0-9,.]*)W)$/;

    function create__createDuration (input, key) {
        var duration = input,
            // matching against regexp is expensive, do it on demand
            match = null,
            sign,
            ret,
            diffRes;

        if (isDuration(input)) {
            duration = {
                ms : input._milliseconds,
                d  : input._days,
                M  : input._months
            };
        } else if (typeof input === 'number') {
            duration = {};
            if (key) {
                duration[key] = input;
            } else {
                duration.milliseconds = input;
            }
        } else if (!!(match = aspNetRegex.exec(input))) {
            sign = (match[1] === '-') ? -1 : 1;
            duration = {
                y  : 0,
                d  : toInt(match[DATE])        * sign,
                h  : toInt(match[HOUR])        * sign,
                m  : toInt(match[MINUTE])      * sign,
                s  : toInt(match[SECOND])      * sign,
                ms : toInt(match[MILLISECOND]) * sign
            };
        } else if (!!(match = create__isoRegex.exec(input))) {
            sign = (match[1] === '-') ? -1 : 1;
            duration = {
                y : parseIso(match[2], sign),
                M : parseIso(match[3], sign),
                d : parseIso(match[4], sign),
                h : parseIso(match[5], sign),
                m : parseIso(match[6], sign),
                s : parseIso(match[7], sign),
                w : parseIso(match[8], sign)
            };
        } else if (duration == null) {// checks for null or undefined
            duration = {};
        } else if (typeof duration === 'object' && ('from' in duration || 'to' in duration)) {
            diffRes = momentsDifference(local__createLocal(duration.from), local__createLocal(duration.to));

            duration = {};
            duration.ms = diffRes.milliseconds;
            duration.M = diffRes.months;
        }

        ret = new Duration(duration);

        if (isDuration(input) && hasOwnProp(input, '_locale')) {
            ret._locale = input._locale;
        }

        return ret;
    }

    create__createDuration.fn = Duration.prototype;

    function parseIso (inp, sign) {
        // We'd normally use ~~inp for this, but unfortunately it also
        // converts floats to ints.
        // inp may be undefined, so careful calling replace on it.
        var res = inp && parseFloat(inp.replace(',', '.'));
        // apply sign while we're at it
        return (isNaN(res) ? 0 : res) * sign;
    }

    function positiveMomentsDifference(base, other) {
        var res = {milliseconds: 0, months: 0};

        res.months = other.month() - base.month() +
            (other.year() - base.year()) * 12;
        if (base.clone().add(res.months, 'M').isAfter(other)) {
            --res.months;
        }

        res.milliseconds = +other - +(base.clone().add(res.months, 'M'));

        return res;
    }

    function momentsDifference(base, other) {
        var res;
        other = cloneWithOffset(other, base);
        if (base.isBefore(other)) {
            res = positiveMomentsDifference(base, other);
        } else {
            res = positiveMomentsDifference(other, base);
            res.milliseconds = -res.milliseconds;
            res.months = -res.months;
        }

        return res;
    }

    function createAdder(direction, name) {
        return function (val, period) {
            var dur, tmp;
            //invert the arguments, but complain about it
            if (period !== null && !isNaN(+period)) {
                deprecateSimple(name, 'moment().' + name  + '(period, number) is deprecated. Please use moment().' + name + '(number, period).');
                tmp = val; val = period; period = tmp;
            }

            val = typeof val === 'string' ? +val : val;
            dur = create__createDuration(val, period);
            add_subtract__addSubtract(this, dur, direction);
            return this;
        };
    }

    function add_subtract__addSubtract (mom, duration, isAdding, updateOffset) {
        var milliseconds = duration._milliseconds,
            days = duration._days,
            months = duration._months;
        updateOffset = updateOffset == null ? true : updateOffset;

        if (milliseconds) {
            mom._d.setTime(+mom._d + milliseconds * isAdding);
        }
        if (days) {
            get_set__set(mom, 'Date', get_set__get(mom, 'Date') + days * isAdding);
        }
        if (months) {
            setMonth(mom, get_set__get(mom, 'Month') + months * isAdding);
        }
        if (updateOffset) {
            utils_hooks__hooks.updateOffset(mom, days || months);
        }
    }

    var add_subtract__add      = createAdder(1, 'add');
    var add_subtract__subtract = createAdder(-1, 'subtract');

    function moment_calendar__calendar (time, formats) {
        // We want to compare the start of today, vs this.
        // Getting start-of-today depends on whether we're local/utc/offset or not.
        var now = time || local__createLocal(),
            sod = cloneWithOffset(now, this).startOf('day'),
            diff = this.diff(sod, 'days', true),
            format = diff < -6 ? 'sameElse' :
                diff < -1 ? 'lastWeek' :
                diff < 0 ? 'lastDay' :
                diff < 1 ? 'sameDay' :
                diff < 2 ? 'nextDay' :
                diff < 7 ? 'nextWeek' : 'sameElse';
        return this.format(formats && formats[format] || this.localeData().calendar(format, this, local__createLocal(now)));
    }

    function clone () {
        return new Moment(this);
    }

    function isAfter (input, units) {
        var inputMs;
        units = normalizeUnits(typeof units !== 'undefined' ? units : 'millisecond');
        if (units === 'millisecond') {
            input = isMoment(input) ? input : local__createLocal(input);
            return +this > +input;
        } else {
            inputMs = isMoment(input) ? +input : +local__createLocal(input);
            return inputMs < +this.clone().startOf(units);
        }
    }

    function isBefore (input, units) {
        var inputMs;
        units = normalizeUnits(typeof units !== 'undefined' ? units : 'millisecond');
        if (units === 'millisecond') {
            input = isMoment(input) ? input : local__createLocal(input);
            return +this < +input;
        } else {
            inputMs = isMoment(input) ? +input : +local__createLocal(input);
            return +this.clone().endOf(units) < inputMs;
        }
    }

    function isBetween (from, to, units) {
        return this.isAfter(from, units) && this.isBefore(to, units);
    }

    function isSame (input, units) {
        var inputMs;
        units = normalizeUnits(units || 'millisecond');
        if (units === 'millisecond') {
            input = isMoment(input) ? input : local__createLocal(input);
            return +this === +input;
        } else {
            inputMs = +local__createLocal(input);
            return +(this.clone().startOf(units)) <= inputMs && inputMs <= +(this.clone().endOf(units));
        }
    }

    function diff (input, units, asFloat) {
        var that = cloneWithOffset(input, this),
            zoneDelta = (that.utcOffset() - this.utcOffset()) * 6e4,
            delta, output;

        units = normalizeUnits(units);

        if (units === 'year' || units === 'month' || units === 'quarter') {
            output = monthDiff(this, that);
            if (units === 'quarter') {
                output = output / 3;
            } else if (units === 'year') {
                output = output / 12;
            }
        } else {
            delta = this - that;
            output = units === 'second' ? delta / 1e3 : // 1000
                units === 'minute' ? delta / 6e4 : // 1000 * 60
                units === 'hour' ? delta / 36e5 : // 1000 * 60 * 60
                units === 'day' ? (delta - zoneDelta) / 864e5 : // 1000 * 60 * 60 * 24, negate dst
                units === 'week' ? (delta - zoneDelta) / 6048e5 : // 1000 * 60 * 60 * 24 * 7, negate dst
                delta;
        }
        return asFloat ? output : absFloor(output);
    }

    function monthDiff (a, b) {
        // difference in months
        var wholeMonthDiff = ((b.year() - a.year()) * 12) + (b.month() - a.month()),
            // b is in (anchor - 1 month, anchor + 1 month)
            anchor = a.clone().add(wholeMonthDiff, 'months'),
            anchor2, adjust;

        if (b - anchor < 0) {
            anchor2 = a.clone().add(wholeMonthDiff - 1, 'months');
            // linear across the month
            adjust = (b - anchor) / (anchor - anchor2);
        } else {
            anchor2 = a.clone().add(wholeMonthDiff + 1, 'months');
            // linear across the month
            adjust = (b - anchor) / (anchor2 - anchor);
        }

        return -(wholeMonthDiff + adjust);
    }

    utils_hooks__hooks.defaultFormat = 'YYYY-MM-DDTHH:mm:ssZ';

    function toString () {
        return this.clone().locale('en').format('ddd MMM DD YYYY HH:mm:ss [GMT]ZZ');
    }

    function moment_format__toISOString () {
        var m = this.clone().utc();
        if (0 < m.year() && m.year() <= 9999) {
            if ('function' === typeof Date.prototype.toISOString) {
                // native implementation is ~50x faster, use it when we can
                return this.toDate().toISOString();
            } else {
                return formatMoment(m, 'YYYY-MM-DD[T]HH:mm:ss.SSS[Z]');
            }
        } else {
            return formatMoment(m, 'YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]');
        }
    }

    function format (inputString) {
        var output = formatMoment(this, inputString || utils_hooks__hooks.defaultFormat);
        return this.localeData().postformat(output);
    }

    function from (time, withoutSuffix) {
        if (!this.isValid()) {
            return this.localeData().invalidDate();
        }
        return create__createDuration({to: this, from: time}).locale(this.locale()).humanize(!withoutSuffix);
    }

    function fromNow (withoutSuffix) {
        return this.from(local__createLocal(), withoutSuffix);
    }

    function to (time, withoutSuffix) {
        if (!this.isValid()) {
            return this.localeData().invalidDate();
        }
        return create__createDuration({from: this, to: time}).locale(this.locale()).humanize(!withoutSuffix);
    }

    function toNow (withoutSuffix) {
        return this.to(local__createLocal(), withoutSuffix);
    }

    function locale (key) {
        var newLocaleData;

        if (key === undefined) {
            return this._locale._abbr;
        } else {
            newLocaleData = locale_locales__getLocale(key);
            if (newLocaleData != null) {
                this._locale = newLocaleData;
            }
            return this;
        }
    }

    var lang = deprecate(
        'moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.',
        function (key) {
            if (key === undefined) {
                return this.localeData();
            } else {
                return this.locale(key);
            }
        }
    );

    function localeData () {
        return this._locale;
    }

    function startOf (units) {
        units = normalizeUnits(units);
        // the following switch intentionally omits break keywords
        // to utilize falling through the cases.
        switch (units) {
        case 'year':
            this.month(0);
            /* falls through */
        case 'quarter':
        case 'month':
            this.date(1);
            /* falls through */
        case 'week':
        case 'isoWeek':
        case 'day':
            this.hours(0);
            /* falls through */
        case 'hour':
            this.minutes(0);
            /* falls through */
        case 'minute':
            this.seconds(0);
            /* falls through */
        case 'second':
            this.milliseconds(0);
        }

        // weeks are a special case
        if (units === 'week') {
            this.weekday(0);
        }
        if (units === 'isoWeek') {
            this.isoWeekday(1);
        }

        // quarters are also special
        if (units === 'quarter') {
            this.month(Math.floor(this.month() / 3) * 3);
        }

        return this;
    }

    function endOf (units) {
        units = normalizeUnits(units);
        if (units === undefined || units === 'millisecond') {
            return this;
        }
        return this.startOf(units).add(1, (units === 'isoWeek' ? 'week' : units)).subtract(1, 'ms');
    }

    function to_type__valueOf () {
        return +this._d - ((this._offset || 0) * 60000);
    }

    function unix () {
        return Math.floor(+this / 1000);
    }

    function toDate () {
        return this._offset ? new Date(+this) : this._d;
    }

    function toArray () {
        var m = this;
        return [m.year(), m.month(), m.date(), m.hour(), m.minute(), m.second(), m.millisecond()];
    }

    function toObject () {
        var m = this;
        return {
            years: m.year(),
            months: m.month(),
            date: m.date(),
            hours: m.hours(),
            minutes: m.minutes(),
            seconds: m.seconds(),
            milliseconds: m.milliseconds()
        };
    }

    function moment_valid__isValid () {
        return valid__isValid(this);
    }

    function parsingFlags () {
        return extend({}, getParsingFlags(this));
    }

    function invalidAt () {
        return getParsingFlags(this).overflow;
    }

    addFormatToken(0, ['gg', 2], 0, function () {
        return this.weekYear() % 100;
    });

    addFormatToken(0, ['GG', 2], 0, function () {
        return this.isoWeekYear() % 100;
    });

    function addWeekYearFormatToken (token, getter) {
        addFormatToken(0, [token, token.length], 0, getter);
    }

    addWeekYearFormatToken('gggg',     'weekYear');
    addWeekYearFormatToken('ggggg',    'weekYear');
    addWeekYearFormatToken('GGGG',  'isoWeekYear');
    addWeekYearFormatToken('GGGGG', 'isoWeekYear');

    // ALIASES

    addUnitAlias('weekYear', 'gg');
    addUnitAlias('isoWeekYear', 'GG');

    // PARSING

    addRegexToken('G',      matchSigned);
    addRegexToken('g',      matchSigned);
    addRegexToken('GG',     match1to2, match2);
    addRegexToken('gg',     match1to2, match2);
    addRegexToken('GGGG',   match1to4, match4);
    addRegexToken('gggg',   match1to4, match4);
    addRegexToken('GGGGG',  match1to6, match6);
    addRegexToken('ggggg',  match1to6, match6);

    addWeekParseToken(['gggg', 'ggggg', 'GGGG', 'GGGGG'], function (input, week, config, token) {
        week[token.substr(0, 2)] = toInt(input);
    });

    addWeekParseToken(['gg', 'GG'], function (input, week, config, token) {
        week[token] = utils_hooks__hooks.parseTwoDigitYear(input);
    });

    // HELPERS

    function weeksInYear(year, dow, doy) {
        return weekOfYear(local__createLocal([year, 11, 31 + dow - doy]), dow, doy).week;
    }

    // MOMENTS

    function getSetWeekYear (input) {
        var year = weekOfYear(this, this.localeData()._week.dow, this.localeData()._week.doy).year;
        return input == null ? year : this.add((input - year), 'y');
    }

    function getSetISOWeekYear (input) {
        var year = weekOfYear(this, 1, 4).year;
        return input == null ? year : this.add((input - year), 'y');
    }

    function getISOWeeksInYear () {
        return weeksInYear(this.year(), 1, 4);
    }

    function getWeeksInYear () {
        var weekInfo = this.localeData()._week;
        return weeksInYear(this.year(), weekInfo.dow, weekInfo.doy);
    }

    addFormatToken('Q', 0, 0, 'quarter');

    // ALIASES

    addUnitAlias('quarter', 'Q');

    // PARSING

    addRegexToken('Q', match1);
    addParseToken('Q', function (input, array) {
        array[MONTH] = (toInt(input) - 1) * 3;
    });

    // MOMENTS

    function getSetQuarter (input) {
        return input == null ? Math.ceil((this.month() + 1) / 3) : this.month((input - 1) * 3 + this.month() % 3);
    }

    addFormatToken('D', ['DD', 2], 'Do', 'date');

    // ALIASES

    addUnitAlias('date', 'D');

    // PARSING

    addRegexToken('D',  match1to2);
    addRegexToken('DD', match1to2, match2);
    addRegexToken('Do', function (isStrict, locale) {
        return isStrict ? locale._ordinalParse : locale._ordinalParseLenient;
    });

    addParseToken(['D', 'DD'], DATE);
    addParseToken('Do', function (input, array) {
        array[DATE] = toInt(input.match(match1to2)[0], 10);
    });

    // MOMENTS

    var getSetDayOfMonth = makeGetSet('Date', true);

    addFormatToken('d', 0, 'do', 'day');

    addFormatToken('dd', 0, 0, function (format) {
        return this.localeData().weekdaysMin(this, format);
    });

    addFormatToken('ddd', 0, 0, function (format) {
        return this.localeData().weekdaysShort(this, format);
    });

    addFormatToken('dddd', 0, 0, function (format) {
        return this.localeData().weekdays(this, format);
    });

    addFormatToken('e', 0, 0, 'weekday');
    addFormatToken('E', 0, 0, 'isoWeekday');

    // ALIASES

    addUnitAlias('day', 'd');
    addUnitAlias('weekday', 'e');
    addUnitAlias('isoWeekday', 'E');

    // PARSING

    addRegexToken('d',    match1to2);
    addRegexToken('e',    match1to2);
    addRegexToken('E',    match1to2);
    addRegexToken('dd',   matchWord);
    addRegexToken('ddd',  matchWord);
    addRegexToken('dddd', matchWord);

    addWeekParseToken(['dd', 'ddd', 'dddd'], function (input, week, config) {
        var weekday = config._locale.weekdaysParse(input);
        // if we didn't get a weekday name, mark the date as invalid
        if (weekday != null) {
            week.d = weekday;
        } else {
            getParsingFlags(config).invalidWeekday = input;
        }
    });

    addWeekParseToken(['d', 'e', 'E'], function (input, week, config, token) {
        week[token] = toInt(input);
    });

    // HELPERS

    function parseWeekday(input, locale) {
        if (typeof input !== 'string') {
            return input;
        }

        if (!isNaN(input)) {
            return parseInt(input, 10);
        }

        input = locale.weekdaysParse(input);
        if (typeof input === 'number') {
            return input;
        }

        return null;
    }

    // LOCALES

    var defaultLocaleWeekdays = 'Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday'.split('_');
    function localeWeekdays (m) {
        return this._weekdays[m.day()];
    }

    var defaultLocaleWeekdaysShort = 'Sun_Mon_Tue_Wed_Thu_Fri_Sat'.split('_');
    function localeWeekdaysShort (m) {
        return this._weekdaysShort[m.day()];
    }

    var defaultLocaleWeekdaysMin = 'Su_Mo_Tu_We_Th_Fr_Sa'.split('_');
    function localeWeekdaysMin (m) {
        return this._weekdaysMin[m.day()];
    }

    function localeWeekdaysParse (weekdayName) {
        var i, mom, regex;

        this._weekdaysParse = this._weekdaysParse || [];

        for (i = 0; i < 7; i++) {
            // make the regex if we don't have it already
            if (!this._weekdaysParse[i]) {
                mom = local__createLocal([2000, 1]).day(i);
                regex = '^' + this.weekdays(mom, '') + '|^' + this.weekdaysShort(mom, '') + '|^' + this.weekdaysMin(mom, '');
                this._weekdaysParse[i] = new RegExp(regex.replace('.', ''), 'i');
            }
            // test the regex
            if (this._weekdaysParse[i].test(weekdayName)) {
                return i;
            }
        }
    }

    // MOMENTS

    function getSetDayOfWeek (input) {
        var day = this._isUTC ? this._d.getUTCDay() : this._d.getDay();
        if (input != null) {
            input = parseWeekday(input, this.localeData());
            return this.add(input - day, 'd');
        } else {
            return day;
        }
    }

    function getSetLocaleDayOfWeek (input) {
        var weekday = (this.day() + 7 - this.localeData()._week.dow) % 7;
        return input == null ? weekday : this.add(input - weekday, 'd');
    }

    function getSetISODayOfWeek (input) {
        // behaves the same as moment#day except
        // as a getter, returns 7 instead of 0 (1-7 range instead of 0-6)
        // as a setter, sunday should belong to the previous week.
        return input == null ? this.day() || 7 : this.day(this.day() % 7 ? input : input - 7);
    }

    addFormatToken('H', ['HH', 2], 0, 'hour');
    addFormatToken('h', ['hh', 2], 0, function () {
        return this.hours() % 12 || 12;
    });

    function meridiem (token, lowercase) {
        addFormatToken(token, 0, 0, function () {
            return this.localeData().meridiem(this.hours(), this.minutes(), lowercase);
        });
    }

    meridiem('a', true);
    meridiem('A', false);

    // ALIASES

    addUnitAlias('hour', 'h');

    // PARSING

    function matchMeridiem (isStrict, locale) {
        return locale._meridiemParse;
    }

    addRegexToken('a',  matchMeridiem);
    addRegexToken('A',  matchMeridiem);
    addRegexToken('H',  match1to2);
    addRegexToken('h',  match1to2);
    addRegexToken('HH', match1to2, match2);
    addRegexToken('hh', match1to2, match2);

    addParseToken(['H', 'HH'], HOUR);
    addParseToken(['a', 'A'], function (input, array, config) {
        config._isPm = config._locale.isPM(input);
        config._meridiem = input;
    });
    addParseToken(['h', 'hh'], function (input, array, config) {
        array[HOUR] = toInt(input);
        getParsingFlags(config).bigHour = true;
    });

    // LOCALES

    function localeIsPM (input) {
        // IE8 Quirks Mode & IE7 Standards Mode do not allow accessing strings like arrays
        // Using charAt should be more compatible.
        return ((input + '').toLowerCase().charAt(0) === 'p');
    }

    var defaultLocaleMeridiemParse = /[ap]\.?m?\.?/i;
    function localeMeridiem (hours, minutes, isLower) {
        if (hours > 11) {
            return isLower ? 'pm' : 'PM';
        } else {
            return isLower ? 'am' : 'AM';
        }
    }


    // MOMENTS

    // Setting the hour should keep the time, because the user explicitly
    // specified which hour he wants. So trying to maintain the same hour (in
    // a new timezone) makes sense. Adding/subtracting hours does not follow
    // this rule.
    var getSetHour = makeGetSet('Hours', true);

    addFormatToken('m', ['mm', 2], 0, 'minute');

    // ALIASES

    addUnitAlias('minute', 'm');

    // PARSING

    addRegexToken('m',  match1to2);
    addRegexToken('mm', match1to2, match2);
    addParseToken(['m', 'mm'], MINUTE);

    // MOMENTS

    var getSetMinute = makeGetSet('Minutes', false);

    addFormatToken('s', ['ss', 2], 0, 'second');

    // ALIASES

    addUnitAlias('second', 's');

    // PARSING

    addRegexToken('s',  match1to2);
    addRegexToken('ss', match1to2, match2);
    addParseToken(['s', 'ss'], SECOND);

    // MOMENTS

    var getSetSecond = makeGetSet('Seconds', false);

    addFormatToken('S', 0, 0, function () {
        return ~~(this.millisecond() / 100);
    });

    addFormatToken(0, ['SS', 2], 0, function () {
        return ~~(this.millisecond() / 10);
    });

    addFormatToken(0, ['SSS', 3], 0, 'millisecond');
    addFormatToken(0, ['SSSS', 4], 0, function () {
        return this.millisecond() * 10;
    });
    addFormatToken(0, ['SSSSS', 5], 0, function () {
        return this.millisecond() * 100;
    });
    addFormatToken(0, ['SSSSSS', 6], 0, function () {
        return this.millisecond() * 1000;
    });
    addFormatToken(0, ['SSSSSSS', 7], 0, function () {
        return this.millisecond() * 10000;
    });
    addFormatToken(0, ['SSSSSSSS', 8], 0, function () {
        return this.millisecond() * 100000;
    });
    addFormatToken(0, ['SSSSSSSSS', 9], 0, function () {
        return this.millisecond() * 1000000;
    });


    // ALIASES

    addUnitAlias('millisecond', 'ms');

    // PARSING

    addRegexToken('S',    match1to3, match1);
    addRegexToken('SS',   match1to3, match2);
    addRegexToken('SSS',  match1to3, match3);

    var token;
    for (token = 'SSSS'; token.length <= 9; token += 'S') {
        addRegexToken(token, matchUnsigned);
    }

    function parseMs(input, array) {
        array[MILLISECOND] = toInt(('0.' + input) * 1000);
    }

    for (token = 'S'; token.length <= 9; token += 'S') {
        addParseToken(token, parseMs);
    }
    // MOMENTS

    var getSetMillisecond = makeGetSet('Milliseconds', false);

    addFormatToken('z',  0, 0, 'zoneAbbr');
    addFormatToken('zz', 0, 0, 'zoneName');

    // MOMENTS

    function getZoneAbbr () {
        return this._isUTC ? 'UTC' : '';
    }

    function getZoneName () {
        return this._isUTC ? 'Coordinated Universal Time' : '';
    }

    var momentPrototype__proto = Moment.prototype;

    momentPrototype__proto.add          = add_subtract__add;
    momentPrototype__proto.calendar     = moment_calendar__calendar;
    momentPrototype__proto.clone        = clone;
    momentPrototype__proto.diff         = diff;
    momentPrototype__proto.endOf        = endOf;
    momentPrototype__proto.format       = format;
    momentPrototype__proto.from         = from;
    momentPrototype__proto.fromNow      = fromNow;
    momentPrototype__proto.to           = to;
    momentPrototype__proto.toNow        = toNow;
    momentPrototype__proto.get          = getSet;
    momentPrototype__proto.invalidAt    = invalidAt;
    momentPrototype__proto.isAfter      = isAfter;
    momentPrototype__proto.isBefore     = isBefore;
    momentPrototype__proto.isBetween    = isBetween;
    momentPrototype__proto.isSame       = isSame;
    momentPrototype__proto.isValid      = moment_valid__isValid;
    momentPrototype__proto.lang         = lang;
    momentPrototype__proto.locale       = locale;
    momentPrototype__proto.localeData   = localeData;
    momentPrototype__proto.max          = prototypeMax;
    momentPrototype__proto.min          = prototypeMin;
    momentPrototype__proto.parsingFlags = parsingFlags;
    momentPrototype__proto.set          = getSet;
    momentPrototype__proto.startOf      = startOf;
    momentPrototype__proto.subtract     = add_subtract__subtract;
    momentPrototype__proto.toArray      = toArray;
    momentPrototype__proto.toObject     = toObject;
    momentPrototype__proto.toDate       = toDate;
    momentPrototype__proto.toISOString  = moment_format__toISOString;
    momentPrototype__proto.toJSON       = moment_format__toISOString;
    momentPrototype__proto.toString     = toString;
    momentPrototype__proto.unix         = unix;
    momentPrototype__proto.valueOf      = to_type__valueOf;

    // Year
    momentPrototype__proto.year       = getSetYear;
    momentPrototype__proto.isLeapYear = getIsLeapYear;

    // Week Year
    momentPrototype__proto.weekYear    = getSetWeekYear;
    momentPrototype__proto.isoWeekYear = getSetISOWeekYear;

    // Quarter
    momentPrototype__proto.quarter = momentPrototype__proto.quarters = getSetQuarter;

    // Month
    momentPrototype__proto.month       = getSetMonth;
    momentPrototype__proto.daysInMonth = getDaysInMonth;

    // Week
    momentPrototype__proto.week           = momentPrototype__proto.weeks        = getSetWeek;
    momentPrototype__proto.isoWeek        = momentPrototype__proto.isoWeeks     = getSetISOWeek;
    momentPrototype__proto.weeksInYear    = getWeeksInYear;
    momentPrototype__proto.isoWeeksInYear = getISOWeeksInYear;

    // Day
    momentPrototype__proto.date       = getSetDayOfMonth;
    momentPrototype__proto.day        = momentPrototype__proto.days             = getSetDayOfWeek;
    momentPrototype__proto.weekday    = getSetLocaleDayOfWeek;
    momentPrototype__proto.isoWeekday = getSetISODayOfWeek;
    momentPrototype__proto.dayOfYear  = getSetDayOfYear;

    // Hour
    momentPrototype__proto.hour = momentPrototype__proto.hours = getSetHour;

    // Minute
    momentPrototype__proto.minute = momentPrototype__proto.minutes = getSetMinute;

    // Second
    momentPrototype__proto.second = momentPrototype__proto.seconds = getSetSecond;

    // Millisecond
    momentPrototype__proto.millisecond = momentPrototype__proto.milliseconds = getSetMillisecond;

    // Offset
    momentPrototype__proto.utcOffset            = getSetOffset;
    momentPrototype__proto.utc                  = setOffsetToUTC;
    momentPrototype__proto.local                = setOffsetToLocal;
    momentPrototype__proto.parseZone            = setOffsetToParsedOffset;
    momentPrototype__proto.hasAlignedHourOffset = hasAlignedHourOffset;
    momentPrototype__proto.isDST                = isDaylightSavingTime;
    momentPrototype__proto.isDSTShifted         = isDaylightSavingTimeShifted;
    momentPrototype__proto.isLocal              = isLocal;
    momentPrototype__proto.isUtcOffset          = isUtcOffset;
    momentPrototype__proto.isUtc                = isUtc;
    momentPrototype__proto.isUTC                = isUtc;

    // Timezone
    momentPrototype__proto.zoneAbbr = getZoneAbbr;
    momentPrototype__proto.zoneName = getZoneName;

    // Deprecations
    momentPrototype__proto.dates  = deprecate('dates accessor is deprecated. Use date instead.', getSetDayOfMonth);
    momentPrototype__proto.months = deprecate('months accessor is deprecated. Use month instead', getSetMonth);
    momentPrototype__proto.years  = deprecate('years accessor is deprecated. Use year instead', getSetYear);
    momentPrototype__proto.zone   = deprecate('moment().zone is deprecated, use moment().utcOffset instead. https://github.com/moment/moment/issues/1779', getSetZone);

    var momentPrototype = momentPrototype__proto;

    function moment__createUnix (input) {
        return local__createLocal(input * 1000);
    }

    function moment__createInZone () {
        return local__createLocal.apply(null, arguments).parseZone();
    }

    var defaultCalendar = {
        sameDay : '[Today at] LT',
        nextDay : '[Tomorrow at] LT',
        nextWeek : 'dddd [at] LT',
        lastDay : '[Yesterday at] LT',
        lastWeek : '[Last] dddd [at] LT',
        sameElse : 'L'
    };

    function locale_calendar__calendar (key, mom, now) {
        var output = this._calendar[key];
        return typeof output === 'function' ? output.call(mom, now) : output;
    }

    var defaultLongDateFormat = {
        LTS  : 'h:mm:ss A',
        LT   : 'h:mm A',
        L    : 'MM/DD/YYYY',
        LL   : 'MMMM D, YYYY',
        LLL  : 'MMMM D, YYYY h:mm A',
        LLLL : 'dddd, MMMM D, YYYY h:mm A'
    };

    function longDateFormat (key) {
        var format = this._longDateFormat[key],
            formatUpper = this._longDateFormat[key.toUpperCase()];

        if (format || !formatUpper) {
            return format;
        }

        this._longDateFormat[key] = formatUpper.replace(/MMMM|MM|DD|dddd/g, function (val) {
            return val.slice(1);
        });

        return this._longDateFormat[key];
    }

    var defaultInvalidDate = 'Invalid date';

    function invalidDate () {
        return this._invalidDate;
    }

    var defaultOrdinal = '%d';
    var defaultOrdinalParse = /\d{1,2}/;

    function ordinal (number) {
        return this._ordinal.replace('%d', number);
    }

    function preParsePostFormat (string) {
        return string;
    }

    var defaultRelativeTime = {
        future : 'in %s',
        past   : '%s ago',
        s  : 'a few seconds',
        m  : 'a minute',
        mm : '%d minutes',
        h  : 'an hour',
        hh : '%d hours',
        d  : 'a day',
        dd : '%d days',
        M  : 'a month',
        MM : '%d months',
        y  : 'a year',
        yy : '%d years'
    };

    function relative__relativeTime (number, withoutSuffix, string, isFuture) {
        var output = this._relativeTime[string];
        return (typeof output === 'function') ?
            output(number, withoutSuffix, string, isFuture) :
            output.replace(/%d/i, number);
    }

    function pastFuture (diff, output) {
        var format = this._relativeTime[diff > 0 ? 'future' : 'past'];
        return typeof format === 'function' ? format(output) : format.replace(/%s/i, output);
    }

    function locale_set__set (config) {
        var prop, i;
        for (i in config) {
            prop = config[i];
            if (typeof prop === 'function') {
                this[i] = prop;
            } else {
                this['_' + i] = prop;
            }
        }
        // Lenient ordinal parsing accepts just a number in addition to
        // number + (possibly) stuff coming from _ordinalParseLenient.
        this._ordinalParseLenient = new RegExp(this._ordinalParse.source + '|' + (/\d{1,2}/).source);
    }

    var prototype__proto = Locale.prototype;

    prototype__proto._calendar       = defaultCalendar;
    prototype__proto.calendar        = locale_calendar__calendar;
    prototype__proto._longDateFormat = defaultLongDateFormat;
    prototype__proto.longDateFormat  = longDateFormat;
    prototype__proto._invalidDate    = defaultInvalidDate;
    prototype__proto.invalidDate     = invalidDate;
    prototype__proto._ordinal        = defaultOrdinal;
    prototype__proto.ordinal         = ordinal;
    prototype__proto._ordinalParse   = defaultOrdinalParse;
    prototype__proto.preparse        = preParsePostFormat;
    prototype__proto.postformat      = preParsePostFormat;
    prototype__proto._relativeTime   = defaultRelativeTime;
    prototype__proto.relativeTime    = relative__relativeTime;
    prototype__proto.pastFuture      = pastFuture;
    prototype__proto.set             = locale_set__set;

    // Month
    prototype__proto.months       =        localeMonths;
    prototype__proto._months      = defaultLocaleMonths;
    prototype__proto.monthsShort  =        localeMonthsShort;
    prototype__proto._monthsShort = defaultLocaleMonthsShort;
    prototype__proto.monthsParse  =        localeMonthsParse;

    // Week
    prototype__proto.week = localeWeek;
    prototype__proto._week = defaultLocaleWeek;
    prototype__proto.firstDayOfYear = localeFirstDayOfYear;
    prototype__proto.firstDayOfWeek = localeFirstDayOfWeek;

    // Day of Week
    prototype__proto.weekdays       =        localeWeekdays;
    prototype__proto._weekdays      = defaultLocaleWeekdays;
    prototype__proto.weekdaysMin    =        localeWeekdaysMin;
    prototype__proto._weekdaysMin   = defaultLocaleWeekdaysMin;
    prototype__proto.weekdaysShort  =        localeWeekdaysShort;
    prototype__proto._weekdaysShort = defaultLocaleWeekdaysShort;
    prototype__proto.weekdaysParse  =        localeWeekdaysParse;

    // Hours
    prototype__proto.isPM = localeIsPM;
    prototype__proto._meridiemParse = defaultLocaleMeridiemParse;
    prototype__proto.meridiem = localeMeridiem;

    function lists__get (format, index, field, setter) {
        var locale = locale_locales__getLocale();
        var utc = create_utc__createUTC().set(setter, index);
        return locale[field](utc, format);
    }

    function list (format, index, field, count, setter) {
        if (typeof format === 'number') {
            index = format;
            format = undefined;
        }

        format = format || '';

        if (index != null) {
            return lists__get(format, index, field, setter);
        }

        var i;
        var out = [];
        for (i = 0; i < count; i++) {
            out[i] = lists__get(format, i, field, setter);
        }
        return out;
    }

    function lists__listMonths (format, index) {
        return list(format, index, 'months', 12, 'month');
    }

    function lists__listMonthsShort (format, index) {
        return list(format, index, 'monthsShort', 12, 'month');
    }

    function lists__listWeekdays (format, index) {
        return list(format, index, 'weekdays', 7, 'day');
    }

    function lists__listWeekdaysShort (format, index) {
        return list(format, index, 'weekdaysShort', 7, 'day');
    }

    function lists__listWeekdaysMin (format, index) {
        return list(format, index, 'weekdaysMin', 7, 'day');
    }

    locale_locales__getSetGlobalLocale('en', {
        ordinalParse: /\d{1,2}(th|st|nd|rd)/,
        ordinal : function (number) {
            var b = number % 10,
                output = (toInt(number % 100 / 10) === 1) ? 'th' :
                (b === 1) ? 'st' :
                (b === 2) ? 'nd' :
                (b === 3) ? 'rd' : 'th';
            return number + output;
        }
    });

    // Side effect imports
    utils_hooks__hooks.lang = deprecate('moment.lang is deprecated. Use moment.locale instead.', locale_locales__getSetGlobalLocale);
    utils_hooks__hooks.langData = deprecate('moment.langData is deprecated. Use moment.localeData instead.', locale_locales__getLocale);

    var mathAbs = Math.abs;

    function duration_abs__abs () {
        var data           = this._data;

        this._milliseconds = mathAbs(this._milliseconds);
        this._days         = mathAbs(this._days);
        this._months       = mathAbs(this._months);

        data.milliseconds  = mathAbs(data.milliseconds);
        data.seconds       = mathAbs(data.seconds);
        data.minutes       = mathAbs(data.minutes);
        data.hours         = mathAbs(data.hours);
        data.months        = mathAbs(data.months);
        data.years         = mathAbs(data.years);

        return this;
    }

    function duration_add_subtract__addSubtract (duration, input, value, direction) {
        var other = create__createDuration(input, value);

        duration._milliseconds += direction * other._milliseconds;
        duration._days         += direction * other._days;
        duration._months       += direction * other._months;

        return duration._bubble();
    }

    // supports only 2.0-style add(1, 's') or add(duration)
    function duration_add_subtract__add (input, value) {
        return duration_add_subtract__addSubtract(this, input, value, 1);
    }

    // supports only 2.0-style subtract(1, 's') or subtract(duration)
    function duration_add_subtract__subtract (input, value) {
        return duration_add_subtract__addSubtract(this, input, value, -1);
    }

    function absCeil (number) {
        if (number < 0) {
            return Math.floor(number);
        } else {
            return Math.ceil(number);
        }
    }

    function bubble () {
        var milliseconds = this._milliseconds;
        var days         = this._days;
        var months       = this._months;
        var data         = this._data;
        var seconds, minutes, hours, years, monthsFromDays;

        // if we have a mix of positive and negative values, bubble down first
        // check: https://github.com/moment/moment/issues/2166
        if (!((milliseconds >= 0 && days >= 0 && months >= 0) ||
                (milliseconds <= 0 && days <= 0 && months <= 0))) {
            milliseconds += absCeil(monthsToDays(months) + days) * 864e5;
            days = 0;
            months = 0;
        }

        // The following code bubbles up values, see the tests for
        // examples of what that means.
        data.milliseconds = milliseconds % 1000;

        seconds           = absFloor(milliseconds / 1000);
        data.seconds      = seconds % 60;

        minutes           = absFloor(seconds / 60);
        data.minutes      = minutes % 60;

        hours             = absFloor(minutes / 60);
        data.hours        = hours % 24;

        days += absFloor(hours / 24);

        // convert days to months
        monthsFromDays = absFloor(daysToMonths(days));
        months += monthsFromDays;
        days -= absCeil(monthsToDays(monthsFromDays));

        // 12 months -> 1 year
        years = absFloor(months / 12);
        months %= 12;

        data.days   = days;
        data.months = months;
        data.years  = years;

        return this;
    }

    function daysToMonths (days) {
        // 400 years have 146097 days (taking into account leap year rules)
        // 400 years have 12 months === 4800
        return days * 4800 / 146097;
    }

    function monthsToDays (months) {
        // the reverse of daysToMonths
        return months * 146097 / 4800;
    }

    function as (units) {
        var days;
        var months;
        var milliseconds = this._milliseconds;

        units = normalizeUnits(units);

        if (units === 'month' || units === 'year') {
            days   = this._days   + milliseconds / 864e5;
            months = this._months + daysToMonths(days);
            return units === 'month' ? months : months / 12;
        } else {
            // handle milliseconds separately because of floating point math errors (issue #1867)
            days = this._days + Math.round(monthsToDays(this._months));
            switch (units) {
                case 'week'   : return days / 7     + milliseconds / 6048e5;
                case 'day'    : return days         + milliseconds / 864e5;
                case 'hour'   : return days * 24    + milliseconds / 36e5;
                case 'minute' : return days * 1440  + milliseconds / 6e4;
                case 'second' : return days * 86400 + milliseconds / 1000;
                // Math.floor prevents floating point math errors here
                case 'millisecond': return Math.floor(days * 864e5) + milliseconds;
                default: throw new Error('Unknown unit ' + units);
            }
        }
    }

    // TODO: Use this.as('ms')?
    function duration_as__valueOf () {
        return (
            this._milliseconds +
            this._days * 864e5 +
            (this._months % 12) * 2592e6 +
            toInt(this._months / 12) * 31536e6
        );
    }

    function makeAs (alias) {
        return function () {
            return this.as(alias);
        };
    }

    var asMilliseconds = makeAs('ms');
    var asSeconds      = makeAs('s');
    var asMinutes      = makeAs('m');
    var asHours        = makeAs('h');
    var asDays         = makeAs('d');
    var asWeeks        = makeAs('w');
    var asMonths       = makeAs('M');
    var asYears        = makeAs('y');

    function duration_get__get (units) {
        units = normalizeUnits(units);
        return this[units + 's']();
    }

    function makeGetter(name) {
        return function () {
            return this._data[name];
        };
    }

    var milliseconds = makeGetter('milliseconds');
    var seconds      = makeGetter('seconds');
    var minutes      = makeGetter('minutes');
    var hours        = makeGetter('hours');
    var days         = makeGetter('days');
    var months       = makeGetter('months');
    var years        = makeGetter('years');

    function weeks () {
        return absFloor(this.days() / 7);
    }

    var round = Math.round;
    var thresholds = {
        s: 45,  // seconds to minute
        m: 45,  // minutes to hour
        h: 22,  // hours to day
        d: 26,  // days to month
        M: 11   // months to year
    };

    // helper function for moment.fn.from, moment.fn.fromNow, and moment.duration.fn.humanize
    function substituteTimeAgo(string, number, withoutSuffix, isFuture, locale) {
        return locale.relativeTime(number || 1, !!withoutSuffix, string, isFuture);
    }

    function duration_humanize__relativeTime (posNegDuration, withoutSuffix, locale) {
        var duration = create__createDuration(posNegDuration).abs();
        var seconds  = round(duration.as('s'));
        var minutes  = round(duration.as('m'));
        var hours    = round(duration.as('h'));
        var days     = round(duration.as('d'));
        var months   = round(duration.as('M'));
        var years    = round(duration.as('y'));

        var a = seconds < thresholds.s && ['s', seconds]  ||
                minutes === 1          && ['m']           ||
                minutes < thresholds.m && ['mm', minutes] ||
                hours   === 1          && ['h']           ||
                hours   < thresholds.h && ['hh', hours]   ||
                days    === 1          && ['d']           ||
                days    < thresholds.d && ['dd', days]    ||
                months  === 1          && ['M']           ||
                months  < thresholds.M && ['MM', months]  ||
                years   === 1          && ['y']           || ['yy', years];

        a[2] = withoutSuffix;
        a[3] = +posNegDuration > 0;
        a[4] = locale;
        return substituteTimeAgo.apply(null, a);
    }

    // This function allows you to set a threshold for relative time strings
    function duration_humanize__getSetRelativeTimeThreshold (threshold, limit) {
        if (thresholds[threshold] === undefined) {
            return false;
        }
        if (limit === undefined) {
            return thresholds[threshold];
        }
        thresholds[threshold] = limit;
        return true;
    }

    function humanize (withSuffix) {
        var locale = this.localeData();
        var output = duration_humanize__relativeTime(this, !withSuffix, locale);

        if (withSuffix) {
            output = locale.pastFuture(+this, output);
        }

        return locale.postformat(output);
    }

    var iso_string__abs = Math.abs;

    function iso_string__toISOString() {
        // for ISO strings we do not use the normal bubbling rules:
        //  * milliseconds bubble up until they become hours
        //  * days do not bubble at all
        //  * months bubble up until they become years
        // This is because there is no context-free conversion between hours and days
        // (think of clock changes)
        // and also not between days and months (28-31 days per month)
        var seconds = iso_string__abs(this._milliseconds) / 1000;
        var days         = iso_string__abs(this._days);
        var months       = iso_string__abs(this._months);
        var minutes, hours, years;

        // 3600 seconds -> 60 minutes -> 1 hour
        minutes           = absFloor(seconds / 60);
        hours             = absFloor(minutes / 60);
        seconds %= 60;
        minutes %= 60;

        // 12 months -> 1 year
        years  = absFloor(months / 12);
        months %= 12;


        // inspired by https://github.com/dordille/moment-isoduration/blob/master/moment.isoduration.js
        var Y = years;
        var M = months;
        var D = days;
        var h = hours;
        var m = minutes;
        var s = seconds;
        var total = this.asSeconds();

        if (!total) {
            // this is the same as C#'s (Noda) and python (isodate)...
            // but not other JS (goog.date)
            return 'P0D';
        }

        return (total < 0 ? '-' : '') +
            'P' +
            (Y ? Y + 'Y' : '') +
            (M ? M + 'M' : '') +
            (D ? D + 'D' : '') +
            ((h || m || s) ? 'T' : '') +
            (h ? h + 'H' : '') +
            (m ? m + 'M' : '') +
            (s ? s + 'S' : '');
    }

    var duration_prototype__proto = Duration.prototype;

    duration_prototype__proto.abs            = duration_abs__abs;
    duration_prototype__proto.add            = duration_add_subtract__add;
    duration_prototype__proto.subtract       = duration_add_subtract__subtract;
    duration_prototype__proto.as             = as;
    duration_prototype__proto.asMilliseconds = asMilliseconds;
    duration_prototype__proto.asSeconds      = asSeconds;
    duration_prototype__proto.asMinutes      = asMinutes;
    duration_prototype__proto.asHours        = asHours;
    duration_prototype__proto.asDays         = asDays;
    duration_prototype__proto.asWeeks        = asWeeks;
    duration_prototype__proto.asMonths       = asMonths;
    duration_prototype__proto.asYears        = asYears;
    duration_prototype__proto.valueOf        = duration_as__valueOf;
    duration_prototype__proto._bubble        = bubble;
    duration_prototype__proto.get            = duration_get__get;
    duration_prototype__proto.milliseconds   = milliseconds;
    duration_prototype__proto.seconds        = seconds;
    duration_prototype__proto.minutes        = minutes;
    duration_prototype__proto.hours          = hours;
    duration_prototype__proto.days           = days;
    duration_prototype__proto.weeks          = weeks;
    duration_prototype__proto.months         = months;
    duration_prototype__proto.years          = years;
    duration_prototype__proto.humanize       = humanize;
    duration_prototype__proto.toISOString    = iso_string__toISOString;
    duration_prototype__proto.toString       = iso_string__toISOString;
    duration_prototype__proto.toJSON         = iso_string__toISOString;
    duration_prototype__proto.locale         = locale;
    duration_prototype__proto.localeData     = localeData;

    // Deprecations
    duration_prototype__proto.toIsoString = deprecate('toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)', iso_string__toISOString);
    duration_prototype__proto.lang = lang;

    // Side effect imports

    addFormatToken('X', 0, 0, 'unix');
    addFormatToken('x', 0, 0, 'valueOf');

    // PARSING

    addRegexToken('x', matchSigned);
    addRegexToken('X', matchTimestamp);
    addParseToken('X', function (input, array, config) {
        config._d = new Date(parseFloat(input, 10) * 1000);
    });
    addParseToken('x', function (input, array, config) {
        config._d = new Date(toInt(input));
    });

    // Side effect imports


    utils_hooks__hooks.version = '2.10.6';

    setHookCallback(local__createLocal);

    utils_hooks__hooks.fn                    = momentPrototype;
    utils_hooks__hooks.min                   = min;
    utils_hooks__hooks.max                   = max;
    utils_hooks__hooks.utc                   = create_utc__createUTC;
    utils_hooks__hooks.unix                  = moment__createUnix;
    utils_hooks__hooks.months                = lists__listMonths;
    utils_hooks__hooks.isDate                = isDate;
    utils_hooks__hooks.locale                = locale_locales__getSetGlobalLocale;
    utils_hooks__hooks.invalid               = valid__createInvalid;
    utils_hooks__hooks.duration              = create__createDuration;
    utils_hooks__hooks.isMoment              = isMoment;
    utils_hooks__hooks.weekdays              = lists__listWeekdays;
    utils_hooks__hooks.parseZone             = moment__createInZone;
    utils_hooks__hooks.localeData            = locale_locales__getLocale;
    utils_hooks__hooks.isDuration            = isDuration;
    utils_hooks__hooks.monthsShort           = lists__listMonthsShort;
    utils_hooks__hooks.weekdaysMin           = lists__listWeekdaysMin;
    utils_hooks__hooks.defineLocale          = defineLocale;
    utils_hooks__hooks.weekdaysShort         = lists__listWeekdaysShort;
    utils_hooks__hooks.normalizeUnits        = normalizeUnits;
    utils_hooks__hooks.relativeTimeThreshold = duration_humanize__getSetRelativeTimeThreshold;

    var _moment = utils_hooks__hooks;

    // return _moment;
    window.moment= _moment;

})();


(function(moment) {

    var zh_cn = moment.defineLocale('zh-cn', {
        months : '一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月'.split('_'),
        monthsShort : '1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月'.split('_'),
        weekdays : '星期日_星期一_星期二_星期三_星期四_星期五_星期六'.split('_'),
        weekdaysShort : '周日_周一_周二_周三_周四_周五_周六'.split('_'),
        weekdaysMin : '日_一_二_三_四_五_六'.split('_'),
        longDateFormat : {
            LT : 'Ah点mm分',
            LTS : 'Ah点m分s秒',
            L : 'YYYY-MM-DD',
            LL : 'YYYY年MMMD日',
            LLL : 'YYYY年MMMD日Ah点mm分',
            LLLL : 'YYYY年MMMD日ddddAh点mm分',
            l : 'YYYY-MM-DD',
            ll : 'YYYY年MMMD日',
            lll : 'YYYY年MMMD日Ah点mm分',
            llll : 'YYYY年MMMD日ddddAh点mm分'
        },
        meridiemParse: /凌晨|早上|上午|中午|下午|晚上/,
        meridiemHour: function (hour, meridiem) {
            if (hour === 12) {
                hour = 0;
            }
            if (meridiem === '凌晨' || meridiem === '早上' ||
                    meridiem === '上午') {
                return hour;
            } else if (meridiem === '下午' || meridiem === '晚上') {
                return hour + 12;
            } else {
                // '中午'
                return hour >= 11 ? hour : hour + 12;
            }
        },
        meridiem : function (hour, minute, isLower) {
            var hm = hour * 100 + minute;
            if (hm < 600) {
                return '凌晨';
            } else if (hm < 900) {
                return '早上';
            } else if (hm < 1130) {
                return '上午';
            } else if (hm < 1230) {
                return '中午';
            } else if (hm < 1800) {
                return '下午';
            } else {
                return '晚上';
            }
        },
        calendar : {
            sameDay : function () {
                return this.minutes() === 0 ? '[今天]Ah[点整]' : '[今天]LT';
            },
            nextDay : function () {
                return this.minutes() === 0 ? '[明天]Ah[点整]' : '[明天]LT';
            },
            lastDay : function () {
                return this.minutes() === 0 ? '[昨天]Ah[点整]' : '[昨天]LT';
            },
            nextWeek : function () {
                var startOfWeek, prefix;
                startOfWeek = moment().startOf('week');
                prefix = this.unix() - startOfWeek.unix() >= 7 * 24 * 3600 ? '[下]' : '[本]';
                return this.minutes() === 0 ? prefix + 'dddAh点整' : prefix + 'dddAh点mm';
            },
            lastWeek : function () {
                var startOfWeek, prefix;
                startOfWeek = moment().startOf('week');
                prefix = this.unix() < startOfWeek.unix()  ? '[上]' : '[本]';
                return this.minutes() === 0 ? prefix + 'dddAh点整' : prefix + 'dddAh点mm';
            },
            sameElse : 'LL'
        },
        ordinalParse: /\d{1,2}(日|月|周)/,
        ordinal : function (number, period) {
            switch (period) {
            case 'd':
            case 'D':
            case 'DDD':
                return number + '日';
            case 'M':
                return number + '月';
            case 'w':
            case 'W':
                return number + '周';
            default:
                return number;
            }
        },
        relativeTime : {
            future : '%s内',
            past : '%s前',
            s : '几秒',
            m : '1 分钟',
            mm : '%d 分钟',
            h : '1 小时',
            hh : '%d 小时',
            d : '1 天',
            dd : '%d 天',
            M : '1 个月',
            MM : '%d 个月',
            y : '1 年',
            yy : '%d 年'
        },
        week : {
            // GB/T 7408-1994《数据元和交换格式·信息交换·日期和时间表示法》与ISO 8601:1988等效
            dow : 1, // Monday is the first day of the week.
            doy : 4  // The week that contains Jan 4th is the first week of the year.
        }
    });

    // return zh_cn;

})(moment);


(function(moment) {
    moment.locale('zh_cn'); // 显示语言
    // moment.tz.setDefault(8);         // 北京时间正8区
})(moment);
/* ========================================================================
 * Bootstrap: transition.js v3.3.5
 * http://getbootstrap.com/javascript/#transitions
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // CSS TRANSITION SUPPORT (Shoutout: http://www.modernizr.com/)
  // ============================================================

  function transitionEnd() {
    var el = document.createElement('bootstrap')

    var transEndEventNames = {
      WebkitTransition : 'webkitTransitionEnd',
      MozTransition    : 'transitionend',
      OTransition      : 'oTransitionEnd otransitionend',
      transition       : 'transitionend'
    }

    for (var name in transEndEventNames) {
      if (el.style[name] !== undefined) {
        return { end: transEndEventNames[name] }
      }
    }

    return false // explicit for ie8 (  ._.)
  }

  // http://blog.alexmaccaw.com/css-transitions
  $.fn.emulateTransitionEnd = function (duration) {
    var called = false
    var $el = this
    $(this).one('bsTransitionEnd', function () { called = true })
    var callback = function () { if (!called) $($el).trigger($.support.transition.end) }
    setTimeout(callback, duration)
    return this
  }

  $(function () {
    $.support.transition = transitionEnd()

    if (!$.support.transition) return

    $.event.special.bsTransitionEnd = {
      bindType: $.support.transition.end,
      delegateType: $.support.transition.end,
      handle: function (e) {
        if ($(e.target).is(this)) return e.handleObj.handler.apply(this, arguments)
      }
    }
  })

}(jQuery);


//弹窗拖拽
(function($){
    $.fn.drags = function(opt) {
        opt = $.extend( { handle:"",cursor:"move" }, opt);
        var $handle = opt.handle === "" ? $(this) : $(this).find(opt.handle);
        var $drag   = $(this);
        var move = function(){
            var z_idx,
                mouseMove;
            return {
                start:function(e){
                    z_idx = $drag.css('z-index');
                    mouseMove = function() {
                        var drg_h = $drag.outerHeight(),
                            drg_w = $drag.outerWidth(),
                            pos_y = $drag.offset().top + drg_h - e.pageY,
                            pos_x = $drag.offset().left + drg_w - e.pageX;
                        return function(e){
                            $drag.offset({
                                             top:e.pageY + pos_y - drg_h,
                                             left:e.pageX + pos_x - drg_w
                                         });
                        };
                    }();
                    $(window).on("mousemove", mouseMove)
                         // .css('z-index', 99999)
                         .addClass('draggable');
                },
                stop:function(){
                    $(window).off("mousemove", mouseMove)
                        // .css('z-index', z_idx)
                        .removeClass('draggable');
                }
            };
        }();

        $handle.css('cursor', opt.cursor)
               .on("mousedown",function(e){
                   $(window).one("mouseup", function(e) {
                       move.stop();
                       $handle.removeClass('active-handle');
                   });
                   move.start(e);
                   $handle.addClass('active-handle');
               });
        return $handle;
    } 
})(jQuery);


$.fn.extend({
        //---元素拖动插件
    dragging:function(data){   
        var $this = $(this);
        var xPage;
        var yPage;
        var X;//
        var Y;//
        var xRand = 0;//
        var yRand = 0;//
        var father = $this.parent();
        var defaults = {
            move : 'both',
            randomPosition : true ,
            hander:1,

        }
        var opt = $.extend({},defaults,data);
        var movePosition = opt.move;
        var random = opt.randomPosition;
        
        var hander = opt.hander;
        if(opt.parent){
            $this=opt.parent;
        }
        if(hander == 1){
            hander = $this; 
        }else{
            hander = $this.find(opt.hander);
        }
        
            
        //---初始化
        //father.css({"position":"relative","overflow":"hidden"});
        father.css({"position":"relative"});
        if(opt.parent){
              $this.css({"position":"absolute",'left':'50%','marginLeft':(-$this.width()/2)});
          }else{
            $this.css({"position":"absolute"});
          }
      
        hander.css({"cursor":"move"});

        var faWidth = father.width();
        var faHeight = father.height();
        var thisWidth = $this.width()+parseInt($this.css('padding-left'))+parseInt($this.css('padding-right'));
        var thisHeight = $this.height()+parseInt($this.css('padding-top'))+parseInt($this.css('padding-bottom'));
        var mDown = false;//
        var positionX;
        var positionY;
        var moveX ;
        var moveY ;
        
        if(random){
            $thisRandom();
        }
        function $thisRandom(){ //随机函数
            $this.each(function(index){
                var randY = parseInt(Math.random()*(faHeight-thisHeight));///
                var randX = parseInt(Math.random()*(faWidth-thisWidth));///
                if(movePosition.toLowerCase() == 'x'){
                    $(this).css({
                        left:randX
                    });
                }else if(movePosition.toLowerCase() == 'y'){
                    $(this).css({
                        top:randY
                    });
                }else if(movePosition.toLowerCase() == 'both'){
                    $(this).css({
                        top:randY,
                        left:randX
                    });
                }
                
            }); 
        }
        
        hander.mousedown(function(e){
          //  father.children().css({"zIndex":"0"});
           // $this.css({"zIndex":"1"});
            mDown = true;
            X = e.pageX;
            Y = e.pageY;
            positionX = $this.position().left;
            positionY = $this.position().top;
            return false;
        });
            
        $(document).mouseup(function(e){
            mDown = false;
        });
            
        $(document).mousemove(function(e){
            xPage = e.pageX;//--
            moveX = positionX+xPage-X;
            
            yPage = e.pageY;//--
            moveY = positionY+yPage-Y;
            
            function thisXMove(){ //x轴移动
                if(mDown === true){
                    $this.css({"left":moveX});
                }else{
                    return;
                }
                if(moveX < 0){
                    $this.css({"left":"0"});
                }
                if(moveX > (faWidth-thisWidth)){
                    $this.css({"left":faWidth-thisWidth});
                }
                return moveX;
            }
            
            function thisYMove(){ //y轴移动
                if(mDown === true){
                    $this.css({"top":moveY});
                }else{
                    return;
                }
                if(moveY < 0){
                    $this.css({"top":"0"});
                }
                if(moveY > (faHeight-thisHeight)){
                    $this.css({"top":faHeight-thisHeight});
                }
                return moveY;
            }

            function thisAllMove(){ //全部移动
                if(mDown === true){
                    $this.css({"left":moveX,"top":moveY});
                }else{
                    return;
                }
               /* if(moveX < 0){
                  //  $this.css({"left":"0"});
                    $this.css({"left":moveX});
                }
                if(moveX > (faWidth-thisWidth)){
                
                    $this.css({"left":faWidth-thisWidth});
                }

                if(moveY < 0){
                   // $this.css({"top":"0"});
                     $this.css({"top":moveY});
                }
                if(moveY > (faHeight-thisHeight)){
                    $this.css({"top":faHeight-thisHeight});
                }*/
            }
            if(movePosition.toLowerCase() == "x"){
                thisXMove();
            }else if(movePosition.toLowerCase() == "y"){
                thisYMove();
            }else if(movePosition.toLowerCase() == 'both'){
                thisAllMove();
            }
        });
    }
}); 
/**
 * jquery.mask.js
 * @version: v1.14.11
 * @author: Igor Escobar
 *
 * Created by Igor Escobar on 2012-03-10. Please report any bug at http://blog.igorescobar.com
 *
 * Copyright (c) 2012 Igor Escobar http://blog.igorescobar.com
 *
 * The MIT License (http://www.opensource.org/licenses/mit-license.php)
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/* jshint laxbreak: true */
/* jshint maxcomplexity:17 */
/* global define */

'use strict';

// UMD (Universal Module Definition) patterns for JavaScript modules that work everywhere.
// https://github.com/umdjs/umd/blob/master/jqueryPluginCommonjs.js
(function (factory, jQuery, Zepto) {

    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        module.exports = factory(require('jquery'));
    } else {
        factory(jQuery || Zepto);
    }

}(function ($) {

    var Mask = function (el, mask, options) {

        var p = {
            invalid: [],
            getCaret: function () {
                try {
                    var sel,
                        pos = 0,
                        ctrl = el.get(0),
                        dSel = document.selection,
                        cSelStart = ctrl.selectionStart;

                    // IE Support
                    if (dSel && navigator.appVersion.indexOf('MSIE 10') === -1) {
                        sel = dSel.createRange();
                        sel.moveStart('character', -p.val().length);
                        pos = sel.text.length;
                    }
                    // Firefox support
                    else if (cSelStart || cSelStart === '0') {
                        pos = cSelStart;
                    }

                    return pos;
                } catch (e) {}
            },
            setCaret: function(pos) {
                try {
                    if (el.is(':focus')) {
                        var range, ctrl = el.get(0);

                        // Firefox, WebKit, etc..
                        if (ctrl.setSelectionRange) {
                            //ctrl.setSelectionRange(pos, pos);
                            ctrl.setSelectionRange(ctrl.value.length, ctrl.value.length);       //zou.oth 修改光标停留位置
                        } else { // IE
                            range = ctrl.createTextRange();
                            range.collapse(true);
                            //range.moveEnd('character', pos);
                            //range.moveStart('character', pos);
                            range.moveEnd('character', range.value.length);     //zou.oth 修改光标停留位置
                            range.moveStart('character', range.value.length);       //zou.oth 修改光标停留位置
                            range.select();
                        }
                    }
                } catch (e) {}
            },
            events: function() {
                el
                .on('keydown.mask', function(e) {
                    el.data('mask-keycode', e.keyCode || e.which);
                    el.data('mask-previus-value', el.val());
                    el.data('mask-previus-caret-pos', p.getCaret());
                    p.maskDigitPosMapOld = p.maskDigitPosMap;
                })
                .on($.jMaskGlobals.useInput ? 'input.mask' : 'keyup.mask', p.behaviour)
                .on('paste.mask drop.mask', function() {
                    setTimeout(function() {
                        el.keydown().keyup();
                    }, 100);
                })
                .on('change.mask', function(){
                    el.data('changed', true);
                })
                .on('blur.mask', function(){
                    if (oldValue !== p.val() && !el.data('changed')) {
                        el.trigger('change');
                    }
                    el.data('changed', false);
                })
                // it's very important that this callback remains in this position
                // otherwhise oldValue it's going to work buggy
                .on('blur.mask', function() {
                    oldValue = p.val();
                })
                // select all text on focus
                .on('focus.mask', function (e) {
                    if (options.selectOnFocus === true) {
                        $(e.target).select();
                    }
                })
                // clear the value if it not complete the mask
                .on('focusout.mask', function() {
                    if (options.clearIfNotMatch && !regexMask.test(p.val())) {
                       p.val('');
                   }
                });
            },
            getRegexMask: function() {
                var maskChunks = [], translation, pattern, optional, recursive, oRecursive, r;

                for (var i = 0; i < mask.length; i++) {
                    translation = jMask.translation[mask.charAt(i)];

                    if (translation) {

                        pattern = translation.pattern.toString().replace(/.{1}$|^.{1}/g, '');
                        optional = translation.optional;
                        recursive = translation.recursive;

                        if (recursive) {
                            maskChunks.push(mask.charAt(i));
                            oRecursive = {digit: mask.charAt(i), pattern: pattern};
                        } else {
                            maskChunks.push(!optional && !recursive ? pattern : (pattern + '?'));
                        }

                    } else {
                        maskChunks.push(mask.charAt(i).replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&'));
                    }
                }

                r = maskChunks.join('');

                if (oRecursive) {
                    r = r.replace(new RegExp('(' + oRecursive.digit + '(.*' + oRecursive.digit + ')?)'), '($1)?')
                         .replace(new RegExp(oRecursive.digit, 'g'), oRecursive.pattern);
                }

                return new RegExp(r);
            },
            destroyEvents: function() {
                el.off(['input', 'keydown', 'keyup', 'paste', 'drop', 'blur', 'focusout', ''].join('.mask '));
            },
            val: function(v) {
                var isInput = el.is('input'),
                    method = isInput ? 'val' : 'text',
                    r;

                if (arguments.length > 0) {
                    if (el[method]() !== v) {
                        el[method](v);
                    }
                    r = el;
                } else {
                    r = el[method]();
                }

                return r;
            },
            calculateCaretPosition: function() {
                var oldVal = el.data('mask-previus-value') || '',
                    newVal = p.getMasked(),
                    caretPosNew = p.getCaret();
                if (oldVal !== newVal) {
                    var caretPosOld = el.data('mask-previus-caret-pos') || 0,
                        newValL = newVal.length,
                        oldValL = oldVal.length,
                        maskDigitsBeforeCaret = 0,
                        maskDigitsAfterCaret = 0,
                        maskDigitsBeforeCaretAll = 0,
                        maskDigitsBeforeCaretAllOld = 0,
                        i = 0;

                    for (i = caretPosNew; i < newValL; i++) {
                        if (!p.maskDigitPosMap[i]) {
                            break;
                        }
                        maskDigitsAfterCaret++;
                    }

                    for (i = caretPosNew - 1; i >= 0; i--) {
                        if (!p.maskDigitPosMap[i]) {
                            break;
                        }
                        maskDigitsBeforeCaret++;
                    }

                    for (i = caretPosNew - 1; i >= 0; i--) {
                        if (p.maskDigitPosMap[i]) {
                            maskDigitsBeforeCaretAll++;
                        }
                    }

                    for (i = caretPosOld - 1; i >= 0; i--) {
                        if (p.maskDigitPosMapOld[i]) {
                            maskDigitsBeforeCaretAllOld++;
                        }
                    }

                    if (caretPosNew > oldValL) {
                      // if the cursor is at the end keep it there
                      caretPosNew = newValL;
                    }
                    else if (caretPosOld >= caretPosNew && caretPosOld !== oldValL) {
                        if (!p.maskDigitPosMapOld[caretPosNew])  {
                          var caretPos = caretPosNew;
                          caretPosNew -= maskDigitsBeforeCaretAllOld - maskDigitsBeforeCaretAll;
                          caretPosNew -= maskDigitsBeforeCaret;
                          if (p.maskDigitPosMap[caretPosNew])  {
                            caretPosNew = caretPos;
                          }
                        }
                    }
                    else if (caretPosNew > caretPosOld) {
                        caretPosNew += maskDigitsBeforeCaretAll - maskDigitsBeforeCaretAllOld;
                        caretPosNew += maskDigitsAfterCaret;
                    }
                }
                return caretPosNew;
            },
            behaviour: function(e) {
                e = e || window.event;
                p.invalid = [];

                var keyCode = el.data('mask-keycode');

                if ($.inArray(keyCode, jMask.byPassKeys) === -1) {
                    var newVal   = p.getMasked(),
                        caretPos = p.getCaret();

                    setTimeout(function() {
                      p.setCaret(p.calculateCaretPosition());
                    }, 10);

                    p.val(newVal);
                    p.setCaret(caretPos);
                    return p.callbacks(e);
                }
            },
            getMasked: function(skipMaskChars, val) {
                var buf = [],
                    value = val === undefined ? p.val() : val + '',
                    m = 0, maskLen = mask.length,
                    v = 0, valLen = value.length,
                    offset = 1, addMethod = 'push',
                    resetPos = -1,
                    maskDigitCount = 0,
                    maskDigitPosArr = [],
                    lastMaskChar,
                    check;

                if (options.reverse) {
                    addMethod = 'unshift';
                    offset = -1;
                    lastMaskChar = 0;
                    m = maskLen - 1;
                    v = valLen - 1;
                    check = function () {
                        return m > -1 && v > -1;
                    };
                } else {
                    lastMaskChar = maskLen - 1;
                    check = function () {
                        return m < maskLen && v < valLen;
                    };
                }

                var lastUntranslatedMaskChar;
                while (check()) {
                    var maskDigit = mask.charAt(m),
                        valDigit = value.charAt(v),
                        translation = jMask.translation[maskDigit];

                    if (translation) {
                        if (valDigit.match(translation.pattern)) {
                            buf[addMethod](valDigit);
                             if (translation.recursive) {
                                if (resetPos === -1) {
                                    resetPos = m;
                                } else if (m === lastMaskChar) {
                                    m = resetPos - offset;
                                }

                                if (lastMaskChar === resetPos) {
                                    m -= offset;
                                }
                            }
                            m += offset;
                        } else if (valDigit === lastUntranslatedMaskChar) {
                            // matched the last untranslated (raw) mask character that we encountered
                            // likely an insert offset the mask character from the last entry; fall
                            // through and only increment v
                            maskDigitCount--;
                            lastUntranslatedMaskChar = undefined;
                        } else if (translation.optional) {
                            m += offset;
                            v -= offset;
                        } else if (translation.fallback) {
                            buf[addMethod](translation.fallback);
                            m += offset;
                            v -= offset;
                        } else {
                          p.invalid.push({p: v, v: valDigit, e: translation.pattern});
                        }
                        v += offset;
                    } else {
                        if (!skipMaskChars) {
                            buf[addMethod](maskDigit);
                        }

                        if (valDigit === maskDigit) {
                            maskDigitPosArr.push(v);
                            v += offset;
                        } else {
                            lastUntranslatedMaskChar = maskDigit;
                            maskDigitPosArr.push(v + maskDigitCount);
                            maskDigitCount++;
                        }

                        m += offset;
                    }
                }

                var lastMaskCharDigit = mask.charAt(lastMaskChar);
                if (maskLen === valLen + 1 && !jMask.translation[lastMaskCharDigit]) {
                    buf.push(lastMaskCharDigit);
                }

                var newVal = buf.join('');
                p.mapMaskdigitPositions(newVal, maskDigitPosArr, valLen);
                return newVal;
            },
            mapMaskdigitPositions: function(newVal, maskDigitPosArr, valLen) {
              var maskDiff = options.reverse ? newVal.length - valLen : 0;
              p.maskDigitPosMap = {};
              for (var i = 0; i < maskDigitPosArr.length; i++) {
                p.maskDigitPosMap[maskDigitPosArr[i] + maskDiff] = 1;
              }
            },
            callbacks: function (e) {
                var val = p.val(),
                    changed = val !== oldValue,
                    defaultArgs = [val, e, el, options],
                    callback = function(name, criteria, args) {
                        if (typeof options[name] === 'function' && criteria) {
                            options[name].apply(this, args);
                        }
                    };

                callback('onChange', changed === true, defaultArgs);
                callback('onKeyPress', changed === true, defaultArgs);
                callback('onComplete', val.length === mask.length, defaultArgs);
                callback('onInvalid', p.invalid.length > 0, [val, e, el, p.invalid, options]);
            }
        };

        el = $(el);
        var jMask = this, oldValue = p.val(), regexMask;

        mask = typeof mask === 'function' ? mask(p.val(), undefined, el,  options) : mask;

        // public methods
        jMask.mask = mask;
        jMask.options = options;
        jMask.remove = function() {
            var caret = p.getCaret();
            p.destroyEvents();
            p.val(jMask.getCleanVal());
            p.setCaret(caret);
            return el;
        };

        // get value without mask
        jMask.getCleanVal = function() {
           return p.getMasked(true);
        };

        // get masked value without the value being in the input or element
        jMask.getMaskedVal = function(val) {
           return p.getMasked(false, val);
        };

       jMask.init = function(onlyMask) {
            onlyMask = onlyMask || false;
            options = options || {};

            jMask.clearIfNotMatch  = $.jMaskGlobals.clearIfNotMatch;
            jMask.byPassKeys       = $.jMaskGlobals.byPassKeys;
            jMask.translation      = $.extend({}, $.jMaskGlobals.translation, options.translation);

            jMask = $.extend(true, {}, jMask, options);

            regexMask = p.getRegexMask();

            if (onlyMask) {
                p.events();
                p.val(p.getMasked());
            } else {
                if (options.placeholder) {
                    el.attr('placeholder' , options.placeholder);
                }

                // this is necessary, otherwise if the user submit the form
                // and then press the "back" button, the autocomplete will erase
                // the data. Works fine on IE9+, FF, Opera, Safari.
                if (el.data('mask')) {
                  el.attr('autocomplete', 'off');
                }

                // detect if is necessary let the user type freely.
                // for is a lot faster than forEach.
                for (var i = 0, maxlength = true; i < mask.length; i++) {
                    var translation = jMask.translation[mask.charAt(i)];
                    if (translation && translation.recursive) {
                        maxlength = false;
                        break;
                    }
                }

                if (maxlength) {
                    el.attr('maxlength', mask.length);
                }

                p.destroyEvents();
                p.events();

                var caret = p.getCaret();
                p.val(p.getMasked());
                p.setCaret(caret);
            }
        };

        jMask.init(!el.is('input'));
    };

    $.maskWatchers = {};
    var HTMLAttributes = function () {
        var input = $(this),
            options = {},
            prefix = 'data-mask-',
            mask = input.attr('data-mask');

        if (input.attr(prefix + 'reverse')) {
            options.reverse = true;
        }

        if (input.attr(prefix + 'clearifnotmatch')) {
            options.clearIfNotMatch = true;
        }

        if (input.attr(prefix + 'selectonfocus') === 'true') {
           options.selectOnFocus = true;
        }

        if (notSameMaskObject(input, mask, options)) {
            return input.data('mask', new Mask(this, mask, options));
        }
    },
    notSameMaskObject = function(field, mask, options) {
        options = options || {};
        var maskObject = $(field).data('mask'),
            stringify = JSON.stringify,
            value = $(field).val() || $(field).text();
        try {
            if (typeof mask === 'function') {
                mask = mask(value);
            }
            return typeof maskObject !== 'object' || stringify(maskObject.options) !== stringify(options) || maskObject.mask !== mask;
        } catch (e) {}
    },
    eventSupported = function(eventName) {
        var el = document.createElement('div'), isSupported;

        eventName = 'on' + eventName;
        isSupported = (eventName in el);

        if ( !isSupported ) {
            el.setAttribute(eventName, 'return;');
            isSupported = typeof el[eventName] === 'function';
        }
        el = null;

        return isSupported;
    };

    $.fn.mask = function(mask, options) {
        options = options || {};
        var selector = this.selector,
            globals = $.jMaskGlobals,
            interval = globals.watchInterval,
            watchInputs = options.watchInputs || globals.watchInputs,
            maskFunction = function() {
                if (notSameMaskObject(this, mask, options)) {
                    return $(this).data('mask', new Mask(this, mask, options));
                }
            };

        $(this).each(maskFunction);

        if (selector && selector !== '' && watchInputs) {
            clearInterval($.maskWatchers[selector]);
            $.maskWatchers[selector] = setInterval(function(){
                $(document).find(selector).each(maskFunction);
            }, interval);
        }
        return this;
    };

    $.fn.masked = function(val) {
        return this.data('mask').getMaskedVal(val);
    };

    $.fn.unmask = function() {
        clearInterval($.maskWatchers[this.selector]);
        delete $.maskWatchers[this.selector];
        return this.each(function() {
            var dataMask = $(this).data('mask');
            if (dataMask) {
                dataMask.remove().removeData('mask');
            }
        });
    };

    $.fn.cleanVal = function() {
        return this.data('mask').getCleanVal();
    };

    $.applyDataMask = function(selector) {
        selector = selector || $.jMaskGlobals.maskElements;
        var $selector = (selector instanceof $) ? selector : $(selector);
        $selector.filter($.jMaskGlobals.dataMaskAttr).each(HTMLAttributes);
    };

    var globals = {
        maskElements: 'input,td,span,div',
        dataMaskAttr: '*[data-mask]',
        dataMask: true,
        watchInterval: 300,
        watchInputs: true,
        // old versions of chrome dont work great with input event
        useInput: !/Chrome\/[2-4][0-9]|SamsungBrowser/.test(window.navigator.userAgent) && eventSupported('input'),
        watchDataMask: false,
        byPassKeys: [9, 16, 17, 18, 36, 37, 38, 39, 40, 91],
        translation: {
            '0': {pattern: /\d/},
            '9': {pattern: /\d/, optional: true},
            '#': {pattern: /\d/, recursive: true},
            'A': {pattern: /[a-zA-Z0-9]/},
            'S': {pattern: /[a-zA-Z]/}
        }
    };

    $.jMaskGlobals = $.jMaskGlobals || {};
    globals = $.jMaskGlobals = $.extend(true, {}, globals, $.jMaskGlobals);

    // looking for inputs with data-mask attribute
    if (globals.dataMask) {
        $.applyDataMask();
    }

    setInterval(function() {
        if ($.jMaskGlobals.watchDataMask) {
            $.applyDataMask();
        }
    }, globals.watchInterval);
}, window.jQuery, window.Zepto));

/*! jQuery UI - v1.12.1 - 2017-01-04
* http://jqueryui.com
* Includes: widget.js, data.js, scroll-parent.js, widgets/draggable.js, widgets/droppable.js, widgets/sortable.js, widgets/mouse.js
* Copyright jQuery Foundation and other contributors; Licensed MIT */

(function( factory ) {
	if ( typeof define === "function" && define.amd ) {

		// AMD. Register as an anonymous module.
		define([ "jquery" ], factory );
	} else {

		// Browser globals
		factory( jQuery );
	}
}(function( $ ) {

$.ui = $.ui || {};

var version = $.ui.version = "1.12.1";


/*!
 * jQuery UI Widget 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */

//>>label: Widget
//>>group: Core
//>>description: Provides a factory for creating stateful widgets with a common API.
//>>docs: http://api.jqueryui.com/jQuery.widget/
//>>demos: http://jqueryui.com/widget/



var widgetUuid = 0;
var widgetSlice = Array.prototype.slice;

$.cleanData = ( function( orig ) {
	return function( elems ) {
		var events, elem, i;
		for ( i = 0; ( elem = elems[ i ] ) != null; i++ ) {
			try {

				// Only trigger remove when necessary to save time
				events = $._data( elem, "events" );
				if ( events && events.remove ) {
					$( elem ).triggerHandler( "remove" );
				}

			// Http://bugs.jquery.com/ticket/8235
			} catch ( e ) {}
		}
		orig( elems );
	};
} )( $.cleanData );

$.widget = function( name, base, prototype ) {
	var existingConstructor, constructor, basePrototype;

	// ProxiedPrototype allows the provided prototype to remain unmodified
	// so that it can be used as a mixin for multiple widgets (#8876)
	var proxiedPrototype = {};

	var namespace = name.split( "." )[ 0 ];
	name = name.split( "." )[ 1 ];
	var fullName = namespace + "-" + name;

	if ( !prototype ) {
		prototype = base;
		base = $.Widget;
	}

	if ( $.isArray( prototype ) ) {
		prototype = $.extend.apply( null, [ {} ].concat( prototype ) );
	}

	// Create selector for plugin
	$.expr[ ":" ][ fullName.toLowerCase() ] = function( elem ) {
		return !!$.data( elem, fullName );
	};

	$[ namespace ] = $[ namespace ] || {};
	existingConstructor = $[ namespace ][ name ];
	constructor = $[ namespace ][ name ] = function( options, element ) {

		// Allow instantiation without "new" keyword
		if ( !this._createWidget ) {
			return new constructor( options, element );
		}

		// Allow instantiation without initializing for simple inheritance
		// must use "new" keyword (the code above always passes args)
		if ( arguments.length ) {
			this._createWidget( options, element );
		}
	};

	// Extend with the existing constructor to carry over any static properties
	$.extend( constructor, existingConstructor, {
		version: prototype.version,

		// Copy the object used to create the prototype in case we need to
		// redefine the widget later
		_proto: $.extend( {}, prototype ),

		// Track widgets that inherit from this widget in case this widget is
		// redefined after a widget inherits from it
		_childConstructors: []
	} );

	basePrototype = new base();

	// We need to make the options hash a property directly on the new instance
	// otherwise we'll modify the options hash on the prototype that we're
	// inheriting from
	basePrototype.options = $.widget.extend( {}, basePrototype.options );
	$.each( prototype, function( prop, value ) {
		if ( !$.isFunction( value ) ) {
			proxiedPrototype[ prop ] = value;
			return;
		}
		proxiedPrototype[ prop ] = ( function() {
			function _super() {
				return base.prototype[ prop ].apply( this, arguments );
			}

			function _superApply( args ) {
				return base.prototype[ prop ].apply( this, args );
			}

			return function() {
				var __super = this._super;
				var __superApply = this._superApply;
				var returnValue;

				this._super = _super;
				this._superApply = _superApply;

				returnValue = value.apply( this, arguments );

				this._super = __super;
				this._superApply = __superApply;

				return returnValue;
			};
		} )();
	} );
	constructor.prototype = $.widget.extend( basePrototype, {

		// TODO: remove support for widgetEventPrefix
		// always use the name + a colon as the prefix, e.g., draggable:start
		// don't prefix for widgets that aren't DOM-based
		widgetEventPrefix: existingConstructor ? ( basePrototype.widgetEventPrefix || name ) : name
	}, proxiedPrototype, {
		constructor: constructor,
		namespace: namespace,
		widgetName: name,
		widgetFullName: fullName
	} );

	// If this widget is being redefined then we need to find all widgets that
	// are inheriting from it and redefine all of them so that they inherit from
	// the new version of this widget. We're essentially trying to replace one
	// level in the prototype chain.
	if ( existingConstructor ) {
		$.each( existingConstructor._childConstructors, function( i, child ) {
			var childPrototype = child.prototype;

			// Redefine the child widget using the same prototype that was
			// originally used, but inherit from the new version of the base
			$.widget( childPrototype.namespace + "." + childPrototype.widgetName, constructor,
				child._proto );
		} );

		// Remove the list of existing child constructors from the old constructor
		// so the old child constructors can be garbage collected
		delete existingConstructor._childConstructors;
	} else {
		base._childConstructors.push( constructor );
	}

	$.widget.bridge( name, constructor );

	return constructor;
};

$.widget.extend = function( target ) {
	var input = widgetSlice.call( arguments, 1 );
	var inputIndex = 0;
	var inputLength = input.length;
	var key;
	var value;

	for ( ; inputIndex < inputLength; inputIndex++ ) {
		for ( key in input[ inputIndex ] ) {
			value = input[ inputIndex ][ key ];
			if ( input[ inputIndex ].hasOwnProperty( key ) && value !== undefined ) {

				// Clone objects
				if ( $.isPlainObject( value ) ) {
					target[ key ] = $.isPlainObject( target[ key ] ) ?
						$.widget.extend( {}, target[ key ], value ) :

						// Don't extend strings, arrays, etc. with objects
						$.widget.extend( {}, value );

				// Copy everything else by reference
				} else {
					target[ key ] = value;
				}
			}
		}
	}
	return target;
};

$.widget.bridge = function( name, object ) {
	var fullName = object.prototype.widgetFullName || name;
	$.fn[ name ] = function( options ) {
		var isMethodCall = typeof options === "string";
		var args = widgetSlice.call( arguments, 1 );
		var returnValue = this;

		if ( isMethodCall ) {

			// If this is an empty collection, we need to have the instance method
			// return undefined instead of the jQuery instance
			if ( !this.length && options === "instance" ) {
				returnValue = undefined;
			} else {
				this.each( function() {
					var methodValue;
					var instance = $.data( this, fullName );

					if ( options === "instance" ) {
						returnValue = instance;
						return false;
					}

					if ( !instance ) {
						return $.error( "cannot call methods on " + name +
							" prior to initialization; " +
							"attempted to call method '" + options + "'" );
					}

					if ( !$.isFunction( instance[ options ] ) || options.charAt( 0 ) === "_" ) {
						return $.error( "no such method '" + options + "' for " + name +
							" widget instance" );
					}

					methodValue = instance[ options ].apply( instance, args );

					if ( methodValue !== instance && methodValue !== undefined ) {
						returnValue = methodValue && methodValue.jquery ?
							returnValue.pushStack( methodValue.get() ) :
							methodValue;
						return false;
					}
				} );
			}
		} else {

			// Allow multiple hashes to be passed on init
			if ( args.length ) {
				options = $.widget.extend.apply( null, [ options ].concat( args ) );
			}

			this.each( function() {
				var instance = $.data( this, fullName );
				if ( instance ) {
					instance.option( options || {} );
					if ( instance._init ) {
						instance._init();
					}
				} else {
					$.data( this, fullName, new object( options, this ) );
				}
			} );
		}

		return returnValue;
	};
};

$.Widget = function( /* options, element */ ) {};
$.Widget._childConstructors = [];

$.Widget.prototype = {
	widgetName: "widget",
	widgetEventPrefix: "",
	defaultElement: "<div>",

	options: {
		classes: {},
		disabled: false,

		// Callbacks
		create: null
	},

	_createWidget: function( options, element ) {
		element = $( element || this.defaultElement || this )[ 0 ];
		this.element = $( element );
		this.uuid = widgetUuid++;
		this.eventNamespace = "." + this.widgetName + this.uuid;

		this.bindings = $();
		this.hoverable = $();
		this.focusable = $();
		this.classesElementLookup = {};

		if ( element !== this ) {
			$.data( element, this.widgetFullName, this );
			this._on( true, this.element, {
				remove: function( event ) {
					if ( event.target === element ) {
						this.destroy();
					}
				}
			} );
			this.document = $( element.style ?

				// Element within the document
				element.ownerDocument :

				// Element is window or document
				element.document || element );
			this.window = $( this.document[ 0 ].defaultView || this.document[ 0 ].parentWindow );
		}

		this.options = $.widget.extend( {},
			this.options,
			this._getCreateOptions(),
			options );

		this._create();

		if ( this.options.disabled ) {
			this._setOptionDisabled( this.options.disabled );
		}

		this._trigger( "create", null, this._getCreateEventData() );
		this._init();
	},

	_getCreateOptions: function() {
		return {};
	},

	_getCreateEventData: $.noop,

	_create: $.noop,

	_init: $.noop,

	destroy: function() {
		var that = this;

		this._destroy();
		$.each( this.classesElementLookup, function( key, value ) {
			that._removeClass( value, key );
		} );

		// We can probably remove the unbind calls in 2.0
		// all event bindings should go through this._on()
		this.element
			.off( this.eventNamespace )
			.removeData( this.widgetFullName );
		this.widget()
			.off( this.eventNamespace )
			.removeAttr( "aria-disabled" );

		// Clean up events and states
		this.bindings.off( this.eventNamespace );
	},

	_destroy: $.noop,

	widget: function() {
		return this.element;
	},

	option: function( key, value ) {
		var options = key;
		var parts;
		var curOption;
		var i;

		if ( arguments.length === 0 ) {

			// Don't return a reference to the internal hash
			return $.widget.extend( {}, this.options );
		}

		if ( typeof key === "string" ) {

			// Handle nested keys, e.g., "foo.bar" => { foo: { bar: ___ } }
			options = {};
			parts = key.split( "." );
			key = parts.shift();
			if ( parts.length ) {
				curOption = options[ key ] = $.widget.extend( {}, this.options[ key ] );
				for ( i = 0; i < parts.length - 1; i++ ) {
					curOption[ parts[ i ] ] = curOption[ parts[ i ] ] || {};
					curOption = curOption[ parts[ i ] ];
				}
				key = parts.pop();
				if ( arguments.length === 1 ) {
					return curOption[ key ] === undefined ? null : curOption[ key ];
				}
				curOption[ key ] = value;
			} else {
				if ( arguments.length === 1 ) {
					return this.options[ key ] === undefined ? null : this.options[ key ];
				}
				options[ key ] = value;
			}
		}

		this._setOptions( options );

		return this;
	},

	_setOptions: function( options ) {
		var key;

		for ( key in options ) {
			this._setOption( key, options[ key ] );
		}

		return this;
	},

	_setOption: function( key, value ) {
		if ( key === "classes" ) {
			this._setOptionClasses( value );
		}

		this.options[ key ] = value;

		if ( key === "disabled" ) {
			this._setOptionDisabled( value );
		}

		return this;
	},

	_setOptionClasses: function( value ) {
		var classKey, elements, currentElements;

		for ( classKey in value ) {
			currentElements = this.classesElementLookup[ classKey ];
			if ( value[ classKey ] === this.options.classes[ classKey ] ||
					!currentElements ||
					!currentElements.length ) {
				continue;
			}

			// We are doing this to create a new jQuery object because the _removeClass() call
			// on the next line is going to destroy the reference to the current elements being
			// tracked. We need to save a copy of this collection so that we can add the new classes
			// below.
			elements = $( currentElements.get() );
			this._removeClass( currentElements, classKey );

			// We don't use _addClass() here, because that uses this.options.classes
			// for generating the string of classes. We want to use the value passed in from
			// _setOption(), this is the new value of the classes option which was passed to
			// _setOption(). We pass this value directly to _classes().
			elements.addClass( this._classes( {
				element: elements,
				keys: classKey,
				classes: value,
				add: true
			} ) );
		}
	},

	_setOptionDisabled: function( value ) {
		this._toggleClass( this.widget(), this.widgetFullName + "-disabled", null, !!value );

		// If the widget is becoming disabled, then nothing is interactive
		if ( value ) {
			this._removeClass( this.hoverable, null, "ui-state-hover" );
			this._removeClass( this.focusable, null, "ui-state-focus" );
		}
	},

	enable: function() {
		return this._setOptions( { disabled: false } );
	},

	disable: function() {
		return this._setOptions( { disabled: true } );
	},

	_classes: function( options ) {
		var full = [];
		var that = this;

		options = $.extend( {
			element: this.element,
			classes: this.options.classes || {}
		}, options );

		function processClassString( classes, checkOption ) {
			var current, i;
			for ( i = 0; i < classes.length; i++ ) {
				current = that.classesElementLookup[ classes[ i ] ] || $();
				if ( options.add ) {
					current = $( $.unique( current.get().concat( options.element.get() ) ) );
				} else {
					current = $( current.not( options.element ).get() );
				}
				that.classesElementLookup[ classes[ i ] ] = current;
				full.push( classes[ i ] );
				if ( checkOption && options.classes[ classes[ i ] ] ) {
					full.push( options.classes[ classes[ i ] ] );
				}
			}
		}

		this._on( options.element, {
			"remove": "_untrackClassesElement"
		} );

		if ( options.keys ) {
			processClassString( options.keys.match( /\S+/g ) || [], true );
		}
		if ( options.extra ) {
			processClassString( options.extra.match( /\S+/g ) || [] );
		}

		return full.join( " " );
	},

	_untrackClassesElement: function( event ) {
		var that = this;
		$.each( that.classesElementLookup, function( key, value ) {
			if ( $.inArray( event.target, value ) !== -1 ) {
				that.classesElementLookup[ key ] = $( value.not( event.target ).get() );
			}
		} );
	},

	_removeClass: function( element, keys, extra ) {
		return this._toggleClass( element, keys, extra, false );
	},

	_addClass: function( element, keys, extra ) {
		return this._toggleClass( element, keys, extra, true );
	},

	_toggleClass: function( element, keys, extra, add ) {
		add = ( typeof add === "boolean" ) ? add : extra;
		var shift = ( typeof element === "string" || element === null ),
			options = {
				extra: shift ? keys : extra,
				keys: shift ? element : keys,
				element: shift ? this.element : element,
				add: add
			};
		options.element.toggleClass( this._classes( options ), add );
		return this;
	},

	_on: function( suppressDisabledCheck, element, handlers ) {
		var delegateElement;
		var instance = this;

		// No suppressDisabledCheck flag, shuffle arguments
		if ( typeof suppressDisabledCheck !== "boolean" ) {
			handlers = element;
			element = suppressDisabledCheck;
			suppressDisabledCheck = false;
		}

		// No element argument, shuffle and use this.element
		if ( !handlers ) {
			handlers = element;
			element = this.element;
			delegateElement = this.widget();
		} else {
			element = delegateElement = $( element );
			this.bindings = this.bindings.add( element );
		}

		$.each( handlers, function( event, handler ) {
			function handlerProxy() {

				// Allow widgets to customize the disabled handling
				// - disabled as an array instead of boolean
				// - disabled class as method for disabling individual parts
				if ( !suppressDisabledCheck &&
						( instance.options.disabled === true ||
						$( this ).hasClass( "ui-state-disabled" ) ) ) {
					return;
				}
				return ( typeof handler === "string" ? instance[ handler ] : handler )
					.apply( instance, arguments );
			}

			// Copy the guid so direct unbinding works
			if ( typeof handler !== "string" ) {
				handlerProxy.guid = handler.guid =
					handler.guid || handlerProxy.guid || $.guid++;
			}

			var match = event.match( /^([\w:-]*)\s*(.*)$/ );
			var eventName = match[ 1 ] + instance.eventNamespace;
			var selector = match[ 2 ];

			if ( selector ) {
				delegateElement.on( eventName, selector, handlerProxy );
			} else {
				element.on( eventName, handlerProxy );
			}
		} );
	},

	_off: function( element, eventName ) {
		eventName = ( eventName || "" ).split( " " ).join( this.eventNamespace + " " ) +
			this.eventNamespace;
		element.off( eventName ).off( eventName );

		// Clear the stack to avoid memory leaks (#10056)
		this.bindings = $( this.bindings.not( element ).get() );
		this.focusable = $( this.focusable.not( element ).get() );
		this.hoverable = $( this.hoverable.not( element ).get() );
	},

	_delay: function( handler, delay ) {
		function handlerProxy() {
			return ( typeof handler === "string" ? instance[ handler ] : handler )
				.apply( instance, arguments );
		}
		var instance = this;
		return setTimeout( handlerProxy, delay || 0 );
	},

	_hoverable: function( element ) {
		this.hoverable = this.hoverable.add( element );
		this._on( element, {
			mouseenter: function( event ) {
				this._addClass( $( event.currentTarget ), null, "ui-state-hover" );
			},
			mouseleave: function( event ) {
				this._removeClass( $( event.currentTarget ), null, "ui-state-hover" );
			}
		} );
	},

	_focusable: function( element ) {
		this.focusable = this.focusable.add( element );
		this._on( element, {
			focusin: function( event ) {
				this._addClass( $( event.currentTarget ), null, "ui-state-focus" );
			},
			focusout: function( event ) {
				this._removeClass( $( event.currentTarget ), null, "ui-state-focus" );
			}
		} );
	},

	_trigger: function( type, event, data ) {
		var prop, orig;
		var callback = this.options[ type ];

		data = data || {};
		event = $.Event( event );
		event.type = ( type === this.widgetEventPrefix ?
			type :
			this.widgetEventPrefix + type ).toLowerCase();

		// The original event may come from any element
		// so we need to reset the target on the new event
		event.target = this.element[ 0 ];

		// Copy original event properties over to the new event
		orig = event.originalEvent;
		if ( orig ) {
			for ( prop in orig ) {
				if ( !( prop in event ) ) {
					event[ prop ] = orig[ prop ];
				}
			}
		}

		this.element.trigger( event, data );
		return !( $.isFunction( callback ) &&
			callback.apply( this.element[ 0 ], [ event ].concat( data ) ) === false ||
			event.isDefaultPrevented() );
	}
};

$.each( { show: "fadeIn", hide: "fadeOut" }, function( method, defaultEffect ) {
	$.Widget.prototype[ "_" + method ] = function( element, options, callback ) {
		if ( typeof options === "string" ) {
			options = { effect: options };
		}

		var hasOptions;
		var effectName = !options ?
			method :
			options === true || typeof options === "number" ?
				defaultEffect :
				options.effect || defaultEffect;

		options = options || {};
		if ( typeof options === "number" ) {
			options = { duration: options };
		}

		hasOptions = !$.isEmptyObject( options );
		options.complete = callback;

		if ( options.delay ) {
			element.delay( options.delay );
		}

		if ( hasOptions && $.effects && $.effects.effect[ effectName ] ) {
			element[ method ]( options );
		} else if ( effectName !== method && element[ effectName ] ) {
			element[ effectName ]( options.duration, options.easing, callback );
		} else {
			element.queue( function( next ) {
				$( this )[ method ]();
				if ( callback ) {
					callback.call( element[ 0 ] );
				}
				next();
			} );
		}
	};
} );

var widget = $.widget;


/*!
 * jQuery UI :data 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */

//>>label: :data Selector
//>>group: Core
//>>description: Selects elements which have data stored under the specified key.
//>>docs: http://api.jqueryui.com/data-selector/


var data = $.extend( $.expr[ ":" ], {
	data: $.expr.createPseudo ?
		$.expr.createPseudo( function( dataName ) {
			return function( elem ) {
				return !!$.data( elem, dataName );
			};
		} ) :

		// Support: jQuery <1.8
		function( elem, i, match ) {
			return !!$.data( elem, match[ 3 ] );
		}
} );

/*!
 * jQuery UI Scroll Parent 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */

//>>label: scrollParent
//>>group: Core
//>>description: Get the closest ancestor element that is scrollable.
//>>docs: http://api.jqueryui.com/scrollParent/



var scrollParent = $.fn.scrollParent = function( includeHidden ) {
	var position = this.css( "position" ),
		excludeStaticParent = position === "absolute",
		overflowRegex = includeHidden ? /(auto|scroll|hidden)/ : /(auto|scroll)/,
		scrollParent = this.parents().filter( function() {
			var parent = $( this );
			if ( excludeStaticParent && parent.css( "position" ) === "static" ) {
				return false;
			}
			return overflowRegex.test( parent.css( "overflow" ) + parent.css( "overflow-y" ) +
				parent.css( "overflow-x" ) );
		} ).eq( 0 );

	return position === "fixed" || !scrollParent.length ?
		$( this[ 0 ].ownerDocument || document ) :
		scrollParent;
};




// This file is deprecated
var ie = $.ui.ie = !!/msie [\w.]+/.exec( navigator.userAgent.toLowerCase() );

/*!
 * jQuery UI Mouse 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */

//>>label: Mouse
//>>group: Widgets
//>>description: Abstracts mouse-based interactions to assist in creating certain widgets.
//>>docs: http://api.jqueryui.com/mouse/



var mouseHandled = false;
$( document ).on( "mouseup", function() {
	mouseHandled = false;
} );

var widgetsMouse = $.widget( "ui.mouse", {
	version: "1.12.1",
	options: {
		cancel: "input, textarea, button, select, option",
		distance: 1,
		delay: 0
	},
	_mouseInit: function() {
		var that = this;

		this.element
			.on( "mousedown." + this.widgetName, function( event ) {
				return that._mouseDown( event );
			} )
			.on( "click." + this.widgetName, function( event ) {
				if ( true === $.data( event.target, that.widgetName + ".preventClickEvent" ) ) {
					$.removeData( event.target, that.widgetName + ".preventClickEvent" );
					event.stopImmediatePropagation();
					return false;
				}
			} );

		this.started = false;
	},

	// TODO: make sure destroying one instance of mouse doesn't mess with
	// other instances of mouse
	_mouseDestroy: function() {
		this.element.off( "." + this.widgetName );
		if ( this._mouseMoveDelegate ) {
			this.document
				.off( "mousemove." + this.widgetName, this._mouseMoveDelegate )
				.off( "mouseup." + this.widgetName, this._mouseUpDelegate );
		}
	},

	_mouseDown: function( event ) {

		// don't let more than one widget handle mouseStart
		if ( mouseHandled ) {
			return;
		}

		this._mouseMoved = false;

		// We may have missed mouseup (out of window)
		( this._mouseStarted && this._mouseUp( event ) );

		this._mouseDownEvent = event;

		var that = this,
			btnIsLeft = ( event.which === 1 ),

			// event.target.nodeName works around a bug in IE 8 with
			// disabled inputs (#7620)
			elIsCancel = ( typeof this.options.cancel === "string" && event.target.nodeName ?
				$( event.target ).closest( this.options.cancel ).length : false );
		if ( !btnIsLeft || elIsCancel || !this._mouseCapture( event ) ) {
			return true;
		}

		this.mouseDelayMet = !this.options.delay;
		if ( !this.mouseDelayMet ) {
			this._mouseDelayTimer = setTimeout( function() {
				that.mouseDelayMet = true;
			}, this.options.delay );
		}

		if ( this._mouseDistanceMet( event ) && this._mouseDelayMet( event ) ) {
			this._mouseStarted = ( this._mouseStart( event ) !== false );
			if ( !this._mouseStarted ) {
				event.preventDefault();
				return true;
			}
		}

		// Click event may never have fired (Gecko & Opera)
		if ( true === $.data( event.target, this.widgetName + ".preventClickEvent" ) ) {
			$.removeData( event.target, this.widgetName + ".preventClickEvent" );
		}

		// These delegates are required to keep context
		this._mouseMoveDelegate = function( event ) {
			return that._mouseMove( event );
		};
		this._mouseUpDelegate = function( event ) {
			return that._mouseUp( event );
		};

		this.document
			.on( "mousemove." + this.widgetName, this._mouseMoveDelegate )
			.on( "mouseup." + this.widgetName, this._mouseUpDelegate );

		event.preventDefault();

		mouseHandled = true;
		return true;
	},

	_mouseMove: function( event ) {

		// Only check for mouseups outside the document if you've moved inside the document
		// at least once. This prevents the firing of mouseup in the case of IE<9, which will
		// fire a mousemove event if content is placed under the cursor. See #7778
		// Support: IE <9
		if ( this._mouseMoved ) {

			// IE mouseup check - mouseup happened when mouse was out of window
			if ( $.ui.ie && ( !document.documentMode || document.documentMode < 9 ) &&
					!event.button ) {
				return this._mouseUp( event );

			// Iframe mouseup check - mouseup occurred in another document
			} else if ( !event.which ) {

				// Support: Safari <=8 - 9
				// Safari sets which to 0 if you press any of the following keys
				// during a drag (#14461)
				if ( event.originalEvent.altKey || event.originalEvent.ctrlKey ||
						event.originalEvent.metaKey || event.originalEvent.shiftKey ) {
					this.ignoreMissingWhich = true;
				} else if ( !this.ignoreMissingWhich ) {
					return this._mouseUp( event );
				}
			}
		}

		if ( event.which || event.button ) {
			this._mouseMoved = true;
		}

		if ( this._mouseStarted ) {
			this._mouseDrag( event );
			return event.preventDefault();
		}

		if ( this._mouseDistanceMet( event ) && this._mouseDelayMet( event ) ) {
			this._mouseStarted =
				( this._mouseStart( this._mouseDownEvent, event ) !== false );
			( this._mouseStarted ? this._mouseDrag( event ) : this._mouseUp( event ) );
		}

		return !this._mouseStarted;
	},

	_mouseUp: function( event ) {
		this.document
			.off( "mousemove." + this.widgetName, this._mouseMoveDelegate )
			.off( "mouseup." + this.widgetName, this._mouseUpDelegate );

		if ( this._mouseStarted ) {
			this._mouseStarted = false;

			if ( event.target === this._mouseDownEvent.target ) {
				$.data( event.target, this.widgetName + ".preventClickEvent", true );
			}

			this._mouseStop( event );
		}

		if ( this._mouseDelayTimer ) {
			clearTimeout( this._mouseDelayTimer );
			delete this._mouseDelayTimer;
		}

		this.ignoreMissingWhich = false;
		mouseHandled = false;
		event.preventDefault();
	},

	_mouseDistanceMet: function( event ) {
		return ( Math.max(
				Math.abs( this._mouseDownEvent.pageX - event.pageX ),
				Math.abs( this._mouseDownEvent.pageY - event.pageY )
			) >= this.options.distance
		);
	},

	_mouseDelayMet: function( /* event */ ) {
		return this.mouseDelayMet;
	},

	// These are placeholder methods, to be overriden by extending plugin
	_mouseStart: function( /* event */ ) {},
	_mouseDrag: function( /* event */ ) {},
	_mouseStop: function( /* event */ ) {},
	_mouseCapture: function( /* event */ ) { return true; }
} );




// $.ui.plugin is deprecated. Use $.widget() extensions instead.
var plugin = $.ui.plugin = {
	add: function( module, option, set ) {
		var i,
			proto = $.ui[ module ].prototype;
		for ( i in set ) {
			proto.plugins[ i ] = proto.plugins[ i ] || [];
			proto.plugins[ i ].push( [ option, set[ i ] ] );
		}
	},
	call: function( instance, name, args, allowDisconnected ) {
		var i,
			set = instance.plugins[ name ];

		if ( !set ) {
			return;
		}

		if ( !allowDisconnected && ( !instance.element[ 0 ].parentNode ||
				instance.element[ 0 ].parentNode.nodeType === 11 ) ) {
			return;
		}

		for ( i = 0; i < set.length; i++ ) {
			if ( instance.options[ set[ i ][ 0 ] ] ) {
				set[ i ][ 1 ].apply( instance.element, args );
			}
		}
	}
};



var safeActiveElement = $.ui.safeActiveElement = function( document ) {
	var activeElement;

	// Support: IE 9 only
	// IE9 throws an "Unspecified error" accessing document.activeElement from an <iframe>
	try {
		activeElement = document.activeElement;
	} catch ( error ) {
		activeElement = document.body;
	}

	// Support: IE 9 - 11 only
	// IE may return null instead of an element
	// Interestingly, this only seems to occur when NOT in an iframe
	if ( !activeElement ) {
		activeElement = document.body;
	}

	// Support: IE 11 only
	// IE11 returns a seemingly empty object in some cases when accessing
	// document.activeElement from an <iframe>
	if ( !activeElement.nodeName ) {
		activeElement = document.body;
	}

	return activeElement;
};



var safeBlur = $.ui.safeBlur = function( element ) {

	// Support: IE9 - 10 only
	// If the <body> is blurred, IE will switch windows, see #9420
	if ( element && element.nodeName.toLowerCase() !== "body" ) {
		$( element ).trigger( "blur" );
	}
};


/*!
 * jQuery UI Draggable 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */

//>>label: Draggable
//>>group: Interactions
//>>description: Enables dragging functionality for any element.
//>>docs: http://api.jqueryui.com/draggable/
//>>demos: http://jqueryui.com/draggable/
//>>css.structure: ../../themes/base/draggable.css



$.widget( "ui.draggable", $.ui.mouse, {
	version: "1.12.1",
	widgetEventPrefix: "drag",
	options: {
		addClasses: true,
		appendTo: "parent",
		axis: false,
		connectToSortable: false,
		containment: false,
		cursor: "auto",
		cursorAt: false,
		grid: false,
		handle: false,
		helper: "original",
		iframeFix: false,
		opacity: false,
		refreshPositions: false,
		revert: false,
		revertDuration: 500,
		scope: "default",
		scroll: true,
		scrollSensitivity: 20,
		scrollSpeed: 20,
		snap: false,
		snapMode: "both",
		snapTolerance: 20,
		stack: false,
		zIndex: false,

		// Callbacks
		drag: null,
		start: null,
		stop: null
	},
	_create: function() {

		if ( this.options.helper === "original" ) {
			this._setPositionRelative();
		}
		if ( this.options.addClasses ) {
			this._addClass( "ui-draggable" );
		}
		this._setHandleClassName();

		this._mouseInit();
	},

	_setOption: function( key, value ) {
		this._super( key, value );
		if ( key === "handle" ) {
			this._removeHandleClassName();
			this._setHandleClassName();
		}
	},

	_destroy: function() {
		if ( ( this.helper || this.element ).is( ".ui-draggable-dragging" ) ) {
			this.destroyOnClear = true;
			return;
		}
		this._removeHandleClassName();
		this._mouseDestroy();
	},

	_mouseCapture: function( event ) {
		var o = this.options;

		// Among others, prevent a drag on a resizable-handle
		if ( this.helper || o.disabled ||
				$( event.target ).closest( ".ui-resizable-handle" ).length > 0 ) {
			return false;
		}

		//Quit if we're not on a valid handle
		this.handle = this._getHandle( event );
		if ( !this.handle ) {
			return false;
		}

		this._blurActiveElement( event );

		this._blockFrames( o.iframeFix === true ? "iframe" : o.iframeFix );

		return true;

	},

	_blockFrames: function( selector ) {
		this.iframeBlocks = this.document.find( selector ).map( function() {
			var iframe = $( this );

			return $( "<div>" )
				.css( "position", "absolute" )
				.appendTo( iframe.parent() )
				.outerWidth( iframe.outerWidth() )
				.outerHeight( iframe.outerHeight() )
				.offset( iframe.offset() )[ 0 ];
		} );
	},

	_unblockFrames: function() {
		if ( this.iframeBlocks ) {
			this.iframeBlocks.remove();
			delete this.iframeBlocks;
		}
	},

	_blurActiveElement: function( event ) {
		var activeElement = $.ui.safeActiveElement( this.document[ 0 ] ),
			target = $( event.target );

		// Don't blur if the event occurred on an element that is within
		// the currently focused element
		// See #10527, #12472
		if ( target.closest( activeElement ).length ) {
			return;
		}

		// Blur any element that currently has focus, see #4261
		$.ui.safeBlur( activeElement );
	},

	_mouseStart: function( event ) {

		var o = this.options;

		//Create and append the visible helper
		this.helper = this._createHelper( event );

		this._addClass( this.helper, "ui-draggable-dragging" );

		//Cache the helper size
		this._cacheHelperProportions();

		//If ddmanager is used for droppables, set the global draggable
		if ( $.ui.ddmanager ) {
			$.ui.ddmanager.current = this;
		}

		/*
		 * - Position generation -
		 * This block generates everything position related - it's the core of draggables.
		 */

		//Cache the margins of the original element
		this._cacheMargins();

		//Store the helper's css position
		this.cssPosition = this.helper.css( "position" );
		this.scrollParent = this.helper.scrollParent( true );
		this.offsetParent = this.helper.offsetParent();
		this.hasFixedAncestor = this.helper.parents().filter( function() {
				return $( this ).css( "position" ) === "fixed";
			} ).length > 0;

		//The element's absolute position on the page minus margins
		this.positionAbs = this.element.offset();
		this._refreshOffsets( event );

		//Generate the original position
		this.originalPosition = this.position = this._generatePosition( event, false );
		this.originalPageX = event.pageX;
		this.originalPageY = event.pageY;

		//Adjust the mouse offset relative to the helper if "cursorAt" is supplied
		( o.cursorAt && this._adjustOffsetFromHelper( o.cursorAt ) );

		//Set a containment if given in the options
		this._setContainment();

		//Trigger event + callbacks
		if ( this._trigger( "start", event ) === false ) {
			this._clear();
			return false;
		}

		//Recache the helper size
		this._cacheHelperProportions();

		//Prepare the droppable offsets
		if ( $.ui.ddmanager && !o.dropBehaviour ) {
			$.ui.ddmanager.prepareOffsets( this, event );
		}

		// Execute the drag once - this causes the helper not to be visible before getting its
		// correct position
		this._mouseDrag( event, true );

		// If the ddmanager is used for droppables, inform the manager that dragging has started
		// (see #5003)
		if ( $.ui.ddmanager ) {
			$.ui.ddmanager.dragStart( this, event );
		}

		return true;
	},

	_refreshOffsets: function( event ) {
		this.offset = {
			top: this.positionAbs.top - this.margins.top,
			left: this.positionAbs.left - this.margins.left,
			scroll: false,
			parent: this._getParentOffset(),
			relative: this._getRelativeOffset()
		};

		this.offset.click = {
			left: event.pageX - this.offset.left,
			top: event.pageY - this.offset.top
		};
	},

	_mouseDrag: function( event, noPropagation ) {

		// reset any necessary cached properties (see #5009)
		if ( this.hasFixedAncestor ) {
			this.offset.parent = this._getParentOffset();
		}

		//Compute the helpers position
		this.position = this._generatePosition( event, true );
		this.positionAbs = this._convertPositionTo( "absolute" );

		//Call plugins and callbacks and use the resulting position if something is returned
		if ( !noPropagation ) {
			var ui = this._uiHash();
			if ( this._trigger( "drag", event, ui ) === false ) {
				this._mouseUp( new $.Event( "mouseup", event ) );
				return false;
			}
			this.position = ui.position;
		}

		this.helper[ 0 ].style.left = this.position.left + "px";
		this.helper[ 0 ].style.top = this.position.top + "px";

		if ( $.ui.ddmanager ) {
			$.ui.ddmanager.drag( this, event );
		}

		return false;
	},

	_mouseStop: function( event ) {

		//If we are using droppables, inform the manager about the drop
		var that = this,
			dropped = false;
		if ( $.ui.ddmanager && !this.options.dropBehaviour ) {
			dropped = $.ui.ddmanager.drop( this, event );
		}

		//if a drop comes from outside (a sortable)
		if ( this.dropped ) {
			dropped = this.dropped;
			this.dropped = false;
		}

		if ( ( this.options.revert === "invalid" && !dropped ) ||
				( this.options.revert === "valid" && dropped ) ||
				this.options.revert === true || ( $.isFunction( this.options.revert ) &&
				this.options.revert.call( this.element, dropped ) )
		) {
			$( this.helper ).animate(
				this.originalPosition,
				parseInt( this.options.revertDuration, 10 ),
				function() {
					if ( that._trigger( "stop", event ) !== false ) {
						that._clear();
					}
				}
			);
		} else {
			if ( this._trigger( "stop", event ) !== false ) {
				this._clear();
			}
		}

		return false;
	},

	_mouseUp: function( event ) {
		this._unblockFrames();

		// If the ddmanager is used for droppables, inform the manager that dragging has stopped
		// (see #5003)
		if ( $.ui.ddmanager ) {
			$.ui.ddmanager.dragStop( this, event );
		}

		// Only need to focus if the event occurred on the draggable itself, see #10527
		if ( this.handleElement.is( event.target ) ) {

			// The interaction is over; whether or not the click resulted in a drag,
			// focus the element
			this.element.trigger( "focus" );
		}

		return $.ui.mouse.prototype._mouseUp.call( this, event );
	},

	cancel: function() {

		if ( this.helper.is( ".ui-draggable-dragging" ) ) {
			this._mouseUp( new $.Event( "mouseup", { target: this.element[ 0 ] } ) );
		} else {
			this._clear();
		}

		return this;

	},

	_getHandle: function( event ) {
		return this.options.handle ?
			!!$( event.target ).closest( this.element.find( this.options.handle ) ).length :
			true;
	},

	_setHandleClassName: function() {
		this.handleElement = this.options.handle ?
			this.element.find( this.options.handle ) : this.element;
		this._addClass( this.handleElement, "ui-draggable-handle" );
	},

	_removeHandleClassName: function() {
		this._removeClass( this.handleElement, "ui-draggable-handle" );
	},

	_createHelper: function( event ) {

		var o = this.options,
			helperIsFunction = $.isFunction( o.helper ),
			helper = helperIsFunction ?
				$( o.helper.apply( this.element[ 0 ], [ event ] ) ) :
				( o.helper === "clone" ?
					this.element.clone().removeAttr( "id" ) :
					this.element );

		if ( !helper.parents( "body" ).length ) {
			helper.appendTo( ( o.appendTo === "parent" ?
				this.element[ 0 ].parentNode :
				o.appendTo ) );
		}

		// Http://bugs.jqueryui.com/ticket/9446
		// a helper function can return the original element
		// which wouldn't have been set to relative in _create
		if ( helperIsFunction && helper[ 0 ] === this.element[ 0 ] ) {
			this._setPositionRelative();
		}

		if ( helper[ 0 ] !== this.element[ 0 ] &&
				!( /(fixed|absolute)/ ).test( helper.css( "position" ) ) ) {
			helper.css( "position", "absolute" );
		}

		return helper;

	},

	_setPositionRelative: function() {
		if ( !( /^(?:r|a|f)/ ).test( this.element.css( "position" ) ) ) {
			this.element[ 0 ].style.position = "relative";
		}
	},

	_adjustOffsetFromHelper: function( obj ) {
		if ( typeof obj === "string" ) {
			obj = obj.split( " " );
		}
		if ( $.isArray( obj ) ) {
			obj = { left: +obj[ 0 ], top: +obj[ 1 ] || 0 };
		}
		if ( "left" in obj ) {
			this.offset.click.left = obj.left + this.margins.left;
		}
		if ( "right" in obj ) {
			this.offset.click.left = this.helperProportions.width - obj.right + this.margins.left;
		}
		if ( "top" in obj ) {
			this.offset.click.top = obj.top + this.margins.top;
		}
		if ( "bottom" in obj ) {
			this.offset.click.top = this.helperProportions.height - obj.bottom + this.margins.top;
		}
	},

	_isRootNode: function( element ) {
		return ( /(html|body)/i ).test( element.tagName ) || element === this.document[ 0 ];
	},

	_getParentOffset: function() {

		//Get the offsetParent and cache its position
		var po = this.offsetParent.offset(),
			document = this.document[ 0 ];

		// This is a special case where we need to modify a offset calculated on start, since the
		// following happened:
		// 1. The position of the helper is absolute, so it's position is calculated based on the
		// next positioned parent
		// 2. The actual offset parent is a child of the scroll parent, and the scroll parent isn't
		// the document, which means that the scroll is included in the initial calculation of the
		// offset of the parent, and never recalculated upon drag
		if ( this.cssPosition === "absolute" && this.scrollParent[ 0 ] !== document &&
				$.contains( this.scrollParent[ 0 ], this.offsetParent[ 0 ] ) ) {
			po.left += this.scrollParent.scrollLeft();
			po.top += this.scrollParent.scrollTop();
		}

		if ( this._isRootNode( this.offsetParent[ 0 ] ) ) {
			po = { top: 0, left: 0 };
		}

		return {
			top: po.top + ( parseInt( this.offsetParent.css( "borderTopWidth" ), 10 ) || 0 ),
			left: po.left + ( parseInt( this.offsetParent.css( "borderLeftWidth" ), 10 ) || 0 )
		};

	},

	_getRelativeOffset: function() {
		if ( this.cssPosition !== "relative" ) {
			return { top: 0, left: 0 };
		}

		var p = this.element.position(),
			scrollIsRootNode = this._isRootNode( this.scrollParent[ 0 ] );

		return {
			top: p.top - ( parseInt( this.helper.css( "top" ), 10 ) || 0 ) +
				( !scrollIsRootNode ? this.scrollParent.scrollTop() : 0 ),
			left: p.left - ( parseInt( this.helper.css( "left" ), 10 ) || 0 ) +
				( !scrollIsRootNode ? this.scrollParent.scrollLeft() : 0 )
		};

	},

	_cacheMargins: function() {
		this.margins = {
			left: ( parseInt( this.element.css( "marginLeft" ), 10 ) || 0 ),
			top: ( parseInt( this.element.css( "marginTop" ), 10 ) || 0 ),
			right: ( parseInt( this.element.css( "marginRight" ), 10 ) || 0 ),
			bottom: ( parseInt( this.element.css( "marginBottom" ), 10 ) || 0 )
		};
	},

	_cacheHelperProportions: function() {
		this.helperProportions = {
			width: this.helper.outerWidth(),
			height: this.helper.outerHeight()
		};
	},

	_setContainment: function() {

		var isUserScrollable, c, ce,
			o = this.options,
			document = this.document[ 0 ];

		this.relativeContainer = null;

		if ( !o.containment ) {
			this.containment = null;
			return;
		}

		if ( o.containment === "window" ) {
			this.containment = [
				$( window ).scrollLeft() - this.offset.relative.left - this.offset.parent.left,
				$( window ).scrollTop() - this.offset.relative.top - this.offset.parent.top,
				$( window ).scrollLeft() + $( window ).width() -
					this.helperProportions.width - this.margins.left,
				$( window ).scrollTop() +
					( $( window ).height() || document.body.parentNode.scrollHeight ) -
					this.helperProportions.height - this.margins.top
			];
			return;
		}

		if ( o.containment === "document" ) {
			this.containment = [
				0,
				0,
				$( document ).width() - this.helperProportions.width - this.margins.left,
				( $( document ).height() || document.body.parentNode.scrollHeight ) -
					this.helperProportions.height - this.margins.top
			];
			return;
		}

		if ( o.containment.constructor === Array ) {
			this.containment = o.containment;
			return;
		}

		if ( o.containment === "parent" ) {
			o.containment = this.helper[ 0 ].parentNode;
		}

		c = $( o.containment );
		ce = c[ 0 ];

		if ( !ce ) {
			return;
		}

		isUserScrollable = /(scroll|auto)/.test( c.css( "overflow" ) );

		this.containment = [
			( parseInt( c.css( "borderLeftWidth" ), 10 ) || 0 ) +
				( parseInt( c.css( "paddingLeft" ), 10 ) || 0 ),
			( parseInt( c.css( "borderTopWidth" ), 10 ) || 0 ) +
				( parseInt( c.css( "paddingTop" ), 10 ) || 0 ),
			( isUserScrollable ? Math.max( ce.scrollWidth, ce.offsetWidth ) : ce.offsetWidth ) -
				( parseInt( c.css( "borderRightWidth" ), 10 ) || 0 ) -
				( parseInt( c.css( "paddingRight" ), 10 ) || 0 ) -
				this.helperProportions.width -
				this.margins.left -
				this.margins.right,
			( isUserScrollable ? Math.max( ce.scrollHeight, ce.offsetHeight ) : ce.offsetHeight ) -
				( parseInt( c.css( "borderBottomWidth" ), 10 ) || 0 ) -
				( parseInt( c.css( "paddingBottom" ), 10 ) || 0 ) -
				this.helperProportions.height -
				this.margins.top -
				this.margins.bottom
		];
		this.relativeContainer = c;
	},

	_convertPositionTo: function( d, pos ) {

		if ( !pos ) {
			pos = this.position;
		}

		var mod = d === "absolute" ? 1 : -1,
			scrollIsRootNode = this._isRootNode( this.scrollParent[ 0 ] );

		return {
			top: (

				// The absolute mouse position
				pos.top	+

				// Only for relative positioned nodes: Relative offset from element to offset parent
				this.offset.relative.top * mod +

				// The offsetParent's offset without borders (offset + border)
				this.offset.parent.top * mod -
				( ( this.cssPosition === "fixed" ?
					-this.offset.scroll.top :
					( scrollIsRootNode ? 0 : this.offset.scroll.top ) ) * mod )
			),
			left: (

				// The absolute mouse position
				pos.left +

				// Only for relative positioned nodes: Relative offset from element to offset parent
				this.offset.relative.left * mod +

				// The offsetParent's offset without borders (offset + border)
				this.offset.parent.left * mod	-
				( ( this.cssPosition === "fixed" ?
					-this.offset.scroll.left :
					( scrollIsRootNode ? 0 : this.offset.scroll.left ) ) * mod )
			)
		};

	},

	_generatePosition: function( event, constrainPosition ) {

		var containment, co, top, left,
			o = this.options,
			scrollIsRootNode = this._isRootNode( this.scrollParent[ 0 ] ),
			pageX = event.pageX,
			pageY = event.pageY;

		// Cache the scroll
		if ( !scrollIsRootNode || !this.offset.scroll ) {
			this.offset.scroll = {
				top: this.scrollParent.scrollTop(),
				left: this.scrollParent.scrollLeft()
			};
		}

		/*
		 * - Position constraining -
		 * Constrain the position to a mix of grid, containment.
		 */

		// If we are not dragging yet, we won't check for options
		if ( constrainPosition ) {
			if ( this.containment ) {
				if ( this.relativeContainer ) {
					co = this.relativeContainer.offset();
					containment = [
						this.containment[ 0 ] + co.left,
						this.containment[ 1 ] + co.top,
						this.containment[ 2 ] + co.left,
						this.containment[ 3 ] + co.top
					];
				} else {
					containment = this.containment;
				}

				if ( event.pageX - this.offset.click.left < containment[ 0 ] ) {
					pageX = containment[ 0 ] + this.offset.click.left;
				}
				if ( event.pageY - this.offset.click.top < containment[ 1 ] ) {
					pageY = containment[ 1 ] + this.offset.click.top;
				}
				if ( event.pageX - this.offset.click.left > containment[ 2 ] ) {
					pageX = containment[ 2 ] + this.offset.click.left;
				}
				if ( event.pageY - this.offset.click.top > containment[ 3 ] ) {
					pageY = containment[ 3 ] + this.offset.click.top;
				}
			}

			if ( o.grid ) {

				//Check for grid elements set to 0 to prevent divide by 0 error causing invalid
				// argument errors in IE (see ticket #6950)
				top = o.grid[ 1 ] ? this.originalPageY + Math.round( ( pageY -
					this.originalPageY ) / o.grid[ 1 ] ) * o.grid[ 1 ] : this.originalPageY;
				pageY = containment ? ( ( top - this.offset.click.top >= containment[ 1 ] ||
					top - this.offset.click.top > containment[ 3 ] ) ?
						top :
						( ( top - this.offset.click.top >= containment[ 1 ] ) ?
							top - o.grid[ 1 ] : top + o.grid[ 1 ] ) ) : top;

				left = o.grid[ 0 ] ? this.originalPageX +
					Math.round( ( pageX - this.originalPageX ) / o.grid[ 0 ] ) * o.grid[ 0 ] :
					this.originalPageX;
				pageX = containment ? ( ( left - this.offset.click.left >= containment[ 0 ] ||
					left - this.offset.click.left > containment[ 2 ] ) ?
						left :
						( ( left - this.offset.click.left >= containment[ 0 ] ) ?
							left - o.grid[ 0 ] : left + o.grid[ 0 ] ) ) : left;
			}

			if ( o.axis === "y" ) {
				pageX = this.originalPageX;
			}

			if ( o.axis === "x" ) {
				pageY = this.originalPageY;
			}
		}

		return {
			top: (

				// The absolute mouse position
				pageY -

				// Click offset (relative to the element)
				this.offset.click.top -

				// Only for relative positioned nodes: Relative offset from element to offset parent
				this.offset.relative.top -

				// The offsetParent's offset without borders (offset + border)
				this.offset.parent.top +
				( this.cssPosition === "fixed" ?
					-this.offset.scroll.top :
					( scrollIsRootNode ? 0 : this.offset.scroll.top ) )
			),
			left: (

				// The absolute mouse position
				pageX -

				// Click offset (relative to the element)
				this.offset.click.left -

				// Only for relative positioned nodes: Relative offset from element to offset parent
				this.offset.relative.left -

				// The offsetParent's offset without borders (offset + border)
				this.offset.parent.left +
				( this.cssPosition === "fixed" ?
					-this.offset.scroll.left :
					( scrollIsRootNode ? 0 : this.offset.scroll.left ) )
			)
		};

	},

	_clear: function() {
		this._removeClass( this.helper, "ui-draggable-dragging" );
		if ( this.helper[ 0 ] !== this.element[ 0 ] && !this.cancelHelperRemoval ) {
			this.helper.remove();
		}
		this.helper = null;
		this.cancelHelperRemoval = false;
		if ( this.destroyOnClear ) {
			this.destroy();
		}
	},

	// From now on bulk stuff - mainly helpers

	_trigger: function( type, event, ui ) {
		ui = ui || this._uiHash();
		$.ui.plugin.call( this, type, [ event, ui, this ], true );

		// Absolute position and offset (see #6884 ) have to be recalculated after plugins
		if ( /^(drag|start|stop)/.test( type ) ) {
			this.positionAbs = this._convertPositionTo( "absolute" );
			ui.offset = this.positionAbs;
		}
		return $.Widget.prototype._trigger.call( this, type, event, ui );
	},

	plugins: {},

	_uiHash: function() {
		return {
			helper: this.helper,
			position: this.position,
			originalPosition: this.originalPosition,
			offset: this.positionAbs
		};
	}

} );

$.ui.plugin.add( "draggable", "connectToSortable", {
	start: function( event, ui, draggable ) {
		var uiSortable = $.extend( {}, ui, {
			item: draggable.element
		} );

		draggable.sortables = [];
		$( draggable.options.connectToSortable ).each( function() {
			var sortable = $( this ).sortable( "instance" );

			if ( sortable && !sortable.options.disabled ) {
				draggable.sortables.push( sortable );

				// RefreshPositions is called at drag start to refresh the containerCache
				// which is used in drag. This ensures it's initialized and synchronized
				// with any changes that might have happened on the page since initialization.
				sortable.refreshPositions();
				sortable._trigger( "activate", event, uiSortable );
			}
		} );
	},
	stop: function( event, ui, draggable ) {
		var uiSortable = $.extend( {}, ui, {
			item: draggable.element
		} );

		draggable.cancelHelperRemoval = false;

		$.each( draggable.sortables, function() {
			var sortable = this;

			if ( sortable.isOver ) {
				sortable.isOver = 0;

				// Allow this sortable to handle removing the helper
				draggable.cancelHelperRemoval = true;
				sortable.cancelHelperRemoval = false;

				// Use _storedCSS To restore properties in the sortable,
				// as this also handles revert (#9675) since the draggable
				// may have modified them in unexpected ways (#8809)
				sortable._storedCSS = {
					position: sortable.placeholder.css( "position" ),
					top: sortable.placeholder.css( "top" ),
					left: sortable.placeholder.css( "left" )
				};

				sortable._mouseStop( event );

				// Once drag has ended, the sortable should return to using
				// its original helper, not the shared helper from draggable
				sortable.options.helper = sortable.options._helper;
			} else {

				// Prevent this Sortable from removing the helper.
				// However, don't set the draggable to remove the helper
				// either as another connected Sortable may yet handle the removal.
				sortable.cancelHelperRemoval = true;

				sortable._trigger( "deactivate", event, uiSortable );
			}
		} );
	},
	drag: function( event, ui, draggable ) {
		$.each( draggable.sortables, function() {
			var innermostIntersecting = false,
				sortable = this;

			// Copy over variables that sortable's _intersectsWith uses
			sortable.positionAbs = draggable.positionAbs;
			sortable.helperProportions = draggable.helperProportions;
			sortable.offset.click = draggable.offset.click;

			if ( sortable._intersectsWith( sortable.containerCache ) ) {
				innermostIntersecting = true;

				$.each( draggable.sortables, function() {

					// Copy over variables that sortable's _intersectsWith uses
					this.positionAbs = draggable.positionAbs;
					this.helperProportions = draggable.helperProportions;
					this.offset.click = draggable.offset.click;

					if ( this !== sortable &&
							this._intersectsWith( this.containerCache ) &&
							$.contains( sortable.element[ 0 ], this.element[ 0 ] ) ) {
						innermostIntersecting = false;
					}

					return innermostIntersecting;
				} );
			}

			if ( innermostIntersecting ) {

				// If it intersects, we use a little isOver variable and set it once,
				// so that the move-in stuff gets fired only once.
				if ( !sortable.isOver ) {
					sortable.isOver = 1;

					// Store draggable's parent in case we need to reappend to it later.
					draggable._parent = ui.helper.parent();

					sortable.currentItem = ui.helper
						.appendTo( sortable.element )
						.data( "ui-sortable-item", true );

					// Store helper option to later restore it
					sortable.options._helper = sortable.options.helper;

					sortable.options.helper = function() {
						return ui.helper[ 0 ];
					};

					// Fire the start events of the sortable with our passed browser event,
					// and our own helper (so it doesn't create a new one)
					event.target = sortable.currentItem[ 0 ];
					sortable._mouseCapture( event, true );
					sortable._mouseStart( event, true, true );

					// Because the browser event is way off the new appended portlet,
					// modify necessary variables to reflect the changes
					sortable.offset.click.top = draggable.offset.click.top;
					sortable.offset.click.left = draggable.offset.click.left;
					sortable.offset.parent.left -= draggable.offset.parent.left -
						sortable.offset.parent.left;
					sortable.offset.parent.top -= draggable.offset.parent.top -
						sortable.offset.parent.top;

					draggable._trigger( "toSortable", event );

					// Inform draggable that the helper is in a valid drop zone,
					// used solely in the revert option to handle "valid/invalid".
					draggable.dropped = sortable.element;

					// Need to refreshPositions of all sortables in the case that
					// adding to one sortable changes the location of the other sortables (#9675)
					$.each( draggable.sortables, function() {
						this.refreshPositions();
					} );

					// Hack so receive/update callbacks work (mostly)
					draggable.currentItem = draggable.element;
					sortable.fromOutside = draggable;
				}

				if ( sortable.currentItem ) {
					sortable._mouseDrag( event );

					// Copy the sortable's position because the draggable's can potentially reflect
					// a relative position, while sortable is always absolute, which the dragged
					// element has now become. (#8809)
					ui.position = sortable.position;
				}
			} else {

				// If it doesn't intersect with the sortable, and it intersected before,
				// we fake the drag stop of the sortable, but make sure it doesn't remove
				// the helper by using cancelHelperRemoval.
				if ( sortable.isOver ) {

					sortable.isOver = 0;
					sortable.cancelHelperRemoval = true;

					// Calling sortable's mouseStop would trigger a revert,
					// so revert must be temporarily false until after mouseStop is called.
					sortable.options._revert = sortable.options.revert;
					sortable.options.revert = false;

					sortable._trigger( "out", event, sortable._uiHash( sortable ) );
					sortable._mouseStop( event, true );

					// Restore sortable behaviors that were modfied
					// when the draggable entered the sortable area (#9481)
					sortable.options.revert = sortable.options._revert;
					sortable.options.helper = sortable.options._helper;

					if ( sortable.placeholder ) {
						sortable.placeholder.remove();
					}

					// Restore and recalculate the draggable's offset considering the sortable
					// may have modified them in unexpected ways. (#8809, #10669)
					ui.helper.appendTo( draggable._parent );
					draggable._refreshOffsets( event );
					ui.position = draggable._generatePosition( event, true );

					draggable._trigger( "fromSortable", event );

					// Inform draggable that the helper is no longer in a valid drop zone
					draggable.dropped = false;

					// Need to refreshPositions of all sortables just in case removing
					// from one sortable changes the location of other sortables (#9675)
					$.each( draggable.sortables, function() {
						this.refreshPositions();
					} );
				}
			}
		} );
	}
} );

$.ui.plugin.add( "draggable", "cursor", {
	start: function( event, ui, instance ) {
		var t = $( "body" ),
			o = instance.options;

		if ( t.css( "cursor" ) ) {
			o._cursor = t.css( "cursor" );
		}
		t.css( "cursor", o.cursor );
	},
	stop: function( event, ui, instance ) {
		var o = instance.options;
		if ( o._cursor ) {
			$( "body" ).css( "cursor", o._cursor );
		}
	}
} );

$.ui.plugin.add( "draggable", "opacity", {
	start: function( event, ui, instance ) {
		var t = $( ui.helper ),
			o = instance.options;
		if ( t.css( "opacity" ) ) {
			o._opacity = t.css( "opacity" );
		}
		t.css( "opacity", o.opacity );
	},
	stop: function( event, ui, instance ) {
		var o = instance.options;
		if ( o._opacity ) {
			$( ui.helper ).css( "opacity", o._opacity );
		}
	}
} );

$.ui.plugin.add( "draggable", "scroll", {
	start: function( event, ui, i ) {
		if ( !i.scrollParentNotHidden ) {
			i.scrollParentNotHidden = i.helper.scrollParent( false );
		}

		if ( i.scrollParentNotHidden[ 0 ] !== i.document[ 0 ] &&
				i.scrollParentNotHidden[ 0 ].tagName !== "HTML" ) {
			i.overflowOffset = i.scrollParentNotHidden.offset();
		}
	},
	drag: function( event, ui, i  ) {

		var o = i.options,
			scrolled = false,
			scrollParent = i.scrollParentNotHidden[ 0 ],
			document = i.document[ 0 ];

		if ( scrollParent !== document && scrollParent.tagName !== "HTML" ) {
			if ( !o.axis || o.axis !== "x" ) {
				if ( ( i.overflowOffset.top + scrollParent.offsetHeight ) - event.pageY <
						o.scrollSensitivity ) {
					scrollParent.scrollTop = scrolled = scrollParent.scrollTop + o.scrollSpeed;
				} else if ( event.pageY - i.overflowOffset.top < o.scrollSensitivity ) {
					scrollParent.scrollTop = scrolled = scrollParent.scrollTop - o.scrollSpeed;
				}
			}

			if ( !o.axis || o.axis !== "y" ) {
				if ( ( i.overflowOffset.left + scrollParent.offsetWidth ) - event.pageX <
						o.scrollSensitivity ) {
					scrollParent.scrollLeft = scrolled = scrollParent.scrollLeft + o.scrollSpeed;
				} else if ( event.pageX - i.overflowOffset.left < o.scrollSensitivity ) {
					scrollParent.scrollLeft = scrolled = scrollParent.scrollLeft - o.scrollSpeed;
				}
			}

		} else {

			if ( !o.axis || o.axis !== "x" ) {
				if ( event.pageY - $( document ).scrollTop() < o.scrollSensitivity ) {
					scrolled = $( document ).scrollTop( $( document ).scrollTop() - o.scrollSpeed );
				} else if ( $( window ).height() - ( event.pageY - $( document ).scrollTop() ) <
						o.scrollSensitivity ) {
					scrolled = $( document ).scrollTop( $( document ).scrollTop() + o.scrollSpeed );
				}
			}

			if ( !o.axis || o.axis !== "y" ) {
				if ( event.pageX - $( document ).scrollLeft() < o.scrollSensitivity ) {
					scrolled = $( document ).scrollLeft(
						$( document ).scrollLeft() - o.scrollSpeed
					);
				} else if ( $( window ).width() - ( event.pageX - $( document ).scrollLeft() ) <
						o.scrollSensitivity ) {
					scrolled = $( document ).scrollLeft(
						$( document ).scrollLeft() + o.scrollSpeed
					);
				}
			}

		}

		if ( scrolled !== false && $.ui.ddmanager && !o.dropBehaviour ) {
			$.ui.ddmanager.prepareOffsets( i, event );
		}

	}
} );

$.ui.plugin.add( "draggable", "snap", {
	start: function( event, ui, i ) {

		var o = i.options;

		i.snapElements = [];

		$( o.snap.constructor !== String ? ( o.snap.items || ":data(ui-draggable)" ) : o.snap )
			.each( function() {
				var $t = $( this ),
					$o = $t.offset();
				if ( this !== i.element[ 0 ] ) {
					i.snapElements.push( {
						item: this,
						width: $t.outerWidth(), height: $t.outerHeight(),
						top: $o.top, left: $o.left
					} );
				}
			} );

	},
	drag: function( event, ui, inst ) {

		var ts, bs, ls, rs, l, r, t, b, i, first,
			o = inst.options,
			d = o.snapTolerance,
			x1 = ui.offset.left, x2 = x1 + inst.helperProportions.width,
			y1 = ui.offset.top, y2 = y1 + inst.helperProportions.height;

		for ( i = inst.snapElements.length - 1; i >= 0; i-- ) {

			l = inst.snapElements[ i ].left - inst.margins.left;
			r = l + inst.snapElements[ i ].width;
			t = inst.snapElements[ i ].top - inst.margins.top;
			b = t + inst.snapElements[ i ].height;

			if ( x2 < l - d || x1 > r + d || y2 < t - d || y1 > b + d ||
					!$.contains( inst.snapElements[ i ].item.ownerDocument,
					inst.snapElements[ i ].item ) ) {
				if ( inst.snapElements[ i ].snapping ) {
					( inst.options.snap.release &&
						inst.options.snap.release.call(
							inst.element,
							event,
							$.extend( inst._uiHash(), { snapItem: inst.snapElements[ i ].item } )
						) );
				}
				inst.snapElements[ i ].snapping = false;
				continue;
			}

			if ( o.snapMode !== "inner" ) {
				ts = Math.abs( t - y2 ) <= d;
				bs = Math.abs( b - y1 ) <= d;
				ls = Math.abs( l - x2 ) <= d;
				rs = Math.abs( r - x1 ) <= d;
				if ( ts ) {
					ui.position.top = inst._convertPositionTo( "relative", {
						top: t - inst.helperProportions.height,
						left: 0
					} ).top;
				}
				if ( bs ) {
					ui.position.top = inst._convertPositionTo( "relative", {
						top: b,
						left: 0
					} ).top;
				}
				if ( ls ) {
					ui.position.left = inst._convertPositionTo( "relative", {
						top: 0,
						left: l - inst.helperProportions.width
					} ).left;
				}
				if ( rs ) {
					ui.position.left = inst._convertPositionTo( "relative", {
						top: 0,
						left: r
					} ).left;
				}
			}

			first = ( ts || bs || ls || rs );

			if ( o.snapMode !== "outer" ) {
				ts = Math.abs( t - y1 ) <= d;
				bs = Math.abs( b - y2 ) <= d;
				ls = Math.abs( l - x1 ) <= d;
				rs = Math.abs( r - x2 ) <= d;
				if ( ts ) {
					ui.position.top = inst._convertPositionTo( "relative", {
						top: t,
						left: 0
					} ).top;
				}
				if ( bs ) {
					ui.position.top = inst._convertPositionTo( "relative", {
						top: b - inst.helperProportions.height,
						left: 0
					} ).top;
				}
				if ( ls ) {
					ui.position.left = inst._convertPositionTo( "relative", {
						top: 0,
						left: l
					} ).left;
				}
				if ( rs ) {
					ui.position.left = inst._convertPositionTo( "relative", {
						top: 0,
						left: r - inst.helperProportions.width
					} ).left;
				}
			}

			if ( !inst.snapElements[ i ].snapping && ( ts || bs || ls || rs || first ) ) {
				( inst.options.snap.snap &&
					inst.options.snap.snap.call(
						inst.element,
						event,
						$.extend( inst._uiHash(), {
							snapItem: inst.snapElements[ i ].item
						} ) ) );
			}
			inst.snapElements[ i ].snapping = ( ts || bs || ls || rs || first );

		}

	}
} );

$.ui.plugin.add( "draggable", "stack", {
	start: function( event, ui, instance ) {
		var min,
			o = instance.options,
			group = $.makeArray( $( o.stack ) ).sort( function( a, b ) {
				return ( parseInt( $( a ).css( "zIndex" ), 10 ) || 0 ) -
					( parseInt( $( b ).css( "zIndex" ), 10 ) || 0 );
			} );

		if ( !group.length ) { return; }

		min = parseInt( $( group[ 0 ] ).css( "zIndex" ), 10 ) || 0;
		$( group ).each( function( i ) {
			$( this ).css( "zIndex", min + i );
		} );
		this.css( "zIndex", ( min + group.length ) );
	}
} );

$.ui.plugin.add( "draggable", "zIndex", {
	start: function( event, ui, instance ) {
		var t = $( ui.helper ),
			o = instance.options;

		if ( t.css( "zIndex" ) ) {
			o._zIndex = t.css( "zIndex" );
		}
		t.css( "zIndex", o.zIndex );
	},
	stop: function( event, ui, instance ) {
		var o = instance.options;

		if ( o._zIndex ) {
			$( ui.helper ).css( "zIndex", o._zIndex );
		}
	}
} );

var widgetsDraggable = $.ui.draggable;


/*!
 * jQuery UI Droppable 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */

//>>label: Droppable
//>>group: Interactions
//>>description: Enables drop targets for draggable elements.
//>>docs: http://api.jqueryui.com/droppable/
//>>demos: http://jqueryui.com/droppable/



$.widget( "ui.droppable", {
	version: "1.12.1",
	widgetEventPrefix: "drop",
	options: {
		accept: "*",
		addClasses: true,
		greedy: false,
		scope: "default",
		tolerance: "intersect",

		// Callbacks
		activate: null,
		deactivate: null,
		drop: null,
		out: null,
		over: null
	},
	_create: function() {

		var proportions,
			o = this.options,
			accept = o.accept;

		this.isover = false;
		this.isout = true;

		this.accept = $.isFunction( accept ) ? accept : function( d ) {
			return d.is( accept );
		};

		this.proportions = function( /* valueToWrite */ ) {
			if ( arguments.length ) {

				// Store the droppable's proportions
				proportions = arguments[ 0 ];
			} else {

				// Retrieve or derive the droppable's proportions
				return proportions ?
					proportions :
					proportions = {
						width: this.element[ 0 ].offsetWidth,
						height: this.element[ 0 ].offsetHeight
					};
			}
		};

		this._addToManager( o.scope );

		o.addClasses && this._addClass( "ui-droppable" );

	},

	_addToManager: function( scope ) {

		// Add the reference and positions to the manager
		$.ui.ddmanager.droppables[ scope ] = $.ui.ddmanager.droppables[ scope ] || [];
		$.ui.ddmanager.droppables[ scope ].push( this );
	},

	_splice: function( drop ) {
		var i = 0;
		for ( ; i < drop.length; i++ ) {
			if ( drop[ i ] === this ) {
				drop.splice( i, 1 );
			}
		}
	},

	_destroy: function() {
		var drop = $.ui.ddmanager.droppables[ this.options.scope ];

		this._splice( drop );
	},

	_setOption: function( key, value ) {

		if ( key === "accept" ) {
			this.accept = $.isFunction( value ) ? value : function( d ) {
				return d.is( value );
			};
		} else if ( key === "scope" ) {
			var drop = $.ui.ddmanager.droppables[ this.options.scope ];

			this._splice( drop );
			this._addToManager( value );
		}

		this._super( key, value );
	},

	_activate: function( event ) {
		var draggable = $.ui.ddmanager.current;

		this._addActiveClass();
		if ( draggable ) {
			this._trigger( "activate", event, this.ui( draggable ) );
		}
	},

	_deactivate: function( event ) {
		var draggable = $.ui.ddmanager.current;

		this._removeActiveClass();
		if ( draggable ) {
			this._trigger( "deactivate", event, this.ui( draggable ) );
		}
	},

	_over: function( event ) {

		var draggable = $.ui.ddmanager.current;

		// Bail if draggable and droppable are same element
		if ( !draggable || ( draggable.currentItem ||
				draggable.element )[ 0 ] === this.element[ 0 ] ) {
			return;
		}

		if ( this.accept.call( this.element[ 0 ], ( draggable.currentItem ||
				draggable.element ) ) ) {
			this._addHoverClass();
			this._trigger( "over", event, this.ui( draggable ) );
		}

	},

	_out: function( event ) {

		var draggable = $.ui.ddmanager.current;

		// Bail if draggable and droppable are same element
		if ( !draggable || ( draggable.currentItem ||
				draggable.element )[ 0 ] === this.element[ 0 ] ) {
			return;
		}

		if ( this.accept.call( this.element[ 0 ], ( draggable.currentItem ||
				draggable.element ) ) ) {
			this._removeHoverClass();
			this._trigger( "out", event, this.ui( draggable ) );
		}

	},

	_drop: function( event, custom ) {

		var draggable = custom || $.ui.ddmanager.current,
			childrenIntersection = false;

		// Bail if draggable and droppable are same element
		if ( !draggable || ( draggable.currentItem ||
				draggable.element )[ 0 ] === this.element[ 0 ] ) {
			return false;
		}

		this.element
			.find( ":data(ui-droppable)" )
			.not( ".ui-draggable-dragging" )
			.each( function() {
				var inst = $( this ).droppable( "instance" );
				if (
					inst.options.greedy &&
					!inst.options.disabled &&
					inst.options.scope === draggable.options.scope &&
					inst.accept.call(
						inst.element[ 0 ], ( draggable.currentItem || draggable.element )
					) &&
					intersect(
						draggable,
						$.extend( inst, { offset: inst.element.offset() } ),
						inst.options.tolerance, event
					)
				) {
					childrenIntersection = true;
					return false; }
			} );
		if ( childrenIntersection ) {
			return false;
		}

		if ( this.accept.call( this.element[ 0 ],
				( draggable.currentItem || draggable.element ) ) ) {
			this._removeActiveClass();
			this._removeHoverClass();

			this._trigger( "drop", event, this.ui( draggable ) );
			return this.element;
		}

		return false;

	},

	ui: function( c ) {
		return {
			draggable: ( c.currentItem || c.element ),
			helper: c.helper,
			position: c.position,
			offset: c.positionAbs
		};
	},

	// Extension points just to make backcompat sane and avoid duplicating logic
	// TODO: Remove in 1.13 along with call to it below
	_addHoverClass: function() {
		this._addClass( "ui-droppable-hover" );
	},

	_removeHoverClass: function() {
		this._removeClass( "ui-droppable-hover" );
	},

	_addActiveClass: function() {
		this._addClass( "ui-droppable-active" );
	},

	_removeActiveClass: function() {
		this._removeClass( "ui-droppable-active" );
	}
} );

var intersect = $.ui.intersect = ( function() {
	function isOverAxis( x, reference, size ) {
		return ( x >= reference ) && ( x < ( reference + size ) );
	}

	return function( draggable, droppable, toleranceMode, event ) {

		if ( !droppable.offset ) {
			return false;
		}

		var x1 = ( draggable.positionAbs ||
				draggable.position.absolute ).left + draggable.margins.left,
			y1 = ( draggable.positionAbs ||
				draggable.position.absolute ).top + draggable.margins.top,
			x2 = x1 + draggable.helperProportions.width,
			y2 = y1 + draggable.helperProportions.height,
			l = droppable.offset.left,
			t = droppable.offset.top,
			r = l + droppable.proportions().width,
			b = t + droppable.proportions().height;

		switch ( toleranceMode ) {
		case "fit":
			return ( l <= x1 && x2 <= r && t <= y1 && y2 <= b );
		case "intersect":
			return ( l < x1 + ( draggable.helperProportions.width / 2 ) && // Right Half
				x2 - ( draggable.helperProportions.width / 2 ) < r && // Left Half
				t < y1 + ( draggable.helperProportions.height / 2 ) && // Bottom Half
				y2 - ( draggable.helperProportions.height / 2 ) < b ); // Top Half
		case "pointer":
			return isOverAxis( event.pageY, t, droppable.proportions().height ) &&
				isOverAxis( event.pageX, l, droppable.proportions().width );
		case "touch":
			return (
				( y1 >= t && y1 <= b ) || // Top edge touching
				( y2 >= t && y2 <= b ) || // Bottom edge touching
				( y1 < t && y2 > b ) // Surrounded vertically
			) && (
				( x1 >= l && x1 <= r ) || // Left edge touching
				( x2 >= l && x2 <= r ) || // Right edge touching
				( x1 < l && x2 > r ) // Surrounded horizontally
			);
		default:
			return false;
		}
	};
} )();

/*
	This manager tracks offsets of draggables and droppables
*/
$.ui.ddmanager = {
	current: null,
	droppables: { "default": [] },
	prepareOffsets: function( t, event ) {

		var i, j,
			m = $.ui.ddmanager.droppables[ t.options.scope ] || [],
			type = event ? event.type : null, // workaround for #2317
			list = ( t.currentItem || t.element ).find( ":data(ui-droppable)" ).addBack();

		droppablesLoop: for ( i = 0; i < m.length; i++ ) {

			// No disabled and non-accepted
			if ( m[ i ].options.disabled || ( t && !m[ i ].accept.call( m[ i ].element[ 0 ],
					( t.currentItem || t.element ) ) ) ) {
				continue;
			}

			// Filter out elements in the current dragged item
			for ( j = 0; j < list.length; j++ ) {
				if ( list[ j ] === m[ i ].element[ 0 ] ) {
					m[ i ].proportions().height = 0;
					continue droppablesLoop;
				}
			}

			m[ i ].visible = m[ i ].element.css( "display" ) !== "none";
			if ( !m[ i ].visible ) {
				continue;
			}

			// Activate the droppable if used directly from draggables
			if ( type === "mousedown" ) {
				m[ i ]._activate.call( m[ i ], event );
			}

			m[ i ].offset = m[ i ].element.offset();
			m[ i ].proportions( {
				width: m[ i ].element[ 0 ].offsetWidth,
				height: m[ i ].element[ 0 ].offsetHeight
			} );

		}

	},
	drop: function( draggable, event ) {

		var dropped = false;

		// Create a copy of the droppables in case the list changes during the drop (#9116)
		$.each( ( $.ui.ddmanager.droppables[ draggable.options.scope ] || [] ).slice(), function() {

			if ( !this.options ) {
				return;
			}
			if ( !this.options.disabled && this.visible &&
					intersect( draggable, this, this.options.tolerance, event ) ) {
				dropped = this._drop.call( this, event ) || dropped;
			}

			if ( !this.options.disabled && this.visible && this.accept.call( this.element[ 0 ],
					( draggable.currentItem || draggable.element ) ) ) {
				this.isout = true;
				this.isover = false;
				this._deactivate.call( this, event );
			}

		} );
		return dropped;

	},
	dragStart: function( draggable, event ) {

		// Listen for scrolling so that if the dragging causes scrolling the position of the
		// droppables can be recalculated (see #5003)
		draggable.element.parentsUntil( "body" ).on( "scroll.droppable", function() {
			if ( !draggable.options.refreshPositions ) {
				$.ui.ddmanager.prepareOffsets( draggable, event );
			}
		} );
	},
	drag: function( draggable, event ) {

		// If you have a highly dynamic page, you might try this option. It renders positions
		// every time you move the mouse.
		if ( draggable.options.refreshPositions ) {
			$.ui.ddmanager.prepareOffsets( draggable, event );
		}

		// Run through all droppables and check their positions based on specific tolerance options
		$.each( $.ui.ddmanager.droppables[ draggable.options.scope ] || [], function() {

			if ( this.options.disabled || this.greedyChild || !this.visible ) {
				return;
			}

			var parentInstance, scope, parent,
				intersects = intersect( draggable, this, this.options.tolerance, event ),
				c = !intersects && this.isover ?
					"isout" :
					( intersects && !this.isover ? "isover" : null );
			if ( !c ) {
				return;
			}

			if ( this.options.greedy ) {

				// find droppable parents with same scope
				scope = this.options.scope;
				parent = this.element.parents( ":data(ui-droppable)" ).filter( function() {
					return $( this ).droppable( "instance" ).options.scope === scope;
				} );

				if ( parent.length ) {
					parentInstance = $( parent[ 0 ] ).droppable( "instance" );
					parentInstance.greedyChild = ( c === "isover" );
				}
			}

			// We just moved into a greedy child
			if ( parentInstance && c === "isover" ) {
				parentInstance.isover = false;
				parentInstance.isout = true;
				parentInstance._out.call( parentInstance, event );
			}

			this[ c ] = true;
			this[ c === "isout" ? "isover" : "isout" ] = false;
			this[ c === "isover" ? "_over" : "_out" ].call( this, event );

			// We just moved out of a greedy child
			if ( parentInstance && c === "isout" ) {
				parentInstance.isout = false;
				parentInstance.isover = true;
				parentInstance._over.call( parentInstance, event );
			}
		} );

	},
	dragStop: function( draggable, event ) {
		draggable.element.parentsUntil( "body" ).off( "scroll.droppable" );

		// Call prepareOffsets one final time since IE does not fire return scroll events when
		// overflow was caused by drag (see #5003)
		if ( !draggable.options.refreshPositions ) {
			$.ui.ddmanager.prepareOffsets( draggable, event );
		}
	}
};

// DEPRECATED
// TODO: switch return back to widget declaration at top of file when this is removed
if ( $.uiBackCompat !== false ) {

	// Backcompat for activeClass and hoverClass options
	$.widget( "ui.droppable", $.ui.droppable, {
		options: {
			hoverClass: false,
			activeClass: false
		},
		_addActiveClass: function() {
			this._super();
			if ( this.options.activeClass ) {
				this.element.addClass( this.options.activeClass );
			}
		},
		_removeActiveClass: function() {
			this._super();
			if ( this.options.activeClass ) {
				this.element.removeClass( this.options.activeClass );
			}
		},
		_addHoverClass: function() {
			this._super();
			if ( this.options.hoverClass ) {
				this.element.addClass( this.options.hoverClass );
			}
		},
		_removeHoverClass: function() {
			this._super();
			if ( this.options.hoverClass ) {
				this.element.removeClass( this.options.hoverClass );
			}
		}
	} );
}

var widgetsDroppable = $.ui.droppable;


/*!
 * jQuery UI Sortable 1.12.1
 * http://jqueryui.com
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 */

//>>label: Sortable
//>>group: Interactions
//>>description: Enables items in a list to be sorted using the mouse.
//>>docs: http://api.jqueryui.com/sortable/
//>>demos: http://jqueryui.com/sortable/
//>>css.structure: ../../themes/base/sortable.css



var widgetsSortable = $.widget( "ui.sortable", $.ui.mouse, {
	version: "1.12.1",
	widgetEventPrefix: "sort",
	ready: false,
	options: {
		appendTo: "parent",
		axis: false,
		connectWith: false,
		containment: false,
		cursor: "auto",
		cursorAt: false,
		dropOnEmpty: true,
		forcePlaceholderSize: false,
		forceHelperSize: false,
		grid: false,
		handle: false,
		helper: "original",
		items: "> *",
		opacity: false,
		placeholder: false,
		revert: false,
		scroll: true,
		scrollSensitivity: 20,
		scrollSpeed: 20,
		scope: "default",
		tolerance: "intersect",
		zIndex: 1000,

		// Callbacks
		activate: null,
		beforeStop: null,
		change: null,
		deactivate: null,
		out: null,
		over: null,
		receive: null,
		remove: null,
		sort: null,
		start: null,
		stop: null,
		update: null
	},

	_isOverAxis: function( x, reference, size ) {
		return ( x >= reference ) && ( x < ( reference + size ) );
	},

	_isFloating: function( item ) {
		return ( /left|right/ ).test( item.css( "float" ) ) ||
			( /inline|table-cell/ ).test( item.css( "display" ) );
	},

	_create: function() {
		this.containerCache = {};
		this._addClass( "ui-sortable" );

		//Get the items
		this.refresh();

		//Let's determine the parent's offset
		this.offset = this.element.offset();

		//Initialize mouse events for interaction
		this._mouseInit();

		this._setHandleClassName();

		//We're ready to go
		this.ready = true;

	},

	_setOption: function( key, value ) {
		this._super( key, value );

		if ( key === "handle" ) {
			this._setHandleClassName();
		}
	},

	_setHandleClassName: function() {
		var that = this;
		this._removeClass( this.element.find( ".ui-sortable-handle" ), "ui-sortable-handle" );
		$.each( this.items, function() {
			that._addClass(
				this.instance.options.handle ?
					this.item.find( this.instance.options.handle ) :
					this.item,
				"ui-sortable-handle"
			);
		} );
	},

	_destroy: function() {
		this._mouseDestroy();

		for ( var i = this.items.length - 1; i >= 0; i-- ) {
			this.items[ i ].item.removeData( this.widgetName + "-item" );
		}

		return this;
	},

	_mouseCapture: function( event, overrideHandle ) {
		var currentItem = null,
			validHandle = false,
			that = this;

		if ( this.reverting ) {
			return false;
		}

		if ( this.options.disabled || this.options.type === "static" ) {
			return false;
		}

		//We have to refresh the items data once first
		this._refreshItems( event );

		//Find out if the clicked node (or one of its parents) is a actual item in this.items
		$( event.target ).parents().each( function() {
			if ( $.data( this, that.widgetName + "-item" ) === that ) {
				currentItem = $( this );
				return false;
			}
		} );
		if ( $.data( event.target, that.widgetName + "-item" ) === that ) {
			currentItem = $( event.target );
		}

		if ( !currentItem ) {
			return false;
		}
		if ( this.options.handle && !overrideHandle ) {
			$( this.options.handle, currentItem ).find( "*" ).addBack().each( function() {
				if ( this === event.target ) {
					validHandle = true;
				}
			} );
			if ( !validHandle ) {
				return false;
			}
		}

		this.currentItem = currentItem;
		this._removeCurrentsFromItems();
		return true;

	},

	_mouseStart: function( event, overrideHandle, noActivation ) {

		var i, body,
			o = this.options;

		this.currentContainer = this;

		//We only need to call refreshPositions, because the refreshItems call has been moved to
		// mouseCapture
		this.refreshPositions();

		//Create and append the visible helper
		this.helper = this._createHelper( event );

		//Cache the helper size
		this._cacheHelperProportions();

		/*
		 * - Position generation -
		 * This block generates everything position related - it's the core of draggables.
		 */

		//Cache the margins of the original element
		this._cacheMargins();

		//Get the next scrolling parent
		this.scrollParent = this.helper.scrollParent();

		//The element's absolute position on the page minus margins
		this.offset = this.currentItem.offset();
		this.offset = {
			top: this.offset.top - this.margins.top,
			left: this.offset.left - this.margins.left
		};

		$.extend( this.offset, {
			click: { //Where the click happened, relative to the element
				left: event.pageX - this.offset.left,
				top: event.pageY - this.offset.top
			},
			parent: this._getParentOffset(),

			// This is a relative to absolute position minus the actual position calculation -
			// only used for relative positioned helper
			relative: this._getRelativeOffset()
		} );

		// Only after we got the offset, we can change the helper's position to absolute
		// TODO: Still need to figure out a way to make relative sorting possible
		this.helper.css( "position", "absolute" );
		this.cssPosition = this.helper.css( "position" );

		//Generate the original position
		this.originalPosition = this._generatePosition( event );
		this.originalPageX = event.pageX;
		this.originalPageY = event.pageY;

		//Adjust the mouse offset relative to the helper if "cursorAt" is supplied
		( o.cursorAt && this._adjustOffsetFromHelper( o.cursorAt ) );

		//Cache the former DOM position
		this.domPosition = {
			prev: this.currentItem.prev()[ 0 ],
			parent: this.currentItem.parent()[ 0 ]
		};

		// If the helper is not the original, hide the original so it's not playing any role during
		// the drag, won't cause anything bad this way
		if ( this.helper[ 0 ] !== this.currentItem[ 0 ] ) {
			this.currentItem.hide();
		}

		//Create the placeholder
		this._createPlaceholder();

		//Set a containment if given in the options
		if ( o.containment ) {
			this._setContainment();
		}

		if ( o.cursor && o.cursor !== "auto" ) { // cursor option
			body = this.document.find( "body" );

			// Support: IE
			this.storedCursor = body.css( "cursor" );
			body.css( "cursor", o.cursor );

			this.storedStylesheet =
				$( "<style>*{ cursor: " + o.cursor + " !important; }</style>" ).appendTo( body );
		}

		if ( o.opacity ) { // opacity option
			if ( this.helper.css( "opacity" ) ) {
				this._storedOpacity = this.helper.css( "opacity" );
			}
			this.helper.css( "opacity", o.opacity );
		}

		if ( o.zIndex ) { // zIndex option
			if ( this.helper.css( "zIndex" ) ) {
				this._storedZIndex = this.helper.css( "zIndex" );
			}
			this.helper.css( "zIndex", o.zIndex );
		}

		//Prepare scrolling
		if ( this.scrollParent[ 0 ] !== this.document[ 0 ] &&
				this.scrollParent[ 0 ].tagName !== "HTML" ) {
			this.overflowOffset = this.scrollParent.offset();
		}

		//Call callbacks
		this._trigger( "start", event, this._uiHash() );

		//Recache the helper size
		if ( !this._preserveHelperProportions ) {
			this._cacheHelperProportions();
		}

		//Post "activate" events to possible containers
		if ( !noActivation ) {
			for ( i = this.containers.length - 1; i >= 0; i-- ) {
				this.containers[ i ]._trigger( "activate", event, this._uiHash( this ) );
			}
		}

		//Prepare possible droppables
		if ( $.ui.ddmanager ) {
			$.ui.ddmanager.current = this;
		}

		if ( $.ui.ddmanager && !o.dropBehaviour ) {
			$.ui.ddmanager.prepareOffsets( this, event );
		}

		this.dragging = true;

		this._addClass( this.helper, "ui-sortable-helper" );

		// Execute the drag once - this causes the helper not to be visiblebefore getting its
		// correct position
		this._mouseDrag( event );
		return true;

	},

	_mouseDrag: function( event ) {
		var i, item, itemElement, intersection,
			o = this.options,
			scrolled = false;

		//Compute the helpers position
		this.position = this._generatePosition( event );
		this.positionAbs = this._convertPositionTo( "absolute" );

		if ( !this.lastPositionAbs ) {
			this.lastPositionAbs = this.positionAbs;
		}

		//Do scrolling
		if ( this.options.scroll ) {
			if ( this.scrollParent[ 0 ] !== this.document[ 0 ] &&
					this.scrollParent[ 0 ].tagName !== "HTML" ) {

				if ( ( this.overflowOffset.top + this.scrollParent[ 0 ].offsetHeight ) -
						event.pageY < o.scrollSensitivity ) {
					this.scrollParent[ 0 ].scrollTop =
						scrolled = this.scrollParent[ 0 ].scrollTop + o.scrollSpeed;
				} else if ( event.pageY - this.overflowOffset.top < o.scrollSensitivity ) {
					this.scrollParent[ 0 ].scrollTop =
						scrolled = this.scrollParent[ 0 ].scrollTop - o.scrollSpeed;
				}

				if ( ( this.overflowOffset.left + this.scrollParent[ 0 ].offsetWidth ) -
						event.pageX < o.scrollSensitivity ) {
					this.scrollParent[ 0 ].scrollLeft = scrolled =
						this.scrollParent[ 0 ].scrollLeft + o.scrollSpeed;
				} else if ( event.pageX - this.overflowOffset.left < o.scrollSensitivity ) {
					this.scrollParent[ 0 ].scrollLeft = scrolled =
						this.scrollParent[ 0 ].scrollLeft - o.scrollSpeed;
				}

			} else {

				if ( event.pageY - this.document.scrollTop() < o.scrollSensitivity ) {
					scrolled = this.document.scrollTop( this.document.scrollTop() - o.scrollSpeed );
				} else if ( this.window.height() - ( event.pageY - this.document.scrollTop() ) <
						o.scrollSensitivity ) {
					scrolled = this.document.scrollTop( this.document.scrollTop() + o.scrollSpeed );
				}

				if ( event.pageX - this.document.scrollLeft() < o.scrollSensitivity ) {
					scrolled = this.document.scrollLeft(
						this.document.scrollLeft() - o.scrollSpeed
					);
				} else if ( this.window.width() - ( event.pageX - this.document.scrollLeft() ) <
						o.scrollSensitivity ) {
					scrolled = this.document.scrollLeft(
						this.document.scrollLeft() + o.scrollSpeed
					);
				}

			}

			if ( scrolled !== false && $.ui.ddmanager && !o.dropBehaviour ) {
				$.ui.ddmanager.prepareOffsets( this, event );
			}
		}

		//Regenerate the absolute position used for position checks
		this.positionAbs = this._convertPositionTo( "absolute" );

		//Set the helper position
		if ( !this.options.axis || this.options.axis !== "y" ) {
			this.helper[ 0 ].style.left = this.position.left + "px";
		}
		if ( !this.options.axis || this.options.axis !== "x" ) {
			this.helper[ 0 ].style.top = this.position.top + "px";
		}

		//Rearrange
		for ( i = this.items.length - 1; i >= 0; i-- ) {

			//Cache variables and intersection, continue if no intersection
			item = this.items[ i ];
			itemElement = item.item[ 0 ];
			intersection = this._intersectsWithPointer( item );
			if ( !intersection ) {
				continue;
			}

			// Only put the placeholder inside the current Container, skip all
			// items from other containers. This works because when moving
			// an item from one container to another the
			// currentContainer is switched before the placeholder is moved.
			//
			// Without this, moving items in "sub-sortables" can cause
			// the placeholder to jitter between the outer and inner container.
			if ( item.instance !== this.currentContainer ) {
				continue;
			}

			// Cannot intersect with itself
			// no useless actions that have been done before
			// no action if the item moved is the parent of the item checked
			if ( itemElement !== this.currentItem[ 0 ] &&
				this.placeholder[ intersection === 1 ? "next" : "prev" ]()[ 0 ] !== itemElement &&
				!$.contains( this.placeholder[ 0 ], itemElement ) &&
				( this.options.type === "semi-dynamic" ?
					!$.contains( this.element[ 0 ], itemElement ) :
					true
				)
			) {

				this.direction = intersection === 1 ? "down" : "up";

				if ( this.options.tolerance === "pointer" || this._intersectsWithSides( item ) ) {
					this._rearrange( event, item );
				} else {
					break;
				}

				this._trigger( "change", event, this._uiHash() );
				break;
			}
		}

		//Post events to containers
		this._contactContainers( event );

		//Interconnect with droppables
		if ( $.ui.ddmanager ) {
			$.ui.ddmanager.drag( this, event );
		}

		//Call callbacks
		this._trigger( "sort", event, this._uiHash() );

		this.lastPositionAbs = this.positionAbs;
		return false;

	},

	_mouseStop: function( event, noPropagation ) {

		if ( !event ) {
			return;
		}

		//If we are using droppables, inform the manager about the drop
		if ( $.ui.ddmanager && !this.options.dropBehaviour ) {
			$.ui.ddmanager.drop( this, event );
		}

		if ( this.options.revert ) {
			var that = this,
				cur = this.placeholder.offset(),
				axis = this.options.axis,
				animation = {};

			if ( !axis || axis === "x" ) {
				animation.left = cur.left - this.offset.parent.left - this.margins.left +
					( this.offsetParent[ 0 ] === this.document[ 0 ].body ?
						0 :
						this.offsetParent[ 0 ].scrollLeft
					);
			}
			if ( !axis || axis === "y" ) {
				animation.top = cur.top - this.offset.parent.top - this.margins.top +
					( this.offsetParent[ 0 ] === this.document[ 0 ].body ?
						0 :
						this.offsetParent[ 0 ].scrollTop
					);
			}
			this.reverting = true;
			$( this.helper ).animate(
				animation,
				parseInt( this.options.revert, 10 ) || 500,
				function() {
					that._clear( event );
				}
			);
		} else {
			this._clear( event, noPropagation );
		}

		return false;

	},

	cancel: function() {

		if ( this.dragging ) {

			this._mouseUp( new $.Event( "mouseup", { target: null } ) );

			if ( this.options.helper === "original" ) {
				this.currentItem.css( this._storedCSS );
				this._removeClass( this.currentItem, "ui-sortable-helper" );
			} else {
				this.currentItem.show();
			}

			//Post deactivating events to containers
			for ( var i = this.containers.length - 1; i >= 0; i-- ) {
				this.containers[ i ]._trigger( "deactivate", null, this._uiHash( this ) );
				if ( this.containers[ i ].containerCache.over ) {
					this.containers[ i ]._trigger( "out", null, this._uiHash( this ) );
					this.containers[ i ].containerCache.over = 0;
				}
			}

		}

		if ( this.placeholder ) {

			//$(this.placeholder[0]).remove(); would have been the jQuery way - unfortunately,
			// it unbinds ALL events from the original node!
			if ( this.placeholder[ 0 ].parentNode ) {
				this.placeholder[ 0 ].parentNode.removeChild( this.placeholder[ 0 ] );
			}
			if ( this.options.helper !== "original" && this.helper &&
					this.helper[ 0 ].parentNode ) {
				this.helper.remove();
			}

			$.extend( this, {
				helper: null,
				dragging: false,
				reverting: false,
				_noFinalSort: null
			} );

			if ( this.domPosition.prev ) {
				$( this.domPosition.prev ).after( this.currentItem );
			} else {
				$( this.domPosition.parent ).prepend( this.currentItem );
			}
		}

		return this;

	},

	serialize: function( o ) {

		var items = this._getItemsAsjQuery( o && o.connected ),
			str = [];
		o = o || {};

		$( items ).each( function() {
			var res = ( $( o.item || this ).attr( o.attribute || "id" ) || "" )
				.match( o.expression || ( /(.+)[\-=_](.+)/ ) );
			if ( res ) {
				str.push(
					( o.key || res[ 1 ] + "[]" ) +
					"=" + ( o.key && o.expression ? res[ 1 ] : res[ 2 ] ) );
			}
		} );

		if ( !str.length && o.key ) {
			str.push( o.key + "=" );
		}

		return str.join( "&" );

	},

	toArray: function( o ) {

		var items = this._getItemsAsjQuery( o && o.connected ),
			ret = [];

		o = o || {};

		items.each( function() {
			ret.push( $( o.item || this ).attr( o.attribute || "id" ) || "" );
		} );
		return ret;

	},

	/* Be careful with the following core functions */
	_intersectsWith: function( item ) {

		var x1 = this.positionAbs.left,
			x2 = x1 + this.helperProportions.width,
			y1 = this.positionAbs.top,
			y2 = y1 + this.helperProportions.height,
			l = item.left,
			r = l + item.width,
			t = item.top,
			b = t + item.height,
			dyClick = this.offset.click.top,
			dxClick = this.offset.click.left,
			isOverElementHeight = ( this.options.axis === "x" ) || ( ( y1 + dyClick ) > t &&
				( y1 + dyClick ) < b ),
			isOverElementWidth = ( this.options.axis === "y" ) || ( ( x1 + dxClick ) > l &&
				( x1 + dxClick ) < r ),
			isOverElement = isOverElementHeight && isOverElementWidth;

		if ( this.options.tolerance === "pointer" ||
			this.options.forcePointerForContainers ||
			( this.options.tolerance !== "pointer" &&
				this.helperProportions[ this.floating ? "width" : "height" ] >
				item[ this.floating ? "width" : "height" ] )
		) {
			return isOverElement;
		} else {

			return ( l < x1 + ( this.helperProportions.width / 2 ) && // Right Half
				x2 - ( this.helperProportions.width / 2 ) < r && // Left Half
				t < y1 + ( this.helperProportions.height / 2 ) && // Bottom Half
				y2 - ( this.helperProportions.height / 2 ) < b ); // Top Half

		}
	},

	_intersectsWithPointer: function( item ) {
		var verticalDirection, horizontalDirection,
			isOverElementHeight = ( this.options.axis === "x" ) ||
				this._isOverAxis(
					this.positionAbs.top + this.offset.click.top, item.top, item.height ),
			isOverElementWidth = ( this.options.axis === "y" ) ||
				this._isOverAxis(
					this.positionAbs.left + this.offset.click.left, item.left, item.width ),
			isOverElement = isOverElementHeight && isOverElementWidth;

		if ( !isOverElement ) {
			return false;
		}

		verticalDirection = this._getDragVerticalDirection();
		horizontalDirection = this._getDragHorizontalDirection();

		return this.floating ?
			( ( horizontalDirection === "right" || verticalDirection === "down" ) ? 2 : 1 )
			: ( verticalDirection && ( verticalDirection === "down" ? 2 : 1 ) );

	},

	_intersectsWithSides: function( item ) {

		var isOverBottomHalf = this._isOverAxis( this.positionAbs.top +
				this.offset.click.top, item.top + ( item.height / 2 ), item.height ),
			isOverRightHalf = this._isOverAxis( this.positionAbs.left +
				this.offset.click.left, item.left + ( item.width / 2 ), item.width ),
			verticalDirection = this._getDragVerticalDirection(),
			horizontalDirection = this._getDragHorizontalDirection();

		if ( this.floating && horizontalDirection ) {
			return ( ( horizontalDirection === "right" && isOverRightHalf ) ||
				( horizontalDirection === "left" && !isOverRightHalf ) );
		} else {
			return verticalDirection && ( ( verticalDirection === "down" && isOverBottomHalf ) ||
				( verticalDirection === "up" && !isOverBottomHalf ) );
		}

	},

	_getDragVerticalDirection: function() {
		var delta = this.positionAbs.top - this.lastPositionAbs.top;
		return delta !== 0 && ( delta > 0 ? "down" : "up" );
	},

	_getDragHorizontalDirection: function() {
		var delta = this.positionAbs.left - this.lastPositionAbs.left;
		return delta !== 0 && ( delta > 0 ? "right" : "left" );
	},

	refresh: function( event ) {
		this._refreshItems( event );
		this._setHandleClassName();
		this.refreshPositions();
		return this;
	},

	_connectWith: function() {
		var options = this.options;
		return options.connectWith.constructor === String ?
			[ options.connectWith ] :
			options.connectWith;
	},

	_getItemsAsjQuery: function( connected ) {

		var i, j, cur, inst,
			items = [],
			queries = [],
			connectWith = this._connectWith();

		if ( connectWith && connected ) {
			for ( i = connectWith.length - 1; i >= 0; i-- ) {
				cur = $( connectWith[ i ], this.document[ 0 ] );
				for ( j = cur.length - 1; j >= 0; j-- ) {
					inst = $.data( cur[ j ], this.widgetFullName );
					if ( inst && inst !== this && !inst.options.disabled ) {
						queries.push( [ $.isFunction( inst.options.items ) ?
							inst.options.items.call( inst.element ) :
							$( inst.options.items, inst.element )
								.not( ".ui-sortable-helper" )
								.not( ".ui-sortable-placeholder" ), inst ] );
					}
				}
			}
		}

		queries.push( [ $.isFunction( this.options.items ) ?
			this.options.items
				.call( this.element, null, { options: this.options, item: this.currentItem } ) :
			$( this.options.items, this.element )
				.not( ".ui-sortable-helper" )
				.not( ".ui-sortable-placeholder" ), this ] );

		function addItems() {
			items.push( this );
		}
		for ( i = queries.length - 1; i >= 0; i-- ) {
			queries[ i ][ 0 ].each( addItems );
		}

		return $( items );

	},

	_removeCurrentsFromItems: function() {

		var list = this.currentItem.find( ":data(" + this.widgetName + "-item)" );

		this.items = $.grep( this.items, function( item ) {
			for ( var j = 0; j < list.length; j++ ) {
				if ( list[ j ] === item.item[ 0 ] ) {
					return false;
				}
			}
			return true;
		} );

	},

	_refreshItems: function( event ) {

		this.items = [];
		this.containers = [ this ];

		var i, j, cur, inst, targetData, _queries, item, queriesLength,
			items = this.items,
			queries = [ [ $.isFunction( this.options.items ) ?
				this.options.items.call( this.element[ 0 ], event, { item: this.currentItem } ) :
				$( this.options.items, this.element ), this ] ],
			connectWith = this._connectWith();

		//Shouldn't be run the first time through due to massive slow-down
		if ( connectWith && this.ready ) {
			for ( i = connectWith.length - 1; i >= 0; i-- ) {
				cur = $( connectWith[ i ], this.document[ 0 ] );
				for ( j = cur.length - 1; j >= 0; j-- ) {
					inst = $.data( cur[ j ], this.widgetFullName );
					if ( inst && inst !== this && !inst.options.disabled ) {
						queries.push( [ $.isFunction( inst.options.items ) ?
							inst.options.items
								.call( inst.element[ 0 ], event, { item: this.currentItem } ) :
							$( inst.options.items, inst.element ), inst ] );
						this.containers.push( inst );
					}
				}
			}
		}

		for ( i = queries.length - 1; i >= 0; i-- ) {
			targetData = queries[ i ][ 1 ];
			_queries = queries[ i ][ 0 ];

			for ( j = 0, queriesLength = _queries.length; j < queriesLength; j++ ) {
				item = $( _queries[ j ] );

				// Data for target checking (mouse manager)
				item.data( this.widgetName + "-item", targetData );

				items.push( {
					item: item,
					instance: targetData,
					width: 0, height: 0,
					left: 0, top: 0
				} );
			}
		}

	},

	refreshPositions: function( fast ) {

		// Determine whether items are being displayed horizontally
		this.floating = this.items.length ?
			this.options.axis === "x" || this._isFloating( this.items[ 0 ].item ) :
			false;

		//This has to be redone because due to the item being moved out/into the offsetParent,
		// the offsetParent's position will change
		if ( this.offsetParent && this.helper ) {
			this.offset.parent = this._getParentOffset();
		}

		var i, item, t, p;

		for ( i = this.items.length - 1; i >= 0; i-- ) {
			item = this.items[ i ];

			//We ignore calculating positions of all connected containers when we're not over them
			if ( item.instance !== this.currentContainer && this.currentContainer &&
					item.item[ 0 ] !== this.currentItem[ 0 ] ) {
				continue;
			}

			t = this.options.toleranceElement ?
				$( this.options.toleranceElement, item.item ) :
				item.item;

			if ( !fast ) {
				item.width = t.outerWidth();
				item.height = t.outerHeight();
			}

			p = t.offset();
			item.left = p.left;
			item.top = p.top;
		}

		if ( this.options.custom && this.options.custom.refreshContainers ) {
			this.options.custom.refreshContainers.call( this );
		} else {
			for ( i = this.containers.length - 1; i >= 0; i-- ) {
				p = this.containers[ i ].element.offset();
				this.containers[ i ].containerCache.left = p.left;
				this.containers[ i ].containerCache.top = p.top;
				this.containers[ i ].containerCache.width =
					this.containers[ i ].element.outerWidth();
				this.containers[ i ].containerCache.height =
					this.containers[ i ].element.outerHeight();
			}
		}

		return this;
	},

	_createPlaceholder: function( that ) {
		that = that || this;
		var className,
			o = that.options;

		if ( !o.placeholder || o.placeholder.constructor === String ) {
			className = o.placeholder;
			o.placeholder = {
				element: function() {

					var nodeName = that.currentItem[ 0 ].nodeName.toLowerCase(),
						element = $( "<" + nodeName + ">", that.document[ 0 ] );

						that._addClass( element, "ui-sortable-placeholder",
								className || that.currentItem[ 0 ].className )
							._removeClass( element, "ui-sortable-helper" );

					if ( nodeName === "tbody" ) {
						that._createTrPlaceholder(
							that.currentItem.find( "tr" ).eq( 0 ),
							$( "<tr>", that.document[ 0 ] ).appendTo( element )
						);
					} else if ( nodeName === "tr" ) {
						that._createTrPlaceholder( that.currentItem, element );
					} else if ( nodeName === "img" ) {
						element.attr( "src", that.currentItem.attr( "src" ) );
					}

					if ( !className ) {
						element.css( "visibility", "hidden" );
					}

					return element;
				},
				update: function( container, p ) {

					// 1. If a className is set as 'placeholder option, we don't force sizes -
					// the class is responsible for that
					// 2. The option 'forcePlaceholderSize can be enabled to force it even if a
					// class name is specified
					if ( className && !o.forcePlaceholderSize ) {
						return;
					}

					//If the element doesn't have a actual height by itself (without styles coming
					// from a stylesheet), it receives the inline height from the dragged item
					if ( !p.height() ) {
						p.height(
							that.currentItem.innerHeight() -
							parseInt( that.currentItem.css( "paddingTop" ) || 0, 10 ) -
							parseInt( that.currentItem.css( "paddingBottom" ) || 0, 10 ) );
					}
					if ( !p.width() ) {
						p.width(
							that.currentItem.innerWidth() -
							parseInt( that.currentItem.css( "paddingLeft" ) || 0, 10 ) -
							parseInt( that.currentItem.css( "paddingRight" ) || 0, 10 ) );
					}
				}
			};
		}

		//Create the placeholder
		that.placeholder = $( o.placeholder.element.call( that.element, that.currentItem ) );

		//Append it after the actual current item
		that.currentItem.after( that.placeholder );

		//Update the size of the placeholder (TODO: Logic to fuzzy, see line 316/317)
		o.placeholder.update( that, that.placeholder );

	},

	_createTrPlaceholder: function( sourceTr, targetTr ) {
		var that = this;

		sourceTr.children().each( function() {
			$( "<td>&#160;</td>", that.document[ 0 ] )
				.attr( "colspan", $( this ).attr( "colspan" ) || 1 )
				.appendTo( targetTr );
		} );
	},

	_contactContainers: function( event ) {
		var i, j, dist, itemWithLeastDistance, posProperty, sizeProperty, cur, nearBottom,
			floating, axis,
			innermostContainer = null,
			innermostIndex = null;

		// Get innermost container that intersects with item
		for ( i = this.containers.length - 1; i >= 0; i-- ) {

			// Never consider a container that's located within the item itself
			if ( $.contains( this.currentItem[ 0 ], this.containers[ i ].element[ 0 ] ) ) {
				continue;
			}

			if ( this._intersectsWith( this.containers[ i ].containerCache ) ) {

				// If we've already found a container and it's more "inner" than this, then continue
				if ( innermostContainer &&
						$.contains(
							this.containers[ i ].element[ 0 ],
							innermostContainer.element[ 0 ] ) ) {
					continue;
				}

				innermostContainer = this.containers[ i ];
				innermostIndex = i;

			} else {

				// container doesn't intersect. trigger "out" event if necessary
				if ( this.containers[ i ].containerCache.over ) {
					this.containers[ i ]._trigger( "out", event, this._uiHash( this ) );
					this.containers[ i ].containerCache.over = 0;
				}
			}

		}

		// If no intersecting containers found, return
		if ( !innermostContainer ) {
			return;
		}

		// Move the item into the container if it's not there already
		if ( this.containers.length === 1 ) {
			if ( !this.containers[ innermostIndex ].containerCache.over ) {
				this.containers[ innermostIndex ]._trigger( "over", event, this._uiHash( this ) );
				this.containers[ innermostIndex ].containerCache.over = 1;
			}
		} else {

			// When entering a new container, we will find the item with the least distance and
			// append our item near it
			dist = 10000;
			itemWithLeastDistance = null;
			floating = innermostContainer.floating || this._isFloating( this.currentItem );
			posProperty = floating ? "left" : "top";
			sizeProperty = floating ? "width" : "height";
			axis = floating ? "pageX" : "pageY";

			for ( j = this.items.length - 1; j >= 0; j-- ) {
				if ( !$.contains(
						this.containers[ innermostIndex ].element[ 0 ], this.items[ j ].item[ 0 ] )
				) {
					continue;
				}
				if ( this.items[ j ].item[ 0 ] === this.currentItem[ 0 ] ) {
					continue;
				}

				cur = this.items[ j ].item.offset()[ posProperty ];
				nearBottom = false;
				if ( event[ axis ] - cur > this.items[ j ][ sizeProperty ] / 2 ) {
					nearBottom = true;
				}

				if ( Math.abs( event[ axis ] - cur ) < dist ) {
					dist = Math.abs( event[ axis ] - cur );
					itemWithLeastDistance = this.items[ j ];
					this.direction = nearBottom ? "up" : "down";
				}
			}

			//Check if dropOnEmpty is enabled
			if ( !itemWithLeastDistance && !this.options.dropOnEmpty ) {
				return;
			}

			if ( this.currentContainer === this.containers[ innermostIndex ] ) {
				if ( !this.currentContainer.containerCache.over ) {
					this.containers[ innermostIndex ]._trigger( "over", event, this._uiHash() );
					this.currentContainer.containerCache.over = 1;
				}
				return;
			}

			itemWithLeastDistance ?
				this._rearrange( event, itemWithLeastDistance, null, true ) :
				this._rearrange( event, null, this.containers[ innermostIndex ].element, true );
			this._trigger( "change", event, this._uiHash() );
			this.containers[ innermostIndex ]._trigger( "change", event, this._uiHash( this ) );
			this.currentContainer = this.containers[ innermostIndex ];

			//Update the placeholder
			this.options.placeholder.update( this.currentContainer, this.placeholder );

			this.containers[ innermostIndex ]._trigger( "over", event, this._uiHash( this ) );
			this.containers[ innermostIndex ].containerCache.over = 1;
		}

	},

	_createHelper: function( event ) {

		var o = this.options,
			helper = $.isFunction( o.helper ) ?
				$( o.helper.apply( this.element[ 0 ], [ event, this.currentItem ] ) ) :
				( o.helper === "clone" ? this.currentItem.clone() : this.currentItem );

		//Add the helper to the DOM if that didn't happen already
		if ( !helper.parents( "body" ).length ) {
			$( o.appendTo !== "parent" ?
				o.appendTo :
				this.currentItem[ 0 ].parentNode )[ 0 ].appendChild( helper[ 0 ] );
		}

		if ( helper[ 0 ] === this.currentItem[ 0 ] ) {
			this._storedCSS = {
				width: this.currentItem[ 0 ].style.width,
				height: this.currentItem[ 0 ].style.height,
				position: this.currentItem.css( "position" ),
				top: this.currentItem.css( "top" ),
				left: this.currentItem.css( "left" )
			};
		}

		if ( !helper[ 0 ].style.width || o.forceHelperSize ) {
			helper.width( this.currentItem.width() );
		}
		if ( !helper[ 0 ].style.height || o.forceHelperSize ) {
			helper.height( this.currentItem.height() );
		}

		return helper;

	},

	_adjustOffsetFromHelper: function( obj ) {
		if ( typeof obj === "string" ) {
			obj = obj.split( " " );
		}
		if ( $.isArray( obj ) ) {
			obj = { left: +obj[ 0 ], top: +obj[ 1 ] || 0 };
		}
		if ( "left" in obj ) {
			this.offset.click.left = obj.left + this.margins.left;
		}
		if ( "right" in obj ) {
			this.offset.click.left = this.helperProportions.width - obj.right + this.margins.left;
		}
		if ( "top" in obj ) {
			this.offset.click.top = obj.top + this.margins.top;
		}
		if ( "bottom" in obj ) {
			this.offset.click.top = this.helperProportions.height - obj.bottom + this.margins.top;
		}
	},

	_getParentOffset: function() {

		//Get the offsetParent and cache its position
		this.offsetParent = this.helper.offsetParent();
		var po = this.offsetParent.offset();

		// This is a special case where we need to modify a offset calculated on start, since the
		// following happened:
		// 1. The position of the helper is absolute, so it's position is calculated based on the
		// next positioned parent
		// 2. The actual offset parent is a child of the scroll parent, and the scroll parent isn't
		// the document, which means that the scroll is included in the initial calculation of the
		// offset of the parent, and never recalculated upon drag
		if ( this.cssPosition === "absolute" && this.scrollParent[ 0 ] !== this.document[ 0 ] &&
				$.contains( this.scrollParent[ 0 ], this.offsetParent[ 0 ] ) ) {
			po.left += this.scrollParent.scrollLeft();
			po.top += this.scrollParent.scrollTop();
		}

		// This needs to be actually done for all browsers, since pageX/pageY includes this
		// information with an ugly IE fix
		if ( this.offsetParent[ 0 ] === this.document[ 0 ].body ||
				( this.offsetParent[ 0 ].tagName &&
				this.offsetParent[ 0 ].tagName.toLowerCase() === "html" && $.ui.ie ) ) {
			po = { top: 0, left: 0 };
		}

		return {
			top: po.top + ( parseInt( this.offsetParent.css( "borderTopWidth" ), 10 ) || 0 ),
			left: po.left + ( parseInt( this.offsetParent.css( "borderLeftWidth" ), 10 ) || 0 )
		};

	},

	_getRelativeOffset: function() {

		if ( this.cssPosition === "relative" ) {
			var p = this.currentItem.position();
			return {
				top: p.top - ( parseInt( this.helper.css( "top" ), 10 ) || 0 ) +
					this.scrollParent.scrollTop(),
				left: p.left - ( parseInt( this.helper.css( "left" ), 10 ) || 0 ) +
					this.scrollParent.scrollLeft()
			};
		} else {
			return { top: 0, left: 0 };
		}

	},

	_cacheMargins: function() {
		this.margins = {
			left: ( parseInt( this.currentItem.css( "marginLeft" ), 10 ) || 0 ),
			top: ( parseInt( this.currentItem.css( "marginTop" ), 10 ) || 0 )
		};
	},

	_cacheHelperProportions: function() {
		this.helperProportions = {
			width: this.helper.outerWidth(),
			height: this.helper.outerHeight()
		};
	},

	_setContainment: function() {

		var ce, co, over,
			o = this.options;
		if ( o.containment === "parent" ) {
			o.containment = this.helper[ 0 ].parentNode;
		}
		if ( o.containment === "document" || o.containment === "window" ) {
			this.containment = [
				0 - this.offset.relative.left - this.offset.parent.left,
				0 - this.offset.relative.top - this.offset.parent.top,
				o.containment === "document" ?
					this.document.width() :
					this.window.width() - this.helperProportions.width - this.margins.left,
				( o.containment === "document" ?
					( this.document.height() || document.body.parentNode.scrollHeight ) :
					this.window.height() || this.document[ 0 ].body.parentNode.scrollHeight
				) - this.helperProportions.height - this.margins.top
			];
		}

		if ( !( /^(document|window|parent)$/ ).test( o.containment ) ) {
			ce = $( o.containment )[ 0 ];
			co = $( o.containment ).offset();
			over = ( $( ce ).css( "overflow" ) !== "hidden" );

			this.containment = [
				co.left + ( parseInt( $( ce ).css( "borderLeftWidth" ), 10 ) || 0 ) +
					( parseInt( $( ce ).css( "paddingLeft" ), 10 ) || 0 ) - this.margins.left,
				co.top + ( parseInt( $( ce ).css( "borderTopWidth" ), 10 ) || 0 ) +
					( parseInt( $( ce ).css( "paddingTop" ), 10 ) || 0 ) - this.margins.top,
				co.left + ( over ? Math.max( ce.scrollWidth, ce.offsetWidth ) : ce.offsetWidth ) -
					( parseInt( $( ce ).css( "borderLeftWidth" ), 10 ) || 0 ) -
					( parseInt( $( ce ).css( "paddingRight" ), 10 ) || 0 ) -
					this.helperProportions.width - this.margins.left,
				co.top + ( over ? Math.max( ce.scrollHeight, ce.offsetHeight ) : ce.offsetHeight ) -
					( parseInt( $( ce ).css( "borderTopWidth" ), 10 ) || 0 ) -
					( parseInt( $( ce ).css( "paddingBottom" ), 10 ) || 0 ) -
					this.helperProportions.height - this.margins.top
			];
		}

	},

	_convertPositionTo: function( d, pos ) {

		if ( !pos ) {
			pos = this.position;
		}
		var mod = d === "absolute" ? 1 : -1,
			scroll = this.cssPosition === "absolute" &&
				!( this.scrollParent[ 0 ] !== this.document[ 0 ] &&
				$.contains( this.scrollParent[ 0 ], this.offsetParent[ 0 ] ) ) ?
					this.offsetParent :
					this.scrollParent,
			scrollIsRootNode = ( /(html|body)/i ).test( scroll[ 0 ].tagName );

		return {
			top: (

				// The absolute mouse position
				pos.top	+

				// Only for relative positioned nodes: Relative offset from element to offset parent
				this.offset.relative.top * mod +

				// The offsetParent's offset without borders (offset + border)
				this.offset.parent.top * mod -
				( ( this.cssPosition === "fixed" ?
					-this.scrollParent.scrollTop() :
					( scrollIsRootNode ? 0 : scroll.scrollTop() ) ) * mod )
			),
			left: (

				// The absolute mouse position
				pos.left +

				// Only for relative positioned nodes: Relative offset from element to offset parent
				this.offset.relative.left * mod +

				// The offsetParent's offset without borders (offset + border)
				this.offset.parent.left * mod	-
				( ( this.cssPosition === "fixed" ?
					-this.scrollParent.scrollLeft() : scrollIsRootNode ? 0 :
					scroll.scrollLeft() ) * mod )
			)
		};

	},

	_generatePosition: function( event ) {

		var top, left,
			o = this.options,
			pageX = event.pageX,
			pageY = event.pageY,
			scroll = this.cssPosition === "absolute" &&
				!( this.scrollParent[ 0 ] !== this.document[ 0 ] &&
				$.contains( this.scrollParent[ 0 ], this.offsetParent[ 0 ] ) ) ?
					this.offsetParent :
					this.scrollParent,
				scrollIsRootNode = ( /(html|body)/i ).test( scroll[ 0 ].tagName );

		// This is another very weird special case that only happens for relative elements:
		// 1. If the css position is relative
		// 2. and the scroll parent is the document or similar to the offset parent
		// we have to refresh the relative offset during the scroll so there are no jumps
		if ( this.cssPosition === "relative" && !( this.scrollParent[ 0 ] !== this.document[ 0 ] &&
				this.scrollParent[ 0 ] !== this.offsetParent[ 0 ] ) ) {
			this.offset.relative = this._getRelativeOffset();
		}

		/*
		 * - Position constraining -
		 * Constrain the position to a mix of grid, containment.
		 */

		if ( this.originalPosition ) { //If we are not dragging yet, we won't check for options

			if ( this.containment ) {
				if ( event.pageX - this.offset.click.left < this.containment[ 0 ] ) {
					pageX = this.containment[ 0 ] + this.offset.click.left;
				}
				if ( event.pageY - this.offset.click.top < this.containment[ 1 ] ) {
					pageY = this.containment[ 1 ] + this.offset.click.top;
				}
				if ( event.pageX - this.offset.click.left > this.containment[ 2 ] ) {
					pageX = this.containment[ 2 ] + this.offset.click.left;
				}
				if ( event.pageY - this.offset.click.top > this.containment[ 3 ] ) {
					pageY = this.containment[ 3 ] + this.offset.click.top;
				}
			}

			if ( o.grid ) {
				top = this.originalPageY + Math.round( ( pageY - this.originalPageY ) /
					o.grid[ 1 ] ) * o.grid[ 1 ];
				pageY = this.containment ?
					( ( top - this.offset.click.top >= this.containment[ 1 ] &&
						top - this.offset.click.top <= this.containment[ 3 ] ) ?
							top :
							( ( top - this.offset.click.top >= this.containment[ 1 ] ) ?
								top - o.grid[ 1 ] : top + o.grid[ 1 ] ) ) :
								top;

				left = this.originalPageX + Math.round( ( pageX - this.originalPageX ) /
					o.grid[ 0 ] ) * o.grid[ 0 ];
				pageX = this.containment ?
					( ( left - this.offset.click.left >= this.containment[ 0 ] &&
						left - this.offset.click.left <= this.containment[ 2 ] ) ?
							left :
							( ( left - this.offset.click.left >= this.containment[ 0 ] ) ?
								left - o.grid[ 0 ] : left + o.grid[ 0 ] ) ) :
								left;
			}

		}

		return {
			top: (

				// The absolute mouse position
				pageY -

				// Click offset (relative to the element)
				this.offset.click.top -

				// Only for relative positioned nodes: Relative offset from element to offset parent
				this.offset.relative.top -

				// The offsetParent's offset without borders (offset + border)
				this.offset.parent.top +
				( ( this.cssPosition === "fixed" ?
					-this.scrollParent.scrollTop() :
					( scrollIsRootNode ? 0 : scroll.scrollTop() ) ) )
			),
			left: (

				// The absolute mouse position
				pageX -

				// Click offset (relative to the element)
				this.offset.click.left -

				// Only for relative positioned nodes: Relative offset from element to offset parent
				this.offset.relative.left -

				// The offsetParent's offset without borders (offset + border)
				this.offset.parent.left +
				( ( this.cssPosition === "fixed" ?
					-this.scrollParent.scrollLeft() :
					scrollIsRootNode ? 0 : scroll.scrollLeft() ) )
			)
		};

	},

	_rearrange: function( event, i, a, hardRefresh ) {

		a ? a[ 0 ].appendChild( this.placeholder[ 0 ] ) :
			i.item[ 0 ].parentNode.insertBefore( this.placeholder[ 0 ],
				( this.direction === "down" ? i.item[ 0 ] : i.item[ 0 ].nextSibling ) );

		//Various things done here to improve the performance:
		// 1. we create a setTimeout, that calls refreshPositions
		// 2. on the instance, we have a counter variable, that get's higher after every append
		// 3. on the local scope, we copy the counter variable, and check in the timeout,
		// if it's still the same
		// 4. this lets only the last addition to the timeout stack through
		this.counter = this.counter ? ++this.counter : 1;
		var counter = this.counter;

		this._delay( function() {
			if ( counter === this.counter ) {

				//Precompute after each DOM insertion, NOT on mousemove
				this.refreshPositions( !hardRefresh );
			}
		} );

	},

	_clear: function( event, noPropagation ) {

		this.reverting = false;

		// We delay all events that have to be triggered to after the point where the placeholder
		// has been removed and everything else normalized again
		var i,
			delayedTriggers = [];

		// We first have to update the dom position of the actual currentItem
		// Note: don't do it if the current item is already removed (by a user), or it gets
		// reappended (see #4088)
		if ( !this._noFinalSort && this.currentItem.parent().length ) {
			this.placeholder.before( this.currentItem );
		}
		this._noFinalSort = null;

		if ( this.helper[ 0 ] === this.currentItem[ 0 ] ) {
			for ( i in this._storedCSS ) {
				if ( this._storedCSS[ i ] === "auto" || this._storedCSS[ i ] === "static" ) {
					this._storedCSS[ i ] = "";
				}
			}
			this.currentItem.css( this._storedCSS );
			this._removeClass( this.currentItem, "ui-sortable-helper" );
		} else {
			this.currentItem.show();
		}

		if ( this.fromOutside && !noPropagation ) {
			delayedTriggers.push( function( event ) {
				this._trigger( "receive", event, this._uiHash( this.fromOutside ) );
			} );
		}
		if ( ( this.fromOutside ||
				this.domPosition.prev !==
				this.currentItem.prev().not( ".ui-sortable-helper" )[ 0 ] ||
				this.domPosition.parent !== this.currentItem.parent()[ 0 ] ) && !noPropagation ) {

			// Trigger update callback if the DOM position has changed
			delayedTriggers.push( function( event ) {
				this._trigger( "update", event, this._uiHash() );
			} );
		}

		// Check if the items Container has Changed and trigger appropriate
		// events.
		if ( this !== this.currentContainer ) {
			if ( !noPropagation ) {
				delayedTriggers.push( function( event ) {
					this._trigger( "remove", event, this._uiHash() );
				} );
				delayedTriggers.push( ( function( c ) {
					return function( event ) {
						c._trigger( "receive", event, this._uiHash( this ) );
					};
				} ).call( this, this.currentContainer ) );
				delayedTriggers.push( ( function( c ) {
					return function( event ) {
						c._trigger( "update", event, this._uiHash( this ) );
					};
				} ).call( this, this.currentContainer ) );
			}
		}

		//Post events to containers
		function delayEvent( type, instance, container ) {
			return function( event ) {
				container._trigger( type, event, instance._uiHash( instance ) );
			};
		}
		for ( i = this.containers.length - 1; i >= 0; i-- ) {
			if ( !noPropagation ) {
				delayedTriggers.push( delayEvent( "deactivate", this, this.containers[ i ] ) );
			}
			if ( this.containers[ i ].containerCache.over ) {
				delayedTriggers.push( delayEvent( "out", this, this.containers[ i ] ) );
				this.containers[ i ].containerCache.over = 0;
			}
		}

		//Do what was originally in plugins
		if ( this.storedCursor ) {
			this.document.find( "body" ).css( "cursor", this.storedCursor );
			this.storedStylesheet.remove();
		}
		if ( this._storedOpacity ) {
			this.helper.css( "opacity", this._storedOpacity );
		}
		if ( this._storedZIndex ) {
			this.helper.css( "zIndex", this._storedZIndex === "auto" ? "" : this._storedZIndex );
		}

		this.dragging = false;

		if ( !noPropagation ) {
			this._trigger( "beforeStop", event, this._uiHash() );
		}

		//$(this.placeholder[0]).remove(); would have been the jQuery way - unfortunately,
		// it unbinds ALL events from the original node!
		this.placeholder[ 0 ].parentNode.removeChild( this.placeholder[ 0 ] );

		if ( !this.cancelHelperRemoval ) {
			if ( this.helper[ 0 ] !== this.currentItem[ 0 ] ) {
				this.helper.remove();
			}
			this.helper = null;
		}

		if ( !noPropagation ) {
			for ( i = 0; i < delayedTriggers.length; i++ ) {

				// Trigger all delayed events
				delayedTriggers[ i ].call( this, event );
			}
			this._trigger( "stop", event, this._uiHash() );
		}

		this.fromOutside = false;
		return !this.cancelHelperRemoval;

	},

	_trigger: function() {
		if ( $.Widget.prototype._trigger.apply( this, arguments ) === false ) {
			this.cancel();
		}
	},

	_uiHash: function( _inst ) {
		var inst = _inst || this;
		return {
			helper: inst.helper,
			placeholder: inst.placeholder || $( [] ),
			position: inst.position,
			originalPosition: inst.originalPosition,
			offset: inst.positionAbs,
			item: inst.currentItem,
			sender: _inst ? _inst.element : null
		};
	}

} );




}));
/* szsewebui jQuery 胶水代码(szsewebui保留关键字) */
Jx().package("s.utilities", function(J){
	
	// 严格模式
	'use strict';

	String.format= function(){
		if(arguments.length === 0){
			return null;
		}

		var str= arguments[0];
		for(var i=1; i<arguments.length; i++){
			var reg= new RegExp('\\{' + (i-1) + '\\}', 'gm');
			str= str.replace(reg, arguments[i]);
		}

		return str;
	};
	this.indexOfArray=function(arr,obj,keys){
		//s.utilities.indexOfArray([{text:'1'},{text:'2'},{text:'3'}],{text:'3'},'text',true);
		var istrue=true;
        if(!Array.isArray(arr))
            arr=[arr];
        if(!Array.isArray(keys))
            keys=[keys];
        if(typeof obj !=='object'){ return arr.indexOf(obj);}
        for(var i=0;i<arr.length;i++){
            istrue=true;
            for(var k=0;k<keys.length;k++){
                if(obj[keys[k]]!=arr[i][keys[k]]){
                    istrue=false;
                }
            }
            if(istrue) return i;
          
        }
        return -1;
	}
	this.removeOneofArray=function(arr,obj,key){
		var index=this.indexOfArray(arr,obj,key);
        if(index>=0)
            arr.splice(index,1);
        return arr;
	}
	this.addSomeOfArray=function(arr,somes,obj,key,flag){
		//s.utilities.addSomeOfArray([1,2,'b','c','xx'],[3]);
		//s.utilities.addSomeOfArray([1,2,'b','c','xx'],[3],'c',null,true);
		//s.utilities.addSomeOfArray([{text:'1'},{text:'2'},{text:'3'}],[{text:'4'},{text:'5'}],{text:'3'},'text',true);
		somes=Array.isArray(somes) ? somes : [somes];
		if(flag){
			var index=this.indexOfArray(arr,obj,key);
			var tems=somes.concat([]);
			tems.unshift(index,0);
			Array.prototype.splice.apply(arr,tems);
		}else{
			arr=arr.concat(somes);
		}
		return arr;
		
	//var a=[1,2,3];var b=['a','b'];var c=b.concat([]);c.unshift(1,0); console.log(b);Array.prototype.splice.apply(a,c); console.log(a); console.log(b);
	}
	this.Modal=function(messageDefault){
		var modal={};
		var defaults={
			onInitialized: function(){  // 初始化完成
                // modal.show();
                this.show();
            }
		};

		
		function open(options){
            var settings= $.extend(true, {}, defaults,options);
            var targetElement= settings.targetElement;
            delete settings.targetElement;
            if($(targetElement).attr("disabled") === undefined){
                var modal= new s.ui.controls.Modal(targetElement, settings);
                return modal;
            }
        }

		 modal={
		 	open:open,
            message: function(options){

                // var htmlTemplate= 
                var newOptions;
                var type= options.type;
                var modalclass = options.modalClass;
                var settings;
                var handler = function(e){
                    this.hide();
                };
                switch(type){
                    case 'error':{
                        settings= {
                            modalTitle: options.modalTitle || '错误信息',
                            buttons: '    <button type="button" class="btn btn-primary c-close">关闭</button>'
                        };                        
                        break;
                    }
                    case 'warn':{
                        settings= {
                            modalTitle: options.modalTitle || '警告信息',
                            buttons:  '    <button type="button" class="btn btn-default cancel">取消</button>'+
                                      '    <button type="button" class="btn btn-primary confirm">确认</button>',
                            btnevent: {
                                    confirm: {
                                        selector: '.confirm',
                                        eventName: 'click',
                                        handler: options.ok || handler
                                    },
                                    cancel: {
                                        selector: '.cancel',
                                        eventName: 'click',
                                        handler: options.cancel || handler
                                    }
                            }
                        };
                        break;
                    }
                    case 'success':{
                        settings= {
                            modalTitle: options.modalTitle || '成功信息',
                            buttons: '    <button type="button" class="btn btn-primary cancel">关闭</button>',
                            btnevent: {
                                    cancel: {
                                        selector: '.cancel',
                                        eventName: 'click',
                                        handler: options.cancel || handler
                                    }
                            }
                        };
                        break;
                    }
                    case 'confirm':{
                        settings= {
                            modalTitle: options.modalTitle || '确认信息',
                            buttons:  '    <button type="button" class="btn btn-default cancel">取消</button>'+
                                      '    <button type="button" class="btn btn-primary confirm">确认</button>',
                            btnevent: {
                                    confirm: {
                                        selector: '.confirm',
                                        eventName: 'click',
                                        handler: options.ok || handler
                                    },
                                    cancel: {
                                        selector: '.cancel',
                                        eventName: 'click',
                                        handler: options.cancel || handler
                                    }
                            }
                        };
                        break;
                    }
                    // case 'info':{
                        
                    // }
                    default: {
                        settings= {
                            modalTitle: options.modalTitle || '提示信息',
                            buttons: '    <button type="button" class="btn btn-primary c-close">关闭</button>'
                        };
                        break;
                    }                  
                }
                newOptions= {
                	targetElement:options.targetElement,
                    modalId: '#c-modal-messagebox-'+type,
                    modalContainer: '#c-modal-messagebox',
                    modalClass:modalclass,
                    content: '' +
                        '<div class="modal-content">'+
                        '    <div class="modal-header">'+
                        '    <button type="button" class="close" aria-label="Close"><span aria-hidden="true">&times;</span></button>'+
                        '        <h4 class="modal-title" id="demoModal3Label">'+settings.modalTitle+'</h4>'+
                        '    </div>'+
                        '    <div class="modal-body">'+
                        '        <div class="c-msg-wrap c-msg-'+type+'">'+
                        '            <i class="icon"></i>'+
                        '            <div class="content">'+
                        '                <div class="c-msg-con">'+
                        options.message+
                        '                </div>'+
                        '            </div>'+
                        '         </div>'+
                        '    </div>'+
                        '    <div class="modal-footer">'+
                        settings.buttons+
                        '    </div>'+
                        '</div>',
                    buttons: settings.btnevent || {}
                }

                newOptions.buttons.cClose={
                    selector: '.c-close',
                    eventName: 'click',
                    handler: function(e){
                        this.hide();
                    }
                };

                var settings= $.extend(true, {}, messageDefault, newOptions);
                return open(settings);
            }
        };
		return modal
	}
	this.buildApi = function(context,funs){
		var api={};
		
		function getFun(funName){
			return function(){
				return context[funName].apply(context, arguments);
			};
		}

        for(var mn in funs){
            api[mn]= getFun(funs[mn]);
        }
        return api;
	};

	this.safeApply = function(scope, fn) {
        (scope.$$phase || scope.$root.$$phase) ? fn() : scope.$apply(fn);
    };
	this.replaceParseData = function(arr){
		//控制台测试
		//s.ui.BaseControl.prototype.replaceParseData([{"value":"110000","text":"<北京","parentCode":"102","unselectable":true}])
		this._replaceParseData(arr);
		return arr;
	};
	this.restoreParseData = function(arr){
		//控制台测试
		//s.ui.BaseControl.prototype.restoreParseData([{"value":"110000","text":"&lt;北京","parentCode":"102","unselectable":true}])
		this._restoreParseData(arr);
		return arr;
	};
	this._replaceParseData = function (arr){
		if(!Array.isArray(arr)) arr=[arr];
		for(var i=0;i<arr.length;i++){
			if(typeof arr[i]!=="object"){
				arr[i]=this.replaceStr(arr[i]);
			}else{
				for(var obj in arr[i] ){
					if(typeof arr[i][obj]==="string"){
						arr[i][obj]=this.replaceStr(arr[i][obj]);
					}else{
						this._replaceParseData(arr[i][obj]);
					}
				}
			}
		}
	};
	this._restoreParseData = function (arr){
		if(!Array.isArray(arr)) arr=[arr];
		for(var i=0;i<arr.length;i++){
			if(typeof arr[i]!=="object"){
				arr[i]=this.restoreStr(arr[i]);
			}else{
				for(var obj in arr[i] ){
					if(typeof arr[i][obj]==="string"){
						arr[i][obj]=this.restoreStr(arr[i][obj]);
					}else{
						this._restoreParseData(arr[i][obj]);
					}
				}
			}

		}
	};
	this.replaceStr = function (str){
		if(typeof str!=='string') {
			return str;
		}
		var replaceMap = [
			{key:"&",value:"&amp;"},	//	&
			{key:"'",value:"&quot;"},	//	'
			{key:'"',value:"&apos;"},	//	"
			{key:"<",value:"&lt;"},		//	<
			{key:">",value:"&gt;"},		//	>
		];
		var newStr = str,reg;
		for(var i = 0; i < replaceMap.length; i++){
			reg = new RegExp(replaceMap[i].key,"g");
			newStr = newStr.replace(reg,replaceMap[i].value);
		}
		return newStr;
	};
	this.restoreStr = function (newStr){
		if(typeof newStr!=='string') {
			return newStr;
		}
		var replaceMap = [
			{key:"&",value:"&amp;"},	//	&
			{key:"'",value:"&quot;"},	//	'
			{key:'"',value:"&apos;"},	//	"
			{key:"<",value:"&lt;"},		//	<
			{key:">",value:"&gt;"},		//	>
		];
		var str = newStr,reg;
		for(var i = 0; i < replaceMap.length; i++){
			reg = new RegExp(replaceMap[i].value,"g");
			str = str.replace(reg,replaceMap[i].key);
		}
		return str;
	};

});

Jx().package("s.ui", function(J){
	
	// 严格模式
	'use strict';

	this.BaseControl = new J.Class({
		/*
		defaults:{},
		attributeMap:{},

		element:{},
		elements:{},

		buildHtml:J.emptyFun,
		bindEvent:J.emptyFun,
		*/
		
		initSettings: function(options){
			//this.settings = $.extend({}, this.defaults, this.element_data, this.parseAttributes(), options);
			var attributes = this.parseAttributes();
			this.settings = $.extend(true, {}, this.defaults, attributes, options);
		},
		replaceParseData:function(arr){
			//控制台测试
			//s.ui.BaseControl.prototype.replaceParseData([{"value":"110000","text":"<北京","parentCode":"102","unselectable":true}])
			return s.utilities.replaceParseData(arr);
		},
		restoreParseData:function(arr){
			//控制台测试
			//s.ui.BaseControl.prototype.restoreParseData([{"value":"110000","text":"&lt;北京","parentCode":"102","unselectable":true}])
			return  s.utilities.restoreParseData(arr);
		},
		/*_replaceParseData :function (arr){
			if(!Array.isArray(arr)) arr=[arr];
			for(var i=0;i<arr.length;i++){
				if(typeof arr[i]!=="object"){
					arr[i]=this.replaceStr(arr[i]);
				}else{
					for(var obj in arr[i] ){
						if(typeof arr[i][obj]==="string"){
							arr[i][obj]=this.replaceStr(arr[i][obj]);
						}else{
							this._replaceParseData(arr[i][obj]);
						}
					}
				}			
			}				
		},
		_restoreParseData :function (arr){
				if(!Array.isArray(arr)) arr=[arr];
				for(var i=0;i<arr.length;i++){
					if(typeof arr[i]!=="object"){
						arr[i]=this.restoreStr(arr[i]);
					}else{
						for(var obj in arr[i] ){
							if(typeof arr[i][obj]==="string"){
								arr[i][obj]=this.restoreStr(arr[i][obj]);
							}else{
								this._restoreParseData(arr[i][obj]);
							}
						}
					}
					
				}
				
			},
		replaceStr :function (str){
			if(typeof str!=='string') {
				return str;
			}
			var replaceMap = [
				{key:"&",value:"&amp;"},	//	&
				{key:"'",value:"&quot;"},	//	'
				{key:'"',value:"&apos;"},	//	"
				{key:"<",value:"&lt;"},		//	<
				{key:">",value:"&gt;"},		//	>
			];
			var newStr = str,reg;
			for(var i = 0; i < replaceMap.length; i++){
				reg = new RegExp(replaceMap[i].key,"g");
				newStr = newStr.replace(reg,replaceMap[i].value);
			}
			return newStr;
		},
		restoreStr :function (newStr){
			if(typeof newStr!=='string') {
				return newStr;
			}
			var replaceMap = [
				{key:"&",value:"&amp;"},	//	&
				{key:"'",value:"&quot;"},	//	'
				{key:'"',value:"&apos;"},	//	"
				{key:"<",value:"&lt;"},		//	<
				{key:">",value:"&gt;"},		//	>
			];
			var str = newStr,reg;
			for(var i = 0; i < replaceMap.length; i++){
				reg = new RegExp(replaceMap[i].value,"g");
				str = str.replace(reg,replaceMap[i].key);
			}
			return str;
		},*/
		parseAttributes: function () {
			var context=this;
			var data = {};
			$.each(this.attributeMap, function(key, value) {
				var attrName = 's-' + value + '';
				if (context.element.is('[data-' + attrName + ']')) {
					data[key] = context.element.data(attrName);
				}
			});
			return data;
		},
		destroy: function(){}
	});
});


// 统一的事件机制
// notifyEvent : function (e) {
//     if (e.type === 'dp.change' && ((e.date && e.date.isSame(e.oldDate)) || (!e.date && !e.oldDate))) {
//         return;
//     }
//     this.element.trigger(e);
// },

// viewUpdate : function (e) {
//     if (e === 'y') {
//         e = 'YYYY';
//     }
//     this.notifyEvent({
//         type: 'dp.update',
//         change: e,
//         viewDate: viewDate.clone()
//     });
// },
// this.notifyEvent({
//     type: 'dp.change',
//     date: date.clone(),
//     oldDate: oldDate
// });
/* =============================================================
 * bootstrap3-typeahead.js v3.1.0
 * https://github.com/bassjobsen/Bootstrap-3-Typeahead
 * =============================================================
 * Original written by @mdo and @fat
 * =============================================================
 * Copyright 2014 Bass Jobsen @bassjobsen
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ============================================================ */


Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        data: [], 
        items: 8, 
        menu: '<ul class="typeahead dropdown-menu" role="listbox"></ul>',
        item: '<li><a href="#" role="option"></a></li>', 
        minLength: 0, 
        scrollHeight: 0, 
        autoSelect: true, 
        afterSelect: $.noop, 
        addItem: false, 
        delay: 0,
        map: '{"text":"text","returnValue":"json"}',
        onSelected:undefined,
        matcher:undefined,
        sorter:undefined,
        template:undefined,
        nofound:'',
        nofoundWidth:150,
        highlighterCustom:undefined,
        showItemList:true,
        maxItems:0// 默认不设置指定条数出现滚动条
    };

    var attributeMap = {
        items:'items',
        minLength: 'min-length',
        delay: 'delay',
        map:'map',
        highlighterCustom:'highlighter-custom',
        showItemList:'show-item-list',
        maxItems:'max-items'
    };

    this.Typeahead = new J.Class({extend : s.ui.BaseControl}, {
        defaults: defaults,
        attributeMap: attributeMap,

        init: function(element, options){
            this.element = $(element);

            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);
            
            //this.container;
            //this.elements;

            // this.value = this.element.val();

            // 初始化选项
            this.initSettings(options);

            this.map = this.settings.map;   
            this.timeId;
            if(typeof this.settings.map === 'string' && this.settings.map !== '')
                this.map = JSON.parse(this.settings.map);

            this.matcher = this.settings.matcher || this.matcher;
            this.sorter = this.settings.sorter || this.sorter;
            this.template=this.settings.template||this.template;
            this.mouseover=this.settings.mouseover||this.mouseover;
            this.mouseout=this.settings.mouseout||this.mouseout;
            this.select = this.settings.select || this.select;
            this.autoSelect = typeof this.settings.autoSelect == 'boolean' ? this.settings.autoSelect : true;
            this.highlighter = this.settings.highlighter || this.highlighter;
            this.render = this.settings.render || this.render;
            this.updater = this.settings.updater || this.updater;
            this.displayText = this.settings.displayText || this.displayText;

            this.nofound =this.settings.nofound&&'<p style="padding:5px 5px;width:'+this.settings.nofoundWidth+'px;" >'+this.settings.nofound+'</p>';

            this.source = this.settings.source || this.settings.data;

            this.sourceMap = {};
            this.delay = this.settings.delay;

            this.buildHtml();
            this.initElements();
            this.bindEvents();
        },

        buildHtml: function(){
            var pluginId= this.element.data('plugin-id');
            var htmlTemplate = '' +
                '<div class="c-typeahead-container">' +
                '   <span class="glyphicon glyphicon-menu-down"></span>'+
                '   <ul id="c-typeahead-menu-'+pluginId+'" class="typeahead dropdown-menu">'+
                '   </ul>'+
                '</div>';
            this.container= $(htmlTemplate);
            this.element.before(this.container);
            this.container.prepend(this.element);
            if(this.element.is('input')){
                this.element.attr("autocomplete","off");
            }

        },

        initElements: function(){
            this.elements={
                view: this.element,
                menu: $('.dropdown-menu',this.container)
            };
            var pluginId= this.elements.view.data('plugin-id');
            this.elements.view.attr("data-s-dropdown-id","#c-typeahead-menu-"+pluginId);
            this.dropdown= new s.ui.controls.Dropdown(this.elements.view, {_pluginId:pluginId});
        },

        bindEvents: function () {
            this.elements.view.on('click',$.proxy(this.focusClick,this));
            this.elements.view.on('focus',$.proxy(this.focusClick,this));
            this.elements.menu.on('mouseover','li',$.proxy(this.proxyMouseover, this));
            this.elements.menu.on('mouseout','li',$.proxy(this.proxyMouseout, this));
            this.elements.menu.on('click','li',$.proxy(this.click, this))
                              .on('mouseenter', 'li', $.proxy(this.mouseenter, this));

            this.elements.view.on('keyup', $.proxy(this.keyup, this));
            this.elements.view.on('keydown', $.proxy(this.keydown, this));

            if (typeof (this.settings.onSelected) === 'function') {
                this.elements.view.on('onSelected', $.proxy(this.settings.onSelected, this));
            }

            if (typeof (this.settings.onChanged) === 'function') {
                this.elements.view.on('onChanged', this.settings.onChanged);
            }

            this.element.unbind('click.bs.dropdown');
        },
        proxyMouseover:function(e){
            var target=$(e.currentTarget);
                if(!target.is('li')||target.hasClass('disabled')) return ;
                if(typeof this.settings.mouseover=='function')
                this.settings.mouseover.apply(this,[e,target.data('value')]);
        },
        proxyMouseout:function(e){
            var target=$(e.currentTarget);
                if(!target.is('li')||target.hasClass('disabled')) return ;
                if(typeof this.settings.mouseout=='function')
                this.settings.mouseout.apply(this,[e,target.data('value')]);
        },
        focusClick: function(e){
            if($(e.target).val()==="" && this.settings.showItemList===false){
                this.elements.menu.html('');
            }else{
                // this.elements.menu.html('');
                this.lookup();
                this.setScrollTop();
            }
        },

        setScrollTop: function(e){
            var liActive = this.elements.menu.find('strong').eq(0).parents("li");
            if(liActive.length > 0){
                var top = liActive.position().top - this.elements.menu.find('li:first').position().top;
                this.elements.menu.scrollTop(top);
            }
        },

        setSource: function (source) {
            this.source = source;
            this.sourceMap = {};
            if(source){
                for (var i = 0; i < source.length; i++) {
                    var item = source[i];
                    this.sourceMap[this.displayText(item)] = item;
                }
            }
            if(this.elements.view.is(':focus'))
                this.lookup();
        },

        setValue: function(value){
            if(typeof value === 'object')
                value = this.displayText(value);
            this.elements.view.val(value);
        },

        /*
         * 关键字匹配，默认根据query查找是否存在
         * 对外公开接口
         */
        matcher: function (item) {
            var it = this.displayText(item);
            return it.toLowerCase().indexOf(this.query.toLowerCase());
        },

        /*
         * 对数组进行排序，默认是按照查询条件排序
         * 对外公开接口
         */
        sorter: function (items) {
            var beginswith = [], 
                caseSensitive = [], 
                caseInsensitive = [], 
                item;

            while ((item = items.shift())) {
                var it = this.displayText(item);
                if (!it.toLowerCase().indexOf(this.query.toLowerCase())) beginswith.push(item);
                else if (it.indexOf(this.query)) caseSensitive.push(item);
                else caseInsensitive.push(item);
            }

            return beginswith.concat(caseSensitive, caseInsensitive);
        },
        //对highlighter的html代码处理
        template:function(html){

            return html;
        },
        /*
         * 高亮显示满足query查询的结果
         * 对外公开接口
         */
        highlighter: function (item,node) {
            var html = $('<div></div>');
            var query = this.query;
            if(query === null || typeof query === 'undefined')
                return html.text(item).html();
            var i = item.toLowerCase().indexOf(query.toLowerCase());
            var len, leftPart, middlePart, rightPart, strong;
            len = query.length;
            if(len === 0){
                    return this.template(html.text(item).html(),node);
            }
            while (i > -1) {
                    leftPart = item.substr(0, i);
                    middlePart = item.substr(i, len);
                    rightPart = item.substr(i + len);
                    if(this.settings.highlighterCustom){
                        strong = $(this.settings.highlighterCustom).text(middlePart);
                    }else{
                        strong = $('<strong></strong>').text(middlePart);
                    }
                    html
                            .append(document.createTextNode(leftPart))
                            .append(strong);
                    item = rightPart;
                    i = item.toLowerCase().indexOf(query.toLowerCase());
            }
            return this.template(html.append(document.createTextNode(item)).html(),node);
        },

        displayText: function(item) {
            if(this.map.hasOwnProperty('text')){
                var text = this.map.text;
                return item[text] || item;
            }
            return item;
        },

        move: function (e) {
            if(!this.isMenuOpen())
                return;
            switch(e.keyCode) {
                case 9: // tab
                case 13: // enter
                    this.select();
                    this.dropdown.toggle(e);
                    break;
                case 27: // escape
                    e.preventDefault();
                    break;

                case 38: // up arrow
                    // with the shiftKey (this is actually the left parenthesis)
                    if (e.shiftKey) return;
                    e.preventDefault();
                    this.prev();
                    break;

                case 40: // down arrow
                    // with the shiftKey (this is actually the right parenthesis)
                    if (e.shiftKey) return;
                    e.preventDefault();
                    this.next();
                    break;
            }

            e.stopPropagation();
        },

        next: function (event) {
            var active = this.elements.menu.find('.active').removeClass('active'), 
                next = active.next();

            if (!next.length) {
                next = $(this.elements.menu.find('li')[0]);
            }

            next.addClass('active');
            this.setScrollTop();
        },

        prev: function (event) {
            var active = this.elements.menu.find('.active').removeClass('active'), 
                prev = active.prev();

            if (!prev.length) {
                prev = this.elements.menu.find('li').last();
            }

            prev.addClass('active');
            this.setScrollTop();
        },
        //keydown主要是处理 上下左右 箭头 的默认事件
         keydown: function (e) {
              switch(e.keyCode) {
                case 38: // up arrow
                    // with the shiftKey (this is actually the left parenthesis)
                    if (e.shiftKey) return;
                    e.preventDefault();
                   
                    break;

                case 40: // down arrow
                    // with the shiftKey (this is actually the right parenthesis)
                    if (e.shiftKey) return;
                    e.preventDefault();
                   
                    break;

                case 13: // enter
                    // 阻止IE中默认触发button的点击事件
                    e.preventDefault();
                    break;
              }
        },
        
        keyup: function (e) {
            var context=this;
          /*  var arr=[8,38,40];//backspace left right
            if(context.timeId){
                    window.clearTimeout(context.timeId);
                }
            if(!e.keyCode in arr){
                context.timeId=setTimeout(function(){
                    var suppressKeyPressRepeat = $.inArray(e.keyCode, [40,38,9,13,27]);
                    if (suppressKeyPressRepeat === -1 && context.elements.view.val().length>= context.settings.minLength) {
                        context.lookup();
                    }
                    else {
                        context.move(e);
                    }
                },200);
            }else{*/
                var suppressKeyPressRepeat = $.inArray(e.keyCode, [40,38,9,13,27]);
                if (suppressKeyPressRepeat === -1 && context.elements.view.val().length>= context.settings.minLength) {
                    context.lookup();
                }
                else {
                    context.move(e);
                }
           // }
            
        },

        select: function () {
            if(this.elements.menu.find('.active').hasClass("disabled"))
                return;
            var val = this.elements.menu.find('.active').data('value');
            this.elements.view.data('active', val);
            this.elements.view.val(this.displayText(val));
            this.elements.view.trigger('onSelected', [this.displayText(val),val]);
        },

        lookup: function (query) {
            var items;
            if (typeof query !== 'undefined' && query !== null) {
                this.query = query;
            } else {
                this.query = this.elements.view.val() ||  '';
            }

            var worker = $.proxy(function() {
                if($.isFunction(this.source)){
                    this.source(this.query,$.proxy(this.process,this));
                }else if(this.source){
                    this.process(this.source);
                }
                //this.process(this.source);
            }, this);

            this.elements.view.trigger('onChanged', this.query);
            clearTimeout(this.lookupWorker);
            this.lookupWorker = setTimeout(worker, this.delay);
        },

        process: function (items) {
            var context = this;

            items = $.grep(items, function (item) {
                return context.matcher(item,context.query) > -1;
            });

            items = this.sorter(items);
            if (!items || items.length === 0) {
                if(this.nofound){
                     this.elements.menu.html(this.nofound);
                }else{
                      if(this.isMenuOpen())
                        this.container.removeClass('open');
                }
                return ;
            }else{
                if(!this.isMenuOpen())
                   this.container.addClass('open');
            }
            
            if (items.length > 0) {
                this.elements.view.data('active', items[0]);
            } else {
                this.elements.view.data('active', null);
            }
            
            if (this.settings.items === 'all') {
                return this.render(items);
            } else {
                return this.render(items.slice(0, this.settings.items));
            }
        },

        render: function (items) {
            var context = this;
            var self = this;
            var activeFound = false;
            var liTemplete = '<li><a></a></li>';
            this.elements.menu.html("");
            //解决ie11的兼容问题
            //var $fragment=$(document.createDocumentFragment());
            items = $(items).map(function (i, item) {
                var text = self.displayText(item);
                i = $(liTemplete).data('value', item);
                if(item.unselectable)
                    i.addClass("disabled");
                i.find('a').html(context.highlighter(text,item));
                if (text === self.elements.view.val()) {
                    i.addClass('active');
                    self.elements.view.data('active', item);
                    activeFound = true;
                }
              //  $fragment.append(i);
               self.elements.menu.append(i);
                return i[0];
            });
            if (this.autoSelect && !activeFound) {        
                items.first().addClass('active');
                this.elements.view.data('active', items.first().data('value'));
            }
            //this.elements.menu.html($fragment);
            if(this.settings.maxItems&&!this.heightSate&&items.length>0){
                var  height=this.elements.menu.find('li').eq(0).height();
                height=height ? height : 30;
                this.elements.menu.css("maxHeight",this.settings.maxItems*height+10);
                this.heightSate=true;
            }
            this.setScrollTop();
            return this;
        },

        returnValue: function(item){
            if(this.map.hasOwnProperty('returnValue')){
                var attr = this.map.returnValue;
                if(attr === 'json'){
                    return item;
                }else{
                    return item[attr] || item;
                }
            }
            return item;
        },

        getValue: function(value){
            var item = this.sourceMap[value];
            if(typeof item === 'undefined')
                return value;
            return this.returnValue(item);
        },

        destroy : function () {
            this.elements.view.data('typeahead',null);
            this.elements.view.data('active',null);

            this.elements.menu.remove();
            
            this.elements.view.off('focus')
                              .off('keyup');

            if (typeof (this.settings.onSelected) === 'function') {
                this.elements.view.off('onSelected');
            }

            if (typeof (this.settings.onChanged) === 'function') {
                this.elements.view.off('onChanged');
            }
        },
        
        click: function (e) {
            this.select();
            this.dropdown.toggle(e);
        },

        mouseenter: function (e) {
            this.elements.menu.find('.active').removeClass('active');
            $(e.currentTarget).addClass('active');
        },

        isMenuOpen: function(){
            if(this.container.hasClass('open'))
                return true;
            return false;
        },

        enable: function() {
            //this.elements.view.removeAttr('disabled');
        },
        disable: function() {
           //this.elements.view.attr('disabled', true);
        },
         //开放接口
        getApi: function(){
            var context = this;
            var funs = {
                setSource: 'setSource',
                getValue: 'getValue',
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });
});


/* typeahead javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'typeahead';
    var pluginClass = s.ui.controls.Typeahead;

    // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
/**
* @version: 2.1.18
* @author: Dan Grossman http://www.dangrossman.info/
* @copyright: Copyright (c) 2012-2015 Dan Grossman. All rights reserved.
* @license: Licensed under the MIT license. See http://www.opensource.org/licenses/mit-license.php
* @website: https://www.improvely.com/
*/
Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    var emptyFun= function(){};
    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        minDate: '1900-01-01',
        maxDate: '2099-12-31',

        showWeekNumbers: false,  //是否显示周数
        weekLabel: 'W',
        //locale
        format: 'YYYY/MM/DD',
        daysOfWeek: moment.weekdaysMin(),               //Su_Mo_Tu_We_Th_Fr_Sa
        monthNames: moment.monthsShort(),               //Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec
        //firstDay: moment.localeData().firstDayOfWeek(),  // 0, Sunday is the first day of the week.
        firstDay:0,
        showLunar:false,
        disabledDates:[],
        onSelected: undefined,
        render: undefined
    };
    var attributeMap = {
        minDate: 'min-date',
        maxDate: 'max-date',

        showWeekNumbers: 'show-week-numbers',
        weekLabel: 'week-label',
        //locale
        firstDay: 'first-day',
        daysOfWeek: 'days-of-week',
        format: 'format',
        showLunar:'show-lunar'
    };

    this.Calendar = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // 构造函数
        init: function(element, options){
            this.element = $(element);

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);

            this.render = this.settings.render || this.render;

            this.currentCalendar = {};

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();

            // 绑定事件
            this.bindEvents();

            this.setDate();
        },
        buildHtml: function(){
            var pluginId= this.element.data('plugin-id');
            var htmlTemplate=''+
                    '<div class="c-calendar-container">'+
                        '<div class="calendar-table">'+
                        '</div>'+
                    '</div>';
            this.container= $(htmlTemplate);
            this.element.before(this.container);
            this.element.hide();

            if(typeof this.settings.daysOfWeek === 'string')
                this.settings.daysOfWeek = JSON.parse(this.settings.daysOfWeek);

            if (typeof this.settings.daysOfWeek === 'object')
                this.settings.daysOfWeek = this.settings.daysOfWeek.slice();

            if (typeof this.settings.monthNames === 'object')
                this.settings.monthNames = this.settings.monthNames.slice();

            if (typeof this.settings.minDate === 'string')
                this.settings.minDate = moment(this.settings.minDate, this.settings.format);

            if (typeof this.settings.maxDate === 'string')
                this.settings.maxDate = moment(this.settings.maxDate, this.settings.format);

            if (typeof this.settings.minDate === 'object')
                this.settings.minDate = moment(this.settings.minDate);

            if (typeof this.settings.maxDate === 'object')
                this.settings.maxDate = moment(this.settings.maxDate);

            // update day names order to firstDay
            if (this.settings.firstDay !== 0) {
                var iterator = this.settings.firstDay;
                while (iterator > 0) {
                    this.settings.daysOfWeek.push(this.settings.daysOfWeek.shift());
                    iterator--;
                }
            }
        },
        initElements: function(){
            this.elements={
                calendar: this.container,
                
                table: $('.calendar-table',this.container),
                td: $('td.available',this.container)
            };
        },
        transferAttributes: function(){
            if(this.element.prop('disabled')){
                this.disable();
            }
        },
        bindEvents: function(){
            this.container.find('.calendar-table').on('click','td',$.proxy(this.clickDate, this));
            if (typeof (this.settings.onSelected) === 'function') {
                this.container.on('onSelected', this.settings.onSelected);
            }
        },
        setMinDate:function(minDate){
            if (typeof minDate === 'string')
                this.settings.minDate = moment(minDate, this.settings.format);
            if (typeof minDate === 'object')
                this.settings.minDate = moment(minDate);
        },
        setMaxDate:function(maxDate){
            if (typeof maxDate === 'string')
                this.settings.maxDate = moment(maxDate, this.settings.format);
            if (typeof maxDate === 'object')
                this.settings.maxDate = moment(maxDate);
        },
        renderCalendar: function() {
            //
            // Build the matrix of dates that will populate the calendar
            //
            var disabledDates=this.settings.disabledDates ?  (Array.isArray(this.settings.disabledDates) ? this.settings.disabledDates : [this.settings.disabledDates])  :[];
            var calendar = this.currentCalendar;
            var month = calendar.month.month();
            var year = calendar.month.year();
            var hour = 0;
            var minute = 0;
            var second = 0;
            var daysInMonth = moment([year, month]).daysInMonth();
           
            var firstDay = moment([year, month, 1]);
            var lastDay = moment([year, month, daysInMonth]);
            var lastMonth = moment(firstDay).subtract(1, 'month').month();
            var lastYear = moment(firstDay).subtract(1, 'month').year();
            var daysInLastMonth = moment([lastYear, lastMonth]).daysInMonth();
            var dayOfWeek = firstDay.day();

            //initialize a 6 rows x 7 columns array for the calendar
            var calendar = [];
            calendar.firstDay = firstDay;
            calendar.lastDay = lastDay;

            for (var i = 0; i < 6; i++) {
                calendar[i] = [];
            }

            //populate the calendar with date objects
            var startDay = daysInLastMonth - dayOfWeek + this.settings.firstDay + 1;
            if (startDay > daysInLastMonth)
                startDay -= 7;

            if (dayOfWeek == this.settings.firstDay)
                startDay = daysInLastMonth - 6;

            var curDate = moment([lastYear, lastMonth, startDay, 12, minute, second]);

            var col, row;
            for (var i = 0, col = 0, row = 0; i < 42; i++, col++, curDate = moment(curDate).add(24, 'hour')) {
                if (i > 0 && col % 7 === 0) {
                    col = 0;
                    row++;
                }
                calendar[row][col] = curDate.clone().hour(hour).minute(minute).second(second);
                curDate.hour(12);

                if (this.settings.minDate && calendar[row][col].format('YYYY-MM-DD') == this.settings.minDate.format('YYYY-MM-DD') && calendar[row][col].isBefore(this.settings.minDate)) {
                    calendar[row][col] = this.settings.minDate.clone();
                }

                if (this.settings.maxDate && calendar[row][col].format('YYYY-MM-DD') == this.settings.maxDate.format('YYYY-MM-DD') && calendar[row][col].isAfter(this.settings.maxDate)) {
                    calendar[row][col] = this.settings.maxDate.clone();
                }

            }

            //make the calendar object available to hoverDate/clickDate
            this.currentCalendar.calendar = calendar;

            var minDate = false;
            if(this.settings.minDate)
                minDate = this.settings.minDate.clone();
            var maxDate = false;
            if(this.settings.maxDate)
                maxDate = this.settings.maxDate.clone();

            //
            // Display the calendar
            //

            var selected = this.currentCalendar.month.clone();
            var currentMonth = selected.month();
            var currentYear = selected.year();

            var selectedDate = this.currentCalendar.selectedDate&&this.currentCalendar.selectedDate.clone();

            var html = '<table class="table-condensed">';
            html += '<thead>';
            html += '<tr>';

            if (this.settings.showWeekNumbers)
                html += '<th class="week">' + this.settings.weekLabel + '</th>';

           var context = this;
            $.each(this.settings.daysOfWeek, function(index, dayOfWeek) {
                if(index === (7 - context.settings.firstDay)%7 || index === (6 - context.settings.firstDay)%7)
                    html += '<th class="weekend">' + dayOfWeek + '</th>';
                else
                    html += '<th>' + dayOfWeek + '</th>';
            });

            html += '</tr>';
            html += '</thead>';
            html += '<tbody>';

            for (var row = 0; row < 6; row++) {
                html += '<tr>';

                if (this.settings.showWeekNumbers)
                    html += '<td class="week"><div class="tdcontainer">' + calendar[row][0].week() + '</div></td>';

                for (var col = 0; col < 7; col++) {

                    var classes = [];

                     //highlight today's date
                    if (calendar[row][col].isSame(selectedDate, "day"))
                        classes.push('selected');

                    //highlight today's date
                    if (calendar[row][col].isSame(new Date(), "day"))
                        classes.push('today');

                    //这里是新增的禁用日期
                    for(var t=0;t<disabledDates.length;t++){
                        if(calendar[row][col].isSame(moment(disabledDates[t],'YYYY-MM-DD'),'day')){
                                classes.push('disable');
                        }
                    } 

                    //highlight weekends
                    if (calendar[row][col].isoWeekday() > 5)
                        classes.push('weekend');

                    //grey out the dates in other months displayed at beginning and end of this.settings calendar
                    if (calendar[row][col].month() != calendar[1][1].month())
                        classes.push('off');

                    //don't allow selection of dates before the minimum date
                    if (this.settings.minDate && calendar[row][col].isBefore(this.settings.minDate, 'day'))
                        classes.push('off', 'disabled');

                    //don't allow selection of dates after the maximum date
                    if (maxDate && calendar[row][col].isAfter(maxDate, 'day'))
                        classes.push('off', 'disabled');

                    var text = this.render(new Date(calendar[row][col]));

                    var cname = '', disabled = false;
                    //  if(/class=('|"){1,2}disable('|"){1,2}/.test(text)){
                    //      classes.push('disable');
                    // }
                    var isContain = new RegExp('disable').test(text);
                    if(isContain){
                        classes.push('disable');
                    }

                    for (var i = 0; i < classes.length; i++) {
                        cname += classes[i] + ' ';
                        if (classes[i] == 'disabled')
                            disabled = true;
                    }
                    if (!disabled)
                        cname += 'available';

                
                    html += '<td class="' + cname.replace(/^\s+|\s+$/g, '') + '" data-title="' + 'r' + row + 'c' + col + '" ><div class="tdcontainer">' + text + '</div></td>';

                }
                html += '</tr>';
            }

            html += '</tbody>';
            html += '</table>';

            this.elements.table.html(html);
        },

        prev: function() {
            this.currentCalendar.month.subtract(1, 'month');
            this.renderCalendar();
        },

        next: function() {
            this.currentCalendar.month.add(1, 'month');
            this.renderCalendar();
        },

        clickDate: function(e) {
            var target = $(e.target);
            if(!target.is('td'))
                target = target.parents('td');
            //新增的禁用
            if(target.hasClass('disable'))
                return;
          
            //if (!target.hasClass('available') || target.hasClass('selected')) 
            if (!target.hasClass('available')) 
                return;

            var title = target.attr('data-title');
            var row = title.substr(1, 1);
            var col = title.substr(3, 1);
            var date = this.currentCalendar.calendar[row][col];
            this.currentCalendar.month = date.clone();
            this.currentCalendar.selectedDate = date.clone();
            // this.container.find('td').removeClass('selected');
            // target.addClass('selected');
            this.renderCalendar();
            this.container.trigger('onSelected',date.format(this.settings.format));
        },

        setDate: function(date){
            var currentDate;
            if(!date){
                // currentDate = new Date();
                currentDate = moment();
            }else if(typeof date === 'string'){
                currentDate = moment(date, this.settings.format);
            }else if(typeof date === 'object'){
                currentDate = moment(date);
            }

            if (this.settings.minDate && currentDate.isBefore(this.settings.minDate))
                currentDate = this.settings.minDate.clone();

            if (this.settings.maxDate && currentDate.isAfter(this.settings.maxDate))
                currentDate = this.settings.maxDate.clone();

            this.currentCalendar.month = currentDate.clone();
            this.currentCalendar.selectedDate = moment(date,this.settings.format);
            this.renderCalendar();
        },

        setYear: function(year){
            this.currentCalendar.month.year(year);
            this.renderCalendar();
        },

        setMonth: function(month){
            this.currentCalendar.month.month(month);
            this.renderCalendar();
        },

        getDate: function(date){
            var date = this.currentCalendar.month.clone();
            date = date.format(this.settings.format);
            return date;
        },

        getYear: function(){

            return this.currentCalendar.month.year();
        },

        getMonth: function(){

            return this.currentCalendar.month.month()+1;
        },

        render: function(date){
            if(this.settings.showLunar){
                return this.lunarRender(date);
            }
            return date.getDate();
        },

        lunarRender: function(date){
            var fullcal = $(this).Calendar('FullCalendar',date);
            var label = (fullcal.solarFestival !== '') ? fullcal.solarFestival : (fullcal.lunarFestival !== '') ? fullcal.lunarFestival : (fullcal.solarTerm !== '') ? fullcal.solarTerm : fullcal.lDayLabel;
            return "<span class='day-number'>" + date.getDate()+ "</span><span class='day-almanac'>"+label + "</span>";
        },

        getShowDate: function(){
            if(this.currentCalendar.calendar){
                var start = this.currentCalendar.calendar[0][0];
                var end = this.currentCalendar.calendar[5][6];
                start = start.format(this.settings.format);
                end = end.format(this.settings.format);
                return [start,end];
            }
            return null;
        },

        refresh: function(){},
        destroy: function(){},

        getApi: function(){
            var context = this;
            var funs = {
                prev: 'prev',
                next: 'next',
                setDate:'setDate',
                setYear:'setYear',
                setMonth:'setMonth',
                getDate:'getDate',
                getYear:'getYear',
                getMonth:'getMonth',
                enable:'enable',
                disable:'disable',
                getShowDate:'getShowDate'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });
});

/* calendar javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'calendar';
    var pluginClass = s.ui.controls.Calendar;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };
})(jQuery);

(function ($) {
    var lunarInfo = new Array(
   0x04bd8, 0x04ae0, 0x0a570, 0x054d5, 0x0d260, 0x0d950, 0x16554, 0x056a0, 0x09ad0, 0x055d2,
   0x04ae0, 0x0a5b6, 0x0a4d0, 0x0d250, 0x1d255, 0x0b540, 0x0d6a0, 0x0ada2, 0x095b0, 0x14977,
   0x04970, 0x0a4b0, 0x0b4b5, 0x06a50, 0x06d40, 0x1ab54, 0x02b60, 0x09570, 0x052f2, 0x04970,
   0x06566, 0x0d4a0, 0x0ea50, 0x06e95, 0x05ad0, 0x02b60, 0x186e3, 0x092e0, 0x1c8d7, 0x0c950,
   0x0d4a0, 0x1d8a6, 0x0b550, 0x056a0, 0x1a5b4, 0x025d0, 0x092d0, 0x0d2b2, 0x0a950, 0x0b557,
   0x06ca0, 0x0b550, 0x15355, 0x04da0, 0x0a5d0, 0x14573, 0x052d0, 0x0a9a8, 0x0e950, 0x06aa0,
   0x0aea6, 0x0ab50, 0x04b60, 0x0aae4, 0x0a570, 0x05260, 0x0f263, 0x0d950, 0x05b57, 0x056a0,
   0x096d0, 0x04dd5, 0x04ad0, 0x0a4d0, 0x0d4d4, 0x0d250, 0x0d558, 0x0b540, 0x0b5a0, 0x195a6,
   0x095b0, 0x049b0, 0x0a974, 0x0a4b0, 0x0b27a, 0x06a50, 0x06d40, 0x0af46, 0x0ab60, 0x09570,
   0x04af5, 0x04970, 0x064b0, 0x074a3, 0x0ea50, 0x06b58, 0x055c0, 0x0ab60, 0x096d5, 0x092e0,
   0x0c960, 0x0d954, 0x0d4a0, 0x0da50, 0x07552, 0x056a0, 0x0abb7, 0x025d0, 0x092d0, 0x0cab5,
   0x0a950, 0x0b4a0, 0x0baa4, 0x0ad50, 0x055d9, 0x04ba0, 0x0a5b0, 0x15176, 0x052b0, 0x0a930,
   0x07954, 0x06aa0, 0x0ad50, 0x05b52, 0x04b60, 0x0a6e6, 0x0a4e0, 0x0d260, 0x0ea65, 0x0d530,
   0x05aa0, 0x076a3, 0x096d0, 0x04bd7, 0x04ad0, 0x0a4d0, 0x1d0b6, 0x0d250, 0x0d520, 0x0dd45,
   0x0b5a0, 0x056d0, 0x055b2, 0x049b0, 0x0a577, 0x0a4b0, 0x0aa50, 0x1b255, 0x06d20, 0x0ada0);

    var solarMonth = new Array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
    var Animals = new Array("鼠", "牛", "虎", "兔", "龙", "蛇", "马", "羊", "猴", "鸡", "狗", "猪");
    var solarTerm = new Array("小寒", "大寒", "立春", "雨水", "惊蛰", "春分", "清明", "谷雨", "立夏", "小满", "芒种", "夏至", "小暑", "大暑", "立秋", "处暑", "白露", "秋分", "寒露", "霜降", "立冬", "小雪", "大雪", "冬至");
    var sTermInfo = new Array(0, 21208, 42467, 63836, 85337, 107014, 128867, 150921, 173149, 195551, 218072, 240693, 263343, 285989, 308563, 331033, 353350, 375494, 397447, 419210, 440795, 462224, 483532, 504758);
    var nStr1 = new Array('日', '一', '二', '三', '四', '五', '六', '七', '八', '九', '十');
    var nStr2 = new Array('初', '十', '廿', '卅');
    //公历节日
    var sFtv = new Array(
    "0101 元旦",
    "0214 情人节",
    "0308 妇女节",
    "0312 植树节",
    //"0315 消费者权益日",
    "0315 三一五",
    "0401 愚人节",
    "0501 劳动节",
    "0504 青年节",
    "0512 护士节",
    "0601 儿童节",
    "0701 建党节",
    "0801 建军节",
    "0910 教师节",
    //"0928 孔子诞辰",
    "1001 国庆节",
    "1006 老人节",
    //"1024 联合国日",
    "1224 平安夜",
    "1225 圣诞节")
    //农历节日
    var lFtv = new Array(
    "0101 春节",
    "0115 元宵节",
    "0505 端午节",
    "0707 七夕",
    "0715 中元节",
    "0815 中秋节",
    "0909 重阳节",
    "1208 腊八节",
    "1224 小年");


    var month = 1;
    var methods = {
        //返回农历y年的总天数
        lYearDays: function (y) {
            var i, sum = 348;
            for (i = 0x8000; i > 0x8; i >>= 1) sum += (lunarInfo[y - 1900] & i) ? 1 : 0;
            return (sum + $(this).Calendar('leapDays', y));
        },
        //返回农历y年闰月的天数
        leapDays: function (y) {
            if ($(this).Calendar('leapMonth', y)) return ((lunarInfo[y - 1900] & 0x10000) ? 30 : 29);
            else return (0);
        },
        //判断y年的农历中那个月是闰月,不是闰月返回0
        leapMonth: function (y) {
            return (lunarInfo[y - 1900] & 0xf);
        },
        //返回农历y年m月的总天数
        monthDays: function (y, m) {
            return ((lunarInfo[y - 1900] & (0x10000 >> m)) ? 30 : 29);
        },

        //用中文显示农历的日期
        cDay: function (d) {
            var s;
            switch (d) {
                case 10:
                    s = '初十'; 
                    break;
                case 20:
                    s = '二十'; 
                    break;
                case 30:
                    s = '三十'; 
                    break;
                default:
                    s = nStr2[Math.floor(d / 10)];
                    s += nStr1[d % 10];
            }
            return (s);
        },


        //返回公历y年m+1月的天数
        solarDays: function (y, m) {
            if (m === 1)
                return (((y % 4 === 0) && (y % 100 !== 0) || (y % 400 === 0)) ? 29 : 28);
            else
                return (solarMonth[m]);
        },
        //返回某年的第n个节气为几日(从0小寒起算)
        sTerm: function (y, n) {
            var offDate = new Date((31556925974.7 * (y - 1900) + sTermInfo[n] * 60000) + Date.UTC(1900, 0, 6, 2, 5));
            return (offDate.getUTCDate())
        },


        //算出当前月第一天的农历日期和当前农历日期下一个月农历的第一天日期
        Dianaday: function (objDate) {
            objDate = new Date(objDate.getFullYear(), objDate.getMonth(), objDate.getDate());
            var ldate = {};
            var i, leap = 0, temp = 0;
            var baseDate = new Date(1900, 0, 31);
            var offset = (objDate - baseDate) / 86400000;
            ldate.dayCyl = offset + 40;
            ldate.monCyl = 14;
            for (i = 1900; i < 2050 && offset > 0; i++) {
                temp = $(this).Calendar('lYearDays', i)
                offset -= temp;
                ldate.monCyl += 12;
            }
            if (offset < 0) {
                offset += temp;
                i--;
                ldate.monCyl -= 12;
            }
            ldate.year = i;
            ldate.yearCyl = i - 1864;
            leap = $(this).Calendar('leapMonth', i); //闰哪个月
            ldate.isLeap = false;
            for (i = 1; i < 13 && offset > 0; i++) {
                if (leap > 0 && i === (leap + 1) && ldate.isLeap === false) {    //闰月
                    --i; ldate.isLeap = true; temp = $(this).Calendar('leapDays', ldate.year);
                }
                else {
                    temp = $(this).Calendar('monthDays', ldate.year, i);
                }
                if (ldate.isLeap === true && i === (leap + 1)) ldate.isLeap = false;    //解除闰月
                offset -= temp;
                if (ldate.isLeap === false) ldate.monCyl++;
            }
            if (offset === 0 && leap > 0 && i === leap + 1)
                if (ldate.isLeap) { ldate.isLeap = false; }
                else { ldate.isLeap = true; --i; --ldate.monCyl; }
            if (offset < 0) { offset += temp; --i; --ldate.monCyl; }
            ldate.month = i;
            ldate.day = offset + 1;

            baseDate = null;
            offset = null;

            return ldate;
        },

        ////全日历（公历&农历）
        FullCalendar: function (objDate) {
            var fulCal = {};
            var ldate = $(this).Calendar('Dianaday', objDate);

            //原始日期
            fulCal.origin = new Date(objDate.valueOf());
            //公历
            fulCal.sYear = objDate.getFullYear();
            fulCal.sMonth = objDate.getMonth() + 1;
            fulCal.sDay = objDate.getDate();
            fulCal.week = nStr1[objDate.getDay()];
            //农历
            fulCal.lYear = ldate.year;
            fulCal.lMonth = ldate.month;
            fulCal.lDay = ldate.day;
            fulCal.isLeap = ldate.isLeap;

            if (fulCal.lDay == 1) { //显示农历月
                var month = (fulCal.lMonth > 10 ? "十" : "") + nStr1[(fulCal.lMonth - 1) % 10 + 1];
                fulCal.lDayLabel = (fulCal.isLeap ? '闰' : '') + month + '月';//+ ($(this).Calendar("monthDays", fulCal.lYear, fulCal.lMonth) == 29 ? '小' : '大');

                month = null;
            }
            else { fulCal.lDayLabel = $(this).Calendar("cDay", fulCal.lDay); }    //显示农历日

            //节日记录
            fulCal.lunarFestival = ''; //农历节日
            fulCal.solarFestival = ''; //公历节日

            for (var ipp = 0; ipp < lFtv.length; ipp++) {    //农历节日
                if (parseInt(lFtv[ipp].substr(0, 2), 10) == (fulCal.lMonth)) {
                    if (parseInt(lFtv[ipp].substr(2, 4), 10) == (fulCal.lDay)) {
                        fulCal.lunarFestival = lFtv[ipp].substr(5);
                    }
                }
            }
            if (fulCal.lMonth == 12 && fulCal.lDay >= 29) {    //判断是否为除夕
                var nextlday = $(this).Calendar('Dianaday', new Date(objDate.getFullYear(), objDate.getMonth(), objDate.getDate() + 1));
                if (nextlday.month == 1 && nextlday.day == 1) { fulCal.lunarFestival = "除夕"; }
                nextlday = null;
            }

            for (var ipp = 0; ipp < sFtv.length; ipp++) {    //公历节日
                if (parseInt(sFtv[ipp].substr(0, 2), 10) == (fulCal.sMonth)) {
                    if (parseInt(sFtv[ipp].substr(2, 4), 10) == (fulCal.sDay)) {
                        fulCal.solarFestival = sFtv[ipp].substr(5);
                    }
                }
            }
            if (fulCal.sMonth == 5 && fulCal.week == '日' && fulCal.sDay >= 8 && fulCal.sDay <= 14) {    //母亲节
                fulCal.solarFestival = "母亲节";
            }
            if (fulCal.sMonth == 6 && fulCal.week == '日' && fulCal.sDay >= 15 && fulCal.sDay <= 21) {    //父亲节
                fulCal.solarFestival = "父亲节";
            }

            fulCal.solarTerm = ''; //节气

            var offset = (new Date(fulCal.sYear, fulCal.sMonth, 1)).getDay();
            //节气
            tmp1 = $(this).Calendar('sTerm', fulCal.sYear, fulCal.sMonth * 2 - 2);
            tmp2 = $(this).Calendar('sTerm', fulCal.sYear, fulCal.sMonth * 2 - 1);
            if (fulCal.sDay == tmp1) fulCal.solarTerm = solarTerm[fulCal.sMonth * 2 - 2];
            if (fulCal.sDay == tmp2) fulCal.solarTerm = solarTerm[fulCal.sMonth * 2 - 1];


            offset = null;
            ldate = null;

            return fulCal;
        }
    };
    //显示月视图被选日期日程
    $.fn.Calendar = function (method) {
        if (methods[method]) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        } else if (typeof method === 'object' || !method) {
            return methods.init.apply(this, arguments);
        } else {
            $.error('Method with name ' + method + ' does not exists for jQuery.Calendar');
        }
    };
})(jQuery);
Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        cascadeNames: '一级名称,二级名称,三级名称',
        dataUrl: '',
        map: {"text":"name","returnValue":"id"},
        onChanged: undefined
    };
    var attributeMap = {
        cascadeNames: 'cascade-names',
        dataUrl: 'data-url',
        map: 'map'
    };

    var CascadeMenu=new J.Class({
        init:function(elements, options, data){
            this.inputElements = elements;

            // 初始化选项
            //this.initSettings(options);
            // 直接使用地址类实例的设置
            this.settings=options;
            this.cascades=this.settings.cascadeNames.split(',');

            // 初始化数据
            //this.getData();
            this.data = data;

            // 构建html DOM
            this.buildHtml();

            // 初始化 html DOM 元素
            this.initElements();
            
            //this.transferAttributes();

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            this.bindEventsInterface();

            // 根据值，状态，设置等刷新视图
            this.refresh();
        },
        buildHtml:function(){
            var htmlTabs='';
            var htmlContents='';
            for(var i=0; i<this.cascades.length; i++){
                var cascadeName=this.cascades[i];
                htmlTabs += '<li data-s-cascade="'+ i +'" class="c-cascade-tab-' + i + '"><a href="#">' + cascadeName + '<span class="caret"></span></a></li>';
                htmlContents += '<div class="c-cascade-' + i + ' c-cascade-content"></div>';
            }

            var htmlTemplate = ''+
                '<div class="c-cascade-menu dropdown-menu">'+
                '    <div class="c-cascade-path">'+
                '        <ul class="c-cascade-tabs">'+
                htmlTabs +
                '        </ul>'+
                '    </div>'+
                htmlContents +
                '    <span class="c-cascade-close glyphicon glyphicon-remove"></span>' +
                '</div>';

            this.container=$(htmlTemplate);
            this.container.insertAfter(this.inputElements.clear);
        },
        initElements:function(){
            var context=this;

            this.elements={
                tabs: $('.c-cascade-tabs li', this.container),
                contents: $('.c-cascade-content', this.container),
                close: $('.c-cascade-close',this.container),

                getTab: function(cascadeIndex){
                    var tabSelector = '.c-cascade-tab-' + cascadeIndex;
                    return $(tabSelector, context.container);
                },
                getContent: function(cascadeIndex){
                    var contentSelector = '.c-cascade-' + cascadeIndex;
                    return $(contentSelector,context.container);
                },
                getNodes: function(cascadeIndex){
                    var nodesSelector='.c-cascade-'+ cascadeIndex +' li';
                    return $(nodesSelector,context.container);
                }
            };
        },
        grep: function(nodes, nodeId){
            for(var i=0; i<nodes.length; i++){
                if(nodeId === nodes[i].id){
                    var node=nodes[i];
                    return node;
                }
            }
        },
        _buildNodes: function(nodes, cascadeIndex){
            var htmlTemplate='<ul class="c-cascade-nodes">';
            for(var i=0; i<nodes.length; i++){
                var node= nodes[i];
                var childs=node.childs ? node.childs.length : node.childs;
                htmlTemplate += '<li data-s-id="'+node.id+'" data-s-child="'+childs +'">'+node.name+'</li>'; 
            }
            htmlTemplate+='</ul>';

            this.elements.getContent(cascadeIndex).empty().append(htmlTemplate);
        },   
        bindEvents:function(){
            var context=this;

            var elements=this.elements;

            this.inputElements.view.on('click',    $.proxy(this.show, this));
            this.inputElements.view.on('blur',     $.proxy(this.blur, this));

            this.container
                .on('mouseenter', $.proxy(this.mouseenter, this))
                .on('mouseleave', $.proxy(this.mouseleave, this));

            elements.tabs
                .on('click',function(e){
                    // $.peoxy 不能取 $(this).data('s-foo'); 
                    // 只有elements.foo是单个控件的情况下，能在这里被引用时才能使用。
                    var cascadeIndex = $(this).data('s-cascade');
                    context.activeCascadeIndex = parseInt(cascadeIndex);
                    context._activeTab();

                    e.preventDefault();
                });

            var treePath = this.getPath();
            for(var i=0; i <= treePath.length; i++){
                this._bindEventsNodes(i);
            }

            elements.close.on('click',function(e){
                context.hide();
            });
        },
        _bindEventsNodes:function(cascadeIndex){
            var context=this;

            var jqNodes=this.elements.getNodes(cascadeIndex);
            jqNodes.on('click',function(e){
                var id = $(this).data('s-id');
                var child = $(this).data('s-child');

                // 更新值
                context.change(id, context.activeCascadeIndex);
                context.refresh($(this));

                // 先点击了下级，再点击上级
                var treePath = context.getPath();
               
                if(treePath.length < context.cascades.length){
                    // 清空下一级后的 tab
                    for(var j=treePath.length; j < context.cascades.length; j++){
                        context._setTab(j, context.cascades[j]);

                    }
                    // 清空下两级后的 content
                    for(var k=treePath.length+1; k < context.cascades.length; k++){
                        context.elements.getContent(k).empty();
                    }
                    if(child==='false'||!child) {
                        // 选完最后一级隐藏 树型菜单
                        context.hide();
                    }
                }
                else{
                    // 选完最后一级隐藏 树型菜单
                    context.hide();
                }
            });
        },
        bindEventsInterface:function () {},        
        refresh: function(nodes){
            var treePath = this.getPath();
            this._refreshInputView(treePath);
            this._refreshTabs(treePath);
            this._refreshTabContents(treePath);

            // 保存当前行政级别
            if(nodes && !$(nodes).data('s-child')){
            }else{
                this.activeCascadeIndex= this.getLastestTabIndex(treePath);
            }
            this._activeTab();
        },
        _refreshTabs: function(treePath){
            var nodes=this.data;
            var node;
            for(var i=0; i<treePath.length; i++){
                var nodeId=treePath[i];
                node=this.grep(nodes, nodeId);
                if(!node) break;
                nodes=node.childs;

                this._setTab(i, node.name);
            }
        },
        _setTab:function(index, text){
            var jqActiveTab= this.elements.getTab(index);
            jqActiveTab.html('<a href="#">'+ text +'<span class="caret"></span>');
        },
        _activeTab:function(){
            this.elements.tabs.removeClass('active');
            this.elements.getTab(this.activeCascadeIndex).addClass('active');
            
            this.elements.contents.hide();
            this.elements.getContent(this.activeCascadeIndex).show();
        },
        _refreshTabContents: function(treePath){
            var nodes=this.data;
            var node;
            var lastestTabIndex= this.getLastestTabIndex(treePath);

            for(var i=0; i<lastestTabIndex+1; i++){
                this._buildNodes(nodes, i);
                this._bindEventsNodes(i);

                var nodeId=treePath[i];
                if(!nodeId){
                    break;
                }
                node=this.grep(nodes, nodeId);
                if(!node) break;
                nodes=node.childs;
            }
        },
        _refreshInputView: function(treePath){
            var nodeNames= [];
            var nodes=this.data;
            var node;
            var text = this.settings.map.text;
            for(var i=0; i<treePath.length; i++){
                var nodeId=treePath[i];
                node=this.grep(nodes, nodeId);
                if(!node) break;
                nodes=node.childs;
                nodeNames.push(node[text]);
            }
            var nodeNamePath= nodeNames.join(' / ');
            // 修改视图中的文字
            this.inputElements.view.val(nodeNamePath);
            if(nodeNamePath){
                this.inputElements.clear.show();
            }else{
                this.inputElements.clear.hide();
            }
        },
        getPath:function(){
            var initValue = this.inputElements.orginal.val();
            var treePath = initValue === '' ? [] : initValue.split(',');
            return treePath;
        },
        getLastestTabIndex: function(treePath){
            var lastestTabIndex= treePath.length === this.cascades.length ? treePath.length-1 : treePath.length;
            return lastestTabIndex;
        },
        change:function(id, cascadeIndex){
            var treePath = this.getPath();
            // 选了低级别再选高级别，树路径回滚
            while(treePath.length > cascadeIndex){
                treePath.pop();
            }
            // 选中当前ID
            treePath.push(id);
            this.inputElements.orginal.val(treePath.join(','));
            this.inputElements.orginal.trigger('onChanged', treePath.join(','));
        },
        blur: function (e) {
            var context = this;
            if (!this.mousedover) {
                setTimeout(function () { 
                    context.hide(); 
                }, 200);
            }
        }, 
        mouseenter: function (e) {
            this.mousedover = true;
        },
        mouseleave: function (e) {
            this.mousedover = false;
        },
        show: function () {
            var pos = $.extend({}, this.inputElements.view.position(), {
                height: this.inputElements.view[0].offsetHeight
            });

            this.container
                .css({
                    top: pos.top + pos.height, 
                    left: pos.left
                })
                .show();
        },
        hide: function(){
            this.container.hide();
        },
        destroy:function(){
             this.inputElements.view.off('click');
             this.inputElements.view.off('blur');
             this.container.off('mouseenter');
             this.container.off('mouseleave');
             this.container.remove();
        },
        clear: function(){
            var tabs = this.elements.tabs;
            this.inputElements.view.val('');
            this.inputElements.clear.hide();
            this.inputElements.orginal.val('');
            this.inputElements.orginal.trigger('onChanged','');
            for(var i=0; i<this.cascades.length; i++){
                var cascadeName=this.cascades[i];
                $(tabs[i]).children("a").html(cascadeName);
            }
            $(tabs[0]).trigger('click');
        }
    });

    this.Cascade = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // 构造函数
        init:function(element, options){
            // 区分一个页面中存在多个控件实例
            _currentPluginId += 1;

            this.element = $(element);
            //this.settings,

            this.container,
            this.elements,

            this.value = this.element.val();

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { return; }
            this.element.data('initialized', true);
            
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);
            this.map = this.settings.map;

            if(typeof this.settings.map === 'string' && this.settings.map !== '')
                this.map = JSON.parse(this.settings.map);

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            this.transferAttributes();

            // // 初始化数据
            // this.getData();
            var context= this;
            this.d= $.Deferred();
            $.when(this.getData(this.d))
                .done(function(){
                    context.getDataMap(context.data);
                    context.render();
                })
            // 创建 树型 菜单对象
            //this.menu=new CascadeMenu(this.elements, this.settings);

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            this.bindEventsInterface();
        },
        buildHtml:function () {
            var htmlTemplate = ''+ 
                '<div class="c-cascade-container">'+ 
                //'    <input type="hidden" />'+
                '    <input type="text" autocomplete="off"  unselectable="on"/>'+  //unselectable="on" 解决在IE有光标的bug
                '    <span class="c-cascade-clear">&times;</span>'+
                '</div>';
            this.container = $(htmlTemplate);
            this.element.before(this.container);
        },
        getData:function(d){
            if(this.settings.data){
                this.data= $.extend(true, [], this.settings.data);
                delete this.settings.data;
                d.resolve();
                return d.promise();
            }
            var context = this;

            $.ajax({
                dataType: 'json',
                url: this.settings.dataUrl,
                data: {},
                success: function(data){
                    context.data= data;

                    d.resolve();
                },
                error: function(request, status, error){
                    d.reject();
                }
            });

            return d.promise();
        },
        updateDate:function(data){
            if(!data) return ;
            if(this.menu){
                this.menu.destroy();
            }
            this.data=data;
            this.getDataMap(this.data);
            this.menu=new CascadeMenu(this.elements, this.settings, this.data);
        },
        render:function(){
            this.menu=new CascadeMenu(this.elements, this.settings, this.data);
        },
        refresh: function(){
            var context= this;
            $.when(this.d.promise())
             .done(function(){
                 context.menu&&context.menu.refresh();
             }).fail(function(){  
                 context.menu&&context.menu.refresh();//这里的异步会比 执行render方法的异步快 
             });
        },
        initElements:function () {
            this.elements = {
                orginal: this.element,
                view: $('input[type=text]', this.container),
                clear: $('.c-cascade-clear',this.container),
            };

            this.elements.orginal.hide();
            this.elements.view.prop("readonly","readonly");
        },
        bindEvents:function () {
            var context=this;
            var elements=context.elements;
            elements.clear.on('click',function(e){
                if(context.element.prop('disabled')){
                    return;
                }
                context.menu.clear();
            });
            if (typeof (this.settings.onChanged) === 'function') {
                elements.orginal.on('onChanged', this.settings.onChanged);
            }
        },        
        bindEventsInterface:function () {
        },
        transferAttributes: function(){
            this.elements.view.attr('required', this.element.attr('required'));
            this.elements.view.attr('rel', this.element.attr('rel'));
            this.elements.view.attr('title', this.element.attr('title'));
            this.elements.view.attr('class', this.element.attr('class'));
            this.elements.view.attr('tabindex', this.element.attr('tabindex'));
            this.elements.orginal.removeAttr('tabindex');
            if(this.element.prop('disabled')){
                this.disable();
            }
        },
        disable: function() {
            this.element.prop('disabled', true);
            this.elements.view.prop('disabled', true);
        }, 
        enable: function() {
            this.element.prop('disabled', false);
            this.elements.view.prop('disabled', false);
        },
        getNode: function(node){
            var newNode = $.extend(true,{},node);
            delete newNode.childs;
            this.sourceMap[node.id] = newNode;
        },
        getNodes: function(nodes){
            if(nodes && nodes.length !== 0){
                for(var i = 0; i < nodes.length;i ++){
                    this.getNode(nodes[i]);
                    this.getNodes(nodes[i].childs);
                }
            }
        },
        getDataMap: function(data){
            this.sourceMap = {};
            var nodes = [{
                childs:data
            }];
            this.getNodes(nodes);
        },
        getValue: function(){
            var value = this.elements.orginal.val();
            if(value === ''){
                return value;
            }
            var type = this.map.returnValue;
            if(type === 'id'){
                return value;
            }
            if(type === 'json'){
                var sourceMap = this.sourceMap,
                    idArr = value.split(','),
                    text = [];
                //返回：根据id到sourceMap里面查找到对象，拼接成一个新对象返回。
                for(var i=0;i<idArr.length;i++){
                    text.push(sourceMap[idArr[i]]);
                }
                return text;
            }
        },
        setValue: function(value){
            if(typeof value === 'undefined' || value === null || value === ''){
                this.elements.orginal.val('');
                this.refresh();
                return '';
            }
            var ids = [];
            if(typeof value === 'object'){
                for(var i=0;i< value.length;i++){
                    ids.push(value[i].id);
                }
            }else{
                var arr = value.split(",");
                for(var i=0;i<arr.length;i++){
                    ids.push(arr[i]);
                }
            }
            ids = ids.join(',');
            this.elements.orginal.val(ids);
            this.refresh();
            return ids;
        }
    });
});

/* cascade javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'cascade';
    var pluginClass = s.ui.controls.Cascade;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
/* ========================================================================
 * Bootstrap: collapse.js v3.3.5
 * http://getbootstrap.com/javascript/#collapse
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */
 Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        // 选项
        // fooOption: true,
        toggle: true,
        //show:true
        // 覆写 类方法
        // parseData: undefined,
        // 事件
        // onFooSelected: undefined,
        // onFooChange: function(e, data){}
    };
    var attributeMap = {
        // fooOption: 'foo-option'
         //show: 'show'
    };

    var TRANSITION_DURATION = 350;
    function getTargetFromTrigger($trigger) {
        var href;
        var target = $trigger.attr('data-s-target')
        || (href = $trigger.attr('href')) && href.replace(/.*(?=#[^\s]+$)/, ''); // strip for ie7

        return $(target);
    }

    this.Collapse = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        settings: {},
        value: '',
        data: {},
        templates: {},

        // 构造函数
        init: function(element, options){
            this.element = $(element);

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);
            this.isTrigger= this.element.hasClass('c-collapse-trigger');    // 按钮还是手风琴



            // var target= getTargetFromTrigger(this.element);
            // if(target.length === 1){
            //     // 按钮
            // }
            // else{
            //     // 手风琴
            // }



            // this.$element      = $(element)
            // this.options       = $.extend({}, Collapse.DEFAULTS, options)
            // this.$trigger      = $('[href="#' + element.id + '"],' + '[data-s-target="#' + element.id + '"]')
            // this.transitioning = null

            // if (this.options.parent) {
            //     this.$parent = this.getParent()
            // } else {
            //     this.addAriaAndCollapsedClass(this.$element, this.$trigger)
            // }

            // if (this.options.toggle) this.toggle()

            

            

            
            // 获取初始值
            // this.value = this.element.val();

            // 构建html DOM
            // this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();   

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            // this.bindEventsInterface();            
        },

        // buildHtml: function(){},
        initElements: function(){
            if(this.isTrigger){
                var target= getTargetFromTrigger(this.element);
               // this.toggle(target);
            }
            else{
                this.container= this.element;
                this.elements={
                    triggers: $('.c-collapse-trigger', this.container), 
                    targets: $('.collapse', this.container)//,
                    // original: this.element//,
                    // view: $('input[type=text]', this.container),
                    // getTab: function(levelIndex){
                    //     var tabSelector = '.c-level-tab-' + levelIndex;
                    //     return $(tabSelector, context.container);
                    // }
                };
            }
        },
        bindEvents: function(){
            if(this.isTrigger){
                this.element.on('click', $.proxy(this.onToggle, this));
            }
            else{
                this.elements.triggers.on('click', $.proxy(this.onToggle, this));
            }
        },
        bindEventsInterface: function(){
            var context=this;
            var element=this.element;

            // element.on('selected.s.template', $.proxy(this.settings.onFooClick, this));
        },
        render: function(){},
        dimension: function (target) {
            // var hasWidth = this.$element.hasClass('width')
            var hasWidth = target.hasClass('width')
            return hasWidth ? 'width' : 'height'
        },
        show: function (target) {
            // if (this.transitioning || this.$element.hasClass('in')) return
            if (this.transitioning || target.hasClass('in')) return
            
            var startEvent = $.Event('show.bs.collapse')
            target.trigger(startEvent)
            if (startEvent.isDefaultPrevented()) {
                return
            }

            if(!this.isTrigger){
                var context= this;
                this.elements.targets.each(function(){
                    context.hide($(this));
                });
            }

            var dimension = this.dimension(target)

            target
                .removeClass('collapse')
                .addClass('collapsing')[dimension](0)
                // .attr('aria-expanded', true)

            this.element
                .removeClass('collapsed')
                // .attr('aria-expanded', true)

            this.transitioning = 1

            var complete = function () {
                target
                    .removeClass('collapsing')
                    .addClass('collapse in')[dimension]('')
                target.parents(".panel").addClass("open")
                this.transitioning = 0
                target
                    .trigger('shown.bs.collapse')
            }

            if (!$.support.transition) return complete.call(this)

            var scrollSize = $.camelCase(['scroll', dimension].join('-'))

            target
                .one('bsTransitionEnd', $.proxy(complete, this))
                .emulateTransitionEnd(TRANSITION_DURATION)[dimension](target[0][scrollSize])
        },
        hide: function (target) {
            // if (this.transitioning || !this.$element.hasClass('in')) return
            if (this.transitioning || !target.hasClass('in')) return

            var startEvent = $.Event('hide.bs.collapse')
            target.trigger(startEvent)
            if (startEvent.isDefaultPrevented()) return

            var dimension = this.dimension(target)

            target[dimension](target[dimension]())[0].offsetHeight

            target
                .addClass('collapsing')
                .removeClass('collapse in')
                target.parents(".panel").removeClass("open")
                // .attr('aria-expanded', false)

            this.element
                .addClass('collapsed')
                // .attr('aria-expanded', false)

            this.transitioning = 1

            var complete = function () {
                this.transitioning = 0
                target
                    .removeClass('collapsing')
                    .addClass('collapse')
                    .trigger('hidden.bs.collapse')
            }

            if (!$.support.transition) return complete.call(this)

            target
                [dimension](0)
                .one('bsTransitionEnd', $.proxy(complete, this))
                .emulateTransitionEnd(TRANSITION_DURATION)
        },
        onToggle: function(e){
            e.preventDefault();
            var elem = $(e.target).hasClass('c-collapse-trigger') ? $(e.target) : $(e.target).parents('.c-collapse-trigger');
            var target= getTargetFromTrigger(elem);
            //var target= getTargetFromTrigger($(e.target));
            if(this.isTrigger){
                this.toggle(target);
            }
            else{
                this.show(target);
            }
        },
        toggle: function (target) {
            // this[this.$element.hasClass('in') ? 'hide' : 'show']()            
            var action= target.hasClass('in') ? 'hide' : 'show';
            this[action](target);
        },
        getParent: function () {
            return $(this.options.parent)
                .find('[data-toggle="collapse"][data-parent="' + this.options.parent + '"]')
                .each($.proxy(function (i, element) {
                    var $element = $(element)
                    this.addAriaAndCollapsedClass(getTargetFromTrigger($element), $element)
                }, this))
                .end()
        },
        addAriaAndCollapsedClass: function ($element, $trigger) {
            var isOpen = $element.hasClass('in')

            // $element.attr('aria-expanded', isOpen)
            $trigger
                .toggleClass('collapsed', !isOpen)
                // .attr('aria-expanded', isOpen)
        },

        // 事件处理
        // onFooClick: function(e, data){
        //     context= this;
        //  context.element.trigger('selected.s.template');
        // }

        // API
        refresh: function(){},
        enable: function(){},
        disable: function(){},
        destroy: function(){}
    });

});


/* template javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'collapse';
    var pluginClass = s.ui.controls.Collapse;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
       
    };
    var attributeMap = {
        // fooOption: 'foo-option'
        nofound:'nofound'
    };

    this.Combobox = new J.Class({extend : s.ui.controls.Typeahead}, {
        defaults: defaults,
        attributeMap: attributeMap,
        init: function(element, options){
            this.element = $(element);

            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);
            
            //this.container;
            //this.elements;

            // this.value = this.element.val();

            // 初始化选项
            this.initSettings(options);

            this.map = this.settings.map;
            this.matchAll=false;
           
            if(typeof this.settings.map === 'string' && this.settings.map !== '')
                this.map = JSON.parse(this.settings.map);

            this.matcher = this.settings.matcher || this.matcher;
            this.sorter = this.settings.sorter || this.sorter;
            this.template=this.settings.template||this.template;
            this.mouseover=this.settings.mouseover||this.mouseover;
            this.mouseout=this.settings.mouseout||this.mouseout;
            this.select = this.settings.select || this.select;
            this.autoSelect = typeof this.settings.autoSelect == 'boolean' ? this.settings.autoSelect : true;
            this.highlighter = this.settings.highlighter || this.highlighter;
            this.render = this.settings.render || this.render;
            this.updater = this.settings.updater || this.updater;
            this.displayText = this.settings.displayText || this.displayText;

            this.nofound =this.settings.nofound&&'<p class="nofound">'+this.settings.nofound+'</p>';

            this.source = this.settings.source || this.settings.data;
            this.clear=this.settings.clear||function(){}
            this.sourceMap = {};
            this.delay = this.settings.delay;

            this.buildHtml();
            this.initElements();
            this.bindEvents();
        },
        
        initElements: function(){
            this.elements={
                view: this.element,
                menu: $('.dropdown-menu',this.container),
                menuDwon:$('.glyphicon-menu-down',this.container),
                menuRemove:$('.glyphicon-remove',this.container)
            };
            var pluginId= this.elements.view.data('plugin-id');
            this.elements.view.attr("data-s-dropdown-id","#c-combobox-menu-"+pluginId);
            this.dropdown= new s.ui.controls.Dropdown(this.elements.view, {_pluginId:pluginId});
        },
        buildHtml: function(){
            var pluginId= this.element.data('plugin-id');
            var htmlTemplate = '' +
                '<div class="c-typeahead-container c-combobox-container">' +
                '   <span class="glyphicon glyphicon-remove hidden"></span>'+
                '   <span class="glyphicon glyphicon-menu-down "></span>'+
                '   <ul id="c-combobox-menu-'+pluginId+'" class="typeahead dropdown-menu combobox">'+
                '   </ul>'+
                '</div>';
            this.container= $(htmlTemplate);
            this.element.before(this.container);
            this.container.prepend(this.element);
            if(this.element.is('input')){
                this.element.attr("autocomplete","off");
            }

        },
        bindEvents: function () {
            this.elements.view.on('click',$.proxy(this.focusClick,this));
            this.elements.view.on('focus',$.proxy(this.focusClick,this));
            this.elements.view.on('blur',$.proxy(this.blur,this));
            this.elements.menuDwon.on('click',$.proxy(this.focusClick,this));
            this.elements.menuRemove.on('click',$.proxy(this.textRomove,this));

            this.elements.menu.on('mouseover','li',$.proxy(this.proxyMouseover, this));
            this.elements.menu.on('mouseout','li',$.proxy(this.proxyMouseout, this));
            this.elements.menu.on('click','li',$.proxy(this.click, this))
                              .on('mouseenter', 'li', $.proxy(this.mouseenter, this));

            this.elements.view.on('keyup', $.proxy(this.keyup, this));
            this.elements.view.on('keydown', $.proxy(this.keydown, this));

            if (typeof (this.settings.onSelected) === 'function') {
                this.elements.view.on('onSelected', $.proxy(this.settings.onSelected, this));
            }

            if (typeof (this.settings.onChanged) === 'function') {
                this.elements.view.on('onChanged', this.settings.onChanged);
            }
            this.element.unbind('click.bs.dropdown');
        },
        matcher: function (item) {
            var it = this.displayText(item);
            if(it===this.query){
                this.matchAll=true;
            }
            return it.toLowerCase().indexOf(this.query.toLowerCase());
        },
        lookup: function (query) {
            var items;
            if (typeof query !== 'undefined' && query !== null) {
                this.query = query;
            } else {
                this.query = this.elements.view.val() ||  '';
            }

            var worker = $.proxy(function() {
                if($.isFunction(this.source)){
                    this.source(this.query,$.proxy(this.process,this));
                }else if(this.source){
                    this.process(this.source);
                }
                //this.process(this.source);
            }, this);

            this.elements.view.trigger('onChanged', this.query);
            clearTimeout(this.lookupWorker);
            this.lookupWorker = setTimeout(worker, this.delay);
            //操作数据都会经过 lookup 方法， 这里来写 清除按钮 显隐
            if(this.query.length>0){
                this.elements.menuRemove.removeClass('hidden');
            }else{
                this.elements.menuRemove.addClass('hidden');
            }
        },
        process: function (items) {
            var context = this;
            var temItems=items;

            items = $.grep(items, function (item) {
                return context.matcher(item,context.query) > -1;
            });
            if(this.matchAll){
                items=[].concat(temItems);
              
            }
            this.matchAll=false;
            items = this.sorter(items);
            if (!items || items.length === 0) {
                if(this.nofound){
                    this.elements.menu.html(this.nofound);
                    if(!this.isMenuOpen())
                        this.container.addClass('open');
                    context.nofoundType=true;
                }else{
                    if(this.isMenuOpen())
                        this.container.removeClass('open');
                }
                return ;
            }else{
                if(!this.isMenuOpen())
                   this.container.addClass('open');
                context.nofoundType=false;
            }
            
            if (items.length > 0) {
                this.elements.view.data('active', items[0]);
            } else {
                this.elements.view.data('active', null);
            }
            
            if (this.settings.items === 'all') {
                return this.render(items);
            } else {
                return this.render(items.slice(0, this.settings.items));
            }
        },
        blur:function(){
            var context=this;
            if(this.nofoundType===true){
                context.clear();
                context.elements.menuRemove.addClass('hidden');
            }
        },
        textRomove:function(e){
            this.clear();
            this.elements.menuRemove.addClass('hidden');
            this.elements.view.focus();
        },
        select: function () {
            if(this.elements.menu.find('.active').hasClass("disabled"))
                return;
            var val = this.elements.menu.find('.active').data('value');
            this.elements.view.data('active', val);
            this.elements.view.val(this.displayText(val));
            this.matchAll=true;
            this.nofoundType=false;
           
            this.elements.menuRemove.removeClass('hidden');    
            this.elements.view.trigger('onSelected', [this.displayText(val),val]);
            
        },
        getApi: function(){
            var context = this;
            var funs = {
                setSource: 'setSource',
                getValue: 'getValue',
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });

});


/* template javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'combobox';
    var pluginClass = s.ui.controls.Combobox;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
    
Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId= 0;
    var collapseMoreTag = false;

    var defaults = {
        minHeight: 100,
        collapsed: true
    };

    var attributeMap = {
        minHeight: 'min-height',
        collapsed:'collapsed'
    };

    this.Cutpannel = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,
        // 构造函数
        init: function(element, options){
            this.element= $(element);
            this.container= this.element;
            
            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { return; }
            this.element.data('initialized', true);
            // 区分一个页面中存在多个控件实例
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);

            // // 初始化数据
            // this.getData();

            // // 构建html DOM
            // this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            //this.transferAttributes();

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            // this.bindEventsInterface();
            var height = this.settings.minHeight;
            //this.elements.content.css('height',height + 'px');
            this.reflash();
        },
        initElements: function(){
            this.elements={
                original: this.element,
                content: $('.c-cutpannel-content', this.container),  // cutpannel:this.element.parent(".c-cutpannel"),
                button: $('.c-cutpannel-button', this.container)   // cutpannerBtn: this.element.parent().next(".c-cutpannel-btn-wrap").find('.c-cutpannel-btn')
            };   
        },
        bindEvents: function(){
            this.elements.button
                .on('click', $.proxy(this.toggle, this));
            this.elements.content.bind('DOMNodeInserted',$.proxy(this.reflash, this));
        },
        reflash: function(){
            if(parseInt(this.elements.content.css('height')) >= this.settings.minHeight) {
                this.elements.button.show();
                if (this.settings.collapsed === true) {

                    this.elements.button.find(".text").text("展开");
                    this.elements.content.css('height', this.settings.minHeight + 'px');
                    this.elements.button.find(".glyphicon").removeClass("glyphicon-menu-up");
                }
                else {
                    this.elements.button.find(".text").text("收起");
                    this.elements.content.css('height', 'auto');
                    this.elements.button.find(".glyphicon").addClass("glyphicon-menu-up");
                }
            }else {
                this.elements.button.hide();
            }

        },
        
        toggle: function(){
            this.settings.collapsed = !this.settings.collapsed;
            this.reflash();
        }
    });
});


/* cutpannel javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'cutpannel';
    var pluginClass = s.ui.controls.Cutpannel;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
/**
 * @version: 2.1.18
 * @author: Dan Grossman http://www.dangrossman.info/
 * @copyright: Copyright (c) 2012-2015 Dan Grossman. All rights reserved.
 * @license: Licensed under the MIT license. See http://www.opensource.org/licenses/mit-license.php
 * @website: https://www.improvely.com/
 */
Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    var emptyFun= function(){};
    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        minDate: '1900-01-01',
        maxDate: '2099-12-31',
        timePicker: false,
        defaultTime: null,
        timePickerIncrement:5,
        showWeekNumbers: false,  //是否显示周数
        weekLabel: 'W',
        //locale
        format: 'YYYY-MM-DD',
        daysOfWeek: moment.weekdaysMin(),               //Su_Mo_Tu_We_Th_Fr_Sa
        monthNames: moment.monthsShort(),               //Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec
        //firstDay: moment.localeData().firstDayOfWeek(),  // 0, Sunday is the first day of the week.
        firstDay:0,
        position:'auto',   //down,left
        down:'dropdown',
        up:'dropup',
        scroller:false,
        dateFrom:undefined,
        change: emptyFun,
        render: undefined,
        intervalYear:20,
        calendarMode:false,  //为true时不显示文本框，用于daterangepicker
        ngMinDate:undefined,
        ngMaxDate:undefined,
        timeShowPanel:true    //为false就不会展开时分秒的提示面板
    };
    var attributeMap = {
        scroller:'scroller',
        minDate: 'min-date',
        maxDate: 'max-date',
        firstDay:'first-day',
        timePicker: 'time-picker',
        defaultTime: 'default-time',
        timePickerIncrement:'time-picker-increment',
        showWeekNumbers: 'show-week-numbers',
        weekLabel: 'week-label',
        //locale
        daysOfWeek: 'days-of-week',
        format: 'format',
        position:'position',
        timeShowPanel:'time-show-panel'
    };

    this.Datepicker = new J.Class({extend : s.ui.BaseControl}, {
        defaults: defaults,
        attributeMap: attributeMap,

        // 构造函数
        init: function (element, options) {
            this.element = $(element);
            this.initSettings(options);
            this.settings.maxDate = this.settings.ngMaxDate || this.settings.maxDate;
            this.settings.minDate = this.settings.ngMinDate || this.settings.minDate;
            _currentPluginId += 1;
            this.currentCalendar = {};

            if (typeof this.settings.daysOfWeek === 'string')
                this.settings.daysOfWeek = JSON.parse(this.settings.daysOfWeek);

            if (typeof this.settings.daysOfWeek === 'object')
                this.settings.daysOfWeek = this.settings.daysOfWeek.slice();

            if (typeof this.settings.monthNames === 'object')
                this.settings.monthNames = this.settings.monthNames.slice();

            if (typeof this.settings.minDate === 'string')
                this.settings.minDate = moment(this.settings.minDate, this.settings.format);

            if (this.settings.minDate)
                this.settings.minYear = moment(this.settings.minDate).year();

            if (this.settings.maxDate)
                this.settings.maxYear = moment(this.settings.maxDate).year();

            if (typeof this.settings.maxDate === 'string')
                this.settings.maxDate = moment(this.settings.maxDate, this.settings.format);

            if (typeof this.settings.minDate === 'object')
                this.settings.minDate = moment(this.settings.minDate);

            if (typeof this.settings.maxDate === 'object')
                this.settings.maxDate = moment(this.settings.maxDate);
            this.element.data('plugin-id', _currentPluginId);
            this.buildHtml();
            this.initElements();
            this.transferAttributes();
            this.bindEvents();
            this.setDate(new Date())

        },
        buildHtml: function () {
            var pluginId = this.element.data('plugin-id');
            var htmlTemplate = '' +
                '<div class="c-datepicker-container" style="display: none">' +
                '   <span class="glyphicon-calendar" data-s-dropdown-id="#c-datepicker-menu-' + pluginId + '"></span>' +
                '   <ul id="c-datepicker-menu-' + pluginId + '" class="datepicker dropdown-menu"> ' +
                '       <div class="calendar-control">' +
                '           <div class="icon-prev" title="上一月">' +
                '               <i class="glyphicon glyphicon-menu-left"></i>' +
                '           </div>' +
                '           <div class="calendar-year">' +
                '               <a class="btn-container"><span>2000年</span><i class="glyphicon glyphicon-menu-down"></i></a>' +
                '               <div class="dropdown-menu calendar-years">' +
                '               </div>' +
                '           </div>' +
                '           <div class="calendar-month">' +
                '               <a class="btn-container"><span>1月</span><i class="glyphicon glyphicon-menu-down"></i></a>' +
                '               <div class="dropdown-menu calendar-months">' +
                '               </div>' +
                '           </div>' +
                '           <div class="icon-next" title="下一月">' +
                '               <i class="glyphicon glyphicon-menu-right"></i>' +
                '           </div>' +
                '       </div>' +
                '       <div class="calendar-table">' +
                '           <input class="datepicker"/>' +
                '       </div>' +
                '       <div class="calendar-bottom">' +
                '           <button class="today" value="今日" type="button">今日</button> ' +
                '           <input type="text" class="timepicker t1 form-control"/>' +
                //'           <input type="text" class="timepicker t2 form-control"/>' +
                '           <button class="btn btn-primary confirm" value="确定" type="button">确定</button>' +
                '       </div>' +
                '   </ul>' +
                '</div>';
            this.container = $(htmlTemplate);
            this.element.before(this.container);
            this.container.prepend(this.element);

            this.element.attr("data-s-dropdown-id", "#c-datepicker-menu-" + pluginId);
            if (this.element.is('input')) {
                this.element.attr("autocomplete", "off");
            }
        },
        initElements: function () {
            this.elements = {
                view: this.element,
                datepicker: this.container,
                menu: $('.dropdown-menu', this.container),
                icon: $('.glyphicon-calendar', this.container),
                prev: $('.calendar-control .icon-prev', this.container),
                next: $('.calendar-control .icon-next', this.container),
                calendarYear: $('.calendar-year', this.container),
                showYear: $('.calendar-year .btn-container span', this.container),
                showYearBtn: $('.calendar-year .btn-container', this.container),
                calendarYears: $('.calendar-year .calendar-years', this.container),

                calendarMonth: $('.calendar-month', this.container),
                showMonth: $('.calendar-month .btn-container span', this.container),
                showMonthBtn: $('.calendar-month .btn-container', this.container),
                calendarMonths: $('.calendar-month .calendar-months', this.container),

                table: $('.calendar-table', this.container),
                td: $('td.available', this.container),
                timepickerInput1: $('input.timepicker.t1', this.container),
                calendarBottom: $('.calendar-bottom', this.container),

                today: $('.calendar-bottom .today', this.container),
                confirm: $('.calendar-bottom .confirm', this.container)
            };

            if (this.settings.position === 'auto') {
                this.position = this.getPosition();
            } else {
                this.position = this.settings.position;
            }
            this.autoUpPosition();
            this.autoRightPosition();
            var pluginId = this.elements.view.data('plugin-id');
            var renderDateFrom;
            if (this.settings.dateFrom && !this.settings.render) {
                var dateFromTime = new Date(this.settings.dateFrom).getTime();
                renderDateFrom = function (date) {
                    var times = date.getTime();
                    var text;
                    if (times <= dateFromTime) {
                        text = "<div class='disable'>" + date.getDate() + "</div>";
                    } else {
                        text = date.getDate();
                    }
                    return text;
                }
            }
            var options = $.extend(true, {
                render: renderDateFrom
            }, this.settings);
            this.calendar = new s.ui.controls.Calendar(this.container.find('input.datepicker'), options);

            if (this.settings.timePicker) {
                this.elements.calendarBottom.addClass('c-time');
                var format = this.settings.format.split(' '),
                    Options = {
                        timePickerIncrement: this.settings.timePickerIncrement,
                        defaultTime: this.settings.defaultTime,
                        timeShowPanel: this.settings.timeShowPanel
                    };
                if (format.length >= 1) {
                    Options.format = format[1];
                    if (Options.format == 'HH:mm:ss.SSS' || Options.format == 'HH:mm:ss') {
                        this.elements.calendarBottom.addClass('reflow');
                    }
                }
                this.timePicker = new s.ui.controls.Timepicker(this.elements.timepickerInput1, Options);

                //两个时间 todo
                /*if(this.settings.timePicker==2){
                    this.timePicker2 = new s.ui.controls.Timepicker(this.elements.timepickerInput2, Options);
                }*/
            }
            //日历模式没有文本框，适用于daterangepicker
            if(this.settings.calendarMode){
                this.elements.view.hide();
                this.elements.icon.hide().removeAttr('data-s-dropdown-id');
                this.elements.datepicker.addClass('open');
                this.renderCalendar();
                this.settings.change.bind(this);
            }else{
                this.dropdown = new s.ui.controls.Dropdown(this.elements.view, {_pluginId: pluginId});
                this.dropdown1 = new s.ui.controls.Dropdown(this.elements.icon, {_pluginId: pluginId});
            }
            this.elements.datepicker.show();
        },

        /*
         *获取日期弹出框的位置：up/down、left/right
         */
        getPosition: function () {
            var offset = this.elements.view.offset();
            var viewHeight = this.elements.view.height();
            var menuWidth = this.elements.menu.width();
            var menuHeight = this.elements.menu.height();
            var documentWidth = $(document).width();
            var documentHeight = $(document).height();
            var down = 'down', left = 'left';
            if (offset.top + viewHeight + menuHeight > documentHeight) {
                down = 'up';
            }
            if (offset.left + menuWidth > documentWidth) {
                left = 'right';
            }
            return down + ',' + left;
        },

        autoUpPosition: function () {
            var positions = this.position.split(',');
            var topPostion = positions[0];
            var leftPosition = positions[1];
            var dropCss = topPostion === 'up' ? this.settings.up : '';
            this.container.addClass(dropCss);
            this.settings.isRightPosition = false;
            if (leftPosition === 'right') {
                this.settings.isRightPosition = true;
            }
        },

        autoRightPosition: function () {
            if (this.settings.isRightPosition) {
                var viewWidth = this.elements.view.width();
                viewWidth += this.elements.icon.width();
                var menuWidth = this.elements.menu.width();
                var width = viewWidth - menuWidth;
                this.elements.menu.css('margin-left', width);
            }
        },

        transferAttributes: function () {
            if (this.element.prop('disabled')) {
                this.disable();
            }
        },

        //绑定事件
        bindEvents: function () {
            this.elements.prev.on('click', $.proxy(this.clickPrev, this));
            this.elements.next.on('click', $.proxy(this.clickNext, this));
            this.elements.datepicker.on('click', function (e) {
                e.preventDefault();
                e.stopPropagation();
            });
            this.elements.showYearBtn.on('click', $.proxy(this.showPanel, this, 'year'));
            this.elements.showMonthBtn.on('click', $.proxy(this.showPanel, this, 'month'));
            this.elements.view.on('focus', $.proxy(this.focusView, this));
            this.elements.icon.on('click', $.proxy(function () {
                this.elements.view.focus();
            }, this));
            this.elements.view.on('keyup', $.proxy(this.updateCalendar, this));
            this.elements.view.on('blur', $.proxy(this.confirmCalendar, this));
            this.container.find('.calendar-year').on('click', 'li', $.proxy(this.yearChanged, this));
            this.container.find('.calendar-month').on('click', 'li', $.proxy(this.monthChanged, this));
            this.elements.view.on('changedate', $.proxy(this.settings.change, this));

            this.elements.today.on('click', $.proxy(this.setToday, this));
            this.elements.confirm.on('click', $.proxy(this.clickApply, this));

            if (this.settings.scroller) {
                var self = this;
                var ul;
                this.container.find('.calendar-years').scroll(function (e) {
                    var target = $(e.target);
                    var top = target[0].scrollTop;
                    var height = target[0].scrollHeight;
                    var clientHeight = target[0].clientHeight;
                    if (top / height === 0) {
                        ul = target.find('ul');
                        if (ul.find('li:first-child').text() * 1 <= self.settings.minYear) return;
                        self.loadYear(ul, ul.find('li:first-child').text(), 'up');
                        target.scrollTop(26 * 10);

                    } else if (top + clientHeight === height) {
                        ul = target.find('ul');

                        if (ul.find('li:last-child').text() * 1 >= self.settings.maxYear) return;

                        self.loadYear(ul, ul.find('li:last-child').text(), 'down');
                    }
                })
            }
            var context = this;
            this.calendar.container.on('onSelected',function(e,date){
                context.currentDate = moment(date, context.settings.format);
                context.currentCalendar.month = context.currentDate;
                var selectedDate = context.currentCalendar.month.clone();
                if (!context.settings.timePicker) {
                    if(!context.settings.calendarMode){
                        context.dropdown.toggle(e);
                        context.dropdown1.toggle(e);
                    }
                } else {
                    var time = $.extend(true, {}, ["0", "0", "0", "0"], context.timePicker.getValue().split(':'));
                    var temp = time[2].split('.');
                    time[2] = temp[0];
                    time[3] = (temp[1]) ? temp[1] : "0";
                    selectedDate.hour(time[0]).minute(time[1]).second(time[2]).millisecond(time[3]);
                }
                context.setDate(selectedDate);
                context.setCurrentDate(selectedDate);
                context.showDate();
                context.getDate();
                context.container.trigger('dateSelected', selectedDate);
            });
            if(this.settings.timePicker){
                if(this.settings.calendarMode) {
                    this.timePicker.elements.view.on('timeChange', function (e, date) {
                        var selectedDate = context.currentCalendar.month.clone();
                        var time = $.extend(true, {}, ["0", "0", "0", "0"], context.timePicker.getValue().split(':'));
                        var temp = time[2].split('.');
                        time[2] = temp[0];
                        time[3] = (temp[1]) ? temp[1] : "0";
                        selectedDate.hour(time[0]).minute(time[1]).second(time[2]).millisecond(time[3]);
                        context.setDate(selectedDate);
                        context.setCurrentDate(selectedDate);
                        context.showDate();
                        context.getDate();
                        context.container.trigger('dateSelected', selectedDate);
                    });
                }
            }
        },

        setMinDate:function(minDate){
            if (minDate)
                this.settings.minYear = moment(minDate).year();
            if (typeof minDate === 'string')
                this.settings.minDate = moment(minDate, this.settings.format);
            if (typeof minDate === 'object')
                this.settings.minDate = moment(minDate);
            this.calendar.setMinDate(minDate);
        },
        setMaxDate:function(maxDate){
            if (maxDate)
                this.settings.maxYear = moment(maxDate).year();
            if (typeof maxDate === 'string')
                this.settings.maxDate = moment(maxDate, this.settings.format);
            if (typeof maxDate === 'object')
                this.settings.maxDate = moment(maxDate);
            this.calendar.setMaxDate(maxDate);
        },
        loadYear: function (ul, currentYear, point) {
            var years = this.getYearData(currentYear, point);
            var yearHtml = '';
            for (var i = 0; i < years.length; i++) {
                yearHtml += '<li value="' + years[i] + '">' + years[i] + '</li>';
            }
            if (point === 'down') {
                ul.append(yearHtml);
            } else {
                ul.find('li:first-child').before(yearHtml);
            }
        },

        getYearData: function (currentYear, point) {
            var year, minYear, maxYear, years = [];
            //   返回 上20年和后20年数据
            if (isNaN(currentYear)) {
                //如果不符合年的数据结构   默认当前年份
                year = new Date().getFullYear();
            } else {
                year = currentYear;
            }
            minYear = year * 1 - this.settings.intervalYear;
            maxYear = year * 1 + this.settings.intervalYear;
            if (minYear < this.settings.minYear) {
                minYear = this.settings.minYear;
            }
            if (maxYear > this.settings.maxYear) {
                maxYear = this.settings.maxYear;
            }
            if (point === "down") {
                for (var i = year * 1 + 1; i <= maxYear; i++) {
                    years.push(i);
                }
            } else {
                for (var i = minYear; i < year; i++) {
                    years.push(i);
                }
            }
            return years;
        },

        setDate: function (currentDate) {
            if (typeof currentDate === 'string')
                this.currentDate = moment(currentDate, this.settings.format);

            if (typeof currentDate === 'object')
                this.currentDate = moment(currentDate);

            if (typeof currentDate === 'number') {
                this.currentDate = moment(currentDate);
            }

            if (!this.settings.timePicker)
                this.currentDate = this.currentDate.startOf('day');

            if (this.settings.minDate && this.currentDate.isBefore(this.settings.minDate))
                this.currentDate = this.settings.minDate.clone();

            if (this.settings.maxDate && this.currentDate.isAfter(this.settings.maxDate))
                this.currentDate = this.settings.maxDate.clone();

            this.currentCalendar.month = this.currentDate.clone();

            if (this.settings.timePicker) {
                var hour, minute, second, millisecond;
                hour = this.currentDate.hour();
                minute = this.currentDate.minute();
                second = this.currentDate.second();
                millisecond = this.currentDate.millisecond();
                this.currentCalendar.month.hour(hour).minute(minute).second(second).millisecond(millisecond);
            }
            this.calendar.currentCalendar.month = this.currentCalendar.month;
            this.calendar.currentCalendar.selectedDate = this.currentCalendar.month;
        },

        setCurrentDate: function (date) {
            if (!date) {
                this.currentDate = null;
                return;
            }
            var currentDate;
            if (typeof date === 'string')
                currentDate = moment(date, this.settings.format);

            if (typeof date === 'object')
                currentDate = moment(date);

            if (typeof date === 'number') {
                currentDate = moment(date);
            }

            if (!this.settings.timePicker)
                currentDate = currentDate.startOf('day');

            if (this.settings.minDate && currentDate.isBefore(this.settings.minDate))
                currentDate = this.settings.minDate.clone();

            if (this.settings.maxDate && currentDate.isAfter(this.settings.maxDate))
                currentDate = this.settings.maxDate.clone();

            this.currentDate = currentDate.clone();
            this.calendar.currentDate = currentDate.clone();
        },

        setCurrentCalendar: function (date) {
            var dateisNull = false;
            if (!date) {
                date = new Date();
                dateisNull = true;
            }
            var currentDate;
            if (typeof date === 'string')
                currentDate = moment(date, this.settings.format);

            if (typeof date === 'object')
                currentDate = moment(date);

            if (typeof date === 'number') {
                currentDate = moment(date);
            }

            if (!this.settings.timePicker)
                currentDate = currentDate.startOf('day');

            if (this.settings.minDate && currentDate.isBefore(this.settings.minDate))
                currentDate = this.settings.minDate.clone();

            if (this.settings.maxDate && currentDate.isAfter(this.settings.maxDate))
                currentDate = this.settings.maxDate.clone();

            this.currentCalendar.month = currentDate.clone();
            // hack 1 如果timepicker为空，就不设置
            if (this.settings.timePicker && !dateisNull) {
                var hour = currentDate.hour() || "00",
                    minute = currentDate.minute() || "00",
                    second = currentDate.second() || "00",
                    millisecond = currentDate.millisecond() || "00";
                this.currentCalendar.month.hour(hour).minute(minute).second(second).millisecond(millisecond);

                var time = hour + ":" + minute + ":" + second + "." + millisecond;
                this.timePicker.setValue(time);

            }

            this.calendar.currentCalendar.month = this.currentCalendar.month;
            this.calendar.currentCalendar.selectedDate = this.currentCalendar.month;
        },

        showDate: function () {
            var date = null;
            if (this.currentDate) {
                date = this.currentDate.clone();
                date = date.format(this.settings.format);
            }
            this.elements.view.val(date);
        },

        setValue: function (date) {
            var temp= moment(date).format('YYYY/MM/DD');
            if(temp==='Invalid Date'||temp==='Invalid date') date='';
            this.setCurrentDate(date);
            this.setCurrentCalendar(date);
            this.showDate();
        },

        setToday: function (e) {
            var date = new Date();
            date = moment(date);
            var calendar = this.currentCalendar;
            var hour = calendar.month.hour();
            var minute = calendar.month.minute();
            var second = calendar.month.second();
            var millisecond = calendar.month.millisecond();
            date.hour(hour).minute(minute).second(second).millisecond(millisecond);
            this.setCurrentCalendar(date);
            if (!this.settings.timePicker) {
                this.clickApply(e);
            } else {
                this.updateView();
            }
        },

        confirmCalendar: function (e) {
            var value = this.elements.view.val();
            if (value.length > 0) {
                var d = this.getInputValue();
                this.setValue(d);
            } else {
                this.setCurrentDate(null);
                this.setCurrentCalendar(null);
            }
            this.getDate();
        },

        getNumResult: function () {
            return this.typeNumResult;
        },

        setNumResult: function (resultType) {
            this.typeNumResult = resultType;
        },

        updateCalendarForValue: function (value) {
            var context = this;
            if (this.timer)
                clearTimeout(this.timer);
            this.timer = setTimeout(function () {
                var mValue = context.parseInputDate(value);  // moment object
                var d = mValue || context.getMoment();
                context.setCurrentCalendar(d);
                context.updateView();
            }, 300);
        },

        updateCalendar: function (e) {
            var context = this;
            if (this.timer)
                clearTimeout(this.timer);
            this.timer = setTimeout(function () {
                var d = context.getInputValue();
                context.setCurrentCalendar(d);
                context.updateView();
            }, 300);
        },
        getMomentDate: function (value) {
            var mValue = this.parseInputDate(value);  // moment object
            var d = mValue || this.getMoment();
            if (d)
                d = d.format(this.settings.format);
            return d;
        },

        getInputValue: function () {
            var value = this.elements.view.val();
            var mValue = this.parseInputDate(value);  // moment object
            var d = mValue || this.getMoment();
            return d;
        },

        parseInputDate: function (inputDate) {
            if (moment.isMoment(inputDate) || inputDate instanceof Date) {
                inputDate = moment(inputDate);
            } else {
                inputDate = this.getMoment(inputDate);
            }
            return inputDate;
        },

        getMoment: function (d) {
            var returnMoment;

            if (typeof d === 'undefined' || d === null) {
                returnMoment = null;
            } else {
                returnMoment = moment(d, this.settings.format, false);
                if (!moment(returnMoment, this.settings.format, true).isValid())
                    returnMoment = null;
            }
            return returnMoment;
        },

        focusView: function () {
            if (this.settings.position === 'auto') {
                this.position = this.getPosition();
                this.autoUpPosition();
                this.autoRightPosition();
            }
            var d = this.getInputValue();
            this.setCurrentCalendar(d);
            this.updateView();
        },

        updateView: function (e) {
            if (this.elements.calendarYear.hasClass("active")) {
                this.elements.calendarYear.removeClass("active");
            }
            if (this.elements.calendarMonth.hasClass("active")) {
                this.elements.calendarMonth.removeClass("active");
            }
            this.elements.calendarYears.hide();
            this.elements.calendarMonths.hide();
            if (this.settings.timePicker) {
                this.timePicker.renderTimePicker();
            }
            this.renderCalendar();
        },

        renderCalendar: function () {
            this.calendar.renderCalendar();
            var calendar = this.calendar.currentCalendar;
            var month = calendar.month.month();
            var year = calendar.month.year();
            var minDate = false;
            if (this.settings.minDate)
                minDate = this.settings.minDate.clone();
            var maxDate = false;
            if (this.settings.maxDate)
                maxDate = this.settings.maxDate.clone();
            var selected = calendar.month.clone();
            var currentMonth = selected.month();
            var currentYear = selected.year();
            var maxYear = (maxDate && maxDate.year()) || this.settings.maxYear;
            var minYear = (minDate && minDate.year()) || this.settings.minYear;
            var inMinYear = currentYear == minYear;
            var inMaxYear = currentYear == maxYear;
            var daysInMonth = moment([year, month]).daysInMonth();
            var firstDay = moment([year, month, 1]);
            var lastDay = moment([year, month, daysInMonth]);

            //initialize a 6 rows x 7 columns array for the calendar
            var calendar = [];
            calendar.firstDay = firstDay;
            calendar.lastDay = lastDay;

            var monthHtml = '<ul class="monthselect">';

            if ((!minDate || minDate.isBefore(calendar.firstDay))) {
                this.elements.prev.show();
            } else {
                this.elements.prev.hide();
            }
            if ((!maxDate || maxDate.isAfter(calendar.lastDay))) {
                this.elements.next.show();
            } else {
                this.elements.next.hide();
            }

            for (var m = 0; m < 12; m++) {
                if ((!inMinYear || m >= minDate.month()) && (!inMaxYear || m <= maxDate.month())) {
                    monthHtml += "<li value='" + m + "'" +
                    (m === currentMonth ? " class='selected'" : "") +
                    ">" + (m + 1) + "</li>";
                }
            }
            monthHtml += "</ul>";
            this.elements.calendarMonths.html(monthHtml);

            if (this.settings.scroller) {
                minYear = currentYear * 1 - this.settings.intervalYear;
                maxYear = currentYear * 1 + this.settings.intervalYear;
                if (minYear < this.settings.minYear) {
                    minYear = this.settings.minYear;
                }
                if (maxYear > this.settings.maxYear) {
                    maxYear = this.settings.maxYear;
                }
                this.scrollerMinYear = minYear;
            }

            var yearHtml = '<ul class="yearselect">';
            for (var y = minYear; y <= maxYear; y++) {
                yearHtml += '<li value="' + y + '"' +
                (y === currentYear ? ' class="selected"' : '') +
                '>' + y + '</li>';
            }
            yearHtml += '</ul>';

            this.elements.calendarYears.html(yearHtml);
            var showMonth = month + 1;
            this.elements.showYear.html(year + "年");
            this.elements.showMonth.html(showMonth + "月");
        },

        yearChanged: function (e) {
            var data = parseInt(e.target.innerHTML);
            this.currentCalendar.month.year(data);
            this.elements.calendarYear.removeClass("active");
            this.elements.calendarYears.hide();
            this.monthOrYearChanged();
        },

        monthChanged: function (e) {
            var data = parseInt(e.target.innerHTML);
            data = data - 1;
            this.currentCalendar.month.month(data);
            this.elements.calendarMonth.removeClass("active");
            this.elements.calendarMonths.hide();
            this.monthOrYearChanged();
        },

        showPanel: function (type, e) {
            e.preventDefault();
            e.stopPropagation();
            this.elements.calendarYears.hide();
            this.elements.calendarMonths.hide();

            var flag = "";
            if (this.elements.calendarYear.hasClass("active")) {
                this.elements.calendarYear.removeClass("active");
                flag = 'year';
            }
            if (this.elements.calendarMonth.hasClass("active")) {
                this.elements.calendarMonth.removeClass("active");
                flag = 'month';
            }

            if (flag == type)
                return;

            var calendar = this.currentCalendar;
            var month = calendar.month.month();
            var year = calendar.month.year();

            var minYear = this.scrollerMinYear || (this.settings.minDate && this.settings.minDate.year()) || this.settings.minYear;

            var distance = year - minYear - 4;
            if (distance < 0)
                distance = 0;
            distance = distance * 26;

            var distance1 = month - 3;
            if (distance1 < 0)
                distance1 = 0;
            distance1 = distance1 * 26;

            switch (type) {
                case 'year':
                    this.elements.calendarYears.show();
                    this.elements.calendarYear.addClass("active");
                    this.elements.calendarYears.scrollTop(distance);
                    break;
                case 'month':
                    this.elements.calendarMonths.show();
                    this.elements.calendarMonth.addClass("active");
                    this.elements.calendarMonths.scrollTop(distance1);
                    break;
                default:
                // do nothing
            }
        },

        clickPrev: function (e) {
            this.calendar.currentCalendar.month.subtract(1, 'month');
            this.renderCalendar();
        },

        clickNext: function (e) {
            this.calendar.currentCalendar.month.add(1, 'month');
            this.renderCalendar();
        },

        clickApply: function (e) {
            var selectedDate = this.currentCalendar.month.clone();
            if (this.settings.timePicker) {
                var time = $.extend(true, {}, ["0", "0", "0", "0"], this.timePicker.getValue().split(':'));
                var temp = time[2].split('.');
                time[2] = temp[0];
                time[3] = (temp[1]) ? temp[1] : "0";
                selectedDate = selectedDate.clone().hour(time[0]).minute(time[1]).second(time[2]).millisecond(time[3]);
            }
            this.setCurrentDate(selectedDate);
            this.showDate();
            this.getDate();
            if(!this.settings.calendarMode){
                this.dropdown.toggle(e);
                this.dropdown1.toggle(e);
            }
        },

        getDate: function () {
            var date;
            if (this.currentDate) {
                date = this.currentDate.clone();
                date = date.format(this.settings.format);
            }
            this.elements.view.trigger('changedate', date);
        },

        monthOrYearChanged: function () {
            var calendar = this.currentCalendar;
            var month = calendar.month.month();
            var year = calendar.month.year();
            if (this.settings.minDate) {
                if (year < this.settings.minDate.year() || (year == this.settings.minDate.year() && month < this.settings.minDate.month())) {
                    month = this.settings.minDate.month();
                    year = this.settings.minDate.year();
                }
            }
            if (this.settings.maxDate) {
                if (year > this.settings.maxDate.year() || (year == this.settings.maxDate.year() && month > this.settings.maxDate.month())) {
                    month = this.settings.maxDate.month();
                    year = this.settings.maxDate.year();
                }
            }
            this.calendar.currentCalendar.month.month(month).year(year);
            this.renderCalendar();
        },

        render: function (date) {
            return date.getDate();
        },

        refresh: function () {
        },

        enable: function () {
            this.elements.view.prop('disabled', false);
            this.container.removeClass("disabled");
            this.elements.icon.removeClass("disabled");
            // this.elements.button.attr('disabled', false);
        },

        disable: function () {
            this.elements.view.prop('disabled', true);
            this.elements.icon.addClass("disabled");
            this.container.addClass("disabled");
            // this.elements.button.attr('disabled', true);
        },
        destroy: function () {
        },
        getApi: function(){
            var context = this;
            var funs = {
                getValue:'getInputValue',
                setValue:'setValue',
                clickApply:'clickApply',
                setMinDate:'setMinDate',
                setMaxDate:'setMaxDate'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });
});

/* datepicker javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'datepicker';
    var pluginClass = s.ui.controls.datepicker;

    // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };
})(jQuery);

/**
* @version: 2.1.18
* @author: Dan Grossman http://www.dangrossman.info/
* @copyright: Copyright (c) 2012-2015 Dan Grossman. All rights reserved.
* @license: Licensed under the MIT license. See http://www.opensource.org/licenses/mit-license.php
* @website: https://www.improvely.com/
*/
Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    var emptyFun= function(){};
    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        minDate: '1900-01-01',
        maxDate: '2099-12-31',
        ngMinDate:undefined,
        ngMaxDate:undefined,
        showWeekNumbers: false,  //是否显示周数
        weekLabel: 'W',
        //locale
        format: 'YYYY-MM-DD',
        daysOfWeek: moment.weekdaysMin(),               //Su_Mo_Tu_We_Th_Fr_Sa
        monthNames: moment.monthsShort(),               //Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec
        //firstDay: moment.localeData().firstDayOfWeek(),  // 0, Sunday is the first day of the week.
        firstDay:0,
        split: '至',
        replace: '...',
        options: [
           //{'text':'最近五个交易日','value':'N+7'},
            {'text':'不限','value':''},
            {'text':'今日','value':'N+0'},
            {'text':'明天','value':'S+1'}, //S 表示 今天+1 =明天
            {'text':'最近1日','value':'N-1'},
            {'text':'最近1月','value':'M-1'},
            {'text':'最近1年','value':'Y-1'}
        ],
        position:'bottom,left',
        bottom:'dropdown',
        up:'dropup',
        change: emptyFun,
        timePicker:false,
        timeShowPanel:true,    //为false就不会展开时分秒的提示面板
        mask:false,//是否启用输入辅助
        defaultTime:null,
        dateChangedLeft:null,
        dateChangedRight:null,
        renderLeft:null,//渲染接口
        renderRight:null//渲染接口
    };
    var attributeMap = {
        minDate: 'min-date',
        maxDate: 'max-date',
        //replace: 'replace',
        showWeekNumbers: 'show-week-numbers',
        weekLabel: 'week-label',
        //locale
        format: 'format',
        options: 'options',
        position:'position',
        timePicker: 'time-picker',
        timeShowPanel:'time-show-panel',
        mask:'mask',
        defaultTime:'default-time'
    };

    this.Daterangepicker = new J.Class({extend : s.ui.BaseControl}, {
        defaults: defaults,
        attributeMap: attributeMap,

        // 构造函数
        init: function (element, options) {
            this.element = $(element);

            /*
             initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
             也不在 parseAttributes 中解析，同理不加 data-s 前缀
             */
            // 防止多次初始化
            if (this.element.data('initialized')) {
                return;
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);
            this.noRestrict=false;
            // 获取选项
            this.initSettings(options);
            this.settings.maxDate = this.settings.ngMaxDate || this.settings.maxDate;
            this.settings.minDate = this.settings.ngMinDate || this.settings.minDate;
            if(options.options){
                this.settings.options = options.options;
            }
            // 获取初始值
            // this.value = this.element.val();

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            // this.transferAttributes();

            // 绑定事件
            this.bindEvents();
        },
        buildHtml: function () {
            var pluginId = this.element.data('plugin-id');
            var htmlTemplate = '' +
                '<div class="c-daterangepicker-container">' +
                '   <div class="input-group-wrap form-control" data-s-dropdown-id="#c-daterangepicker-menu-' + pluginId + '"> ' +
                '       <input value="" class="input-left"  type="text" autocomplete="off" data-s-dropdown-id="#c-daterangepicker-menu-' + pluginId + '" placeholder="' + this.settings.format + '"/> ' +
                '       <span class="split">' + this.settings.split + '</span>' +
                '       <input value="" class="input-right"  type="text" autocomplete="off" data-s-dropdown-id="#c-daterangepicker-menu-' + pluginId + '" placeholder="' + this.settings.format + '"/> ' +
                '       <i class="glyphicon glyphicon-calendar"></i>' +
                '   </div> ' +
                '   <ul id="c-daterangepicker-menu-' + pluginId + '" class="daterangepicker dropdown-menu"> ' +
                '       <div class="datepicker-each calendar-left">' +
                '           <div class="calendar-title">开始日期</div>' +
                '           <input class="dtpicker-left" />' +
                '       </div>' +
                '       <div class="datepicker-each calendar-right">' +
                '          <div class="calendar-title">结束日期</div>' +
                '           <input class="dtpicker-right" />' +
                '       </div>' +
                '       <div class="calendar-bottom">' +
                '           <div class="calendar-option">' +
                '           </div>' +
                '           <button class="btn btn-primary confirm" value="确定" type="button">确定</button>' +
                '       </div>' +
                '   </ul>' +
                '</div>';
            //this.element.after(this.container);
            this.container = $(htmlTemplate);
            this.element.before(this.container);
            // this.container.prepend(this.element);
            this.element.hide();
        },
        initElements: function () {
            this.elements = {
                original: this.element,
                daterangepicker: this.container,
                view: $('.input-group-wrap', this.container),
                inputLeftView: $('.input-left', this.container),//可输入文本框
                inputRightView: $('.input-right', this.container),
                menu: $('.dropdown-menu', this.container),
                icon: $('.glyphicon-calendar', this.container),
                dpkLeftInput: $('.dtpicker-left', this.container),//隐藏文本框
                dpkRightInput: $('.dtpicker-right', this.container),
                options: $('.calendar-option', this.container),
                confirm: $('.calendar-bottom .confirm', this.container)
            };

            var pluginId = this.elements.view.data('plugin-id');
            var self=this;
            this.dropdown = new s.ui.controls.Dropdown(this.elements.view, {_pluginId: pluginId});
            this.currentDate = {};
            var defaultTimes=this.settings.defaultTime ?  this.settings.defaultTime.split(',') : [null,null];
            var options1 = $.extend(true,{},this.settings,{
                calendarMode: true,
                defaultTime:defaultTimes[0],
                render: $.proxy(this.renderDateLeft, this),
                change: emptyFun,
                dateChanged:self.settings.dateChangedLeft,
                changedRender:function(date){
                    self.datepickerRight.setMinDate(date);
                    self.datepickerRight.renderCalendar();
                }
            });
            var options2 = $.extend(true,{},this.settings,{
                calendarMode: true,
                defaultTime:defaultTimes[1],
                render: $.proxy(this.renderDateRight, this),
                change: emptyFun,
                dateChanged:self.settings.dateChangedRight,
                changedRender:function(date){
                    self.datepickerLeft.setMaxDate(date);
                    self.datepickerLeft.renderCalendar();
                }
            });
            this.datepickerLeft = new s.ui.controls.Datepicker(this.elements.dpkLeftInput, options1);
            this.datepickerRight = new s.ui.controls.Datepicker(this.elements.dpkRightInput, options2);
            if(this.settings.timePicker){
                var str='<div class="time-label">开始时间：</div>',
                    str2='<div class="time-label">结束时间：</div>';
                this.datepickerLeft.timePicker.elements.timepicker.prepend(str);
                this.datepickerRight.timePicker.elements.timepicker.prepend(str2);
            }else{
                this.datepickerLeft.elements.calendarBottom.hide();
                this.datepickerRight.elements.calendarBottom.hide();
            }
            if (this.settings.options) {
                for (var i = 0; i < this.settings.options.length; i++) {
                    var option = this.settings.options[i];
                    var btn = '<button class="recentdays" value="' + option.text + '" type="button">' + option.text + '</button>';
                    btn = $(btn);
                    this.elements.options.prepend(btn);
                    btn.on('click', $.proxy(this.setRecentdays, this, option.value));
                }
            }
            this.autoUpPosition();
            this.autoRightPosition();
        },
        //左
        renderDateLeft:function(date){
            var thisDate = moment(date);
            if(this.currentDate.left && this.currentDate.right){
                if(thisDate.isBefore(moment(this.currentDate.right)) && thisDate.isAfter(moment(this.currentDate.left))){
                    if(this.settings.renderLeft){
                        return '<div class="selected">'+this.settings.renderLeft(date)+'</div>';
                    }
                    else{
                        return '<div class="selected">'+date.getDate()+'</div>';
                    }
                    
                }else if(thisDate.format('YYYY-MM-DD') == moment(this.currentDate.right).format('YYYY-MM-DD') || thisDate.format('YYYY-MM-DD') == moment(this.currentDate.left).format('YYYY-MM-DD')){
                    if(this.settings.renderLeft){
                        return '<div class="selected">'+this.settings.renderLeft(date)+'</div>';
                    }
                    else{
                         return '<div class="selected">'+date.getDate()+'</div>';
                    }
                   
                }
            }
            if(this.settings.renderLeft){
                        return this.settings.renderLeft(date);
                    }

            return date.getDate();
        },
        //右
        renderDateRight:function(date){
            var thisDate = moment(date);
            if(this.currentDate.left && this.currentDate.right){
                if(thisDate.isBefore(moment(this.currentDate.right)) && thisDate.isAfter(moment(this.currentDate.left))){
                    if(this.settings.renderRight){
                        return '<div class="selected">'+this.settings.renderRight(date)+'</div>';
                    }
                    else{
                        return '<div class="selected">'+date.getDate()+'</div>';
                    }
                    
                }else if(thisDate.format('YYYY-MM-DD') == moment(this.currentDate.right).format('YYYY-MM-DD') || thisDate.format('YYYY-MM-DD') == moment(this.currentDate.left).format('YYYY-MM-DD')){
                    if(this.settings.renderRight){
                        return '<div class="selected">'+this.settings.renderRight(date)+'</div>';
                    }
                    else{
                         return '<div class="selected">'+date.getDate()+'</div>';
                    }
                   
                }
            }
            if(this.settings.renderRight){
                        return this.settings.renderRight(date);
                    }
                    
            return date.getDate();
        },
        autoUpPosition: function () {
            var positions = this.settings.position.split(',');
            var topPostion = positions[0];
            var leftPosition = positions[1];
            var dropCss = topPostion === 'up' ? this.settings.up : '';
            this.elements.view.addClass(dropCss);
            this.settings.isRightPosition = false;
            if (leftPosition === 'right') {
                this.settings.isRightPosition = true;
            }
        },
        autoRightPosition: function () {
            if (this.settings.isRightPosition) {
                var viewWidth = this.elements.view.width();
                viewWidth += this.elements.icon.width();
                var menuWidth = this.elements.menu.width();
                var width = viewWidth - menuWidth;
                this.elements.menu.css('margin-left', width);
            }
        },  
        transferAttributes: function () {
        },
        bindEvents: function () {
            this.elements.inputRightView.on('keyup', $.proxy(this.setInputValue,this));
         
            this.elements.inputLeftView.on('keyup', $.proxy(this.setInputValue,this));
           
            this.elements.view.on('keyup',$.proxy(this.updateView, this));
            this.elements.view.on('changedate2', $.proxy(this.settings.change, this));
            this.elements.inputLeftView.on('blur', $.proxy(this.inputBlurLeft,this));
            this.elements.inputRightView.on('blur', $.proxy(this.inputBlurright,this));
            this.elements.confirm.on('click', $.proxy(this.clickApply, this));

            var context = this;
            this.elements.daterangepicker.on('click', function (e) {
                e.preventDefault();
                e.stopPropagation();
            });
            this.datepickerLeft.container.on('dateSelected',function(e,date,flag){
                context.currentDate.left = date;
                if(date!==""&&!moment(context.settings.minDate).isAfter(moment(date))&&!(moment(context.settings.maxDate).isBefore(moment(date)))){
                    context.datepickerRight.setMinDate(date);
                }
                var d = context.datepickerLeft.getInputValue();
                context.datepickerLeft.setValue(d);
                context.renderCalendar();
               
            });
            this.datepickerRight.container.on('dateSelected',function(e,date,flag){
                context.currentDate.right = date;
                if(date!==""&&!moment(context.settings.minDate).isAfter(moment(date))&&!(moment(context.settings.maxDate).isBefore(moment(date)))){
                    context.datepickerLeft.setMaxDate(date);
                }
                var d = context.datepickerRight.getInputValue();
                context.datepickerRight.setValue(d);
                context.renderCalendar();
               
            });
        },//新增 双向绑定最小日期
        setMinDate:function(minDate){
            this.settings.minDate=minDate;
            this.datepickerLeft.setMinDate(minDate);
            this.datepickerLeft.renderCalendar();
            //this.datepickerRight.renderCalendar();
        },
        ////新增 双向绑定最大日期
        setMaxDate:function(maxDate){
            this.settings.maxDate=maxDate;
            this.datepickerRight.setMaxDate(maxDate);
            //this.datepickerLeft.renderCalendar();
            this.datepickerRight.renderCalendar();
        },
        //重置最小最大日期
        restoreDate :function(){
            this.container.find('td');
            this.container.find('td.selected').removeClass('selected');
            this.datepickerLeft.setMaxDate(this.settings.maxDate);
            this.datepickerLeft.setMinDate(this.settings.minDate);
            this.datepickerRight.setMinDate(this.settings.minDate);
            this.datepickerRight.setMaxDate(this.settings.maxDate);
            
        },
        renderCalendar:function(){
            this.datepickerRight.renderCalendar();
            this.datepickerLeft.renderCalendar();
            
        },
        setViewValue:function(){
            this.elements.inputLeftView.val(this.elements.dpkLeftInput.val());
            this.elements.inputRightView.val(this.elements.dpkRightInput.val());
        },
        format:function(datestr){
            var len=datestr.length-1;
            switch(len) {
                case 4:
                case 7:
                        datestr=datestr.replace(new RegExp('([\\d\\/-\\s\\:]{'+len+'})'),'$1'+'-');
                        datestr=datestr.replace(/[\-\/]{2,}/g,'-');
                    break;
                case 10:
                    if(this.settings.timePicker){
                        datestr=datestr.replace(new RegExp('([\\d\\/-\\s\\:]{'+len+'})'),'$1'+' ');
                        datestr=datestr.replace(/\s{2,}/g,' ')
                    }  
                    break;
                case 13:
                case 16:
                    if(this.settings.timePicker){
                        datestr=datestr.replace(new RegExp('([\\d\\/-\\s\\:]{'+len+'})'),'$1'+':');
                        datestr=datestr.replace(/\:{2,}/g,':')
                    }
                    break;
                case 19:
                    if(this.settings.timePicker){
                        datestr=datestr.replace(new RegExp('([\\d\\/-\\s\\:]{'+len+'})'),'$1'+'.');
                        datestr=datestr.replace(/\.{2,}/g,'.')
                    }
                    break;
            }
            return  datestr;
        },
        setInputValue:function(e){
           
            if(this.settings.mask&&e&&[8,37,38,39,40].indexOf(e.keyCode)===-1) {
                var target=$(e.target);
                target.val(this.format(target.val()));
            }   
            var inputLeftView = this.elements.inputLeftView.val(),
                inputRightView = this.elements.inputRightView.val();
            var lValue = this.parseInputDate(inputLeftView) || this.getMoment(inputLeftView);  // moment object
            var rValue = this.parseInputDate(inputRightView) || this.getMoment(inputRightView);
            if(!lValue){
                this.elements.dpkLeftInput.val('');
            }else if(!rValue){
                this.elements.dpkRightInput.val('');
            }else if(lValue.isAfter(rValue)){

                return;
            }
            this.elements.dpkLeftInput.val(inputLeftView);
            this.elements.dpkRightInput.val(inputRightView);
            this.datepickerLeft.container.trigger('dateSelected', [lValue,true]);
            this.datepickerRight.container.trigger('dateSelected', [rValue,true]);
        },
       
        /* 设置最近几天
         * value形式为： N+1，最近一天，start：当天，+ ：从当前往后计算，1：一天
         * 支持规则：T+1（交易日，暂未实现），M+1（月），Y+1（年）
         */
        setRecentdays: function (value, e) {
            var rule = value.charAt(0);
            var plus = value.charAt(1);
            var offset = value.charAt(2);
            this.restoreDate();
            if (rule !== 'T') {
                if(!rule){
                    this.noRestrict=true;
                    this.setValue(["",""]);
                   // this.elements.view.trigger('changedate2', ["",""]);
                    return;
                }
                var dates=this.getNRecentdays(startDate, rule, plus, offset);
                var startDate=dates[0];
                var endDate=dates[1];
                this.elements.dpkRightInput.val(endDate.format(this.settings.format));
                this.elements.dpkLeftInput.val(startDate.format(this.settings.format));
                this.datepickerLeft.container.trigger('dateSelected', [startDate,true]);
                this.datepickerRight.container.trigger('dateSelected', [endDate,true]);
                
            }
        },

        /* 获取自然日
         *
         *
         */
        getNRecentdays: function (date, rule, plus, offset) {
            var ndate;
            offset = parseInt(offset);
            switch (rule) {
                case 'N':
                    if (plus == '+') {
                        ndate = moment(date).add(offset, 'day');
                    }
                    else {
                        ndate = moment(date).subtract(offset, 'day');
                    }
                    break;
                case 'M':
                    if (plus == '+') {
                        ndate = moment(date).add(offset, 'month');
                    }
                    else {
                        ndate = moment(date).subtract(offset, 'month');
                    }
                    break;
                case 'Y':
                    if (plus == '+') {
                        ndate = moment(date).add(offset, 'year');
                    }
                    else {
                        ndate = moment(date).subtract(offset, 'year');
                    }
                    break;
                case 'S':
                    if (plus == '+') {
                        ndate = moment(date).add(offset, 'day');
                    } else {
                        ndate = moment(date).subtract(offset, 'day');
                    }   
            }
           // return ndate;
            var startDate = moment();
           // var endDate = this.getNRecentdays(startDate, rule, plus, offset);
           if(rule==='S'){
                startDate=ndate;
           }else if(ndate.isBefore(startDate)) {
                var tmp = startDate.clone();
                startDate = ndate.clone();
                ndate = tmp.clone();
            }
            return [startDate,ndate];
        },

        /* 判断是否是日期格式
         *
         *
         */
        parseInputDate: function (inputDate) {
            if (moment.isMoment(inputDate) || inputDate instanceof Date) {
                inputDate = moment(inputDate);
            } else {
                inputDate = this.getMoment(inputDate);
            }
            return inputDate;
        },
        getMoment: function (d) {
            var returnMoment;
            if (typeof d === 'undefined' || d === null) {
                returnMoment = null;
            } else {
                returnMoment = moment(d, this.settings.format, false);
                if (!moment(returnMoment, this.settings.format, true).isValid())
                    returnMoment = null;
            }
            return returnMoment;
        },

        /* 更新calendar面板
         *
         *
         */
        updateView: function (e) {
           // if($(e.target).is('input')) return;
            var value = this.getDatepickerValue();
            this.elements.view.trigger('changedate2', [value]);
            this.noRestrict=false;
        },

        /* 点击确认后触发事件
         *
         *
         */
        clickApply: function (e) {
            var value = this.getDatepickerValue();
           /* if(value&&!value[0]&&!value[1]&&!this.noRestrict){
                var date=moment(new Date()).format('YYYY-MM-DD');
                value=[date,date];
                this.elements.dpkLeftInput.val(date);
                this.elements.dpkRightInput.val(date)
            }*/
            this.elements.view.trigger('changedate2', [value]);
            this.setViewValue();
            this.dropdown.toggle(e);
        },

        /* 设置值
         *
         *
         */
        setCalendarValue: function (value) {
            this.value = value;
            if(value) {
                if (value.length != 2)
                    return;
                var startDate = value[0];
                var endDate = value[1];
                var sdate = this.getMoment(startDate);
                if (!sdate)
                    startDate = this.settings.replace;
                var edate = this.getMoment(endDate);
                if (!edate)
                    endDate = this.settings.replace;
                if (!sdate && !edate) {
                    this.value = [null, null];
                    startDate = endDate = '';
                }
                this.elements.dpkLeftInput.val(startDate);
                this.elements.dpkRightInput.val(endDate);
                this.datepickerLeft.container.trigger('dateSelected', [startDate,true]);
                this.datepickerRight.container.trigger('dateSelected', [endDate,true]);
                
            } else {
                this.elements.dpkLeftInput.val('');
                this.elements.dpkRightInput.val('');
                this.datepickerLeft.container.trigger('dateSelected', ['',true]);
                this.datepickerRight.container.trigger('dateSelected', ['',true]);
            }
            this.setViewValue();
        },

        /* 供外部调用接口,设置值，同时刷新
         *
         *
         */
        setValue: function (value) { 
            this.restoreDate();
            this.setCalendarValue(value);
        },
        /* 返回值[左边日期,右边日期]
         *
         */
        getCaleandarValue: function () {
            var startDate = this.datepickerLeft.getInputValue();
            if (!startDate)
                startDate = null;
            else
                startDate = this.datepickerLeft.getInputValue().clone().format(this.settings.format);
            var endDate = this.datepickerRight.getInputValue();
            if (!endDate)
                endDate = null;
            else
                endDate = this.datepickerRight.getInputValue().clone().format(this.settings.format);
            var value = [startDate, endDate];
            return value;
        },
        getDatepickerValue:function(){
            this.value = this.getCaleandarValue();
            return this.value;
        },
        getValue: function () {
            return [this.elements.inputLeftView.val(),this.elements.inputRightView.val()];
        },
        renderCalendarLeft:function(){
            this.datepickerLeft.renderCalendar();
        },
        renderCalendarRight:function(){
            this.datepickerRight.renderCalendar();
        },  
        inputBlurLeft:function(){
          var left=this.elements.inputLeftView.val(); 
          var right=this.elements.inputRightView.val();
          var leftDate=moment(left); 
          var rightDate=moment(right); 
          if(leftDate.isAfter(rightDate)){
    
            this.elements.inputLeftView.val("")
            this.setViewValue();
          }
         
        
        }  ,
        inputBlurright:function(){
          var left=this.elements.inputLeftView.val(); 
          var right=this.elements.inputRightView.val();
          var leftDate=moment(left); 
          var rightDate=moment(right); 
          if(rightDate.isBefore(leftDate)){
         
            this.elements.inputRightView.val("");
            this.setViewValue();
            
          }
         
          
        }  ,
        enable: function () {
            this.elements.inputLeftView.prop('disabled', false);
            this.elements.inputRightView.prop('disabled', false);
            this.elements.view.removeClass("disabled");
            this.container.removeClass("disabled");
        },
        disable: function () {
            this.elements.inputLeftView.prop('disabled', true);
            this.elements.inputRightView.prop('disabled', true);
            this.elements.view.addClass("disabled");
            this.container.addClass("disabled");
        },
        destroy: function () {
        },

        //开放接口
        getApi: function () {
            var context = this;
            var funs = {
                setValue: 'setValue',
                getValue: 'getValue',
                renderCalendarLeft:'renderCalendarLeft',
                renderCalendarRight:'renderCalendarRight'
            };
            var api = s.utilities.buildApi(context, funs);
            return api;
        }
    });
});


/* datepicker javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'daterangepicker';
    var pluginClass = s.ui.controls.Daterangepicker;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };
})(jQuery);

/* ========================================================================
 * Bootstrap: dropdown.js v3.3.5
 * http://getbootstrap.com/javascript/#dropdowns
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */

Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        // 选项
        // fooOption: true,
        // 覆写 类方法
        // parseData: undefined,
        // 事件
        // onFooSelected: undefined,
        // onFooChange: function(e, data){}
        multiDropdown: false
    };
    var attributeMap = {
        // fooOption: 'foo-option'
        dropdownId: 'dropdown-id',
        multiDropdown: 'multi-dropdown'
    };

    // var toggle   = '[data-toggle="dropdown"]';
    //var toggle= '[data-s-dropdown-id]';
    var dropdownIds= [],unDropdownIds={};

    function clearMenus(e) {
        if (e && e.which === 3) return;
        // $(backdrop).remove()
        // $(toggle).each(function () {
        for(var i=0; i<dropdownIds.length; i++){
            // var $this         = $(this)
            var id= dropdownIds[i];
            var dropdownMenu= $(id);
            var parent       = dropdownMenu.parent();
            // var relatedTarget = { relatedTarget: this }

            if (!parent.hasClass('open')){
                continue
            }
            if (e && e.type == 'click' && /input|textarea/i.test(e.target.tagName) && $.contains(parent[0], e.target)){
                continue
            }

            if (unDropdownIds[id]){
                if((e && $.contains(parent[0], e.target)) || (parent[0] == e.target)){
                    parent.toggleClass('open');
                    continue
                }
            }
            // $parent.trigger(e = $.Event('hide.bs.dropdown', relatedTarget))

            // if (e.isDefaultPrevented()) return
            if(!unDropdownIds[id])
                parent.removeClass('open')
        }
    }

    this.Dropdown = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        settings: {},
        value: '',
        data: {},
        templates: {},

        // 构造函数
        init: function(element, options){
            this.element = $(element);
            this.element.addClass("dropdown-btn");
            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            var isInnerPlugin= options && options._pluginId;
            // 防止多次初始化
            if (!isInnerPlugin && this.element.data('initialized')) { 
                return; 
            }
            if(!isInnerPlugin){
                this.element.data('initialized', true);
                _currentPluginId += 1;
                this.element.data('plugin-id', _currentPluginId);
            }

            // 获取选项
            this.initSettings(options);
            // 获取初始值
            // this.value = this.element.val();
            if(this.element.is('a')){
                if(this.element.attr('href'))
                    this.settings.dropdownId= this.element.attr('href');
            }
            dropdownIds.push(this.settings.dropdownId);
            if(this.settings.multiDropdown){
                unDropdownIds[this.settings.dropdownId] = 1;
            }

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            // this.transferAttributes();

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            // this.bindEventsInterface();            
        },

        // 模板模式 方法
        buildHtml: function(){},
        initElements: function(){
            this.elements={
                original: this.element//,
                // view: $('input[type=text]', this.container),
                // getTab: function(levelIndex){
                //     var tabSelector = '.c-level-tab-' + levelIndex;
                //     return $(tabSelector, context.container);
                // }
            };
        },
        bindEvents: function(){
            this.element.on('click.bs.dropdown', this.toggle);
        },
        bindEventsInterface: function(){
            var context=this;
            var element=this.element;

            // element.on('selected.s.dropdown', $.proxy(this.settings.onFooClick, this));
        },
        render: function(){},

        // 事件处理
        // onFooClick: function(e, data){
        //     context= this;
        //  context.element.trigger('selected.s.template');
        // }

        // API
        toggle: function (e) {
            var $this = $(this);

            if ($this.is('.disabled, :disabled')) return;

            var $parent  = $this.parent();

            var isActive = $parent.hasClass('open');

            clearMenus(e);

            if (!isActive) {
                //var relatedTarget = { relatedTarget: this };
                // $parent.trigger(e = $.Event('show.bs.dropdown', relatedTarget));

                // if (e.isDefaultPrevented()) return;

                $this.trigger('focus');
                    // .attr('aria-expanded', 'true');
                //if($parent.find('ul').length>0&&$parent.find('ul').find('li').length>0){
                     $parent.toggleClass('open')
              //  }
                 
                    // .trigger('shown.bs.dropdown', relatedTarget);
            }

            return false;
        },

        // keydown: function (e) {
        //     if (!/(38|40|27|32)/.test(e.which) || /input|textarea/i.test(e.target.tagName)) return;

        //     var $this = $(this);

        //     e.preventDefault();
        //     e.stopPropagation();

        //     if ($this.is('.disabled, :disabled')) return;

        //     var $parent  = getParent($this);
        //     var isActive = $parent.hasClass('open');

        //     if (!isActive && e.which != 27 || isActive && e.which == 27) {
        //         if (e.which == 27) $parent.find(toggle).trigger('focus');
        //         return $this.trigger('click');
        //     }

        //     var desc = ' li:not(.disabled):visible a';
        //     var $items = $parent.find('.dropdown-menu' + desc);

        //     if (!$items.length) return;

        //     var index = $items.index(e.target);

        //     if (e.which == 38 && index > 0)                 index--;         // up
        //     if (e.which == 40 && index < $items.length - 1) index++;         // down
        //     if (!~index)                                    index = 0;

        //     $items.eq(index).trigger('focus');
        // }
        refresh: function(){},
        enable: function(){},
        disable: function(){},
        destroy: function(){}
    });

    $(document)
        .on('click.bs.dropdown.data-api', clearMenus);
    //     .on('click.bs.dropdown.data-api', '.dropdown form', function (e) { e.stopPropagation() })
    //     .on('click.bs.dropdown.data-api', toggle, this.Dropdown.prototype.toggle)
    //     .on('keydown.bs.dropdown.data-api', toggle, this.Dropdown.prototype.keydown);
        // .on('keydown.bs.dropdown.data-api', '.dropdown-menu', this.Dropdown.prototype.keydown);

});


/* dropdown javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'dropdown';
    var pluginClass = s.ui.controls.Dropdown;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package("s.ui.controls", function(J) {
    // 严格模式
    'use strict';
    // 全局变量、函数、对象
    //var currentPluginId = 0;
    var emptyFn = function(){};
    var defaults = {
        mask: 'N',
        rule: '',
        maskExtend: emptyFn,
        translation: {},
        event: {},
        email: false
    };
    var attributeMap = {
        mask: 'mask',
        email: 'email'
    };
    this.Inputmask = new J.Class({extend : s.ui.BaseControl}, {
        defaults: defaults,
        attributeMap: attributeMap,

        // 构造函数
        init: function (element, options) {
            this.element = $(element);
            this.initSettings(options);
            this.event = {};
            this.globals = {
                maskElements: 'input,td,span,div',
                dataMaskAttr: '*[data-mask]',
                dataMask: true,
                watchInterval: 300,
                watchInputs: true,
                watchDataMask: false,
                byPassKeys: [9, 16, 17, 18, 36, 37, 38, 39, 40, 91],
                translation: {
                    'N': {pattern:/\S/, recursive: true},
                    '0': {pattern: /\d/},
                    '1': {pattern: /[1-9]/},
                    '9': {pattern: /\d/, optional: true},
                    '#': {pattern: /\d/, recursive: true},
                    'A': {pattern: /[a-zA-Z0-9]/},
                    'S': {pattern: /[a-zA-Z]/},
                    'X': {pattern: /\S/},
                    'P': {pattern: /[0-9()+-]/}
                }
            };
            this.rule = this.settings.rule;
            $.extend(true, this.globals.translation, this.settings.translation);
            this.mask = {
                decimal:'',
                prefix: '',
                integer: this.settings.mask
            };
            this.extends = false;
            if(this.settings.email){
                this.bindEvent()
            }else if(this.settings.maskExtend !== emptyFn){
                this.setMask();
                this.render(true);
            }else{
                this.bindEvent();
                this.setMask();
                this.render();
            }
            //console.log(this.settings)
        },
        bindEvent: function(){
            if(this.rule && !this.settings.email){
                this.event = {
                    onKeyPress: this.onKeyPress.bind(this)
                };
            }
            this.element.on('blur', this.onBlur.bind(this));
        },
        onBlur: function(){
            if(this.settings.email){
                this.setEmail();
            }else if(this.rule.decimal){
                this.setDecimal()
            }
        },
        onKeyPress: function(cep, event, field, options){
            field.mask(this.getMask.apply(this, arguments), options)
                .val(this.changeVal(field.val(), cep));
        },
        setEmail: function(){
            var getVal = this.element.val().replace(/ /g, '');
            var atIndex = getVal.lastIndexOf('@');
            var pointIndex = getVal.lastIndexOf('.');
            if(atIndex === -1){
                getVal = getVal + '@_._'
            }else if(pointIndex < atIndex){
                getVal += (getVal.substring(atIndex + 1, getVal.length) === '') ? '_._' : '._'
            }else if(getVal.substring(atIndex + 1, pointIndex) === ''){
                getVal = getVal.substring(0, atIndex + 1) + '_' + getVal.substring(pointIndex, getVal.length)
            }else if(getVal.substring(pointIndex + 1, getVal.length) === ''){
                getVal += '_'
            }
            if(getVal.lastIndexOf('@') === 0){
                getVal = '_' + getVal
            }
            this.element.val(getVal)
        },
        integerMask: function(val){
            var len = val.replace(/\D/g, '').length;
            var integerMask = '9';
            var styleObj = {
                false: false,
                'wan': 4,
                'thousand': 3,
                'hundred':2
            };
            if(len === 0){
                return integerMask
            }
            for(var i = 0; i < len; i++){
                if(i % styleObj[this.rule.numeralStyle] === 0 && i !== 0){
                    integerMask = '0,' + integerMask
                }else{
                    integerMask = '0' + integerMask
                }
                if (i + 1 === this.rule.maxLength && this.rule.maxLength) {
                    return integerMask.substring(0,  integerMask.length - 1)
                }
            }
            return integerMask
        },
        decimalMask: function(){
            var decimalMask = '.';
            for(var i = 0; i < this.rule.decimal; i++){
                decimalMask = decimalMask + '0';
            }
            return decimalMask
        },
        prefixMask: function(){
            var prefixMask = '';
            for(var i = 0; i < this.rule.prefix.length; i++){
                prefixMask += 'X'
            }
            return prefixMask
        },
        setMask: function(){
            if(this.rule.decimal){
                this.mask.decimal = this.decimalMask();
            }
            if(this.rule.prefix){
                this.mask.prefix = this.prefixMask();
            }
        },
        getMask: function(val){
            if(this.rule.decimal){
                val = val.split('.')[0]
            }
            if(this.rule.prefix){
                var prefix = this.rule.prefix;
                if(val.substring(0, prefix.length) !== prefix) {
                    val = this.unPrefix(val);
                }else{
                    val = val.replace(prefix, '');
                }
            }
            if(this.rule){
                this.mask.integer = this.integerMask(val);
            }
            //console.log(this.mask.prefix + this.mask.integer + this.mask.decimal , val);
            return this.mask.prefix + this.mask.integer + this.mask.decimal
        },
        setDecimal: function(){
            var getVal = this.element.val();
            var getDecimal = getVal.split('.')[1] || '';
            var prefix = this.rule.prefix || '';
            if(prefix){
                getVal = getVal.replace(prefix, '');
            }

            if(!getVal){
                getDecimal = prefix
            }else{
                for(var i = 0; i < this.rule.decimal; i++){
                    if(!getDecimal.charAt(i)){
                        getDecimal += '0'
                    }
                }
                getVal = getVal.split('.')[0] || '0';
                getDecimal = prefix + getVal + '.' + getDecimal
            }
            this.element.val(getDecimal)

        },
        changeVal: function(val, cep){
            var getDecimal = '';
            if(this.rule.decimal && val.indexOf('.') !== -1){
                getDecimal = (val.split('.')[1]) ?  '.' + val.split('.')[1] : '.';
            }
            val = val.split('.')[0];
            if(this.rule.prefix){
                var prefix = this.rule.prefix;
                if(!val){
                    return prefix
                }
                if(val.substring(0, prefix.length) !== prefix){
                    val = prefix + this.unPrefix(val);
                }
            }
            return val + getDecimal
        },
        unPrefix: function(val){
            var prefix = this.rule.prefix;
            var getPrefix = val.substring(0, prefix.length);
            var index;
            for(var i = 0; i < prefix.length; i++){
                if(getPrefix.charAt(i) !== prefix.charAt(i)){
                    index = i;
                    break
                }
            }
            return val.substring(index, val.length)
        },
        setValue: function(val){
            var mask;
            if(this.settings.email){
                mask = 'N'
            }else if(this.settings.maskExtend !== emptyFn){
                mask = this.settings.maskExtend.bind(this).apply(this, arguments)
            }else{
                val = this.changeVal(val);
                mask = this.getMask.bind(this).apply(this, arguments)
            }
            this.element.val(val)
                .mask(mask, $.extend({}, this.event, this.globals))
                .val(this.element.masked(val));
            if(this.settings.email){
                this.setEmail()
            }else if(this.rule.decimal && this.settings.maskExtend == emptyFn){
                this.setDecimal()
            }
        },
        getValue: function(){
            return this.element.val()
        },
        render: function(extend){
            var val = this.element.val();
            var mask = (extend) ? this.settings.maskExtend.bind(this) : this.getMask.bind(this);
            if(this.rule.prefix && !extend){
                val = this.rule.prefix + val
            }
            $.extend(true, this.event, this.settings.event);
            this.element.val(val)
                .mask(mask, $.extend({}, this.event, this.globals));
        }
    })
});


/* template javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'inputmask';
    var pluginClass = s.ui.controls.Inputmask;

    // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });
        return this;
    };
})(jQuery);
/**
 * Created by zbfeng.oth on 2017/8/14.
 */
Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        // 选项
        checkedIcon: 'glyphicon glyphicon-check',
        uncheckedIcon: 'glyphicon glyphicon-unchecked',
    };
    var attributeMap = {
    };
    //
    var enumCheckState= {
        check: 1,     // 勾选
        unCheck: -1    // 取消勾选
    };
    this.InputMultiSelect = new J.Class({extend : s.ui.controls.Typeahead}, {
        defaults: defaults,
        attributeMap: attributeMap,

        // 构造函数
        init: function (element, options) {
            this.element = $(element);


            /*initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
             也不在 parseAttributes 中解析，同理不加 data-s 前缀*/

            // 防止多次初始化
            if (this.element.data('initialized')) {
                return;
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);
            //this.settings.width=this.element.width()&&this.element.width()>=248 ? this.element.width():this.settings.width;

            this.map = this.settings.map;
            if (typeof this.settings.map === 'string' && this.settings.map !== '')
                this.map = JSON.parse(this.settings.map);

            this.matcher = this.settings.matcher || this.matcher;
            this.sorter = this.settings.sorter || this.sorter;
            this.template = this.settings.template || this.template;
            this.mouseover = this.settings.mouseover || this.mouseover;
            this.mouseout = this.settings.mouseout || this.mouseout;
            this.select = this.settings.select || this.select;
            this.autoSelect = typeof this.settings.autoSelect == 'boolean' ? this.settings.autoSelect : true;
            this.highlighter = this.settings.highlighter || this.highlighter;
            this.render = this.settings.render || this.render;
            this.updater = this.settings.updater || this.updater;
            this.displayText = this.settings.displayText || this.displayText;

            this.nofound = this.settings.nofound && '<p style="padding:5px 5px;width:' + this.settings.nofoundWidth + 'px;" >' + this.settings.nofound + '</p>';

            this.source = this.settings.data;
            this.sourceMap = {};
            this.delay = this.settings.delay;

            this.chooseItems = [];
            this.selectedItems = []; //选中了变蓝色准备删除的元素
            this.buildHtml();
            this.initElements();
            this.transferAttributes();

            //this.settings.inputWidth=this.elements.view.width();

            this.bindEvents();
            this.bindEventsEx();
        },
        buildHtml: function () {
            this.element.hide();
            var pluginId = this.element.data('plugin-id');
            var htmlTemplate = '' +
                '<div class="c-input-multi-select-container">' +
                '   <div class="c-input-multi-select-panel form-control">' +
                '       <span class="glyphicon glyphicon-menu-down"></span>'+
                '       <div class="c-ims-placeholder">请输入..</div>' +
                '       <div class="c-ims-textlist" title=""></div>' +
                '       <dl class="c-ims-list">' +
                //'           <dd class="c-ims-item">' +
                //'               <span class="c-ims-name">testvalue1</span>' +
                //'               <span class="c-ims-delete">x</span>' +
                //'           </dd>' +
                '       <input class="c-ims-input" data-s-dropdown-id="#c-input-multi-select-menu-'+ pluginId+'"/>' +
                '       </dl>' +
                //                                      '+(this.settings.width-28)+'
                '   </div>' +
                '   <ul id="c-input-multi-select-menu-' + pluginId + '" class="typeahead dropdown-menu">' +
                '   </ul>' +
                '</div>';
            this.container = $(htmlTemplate);
            this.element.before(this.container);

            if (this.element.is('input')) {
                this.element.attr("autocomplete", "off");
            }
        },
        initElements: function(){
            this.elements={
                view:  $('.c-ims-input',this.container),
                panel: $('.c-input-multi-select-panel',this.container),
                placeholder:$('.c-ims-placeholder',this.container),
                menu: $('.dropdown-menu',this.container),
                dl: $('.c-ims-list',this.container),
                textlist:$('.c-ims-textlist',this.container),
            };
            var pluginId= this.elements.view.data('plugin-id');
            //this.elements.view.attr("data-s-dropdown-id", "#c-input-multi-select-menu-" + pluginId);
            this.dropdown= new s.ui.controls.Dropdown(this.elements.view, {_pluginId:pluginId});
        },
        transferAttributes: function(){
            this.elements.placeholder.html(this.element.attr('placeholder'));
            if (this.element.attr('disabled')!==undefined){
                this.disable();
            }
        },
        bindEventsEx:function(){
            this.elements.panel.on('click',$.proxy(this.clickView,this));
            this.elements.view.on('blur',$.proxy(this.inputBlur,this));
        },
        focusClick: function(e){
            this.elements.placeholder.hide();
            if($(e.target).val()==="" && this.settings.showItemList===false){
                this.elements.menu.html('');
            }else{
                // this.elements.menu.html('');
                this.lookup();
                this.setScrollTop();
            }
        },
        inputBlur:function(){
            this.elements.view.val('');
            if(this.chooseItems.length==0){
                this.elements.placeholder.show();
            }
        },
        clickView:function(){
            if(!this.elements.view.is(":focus")){
                this.elements.view.trigger('focus');
            }
        },
        //重写 typeahead 的 setSource
        setSourceEx: function (source) {
            this.setSource(source);
            this._sourceMap = $.extend(true, {}, this.sourceMap);
        },
        //重写 typeahead 的 render
        render: function (items) {
            var context = this;
            var self = this;
            var activeFound = false;
            var liTemplete = '<li><span></span><a></a></li>';
            items = $(items).map(function (i, item) {
                var text = self.displayText(item);
                i = $(liTemplete).data('value', item);
                var glyphicon = i.find('span');
                if (item.unselectable)
                    i.addClass("disabled");
                if (self._sourceMap[text].checked) {
                    glyphicon.addClass(self.settings.checkedIcon);
                    i.find("a").attr("class", "checked");
                } else {
                    glyphicon.addClass(self.settings.uncheckedIcon);
                    i.find("a").attr("class", "");
                }
                i.find('a').html(context.highlighter(text, item));
                i.attr('data-value',item.value);
                if (text === self.elements.view.val()) {
                    i.addClass('active');
                    activeFound = true;
                }
                return i[0];
            });
            if (this.autoSelect && !activeFound) {
                items.first().addClass('active');
                //this.elements.view.data('active', items.first().data('value'));
            }
            this.elements.menu.html(items);
            this.setScrollTop();
            return this;
        },
        //重写 typeahead 的 keyup 方法
        keyup: function (e) {
            var suppressKeyPressRepeat = $.inArray(e.keyCode, [8,13,27,38,40,46]);
            if (suppressKeyPressRepeat != -1) {
                this.move(e);
            }else{
                this.lookup();
            }
        },
        //重写 typeahead 的 move 方法
        move: function (e) {
            if(e.keyCode===8){
                this.batchDelete();
                return;
            }
            if(!this.isMenuOpen())
                return;
            switch(e.keyCode) {
                //case 8: //Backspace
                //    this.batchDelete();
                //    break;
                case 13: // enter
                    var currentLi = this.elements.menu.find('li.active'),
                        text = currentLi.attr('data-value');
                    if (this._sourceMap[text].checked) {  //已勾选
                        this.select(enumCheckState.unCheck,currentLi);//取消勾选
                    } else {
                        this.select(enumCheckState.check,currentLi);//勾选
                    }
                    //this.dropdown.toggle(e); 回车操作不需要关闭dropdown
                    break;
                case 27: // Esc
                    e.preventDefault();
                    break;

                case 38: // up arrow
                    // with the shiftKey (this is actually the left parenthesis)
                    if (e.shiftKey) return;
                    e.preventDefault();
                    this.prev();
                    break;

                case 40: // down arrow
                    // with the shiftKey (this is actually the right parenthesis)
                    if (e.shiftKey) return;
                    e.preventDefault();
                    this.next();
                    break;
                case 46:    //Delete
                    //console.log(this.selectedItems);
                    this.batchDelete();
                    break;
            }

            e.stopPropagation();
        },
        // 重写
        click: function (e) {
            this.toggleClick(e);
            //this.dropdown.toggle(e);
        },
        toggleClick: function (e) {
            var currentTarget = $(e.currentTarget),
                currentLi = currentTarget.closest('li'),
                text = currentLi.attr('data-value');
            if (this._sourceMap[text].checked) {  //已勾选
                this.select(enumCheckState.unCheck,currentLi);//取消勾选
            } else {
                this.select(enumCheckState.check,currentLi);//勾选
            }
            e.preventDefault();
            e.stopPropagation();
        },
        /*
         * 勾选操作 重写 typeahead
         */
        select: function (option,li,value) {
            //var li = this.elements.menu.find('.active'),

            if (li.hasClass("disabled"))
                return;
            var data = li.data('value') || this.sourceMap[value],
                text = li.attr('data-value') || value;

            if (option === enumCheckState.check) {//勾选
                li.find("span").attr("class", this.settings.checkedIcon);
                li.find("a").attr("class", "checked");
                this._sourceMap[text].checked = true;
                this.chooseItems.push(data);
                this.elements.placeholder.hide();
            }
            if (option === enumCheckState.unCheck) {//取消勾选
                li.find("span").attr("class", this.settings.uncheckedIcon);
                li.find("a").attr("class", "");
                this._sourceMap[text].checked = false;
                this._sourceMap[text].selected = false;
                var index = this._getIndexInArray(data, this.chooseItems);
                this.chooseItems.splice(index, 1);
                if(this.chooseItems.length===0){
                    this.elements.placeholder.hide();
                }
            }
            this.setChooseItems();
            this.elements.view.val('');
            this.elements.view.trigger('onSelected', [this.chooseItems, data]);
        },
        /*
         * 渲染文本框中的已选项
         */
        setChooseItems:function(){
            var arr = this.chooseItems,
                htmls='';
            this.elements.dl.find("dd").remove();
            for(var i = 0;i<arr.length;i++) {
                htmls += this.displayText(arr[i]) + " ; ";
                var object = {
                    dd: $("<dd/>").attr("class", "c-ims-item"),
                    spanName: $("<span/>").attr("class", "c-ims-name").text(arr[i].value),
                    spanDel: $("<span/>").attr("class", "c-ims-delete").text("×")
                };
                if(this._sourceMap[arr[i].value].selected === true){
                    object.dd.addClass('active');
                }
                object.dd.data(arr[i]);
                object.spanName.on('click', $.proxy(this.clickSpanName, this));
                object.spanDel.on('click', $.proxy(this.clickSpanDel, this));
                object.dd.append(object.spanName);
                object.dd.append(object.spanDel);
                //this.elements.dl.prepend(object.dd);
                this.elements.view.before(object.dd);
            }
            htmls=htmls.substring(0,htmls.length-3);
            if(htmls!=''){
                this.elements.placeholder.hide();
            }else{
                this.elements.placeholder.show();
            }
            this.elements.textlist.html(htmls);
        },
        /*
         * 批量删除
         */
        batchDelete:function(){
            if(this.selectedItems.length===0)
                return;
            var prepDeleteItems = this.selectedItems;
            for(var i=0;i<prepDeleteItems.length;i++){
                var li = $("li[data-value='"+prepDeleteItems[i].value+"']",this.container);
                this.select(enumCheckState.unCheck, li,prepDeleteItems[i].value);
            }
            this.selectedItems = [];
        },
        /*
         * 点击文本框标签，选中变蓝，添加到全选操作数组中
         */
        clickSpanName:function(e){
            e.stopPropagation();
            var target = $(e.currentTarget);
            if(!$(target.is('span'))) return;
            var dd = target.closest('dd'),
                data = dd.data();
            if(dd.hasClass('active')){
                dd.removeClass('active');
                var index = this._getIndexInArray(data,this.selectedItems);
                this.selectedItems.splice(index,1);
                this._sourceMap[data.value].selected = false;
            }else{
                dd.addClass('active');
                this.selectedItems.push(data);
                this._sourceMap[data.value].selected = true;
            }
        },
        /*
         * 删除
         */
        clickSpanDel:function(e){
            e.stopPropagation();
            var target = $(e.currentTarget);
            if(!$(target.is('span'))) return;
            var value = target.closest('dd').data().value;
            var li = $("li[data-value='"+value+"']",this.container);
            if(li.length===0){  //dom还没渲染出来
                var data = target.closest('dd').data();
                this._sourceMap[value].checked = false;
                this._sourceMap[value].selected = false;
                var index = this._getIndexInArray(data,this.chooseItems);
                this.chooseItems.splice(index, 1);
                this.setChooseItems();
                this.elements.view.val('');
                this.elements.view.trigger('onSelected', [this.chooseItems, data]);
            }else {
                this.select(enumCheckState.unCheck, li);
            }
        },
        //重写 typeahead 的 setValue 方法
        setValue:function(value){
            var list = value;
            if(!list || list.length==0){
                this.chooseItems = [];
                this._sourceMap = $.extend(true, {}, this.sourceMap);
                this.elements.textlist.html('');
                this.elements.dl.find("dd").remove();
                this.elements.placeholder.show();
            }
            if(list && list.length>0){
                var self = this;
                //置空已勾选
                var chooseItems = this.chooseItems;
                for(var i=0;i<chooseItems.length;i++){
                    this._sourceMap[chooseItems[i].value].checked = false;
                    this._sourceMap[chooseItems[i].value].selected = false;
                }
                this.chooseItems = [];//置空已选列表
                if(this.elements.menu.children().length===0){
                    this.chooseItems = list;
                    for(var i=0;i<list.length;i++){
                        this._sourceMap[list[i].value].checked = true;
                    }
                    this.setChooseItems();
                }else {
                    for (var i = 0; i < list.length; i++) {
                        var li = $("li[data-value='" + list[i].value + "']", self.container);
                        self.select(enumCheckState.check, li);
                    }
                }
            }
        },
        //查找obj在arr的下标
        _getIndexInArray:function(obj,arr){
            for(var i=0;i<arr.length;i++){
                if(JSON.stringify(obj) == JSON.stringify(arr[i]))
                    return i;
            }
            return -1;
        },
        getApi: function () {
            var context = this;
            var funs = {
                setSource: 'setSourceEx',
                setValue:'setValue',
                getValue:'getValue'
            };
            var api = s.utilities.buildApi(context, funs);
            return api;
        }
    });

});


/* template javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'InputMultiSelect';
    var pluginClass = s.ui.controls.InputMultiSelect;

    // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;

    var defaults = {
        liked: false,
        dataUrl: '',
        value:''
    };

    var attributeMap = {
        liked: 'liked',
        dataUrl: 'data-url',
        value:'value'
    };

    this.LikeIt = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,
        // 构造函数
        init: function(element, options){
            this.element = $(element);
            //this.settings,
			//this.container;
            //this.elements;
            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { return; }
            this.element.data('initialized', true);
            // 区分一个页面中存在多个控件实例
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);
           
            // // 初始化数据
            // this.getData();
            this.value =  parseInt(this.settings.value ? this.settings.value : this.element.text());

            // // 构建html DOM
            // this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();

            //this.transferAttributes();

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            // this.bindEventsInterface();
            this.reflash();
        },

        initElements: function(){
            this.elements={
                original: this.element//,
                // valueObj: this.element.next(".c-like-value")
            };
        },

        bindEvents: function(){
        	this.element.on('click', $.proxy(this.toggle, this));
        },
        // getValue:function(){
        // 	return this.value;
        // },
        toggle:function(){
        	this.value = this.settings.liked ? this.value-1 : this.value+1;
        	this.settings.liked = !this.settings.liked;
        	this.postback();
        },
        postback:function(){
            var context = this;
            context.reflash();
            if(!this.settings.dataUrl || this.settings.dataUrl.length === 0){
                return;
            }
        	$.ajax({
                dataType: 'json',
                url: this.settings.dataUrl,
                data: {},
                success: function(data){
                    //context.reflash();
                   // console.log(data);
                },
                error: function(){
                    alert('控件id: ' + context.element.attr('id')+' , ajax发送数据失败！');
                }
            });
        },
        reflash: function(){

        	// this.elements.valueObj.text(this.value);
            this.element.text(this.value);
        	if(this.settings.liked){
    			this.element.addClass("liked");
    		}
    		else{
    			this.element.removeClass("liked");
    		}
 			
        }
    });

});


/* likeit javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'likeit';
    var pluginClass = s.ui.controls.LikeIt;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
/*
 * loading-overlay
 * https://github.com/jgerigmeyer/jquery-loading-overlay
 *
 * Copyright (c) 2014 Jonny Gerig Meyer
 * Licensed under the MIT license.
 */

Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;

    var defaults = {
        // show: false,                                    //初始化时是否默认显示
        loadingText: 'loading',                         //遮罩层中显示的字符串
        maskedTarget: ''                                //被遮罩的对象，传对象id进来
    };

    var attributeMap = {
        // show:'show',
        loadingText:'loading-text',
        maskedTarget:'masked-target'
    };

    this.Loading = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // 构造函数
        init: function(element, options){
            this.element = $(element);

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();

        },

        buildHtml: function(){
            var overlay = '<div class="c-loading-overlay">' +
                             '<p class="c-loading-spinner">' +
                                 '<span class="c-loading-icon"></span>' +
                                 '<span class="c-loading-text">' + this.settings.loadingText + '</span>' +
                             '</p>'+
                           '</div>';
            this.overlay = $(overlay);
        },

        initElements:function(){
            this.elements={
                target: $(this.settings.maskedTarget)
            };
        },

        start: function(){
            this.elements.target.addClass('c-loading');
            if (!this.elements.target.data('loading-overlay')) {
               this.elements.target.prepend(this.overlay).data('loading-overlay', true);
            }
        },

        stop: function(){
            this.elements.target.data('loading-overlay', false);
            this.elements.target.find('.c-loading-overlay').detach();
            if (this.elements.target.hasClass('c-loading')) {
                this.elements.target.removeClass('c-loading');
            } else {
                this.elements.target.find('.c-loading').removeClass('c-loading');
            }
        }

    });

});


/* template javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'loading';
    var pluginClass = s.ui.controls.Loading;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
// $(function () {
// 	// 严格模式
//     'use strict';
	
// 	var glyphiconMenudownHtml = '<i class="glyphicon glyphicon-menu-down"></i>';
// 	//var indexItem;
// 	$(".c-menu > li.sub-item > a").append('<i class="glyphicon glyphicon-menu-down"></i>');
//     $(".c-menu > li.sub-item > .dropdown > ul > li").click(function(){
//     	//indexItem = $(".c-menu > li.sub-item").index($(this).parents("li"));
//     	$(".c-menu > li > .dropdown > ul > li").removeClass("cur");
//     	$(this).addClass("cur").siblings();
//     	$(this).parents("li").addClass("cur").siblings().removeClass("cur");
//     });
//     $(".c-menu > li.sub-item").hover(function(){
    	
//     	$(this).addClass("cur").siblings().removeClass("cur");
//     	//$(".c-menu > li.sub-item").eq(indexItem).addClass("cur");
//     },function(){
//     	//$(this).parents("li").removeClass("cur");
//     });
// });
/* ========================================================================
 * Bootstrap: modal.js v3.3.5
 * http://getbootstrap.com/javascript/#modals
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;

    var TRANSITION_DURATION = 300;
    var BACKDROP_TRANSITION_DURATION = 150;


    var defaults = {
        // 参数
        modalId: '',        // 窗口div的id，指定id以后不会重新往modalContainer里面加内容
        modalClass: '',  // 窗口div的自定义css class
        valign:'top',
        height:'',
        show: false,        // 创建后直接显示
        bindTarget: true,   // 绑定element元素(button等)的click事件
        remote: '',         // 远程内容
        content: '',        // 本地内容
        parseData: undefined,   // 解析数据。 angularjs编译页面等
        backdrop: 'static',     // 遮罩(true/false/'static')
        keyboard: true,     // 键盘操作支持
        modalContainer: '#c-modal-base',     // 放在html文件</body>前的容器id
        isdrag:'true',//是否支持拖动
        buttons: {
            close: {
                selector: '.close',
                eventName: 'click',
                handler: function(e){
                    this.hide();
                }
            },
            cancel: {
                selector: '.cancel',
                eventName: 'click',
                handler: function(e){
                    this.hide();
                }
            }
        },
        // 覆写
        // 事件
        onInitialized: function(){}     // 初始化完成
    };
    var attributeMap = {
        modalId: 'modal-id',
        show: 'show',
        remote: 'remote',
        backdrop: 'backdrop',
        keyboard: 'keyboard',
        modalClass:'modal-class',
        valign:'valign',
        height:'height'
    };

    this.Modal = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // 构造函数
        init:function(element, options){
            this.element = $(element);

            // 区分一个页面中存在多个控件实例
            _currentPluginId += 1;
            if(this.element && this.element.length > 0){
                this._currentPluginId= this.element.data('plugin-id');
                if(!this._currentPluginId){
                    this._currentPluginId= _currentPluginId;
                    this.element.data('plugin-id', _currentPluginId);
                }
            }
            // this.element.data('plugin-id', _currentPluginId);

            // 设置初始值/状态
            this.isShown             = false;
            this.originalBodyPad     = null;
            this.scrollbarWidth      = 0;
            this.ignoreBackdropClick = false;   

            // // 初始化选项
            this.initSettings(options);
            if(!this.settings.remote){
                var href= this.element.attr('href');
                this.settings.remote= !/#/.test(href) && href;
            }
            // this.transferAttributes();

            this.modalContainer= $(this.settings.modalContainer);
            this.settings.modalId= this.settings.modalId || this.settings.modalContainer +'-m'+this._currentPluginId;
            if (typeof (this.settings.parseData) === 'function') {
                this.parseData = this.settings.parseData;
                delete this.settings.parseData;
            }

            // 初始化数据
            // this.value = this.element.val();
            this.getData();

            // // 构建html DOM
            // this.buildHtml();
            // // 初始化 html DOM 元素
            // this.initElements();

            // // 绑定事件
            // this.bindEvents();
            // // 绑定事件接口
            // this.bindEventsInterface();
        },

        getData: function(){
            var context= this;

            // if(this.modalContainer.find(this.settings.modalId).length === 0){
            //     context.elements.modal.trigger('modal.on.initialized');
            // }

            if (this.settings.remote) {                
                $.ajax({
                    url: this.settings.remote,
                    type: 'GET',
                    dataType: "html"//,
                    // data: params
                }).done(function(responseText) {
                    var innerHtml= context.parseData(responseText);
                    context.render(innerHtml);
                });
            }
            else{
                this.render(this.settings.content);
            }
        },
        parseData: function(data){
            return data;
        },
        buildHtml: function(){},
        render: function(innerHtml){
            var jqModal= this.modalContainer.find(this.settings.modalId);
            var jqHeightNum = parseInt(this.settings.height) + 2;
            var jqHeight = jqHeightNum + 'px';
            var jqMarginTop = -(jqHeightNum/2) + 'px';
           
            //jqModal.css('height',jqHeight);
           
            if(jqModal.length === 0)
            {
                // dynamic build modal
            
                var cssClass= this.settings.modalClass ? ' '+this.settings.modalClass : '';
                var cssValign = this.settings.valign == 'middle' ? ' valign-middle' : '';
                var htmlTempate= ''+
                    '<div '+
                    '   class="modal c-modal fade'+ cssClass + cssValign +'" '+
                    '   id="'+ this.settings.modalId.substring(1) +'" '+
                    '   tabindex="-1">'+
                    '    <div class="modal-dialog">'+
                    '    </div>'+
                    '</div>';
                this.modalContainer.append(htmlTempate);
                jqModal= this.modalContainer.find(this.settings.modalId);


                /*if(this.settings.height !== '' && this.settings.valign == 'middle'){

                    jqModal.css({
                        'top':'50%',
                        //'height':jqHeight,
                        'margin-top': jqMarginTop
                    });
                }*/
                
            }
            var jqDialog= jqModal.find('.modal-dialog:first');// content
            if(innerHtml){
                jqDialog.empty();
                jqDialog.append(innerHtml); 
            }
          
            if(this.settings.isdrag==='true'){
                // 绑定拖动
                $(jqDialog).draggable({
                    handle:'.modal-header',
                    containment: ".modal-open .modal.in"
                });
            }
            this.initElements();

            // 绑定事件
            this.bindEvents();
            
            this.elements.modal.trigger('modal.on.initialized');

            if(this.settings.show){
                this.show(this.element);
            }
        },
        refresh: function(){},
        initElements: function(){
            var context=this;


            var jqModal= this.modalContainer.find(this.settings.modalId);

            this.elements= {
                original: this.element,
                body: $(document.body),
                modal: jqModal,
                dialog: jqModal.find('.modal-dialog:first'),    // 嵌套modal必须加:first选择器
                backdrop: null
            };
        },
        bindEvents: function(){
            this.unbindEvents();

            if(this.settings.bindTarget){
                var context= this;
                this.elements.original.on('click', function(e){
                    if($(this).is('a')){
                        e.preventDefault();
                    }
                 
                    if(!$(this).prop('disabled')){
                         context.show(context.elements.original);
                    }


                   
                });
            }

            this.elements.modal.on('modal.on.initialized', $.proxy(this.settings.onInitialized, this));

            for(var buttonName in this.settings.buttons){
                var button= this.settings.buttons[buttonName];
                this.elements.modal.off(button.eventName, button.selector); // unbindEvents
                this.elements.modal.on(button.eventName, button.selector, $.proxy(button.handler, this));
            }
        },
        unbindEvents: function(){
            this.elements.modal.off('modal.on.initialized');
            if(this.settings.bindTarget)
            {
                this.elements.modal.off('click');
            }
        },

        // escape: function () {
        //     if (this.isShown && this.settings.keyboard) {
        //         this.elements.modal.on('keydown.modal.on.dismiss', $.proxy(function (e) {
        //             e.which == 27 && this.hide()
        //         }, this))
        //     } else if (!this.isShown) {
        //         this.elements.modal.off('keydown.modal.on.dismiss')
        //     }
        // },

        // toggle: function (_relatedTarget) {
        //     return this.isShown ? this.hide() : this.show(_relatedTarget);
        // },

        show: function (_relatedTarget) {
            var context= this;

            var e= $.Event('modal.on.show', { relatedTarget: _relatedTarget });
            this.elements.modal.trigger(e);

            // 嵌套madal
            var zIndex = 1060 + (10 * $('.modal:visible').length);
            this.elements.modal.css('z-index', zIndex);
            // setTimeout(function() {
            //     $('.modal-backdrop').not('.modal-stack').css('z-index', zIndex - 1).addClass('modal-stack');
            // }, 0);

            if (this.isShown || e.isDefaultPrevented()) return;

            this.isShown = true;

            // checkScrollbar
            var fullWindowWidth = window.innerWidth;
            if (!fullWindowWidth) { // workaround for missing window.innerWidth in IE8
                var documentElementRect = document.documentElement.getBoundingClientRect();
                fullWindowWidth = documentElementRect.right - Math.abs(documentElementRect.left);
            }
            this.bodyIsOverflowing = document.body.clientWidth < fullWindowWidth;
            // measureScrollbar
            var scrollDiv = document.createElement('div');
            scrollDiv.className = 'modal-scrollbar-measure';
            this.elements.body.append(scrollDiv);
            this.scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
            this.elements.body[0].removeChild(scrollDiv);
            // setScrollbar
            var bodyPad = parseInt((this.elements.body.css('padding-right') || 0), 10)
            this.originalBodyPad = document.body.style.paddingRight || ''
           // if (this.bodyIsOverflowing) this.elements.body.css('padding-right', bodyPad + this.scrollbarWidth)
            this.elements.body.addClass('modal-open')

            // // 绑定esc键事件
            // this.escape();
            $(window).on('resize.modal', $.proxy(this.adjustDialog, this))

            this.elements.dialog.on('mousedown.modal.on.dismiss', function () {
                context.elements.modal.one('mouseup.modal.on.dismiss', function (e) {
                    if ($(e.target).is(context.elements.modal)){
                        context.ignoreBackdropClick = true;
                    }
                })
            })

            var e = $.Event('modal.on.shown', { relatedTarget: _relatedTarget });
            this.backdrop($.proxy(this.backdropCallback, this, e));
        },

        hide: function (e) {
            if(e){
                e.preventDefault();
            }

            e = $.Event('modal.on.hide');
            this.elements.modal.trigger(e);

            if (!this.isShown || e.isDefaultPrevented()) {
                return;
            }

            this.isShown = false;

            // // 绑定esc键事件
            // this.escape();
            $(window).off('resize.modal');
            $(document).off('modal.on.focusin');

            this.elements.modal
                .removeClass('in')
                .off('click.modal.on.dismiss')
                .off('mouseup.modal.on.dismiss');

            this.elements.dialog.off('mousedown.modal.on.dismiss');

            if($.support.transition && this.elements.modal.hasClass('fade')){
                this.elements.modal
                    .one('bsTransitionEnd', $.proxy(this.hideModal, this))
                    .emulateTransitionEnd(TRANSITION_DURATION);
            }
            else
            {
                this.hideModal();
            }
        },

        hideModal: function () {
            var context = this
            this.elements.modal.hide()
            this.backdrop(function () {
                context.elements.body.removeClass('modal-open');
                // 修改bug：点击取消、确认按钮，modal窗口会移动
                // context.elements.modal.css({
                //     paddingLeft: '',
                //     paddingRight: ''
                // });
                context.elements.body.css('padding-right', context.originalBodyPad);
                context.elements.modal.trigger('modal.on.hidden');

                // var jqModal= context.modalContainer.find(context.settings.modalId);
                // jqModal.remove();
            })
        },

        backdrop: function (callback) {
            var context = this
            var animate = this.elements.modal.hasClass('fade') ? 'fade' : ''

            if (this.isShown && this.settings.backdrop) {
                var doAnimate = $.support.transition && animate;

                this.elements.backdrop = $(document.createElement('div'))
                    .addClass('modal-backdrop ' + animate)
                    .appendTo(this.elements.body)

                this.elements.modal.on('click.modal.on.dismiss', $.proxy(function (e) {
                    if (this.ignoreBackdropClick) {
                        this.ignoreBackdropClick = false
                        return
                    }
                    if (e.target !== e.currentTarget) return
                    this.settings.backdrop == 'static'
                        ? this.elements.modal[0].focus()
                        : this.hide()
                }, this))

                // http://stackoverflow.com/questions/9016307/force-reflow-in-css-transitions-in-bootstrap
                if (doAnimate) this.elements.backdrop[0].offsetWidth; // force reflow

                this.elements.backdrop.addClass('in');

                if (!callback){
                    return;
                }

                if(doAnimate){
                    this.elements.backdrop
                        .one('bsTransitionEnd', callback)
                        .emulateTransitionEnd(BACKDROP_TRANSITION_DURATION);
                }
                else{
                    callback();
                }
            } 
            else if (!this.isShown && this.elements.backdrop) 
            {
                this.elements.backdrop.removeClass('in')

                var callbackRemove = function () {
                    // removeBackdrop
                    context.elements.backdrop && context.elements.backdrop.remove()
                    context.elements.backdrop = null
                    callback && callback()
                }
                $.support.transition && this.elements.modal.hasClass('fade') ?
                    this.elements.backdrop
                        .one('bsTransitionEnd', callbackRemove)
                        .emulateTransitionEnd(BACKDROP_TRANSITION_DURATION) :
                    callbackRemove()

            } 
            else if (callback) {
                callback();
            }
        },

        backdropCallback: function (e) {
            if (!this.elements.modal.parent().length) {
                this.elements.modal.appendTo(this.elements.body); // don't move modals dom position
            }

            this.elements.modal.show();
            this.elements.modal.scrollTop(0);
            var jqDialog= this.elements.modal.find('.modal-dialog:first');// content
            //垂直居中
            //jqDialog.css({"position":"absolute",'left':'50%','top':'50%','marginTop':(-jqDialog.height()/2),'marginLeft':(-jqDialog.width()/2),'marginRight':'0','marginBottom':'0'});

            var clientHeight = document.documentElement.clientHeight,
                clientWidth = document.documentElement.clientWidth,
                elmHeight = jqDialog.height(),
                elmWidth = jqDialog.width();
            if(clientHeight>elmHeight){
                jqDialog.css({"position":"absolute",'left':(clientWidth/2) - (elmWidth/2),'top':(clientHeight/2) - (elmHeight/2),'margin':0});
            }
            // 修改bug：点击取消、确认按钮，modal窗口会移动
            //this.adjustDialog();

            var transition = $.support.transition && this.elements.modal.hasClass('fade');
            if (transition) {
                this.elements.modal[0].offsetWidth; // force reflow
            }

            this.elements.modal.addClass('in');

            $(document).off('modal.onfocusin'); // guard against infinite focus loop
            $(document).on('modal.onfocusin', $.proxy(function (e) {
                if (this.elements.modal[0] !== e.target && !this.elements.modal.has(e.target).length) {
                    this.elements.modal.trigger('focus')
                }
            }, this));

            if(transition){
                var context= this;
                this.elements.dialog.one('bsTransitionEnd', function () {
                    context.elements.modal.trigger('focus').trigger(e)
                });
                this.elements.dialog.emulateTransitionEnd(TRANSITION_DURATION)
            }
            else{
                this.elements.modal.trigger('focus').trigger(e);
            }                
        },

        adjustDialog: function () {
            var modalIsOverflowing = this.elements.modal[0].scrollHeight > document.documentElement.clientHeight;

            this.elements.modal.css({
                paddingLeft:  !this.bodyIsOverflowing && modalIsOverflowing ? this.scrollbarWidth : '',
                paddingRight: this.bodyIsOverflowing && !modalIsOverflowing ? this.scrollbarWidth : ''
            });
        }
    });
});

/* modal javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'modal';
    var pluginClass = s.ui.controls.Modal;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;

    var emptyFun= function(){};

    var defaults = {
        minYear: '1900',
        maxYear: '2099',
        position:'auto',  //down,left

        monthsArr: ['一月','二月','三月','四月','五月','六月','七月','八月','九月','十月','十一月','十二月'],
        split: '-',
        change: emptyFun
    };
    var attributeMap = {
        minYear: 'min-year',
        maxYear: 'max-year',
        position:'position',
        split: 'split',
    };

    this.Monthpicker = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        settings: {},
        value: '',
        data: {},
        templates: {},

        // 构造函数
        init: function(element, options){
            this.element = $(element);

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);
            // 获取初始值
            // this.value = this.element.val();
            this.year = (new Date()).getFullYear();

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();

            this.render();
            this.transferAttributes();

            // 获取数据
            // 同步获取数据
            // this.getData();
            var context= this;
            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            // this.bindEventsInterface();            
        },

        buildHtml: function(){
            var pluginId= this.element.data('plugin-id');
            var htmlTemplate=''+
                    '<div class="c-monthpicker-container">'+
                        '<span class="glyphicon-calendar" data-s-dropdown-id="#c-monthpicker-menu-'+pluginId+'"></span>'+
                        '<ul id="c-monthpicker-menu-'+pluginId+'" class="monthpicker dropdown-menu"> '+
                            '<div class="monthpicker-control">'+
                              '<div class="icon-prev" title="上一年">'+
                                '<i class="glyphicon glyphicon-menu-left"></i>'+
                              '</div>'+
                              '<div class="calendar-year">'+
                                '<a class="btn-container"><span></span>年<i class="glyphicon glyphicon-menu-down"></i></a>'+
                                '<div class="dropdown-menu calendar-years">'+
                                '</div>'+
                              '</div>'+
                              '<div class="icon-next" title="下一年">'+
                                '<i class="glyphicon glyphicon-menu-right"></i>'+
                              '</div>'+
                            '</div>'+
                            '<div class="calendar-months">'+
                            '</div>'+
                        '</ul>'+
                    '</div>';
            this.container= $(htmlTemplate);
            this.element.before(this.container);
            this.container.prepend(this.element);

            this.element.attr("data-s-dropdown-id","#c-monthpicker-menu-"+pluginId);

            if(this.element.is('input')){
                this.element.attr("autocomplete","off");
            }
        },
        
        initElements: function(){
            this.elements={
                view: this.element,
                monthpicker: this.container,
                menu: $('.monthpicker',this.container),
                icon: $('.glyphicon-calendar',this.container),
                prev: $('.monthpicker-control .icon-prev',this.container),
                next: $('.monthpicker-control .icon-next',this.container),
                calendarYear: $('.calendar-year',this.container),
                showYear: $('.calendar-year .btn-container span',this.container),
                showYearBtn:$('.calendar-year .btn-container',this.container),
                calendarYears: $('.calendar-year .calendar-years',this.container),
            
                calendarMonths: $('.calendar-months',this.container),
            };
            this.initMonthPicker();

            var pluginId= this.elements.view.data('plugin-id');
            this.dropdown= new s.ui.controls.Dropdown(this.elements.view, {_pluginId:pluginId});
            this.dropdown1= new s.ui.controls.Dropdown(this.elements.icon, {_pluginId:pluginId});

            var context = this;
            this.elements.menu.on('click',function(e){
                e.preventDefault();
                e.stopPropagation();

                context.elements.calendarYears.hide();
            });
        },

        bindEvents: function(){
            var context = this;
            this.elements.view.on('focus',$.proxy(function(e){
                context.elements.calendarYears.hide();
                context.render();
            }, this));
            //this.elements.icon.on('click',$.proxy(this.focusView, this));
            this.elements.icon.on('click',$.proxy(function(){
                this.elements.view.focus();
            }, this));
            this.elements.showYearBtn.on('click',$.proxy(this.showYearPanel, this));
            this.elements.calendarYear.on('click', 'li', $.proxy(this.yearChanged, this));
            this.elements.calendarMonths.on('click', 'li', $.proxy(this.monthChanged, this));

            this.elements.view.on('blur',$.proxy(this.confirm, this));
            this.elements.view.on('onChange',$.proxy(this.settings.change, this));
            this.elements.prev.on('click',$.proxy(this.clickPrev, this));
            this.elements.next.on('click',$.proxy(this.clickNext, this));

        },

        initMonthPicker: function(){
            var maxYear =  this.settings.maxYear;
            var minYear =  this.settings.minYear;

            //month
            var monthHtml = '<ul class="monthselect">'; 
            for (var m = 0; m < 12; m++) {
                monthHtml += "<li data-value="+ (m + 1) +">" + this.settings.monthsArr[m]+"</li>";
            }
            monthHtml += "</ul>";
            this.elements.calendarMonths.html(monthHtml);

            //year
            var yearHtml = '<ul class="yearselect">';
            for (var y = minYear; y <= maxYear; y++) {
                yearHtml += "<li data-value="+ y + ">" + y + "</li>";
            }
            yearHtml += '</ul>';
            this.elements.calendarYears.html(yearHtml);
        },

        transferAttributes: function(){
            if(this.element.prop('disabled')){
                this.disable();
            }
        },
        
        render: function(){
            if(!this.year){
                this.year = (new Date()).getFullYear();
            }
            this.elements.showYear.text(this.year);
            var item = this.elements.calendarYears.find('li[data-value='+this.year+']');
            item.addClass("active").siblings().removeClass('active');
            var item2 = this.elements.calendarMonths.find('li[data-value='+this.month+']');
            item2.addClass("active").siblings().removeClass('active');
        },

        showYearPanel:function(e){
            e.preventDefault();
            e.stopPropagation();
            this.elements.calendarYears.show();
            var minYear =  this.settings.minYear;
            var selectedYear = this.year;
            var distance = selectedYear - minYear - 3;
            if(distance < 0)
                distance = 0;
            distance = distance * 26;
            this.elements.calendarYears.scrollTop(distance);
        },

        yearChanged: function(e){
            var target = $(e.target);
            var year = target.attr('data-value');
            this.year = year;
            this.elements.showYear.text(year);
            target.addClass("active").siblings().removeClass("active");
            this.elements.calendarYears.hide();
            this.elements.view.trigger('onChange',this.getValue());
        },

        monthChanged: function(e){
            var target = $(e.target);
            var month = target.attr('data-value');
            this.month = month;
            target.addClass("active").siblings().removeClass("active");
            this.elements.view.trigger('onChange',this.getValue());
        },

        confirm: function(e){
            var value = this.elements.view.val();
            if(value === '' || value.length === 0)
                value = null;
            this.setBindValue(value);
            this.render();
            this.elements.view.trigger('onChange',this.getValue());
        },

        setBindValue: function(value){
            if(!value){
                this.year = null;
                this.month = null;
            }else{
                var date=moment(value,'YYYY-MM-DD');
                this.year = date.toObject().years;
                this.month =date.toObject().months+1;
                if(this.settings.minYear*1>this.year*1){
                    this.year=this.settings.minYear;
                }else if(this.settings.maxYear*1<this.year*1){
                    this.year=this.settings.maxYear;
                }
            }
        },

        setValue:function(value){
            this.setBindValue(value);
            this.elements.view.val(value);
        },

        getValue:function(){
            if(this.year && this.month){
                var showMonth = parseInt(this.month);
                if(showMonth < 10)
                    showMonth = '0'+showMonth;
                return this.year+this.settings.split+showMonth;
            }else{
                return null;
            }
        },

        clickPrev: function(e) {
            if(this.year > this.settings.minYear){
                this.year = parseInt(this.year) - 1;
            }
            this.elements.view.trigger('onChange',this.getValue());
            this.render();

        },

        clickNext: function(e) {
            if(this.year < this.settings.maxYear){
                this.year = parseInt(this.year) + 1;
            }
            this.elements.view.trigger('onChange',this.getValue());
            this.render();
        },
       
        // API
        refresh: function(){},
        enable: function(){
            this.elements.view.prop('disabled',false);
            this.container.removeClass("disabled");
            this.elements.icon.removeClass("disabled");
            // this.elements.button.attr('disabled', false);
        },
        disable: function() {
            this.elements.view.prop('disabled', true);
            this.elements.icon.addClass("disabled");
            this.container.addClass("disabled");
            // this.elements.button.attr('disabled', true);
        },
        destroy: function(){},
        getApi: function(){
            var context = this;
            var funs = {
                setValue: 'setValue',
                getValue: 'getValue'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });

});


/* monthpicker javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'monthpicker';
    var pluginClass = s.ui.controls.Monthpicker;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        // 选项
        // fooOption: true,
        // 覆写 类方法
        // parseData: undefined,
        // 事件
        // onFooSelected: undefined,
        // onFooChange: function(e, data){}
        group: true,
        checkedIcon: 'glyphicon glyphicon-check',
        uncheckedIcon: 'glyphicon glyphicon-unchecked',
        change: undefined,
        placeholderValue:'',
        splitFormat:'; ',
        showCheckall:false,
        groupCheck:false,
        sorter:undefined,
    };
    var attributeMap = {
        group: 'group',
        placeholderValue:'placeholder-value',
        splitFormat:'split-format',
        showCheckall:'show-checkall',
        groupCheck:'group-check'
    };

    this.Multiselect = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        settings: {},
        value: '',
        data: {},
        selects: {},

        // 构造函数
        init: function(element, options){
            this.element = $(element);

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);
            // 获取初始值
            // this.value = this.element.val();

            this.chooseValue = [];
            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            this.transferAttributes();
           
            this.bindEvents();

        },

        buildHtml: function(){
            var pluginId= this.element.data('plugin-id');
            var htmlTemplate=''+
                '<div class="c-multiselect-container"> '+
                '   <div class="dropdown"> '+
                '       <a class="btn btn-default c-selectex-btn c-multiselect-btn" data-s-dropdown-id="#c-multiselect-menu-'+pluginId+'">'+
                '           <span class="c-selectex-btn-text placeholder">' + this.settings.placeholderValue + '</span>'+
                '           <span class="glyphicon glyphicon-menu-down"></span>'+
                '       </a>'+
                '       <ul id="c-multiselect-menu-'+pluginId+'" class="dropdown-menu"> '+
                '       </ul>' +
                '   </div> '+
                '</div>';
            this.container= $(htmlTemplate);
            this.element.before(this.container);
            this.element.hide();

        },
        initElements: function(){
            this.elements= {
                original: this.element,
                viewContainer: $('.c-multiselect-btn', this.container),
                view: $('.c-selectex-btn-text', this.container),
                menu: $('.dropdown-menu', this.container),
            };
            this.dropdown= new s.ui.controls.Dropdown(this.elements.viewContainer, {});

            this.elements.menu.on('click',function(e){
                e.preventDefault();
                e.stopPropagation();
            });
            //this.refreshPlaceholder();
        },
        transferAttributes: function(){

            this.elements.view.attr('style', this.element.attr('style'));
            this.elements.view.show();
            this.elements.view.attr('title', this.element.attr('title'));
            this.elements.view.attr('class', this.elements.view.attr('class') + ' ' + this.element.attr('class') );
    
            if(this.element.prop('disabled')){
                this.disable();
            }
        },
        bindEvents: function(){
            this.elements.menu.on('click','a',$.proxy(this.clickItem, this));

            if (typeof (this.settings.change) === 'function') {
                this.elements.original.on('change', $.proxy(this.settings.change, this));
            }
        },
        bindEventsInterface: function(){
            var context=this;
            var element=this.element;

            // element.on('selected.s.select', $.proxy(this.settings.onFooClick, this));
        },

        updateData: function(_data) {
            var groupSource = [];
            var map = {};
            var showMap = {};
            var groupMap = {};
            if (!_data)
                return;
            var data = _data;
            if (!this.settings.group) {
                data = [{options: _data}];
            }
            var k = 0;
            for (var i = 0; i < data.length; i++) {
                var item = data[i];
                groupSource.push(item);
                for (var j = 0; j < item.options.length; j++) {
                    var option = item.options[j];
                    /*
                     map[k] = option;
                     if(option.value+''){
                     showMap[option.value] = k;
                     }
                     k++;*/
                    if (option.value + '') {
                        showMap[option.value] = option.value;
                    }
                    if (this.settings.group) {
                        if (!groupMap[option.value]) {
                            groupMap[option.value] = []
                        }
                        groupMap[option.value].push(i);
                    }
                    map[option.value] = option;
                }
            }
            this.groupSource = groupSource;
            this.map = map;
            this.showMap = showMap;
            this.lists = _data;
            this.groupMap = groupMap;
        },
        render: function(){
            var that = this;
            if(!this.groupSource)
                return;
            var items = this.groupSource;
            var menuHtml = '';
            if(this.settings.showCheckall){
                menuHtml +='<li><dl><dd><a data-id="-1" data-value="-1"> <span data-s-checkall class="'+this.settings.uncheckedIcon+'"></span><label>全部</label></a></dd></dl></li>';
            }
            var k = 0;
            for(var i = 0;i < items.length; i++){
                var item = items[i];
                menuHtml += '<li><dl>';
                if(this.settings.group){
                    if(this.settings.groupCheck) {
                        menuHtml += '<dt><a data-id="-1" data-value="-1" data-group-id="' + i + '"><span class="' + this.settings.uncheckedIcon + '"></span><label>' + item.text + '</label></a></dt>';
                    }else {
                        menuHtml += '<dt><label>'+item.text+'</label></dt>';
                    }
                }
                menuHtml += '<dd>';
                for(var j = 0;j < item.options.length;j++){
                    menuHtml += '<a data-id='+item.options[j].value+' data-value='+item.options[j].value+' title='+item.options[j].text+'>' + '<span class="'+this.settings.uncheckedIcon+'"></span><label>'+ item.options[j].text + '</label></a>';
                    k++;
                }
                menuHtml += '</dd></dl></li>';
            }
            this.elements.menu.html(menuHtml);
        },

        setValue: function(value){
            if(value || value === 0){
                this.elements.menu.find('span').attr("class",this.settings.uncheckedIcon);

                var ids = value;
                if(typeof value === 'number'){
                    ids= value + '';
                }
                var arrId= ids.split(',');
                var tempChoose = [];
                for(var i=0;i < arrId.length;i++){
                    var id = this.showMap[arrId[i]];
                    if(this.map[id]){
                        tempChoose.push(this.map[id]);
                        this.checkOption(id);
                    } 
                }
                this.chooseValue =tempChoose;
                this.relateCheck();
                var checkboxs=this.elements.menu.find('a>span').length;
                var check=this.elements.menu.find('a>span.glyphicon.glyphicon-check').length;
                if(checkboxs-1===check){
                    this.elements.menu.find('span[data-s-checkall]').attr("class",this.settings.checkedIcon);
                }
                /*if(this.lists.length===tempChoose.length){
                    this.elements.menu.find('span[data-s-checkall]').attr("class",this.settings.checkedIcon);
                }*/
                this.elements.view.text(this.getShowValue()); 
                this.container.attr("title",this.getShowValue());
                this.refreshPlaceholder();
              /*  if(this.settings.sorter)
                this.tem();*/
            }else{  
                this.chooseValue=[];
                this.elements.view.text(value);
                this.elements.view.text(value);
                this.container.attr("title",value);
                this.refreshPlaceholder();
            }
        },
        sorterChooseValue:function(tempChoose){
            return this.chooseValue =this.settings.sorter ? this.settings.sorter(tempChoose) : tempChoose;
        },
        //分组关系检查
        relateCheck:function(){
            var choosValue = this.chooseValue,
                groupSource = this.groupSource,
                groupMap = this.groupMap;//{'value':[groupIndex,groupIndex]},{},{}...
            if(!this.settings.group)
                return;
            var groupIndex=[],
                context = this;
            for(var i=0;i<choosValue.length;i++){
                groupIndex = groupMap[choosValue[i].value];//[groupIndex,groupIndex]
                groupIndex.forEach(function(index){
                    var groupIndexChildren = groupSource[index].options;
                    var isCheckAll = true;
                    groupIndexChildren.forEach(function(index){
                        if($('a[data-id="'+index.value+'"]', context.container).find('span').hasClass(context.settings.uncheckedIcon)){
                            isCheckAll = false;
                        }
                    });
                    if(isCheckAll){
                        $('a[data-group-id="'+index+'"]', context.container).find('span').attr("class",context.settings.checkedIcon);
                    }
                });
            }
        },
        getValue: function(){
            return this.chooseValue;
        },

        getShowValue: function(){
            var texts = [];
            for (var i = 0; i < this.chooseValue.length; i++) {
                texts.push(this.chooseValue[i].text);
            }
            return texts.join(this.settings.splitFormat);
        },

        checkOption: function(id){
            var option= $('a[data-id="'+id+'"]', this.container);
            var check= option.find('span');
            check.attr("class",this.settings.checkedIcon);
        },

        clickItem: function(e){
            if($(e.target).closest("a").hasClass("disabled"))
                return;
            var a =  $(e.target).closest("a");
            var id =  a.attr('data-id');
            var check= a.find('span');
            var optionValue = this.map[id];
            if(a.attr('data-group-id')){//分组复选框
                var groupid = a.attr('data-group-id');
                var groups = this.lists[groupid];
                var nodes = $(e.target).closest("dl").find('dd');
                if(check.hasClass(this.settings.uncheckedIcon)){    //勾选整个分组
                    for(i=0;i<groups.options.length;i++){
                        //var nodecheck = $(nodes[i]).find('a > span');
                        var nodecheck = $('a[data-id="'+groups.options[i].value+'"]', this.container).find('span');
                        if(nodecheck.hasClass(this.settings.uncheckedIcon)){    //没有勾选的，才加到数组中
                            nodecheck.attr("class",this.settings.checkedIcon);
                            this.chooseValue.push(groups.options[i]);
                        }
                    }
                    check.attr("class",this.settings.checkedIcon);
                }else{                                              //取消勾选分组
                    for(i=0;i<groups.options.length;i++){
                        //this.checkItem()
                        var nodecheck = $(nodes[i]).find('a > span');
                        nodecheck.attr("class",this.settings.uncheckedIcon);
                        //this.chooseValue.push(groups.options[i]);
                        this.chooseValue.splice(this.contains(this.chooseValue,groups.options[i],'value'),1);
                    }
                    check.attr("class",this.settings.uncheckedIcon);
                }
                this.elements.view.text(this.getShowValue());
                this.refreshPlaceholder();
                this.elements.original.trigger('change', JSON.stringify(this.sorterChooseValue(this.chooseValue)));
                return;
            }
            if(id==='-1') {
                if(check.hasClass(this.settings.uncheckedIcon)){
                    this.checkAll();
                }else{
                     this.unCheckAll();
                }
                return ;
            }
            if(check.hasClass(this.settings.uncheckedIcon)){
                check.attr("class",this.settings.checkedIcon);
                this.chooseValue.push(optionValue);
            }else{
                check.attr("class",this.settings.uncheckedIcon);
                for(var i=0;i < this.chooseValue.length;i++){
                    if(this.chooseValue[i].value === optionValue.value){
                        this.chooseValue.splice(i,1);
                        break;
                    }
                }
            }
            this.elements.view.text(this.getShowValue());
            this.refreshPlaceholder();
            this.elements.original.trigger('change', JSON.stringify(this.sorterChooseValue(this.chooseValue)));
        },

        duplicate:function(arr,obj,key){
            for(var i=0;i<arr.length;i++){
                if(arr[i][key]===obj[key]){
                    return true
                }
            }
            return false;
        },
        //
        contains:function(arr,obj,key){
            var istrue=true;
            if(!Array.isArray(arr))
                arr=[arr];
            if(typeof obj !=='object'){ return arr.indexOf(obj);}
            for(var i=0;i<arr.length;i++){
                istrue=true;
                for(var x in key){
                    if(obj[key[x]]!=arr[i][key[x]]){
                        istrue=false;
                    }
                }
                if(istrue) return i;
                
            }
            return undefined;
        },
        checkAll:function(){
            var arr=[];
            if(this.settings.groupCheck){
                this.chooseValue.splice(0,this.chooseValue.length);
                for(var i=0;i<this.lists.length;i++){
                    var optins=this.lists[i].options;
                    for(var j=0;j<optins.length;j++){

                        if(!this.duplicate(arr,optins[j],'value')){
                            arr.push(optins[j]);
                            this.chooseValue.push(optins[j]);
                        }
                       
                    }
                   
                }
                this.elements.menu.find('dd>a>span').attr("class",this.settings.checkedIcon);
                this.elements.view.text(this.getShowValue());
                this.refreshPlaceholder();
                this.elements.original.trigger('change', JSON.stringify(this.sorterChooseValue(this.chooseValue)));
            }else{
                this.chooseValue.splice(0,this.chooseValue.length);
                for(var i=0;i<this.lists.length;i++){
                    this.chooseValue.push(this.lists[i]);
                }
                this.elements.menu.find('dd>a>span').attr("class",this.settings.checkedIcon);
                this.elements.view.text(this.getShowValue());
                this.refreshPlaceholder();
                this.elements.original.trigger('change', JSON.stringify(this.sorterChooseValue(this.chooseValue)));
            }
           //  this.chooseValue.push(optionValue);
        
        },
        unCheckAll:function(){
           this.chooseValue.splice(0,this.chooseValue.length);
           this.elements.menu.find('dd>a>span').attr("class",this.settings.uncheckedIcon);
           this.elements.view.text(this.getShowValue());
           this.refreshPlaceholder();
           this.elements.original.trigger('change', JSON.stringify(this.chooseValue));
        },
        refreshPlaceholder:function(){
            if( this.elements.view.text() === this.settings.placeholderValue || this.elements.view.text() ===''){
                this.elements.view.addClass("placeholder");
                this.elements.view.text(this.settings.placeholderValue);
            }
            else{
                this.elements.view.removeClass("placeholder");
            }
        },

        // API
        refresh: function(){},
        enable: function(){
            this.elements.viewContainer.removeClass('disabled');
            // this.elements.button.attr('disabled', false);
        },
        disable: function() {
            //this.elements.viewContainer.attr('disabled', true);
            // this.elements.button.attr('disabled', true);
             this.elements.viewContainer.addClass('disabled');
        },
        destroy: function(){}
    });

});


/* select javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'multiselect';
    var pluginClass = s.ui.controls.Multiselect;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
 Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;

    var defaults = {
       
    };
    var attributeMap = {
        
    };

    this.Radio = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        settings: {},
        value: '',
        data: {},
        templates: {},

        // 构造函数
        init: function(element, options){
            this.element = $(element);
            
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);
            this.buildHtml();
            this.buildEvent();
        },

        // 构建组件
        buildHtml: function(){
            var type = this.element.find("input").attr("type");
            
            this.element.find("li").each(function(){
                $(this).find("input").attr({"class":"labelauty","style":"display: none"});
                var nodeName = $(this).find("input").attr("data-labelauty")
                var label = "<label>" +
                                "<span class='labelauty-unchecked-image'></span>" +
                                "<span class='labelauty-unchecked'>"+nodeName+"</span>" +
                                "<span class='labelauty-checked-image'></span>" +
                                "<span class='labelauty-checked'>"+nodeName+"</span>" +
                            "</label>";
                $(this).append(label);

            });

        },

        // 绑定事件
        buildEvent: function(){
            var el =  this.element;
            this.element.find("label").click(function(){

                if($(this.previousElementSibling).attr("disabled") != "disabled"){

                    // checkbox
                    if($(this.previousElementSibling).attr("type") == "checkbox"){
                        if(this.previousElementSibling.checked === true){
                            this.previousElementSibling.checked = false;
                        }else{
                            this.previousElementSibling.checked = true;
                        }   
                    }

                    // radio
                    if($(this.previousElementSibling).attr("type") == "radio"){
                        this.previousElementSibling.checked = true;
                    }

                }

            });    
        }
    });

});


/* template javascript jQuery */

(function($) {
    // 严格模式
    'use strict';
    // 控件类名
    var pluginName = 'Radio';
    var pluginClass = s.ui.controls.Radio;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
/*
 * @license
 *
 * Multiselect v2.1.4
 * http://crlcu.github.io/multiselect/
 *
 * Copyright (c) 2015 Adrian Crisan
 * Licensed under the MIT license (https://github.com/crlcu/multiselect/blob/master/LICENSE)
 */



 Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    
    var defaults = {
        left: '',
        leftAll: '',
        leftSelected: '',
        right: '',
        rightAll: '',
        rightSelected: '',
        undo: '',
        redo: '',

        submitAllLeft: false,   // 在form postback时，将左边的选项全部(选中)提交
        submitAllRight: false,   // 在form postback时，将右边的选项全部(选中)提交

        // dataUrl: 'data-url'
        // keepRenderingSort: false
        

        /** will be executed once - remove from $left all options that are already in $right
         *
         *  @method startUp
        **/
        startUp: function( $left, $right ) {
            $right.find('option').each(function(index, option) {
                $left.find('option[value="' + option.value + '"]').remove();
            });
        },

        /** will be executed each time before moving option[s] to right
         *  
         *  IMPORTANT : this method must return boolean value
         *      true    : continue to moveToRight method
         *      false   : stop
         * 
         *  @method beforeMoveToRight
         *  @attribute $left jQuery object
         *  @attribute $right jQuery object
         *  @attribute options HTML object (the option[s] which was selected to be moved)
         *  
         *  @default true
         *  @return {boolean}
        **/
        beforeMoveToRight: function($left, $right, options) { return true; },

        /*  will be executed each time after moving option[s] to right
         * 
         *  @method afterMoveToRight
         *  @attribute $left jQuery object
         *  @attribute $right jQuery object
         *  @attribute options HTML object (the option[s] which was selected to be moved)
        **/
        afterMoveToRight: function($left, $right, options){},

        /** will be executed each time before moving option[s] to left
         *  
         *  IMPORTANT : this method must return boolean value
         *      true    : continue to moveToRight method
         *      false   : stop
         * 
         *  @method beforeMoveToLeft
         *  @attribute $left jQuery object
         *  @attribute $right jQuery object
         *  @attribute options HTML object (the option[s] which was selected to be moved)
         *  
         *  @default true
         *  @return {boolean}
        **/
        beforeMoveToLeft: function($left, $right, option){ return true; },

        /*  will be executed each time after moving option[s] to left
         * 
         *  @method afterMoveToLeft
         *  @attribute $left jQuery object
         *  @attribute $right jQuery object
         *  @attribute options HTML object (the option[s] which was selected to be moved)
        **/
        afterMoveToLeft: function($left, $right, option){},

        /** sort options by option text
         * 
         *  @method sort
         *  @attribute a HTML option
         *  @attribute b HTML option
         *
         *  @return 1/-1
        **/
        sort: function(a, b) {
            if (a.innerHTML == 'NA') {
                return 1;   
            } else if (b.innerHTML == 'NA') {
                return -1;   
            }
            
            return (a.innerHTML > b.innerHTML) ? 1 : -1;
        }
    };

    var attributeMap = {
        left: 'left',
        leftAll: 'left-all',
        leftSelected: 'left-selected',
        right: 'right',
        rightSelected: 'right-selected',
        rightAll: 'right-all' //,
        // dataUrl: 'data-url'
    };

    this.RightSelect = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // Vars
        undoStack: [],
        redoStack: [],

        init: function(element, options){
            this.element = $(element);

            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // this.settings,
            // this.container,
            // this.elements,

            // this.value = this.element.val();   

            // 初始化选项
            this.initSettings(options);
            // 初始化默认相关控件id
            var id = this.element.prop('id') || 'rightselect';
            this.settings.left= this.settings.left.length ? this.settings.left : '#' + id + '_leftSelect';
            this.settings.leftSelected= this.settings.leftSelected.length ? this.settings.leftSelected : '#' + id + '_leftSelected';
            this.settings.leftAll= this.settings.leftAll.length ? this.settings.leftAll : '#' + id + '_leftAll';
            this.settings.right= this.settings.right.length ? this.settings.right : '#' + id + '_rightSelect';
            this.settings.rightSelected= this.settings.rightSelected.length ? this.settings.rightSelected : '#' + id + '_rightSelected';
            this.settings.rightAll= this.settings.rightAll.length ? this.settings.rightAll : '#' + id + '_rightAll';
            this.settings.undo= this.settings.undo.length ? this.settings.undo : '#' + id + '_undo';
            this.settings.redo= this.settings.redo.length ? this.settings.redo : '#' + id + '_redo';

            // this.undoStack = [];
            // this.redoStack = [];            
            
            // var context= this;

            // 构建html DOM
            this.buildHtml();

            // 初始化 html DOM 元素
            this.initElements();

            // 绑定事件
            this.bindEvents();
            // // 绑定事件接口
            // this.bindEventsInterface();

            // this.initialized();

            // this.reflash();

            if (this.settings.keepRenderingSort) {

                this.settings.sort = function(a, b) {
                    return $(a).data('position') > $(b).data('position') ? 1 : -1;
                };

                this.elements.left.find('option').each(function(index, option) {
                    $(option).data('position', index);
                });

                this.elements.right.find('option').each(function(index, option) {
                    $(option).data('position', index);
                });
            }
            
            // // 初始化数据
            // $.when(this.getData())
            //  .done(function(){
                
            //     context.setInitialStates({ nodes: context.tree }, 0);
            //     context.reflash();

            //     context.bindEvents();
            //     context.initialized();
            // });

            // this.transferAttributes();            
        },
        buildHtml: function(){
            var searchTemplate = '<input type="text" name="q" class="form-control" placeholder="Search..." />';
            // search_right: '<input type="text" name="q" class="form-control" placeholder="Search..." />',

            if (this.settings.search_left) {
                // this.element.before($(searchTemplate));
                $( this.settings.left ).before($(searchTemplate));
            }

            if (this.settings.search_right) {
                $( this.settings.right ).before($(searchTemplate));
            }
        },
        initElements: function(){
            this.elements= {
                original: this.element, // left

                left           : $( this.settings.left ),
                leftSelected    : $( this.settings.leftSelected ),
                rightSelected   : $( this.settings.rightSelected ),
                right           : $( this.settings.right ),
                leftAll         : $( this.settings.leftAll ),
                rightAll        : $( this.settings.rightAll ),

                undo            : $( this.settings.undo ),
                redo            : $( this.settings.redo )
            };

            if (this.settings.search_left) {
                this.elements.search_left = this.elements.left.prev();    // $(this.settings.search_left);
            }

            if (this.settings.search_right) {
                this.elements.search_right = this.elements.right.prev();     // $(this.settings.search_right);
            }

            // 将右边select有的option，在左边移除
            if ( typeof this.settings.startUp == 'function' ) {
                this.settings.startUp( this.elements.left, this.elements.right );
            }
            
            // 排序
            if ( this.settings.keepRenderingSort && typeof this.settings.sort == 'function' ) {
                this.elements.left.find('option')
                    .sort(this.settings.sort)
                    .appendTo(this.elements.left);
                
                this.elements.right
                    .each(function(i, select) {
                        // 右边可能有多个
                        if($(select).find('option').length === 0){
                            return;
                        }
                        $(select).find('option')
                            .sort(this.settings.sort)
                            .appendTo(select);
                    });
            }
        },
        bindEvents: function() {
            var context = this;
            
            this.elements.left.on('dblclick', 'option', function(e) {
                e.preventDefault();
                context.moveToRight(this, e);
            });
            
            this.elements.right.on('dblclick', 'option', function(e) {
                e.preventDefault();
                context.moveToLeft(this, e);
            });

            // append left filter
            if (this.elements.search_left) {
                this.elements.search_left.on('keyup', $.proxy(this._filteOptions, this, this.elements.left));
            }

            // append right filter
            if (this.elements.search_right) {
                this.elements.search_right.on('keyup', $.proxy(this._filteOptions, this, this.elements.right));
            }

            // select all the options from left and right side
            // when submiting the parent form
            this.elements.right.closest('form').on('submit', function(e) {
                context.elements.left.children().prop('selected', context.settings.submitAllLeft);
                context.elements.right.children().prop('selected', context.settings.submitAllRight);
            });
            
            // dblclick support for IE
            if (navigator.userAgent.match(/MSIE/i)  || 
                navigator.userAgent.indexOf('Trident/') > 0 || 
                navigator.userAgent.indexOf('Edge/') > 0) {
                this.elements.left.dblclick(function(e) {
                    context.elements.rightSelected.trigger('click');
                });
                
                this.elements.right.dblclick(function(e) {
                    context.elements.leftSelected.trigger('click');
                });
            }
            
            this.elements.rightSelected.on('click', function(e) {
                e.preventDefault();
                var options = context.elements.left.find('option:selected');
                
                if ( options ) {
                    context.moveToRight(options, e);
                }

                $(this).blur();
            });
            
            this.elements.leftSelected.on('click', function(e) {
                e.preventDefault();
                var options = context.elements.right.find('option:selected');
                
                if ( options ) {
                    context.moveToLeft(options, e);
                }

                $(this).blur();
            });
            
            this.elements.rightAll.on('click', function(e) {
                e.preventDefault();
                var options = context.elements.left.find('option');
                
                if ( options ) {
                    context.moveToRight(options, e);
                }

                $(this).blur();
            });
            
            this.elements.leftAll.on('click', function(e) {
                e.preventDefault();
                
                var options = context.elements.right.find('option');
                
                if ( options ) {
                    context.moveToLeft(options, e);
                }

                $(this).blur();
            });

            this.elements.undo.on('click', function(e) {
                e.preventDefault();

                context.undo(e);
            });

            this.elements.redo.on('click', function(e) {
                e.preventDefault();

                context.redo(e);
            });
        },
        _filteOptions: function(filterSelect, e) {
            var regex = new RegExp($(e.target).val(),"ig");

            // 兼容ie隐藏 select option
            if (navigator.userAgent.match(/MSIE/i)  || 
                        navigator.userAgent.indexOf('Trident/') > 0 || 
                        navigator.userAgent.indexOf('Edge/') > 0) {
                filterSelect.find('option, span').each(function(i, item) {
                    var jqItem = $(item);
                    if (jqItem.text().search(regex) >= 0) {
                        // 显示
                        if(jqItem.is('option')){
                            return;
                        }
                        if (jqItem.data('ref') ){
                            jqItem.replaceWith( jqItem.data('ref') );
                        }                                
                    } else {
                        // 隐藏
                        if(jqItem.is('span')){
                            return;
                        }
                        var text = jqItem.text(), 
                            val = jqItem.val(), 
                            span = $('<span/>').text(text).val(val).hide();
                                
                        var old = jqItem.clone();
                        jqItem.replaceWith( span );
                        span.data('ref', old);
                    }
                });

                return;
            }
            
            filterSelect.find('option').each(function(i, option) {
                if (option.text.search(regex) >= 0) {
                    $(option).show();
                } else {
                    $(option).hide();
                }
            });
        },
        moveToRight: function( options, event, silent, skipStack ) {
            if ( typeof this.settings.moveToRight == 'function' ) {
                this.settings.moveToRight( this, options, event, silent, skipStack );
            } else {
                if ( typeof this.settings.beforeMoveToRight == 'function' && !silent ) {
                    if ( !this.settings.beforeMoveToRight( this.elements.left, this.elements.right, options ) ) {
                        return false;
                    }
                }
                
                this.elements.right.append(options);

                if ( !skipStack ) {
                    this.undoStack.push(['right', options ]);
                    this.redoStack = [];
                }
                
                if ( typeof this.settings.sort == 'function' && !silent ) {
                    this.elements.right.find('option').sort(this.settings.sort).appendTo(this.elements.right);
                }
                
                if ( typeof this.settings.afterMoveToRight == 'function' && !silent ){
                    this.settings.afterMoveToRight( this.elements.left, this.elements.right, options );
                }
            }
        },
        
        moveToLeft: function( options, event, silent, skipStack ) {            
            if ( typeof this.settings.moveToLeft == 'function' ) {
                this.settings.moveToLeft( this, options, event, silent, skipStack );
            } else {
                if ( typeof this.settings.beforeMoveToLeft == 'function' && !silent ) {
                    if ( !this.settings.beforeMoveToLeft( this.elements.left, this.elements.right, options ) ) {
                        return false;
                    }
                }

                this.elements.left.append(options);
                
                if ( !skipStack ) {
                    this.undoStack.push(['left', options ]);
                    this.redoStack = [];
                }
                
                if ( typeof this.settings.sort == 'function' && !silent ) {
                    this.elements.left.find('option').sort(this.settings.sort).appendTo(this.elements.left);     
                }
                
                if ( typeof this.settings.afterMoveToLeft == 'function' && !silent ) {
                    this.settings.afterMoveToLeft( this.elements.left, this.elements.right, options );
                }
            }
        },

        undo: function(event) {
            var last = this.undoStack.pop();

            if ( last ) {
                this.redoStack.push(last);

                switch(last[0]) {
                    case 'left':
                        this.moveToRight(last[1], event, false, true);
                        break;
                    case 'right':
                        this.moveToLeft(last[1], event, false, true);
                        break;
                }
            }
        },
        redo: function(event) {
            var last = this.redoStack.pop();

            if ( last ) {
                this.undoStack.push(last);

                switch(last[0]) {
                    case 'left':
                        this.moveToLeft(last[1], event, false, true);
                        break;
                    case 'right':
                        this.moveToRight(last[1], event, false, true);
                        break;
                }
            }
        }
    });
});


/* rightselect javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'rightselect';
    var pluginClass = s.ui.controls.RightSelect;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        // 选项
        // fooOption: true,
        // 覆写 类方法
        // parseData: undefined,
        // 事件
        // onFooSelected: undefined,
        // onFooChange: function(e, data){}
        onSelected: undefined,
        placeholderValue:'',
        position:'down,left',

        down:'dropdown',
        up:'dropup',
        selectFirst:false
    };
    var attributeMap = {
        // fooOption: 'foo-option'
        placeholderValue:'placeholder-value',
        position:'position',
        selectFirst:'select-first'
    };

    this.Selectex = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        settings: {},
        value: '',
        data: {},
        selects: {},

        // 构造函数
        init: function(element, options){
            this.element = $(element);

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);
            // 获取初始值
            // this.value = this.element.val();

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            this.transferAttributes();

            this.bindEvents();
            this.parse();
            // this.render();

            var context = this;
            // 初始化数据
            $.when(this.getData())
             .done(function(){
                context.render();
            });
        },

        getData: function(){
            var d = $.Deferred();
            if (this.settings.data) {
                var data = $.extend(true, [], this.settings.data);
                this.updateData(data);
                delete this.settings.data;

                d.resolve();
                return d.promise();
            }
            d.resolve();
            return d.promise();
        },

        buildHtml: function(){
            //var elementClasses = this.element.attr("class");
            var pluginId= this.element.data('plugin-id');
            var htmlTemplate=''+
                '<div class="c-selectex-container"> '+
                '   <div> '+
                '       <a class="btn btn-default c-selectex-btn" data-s-dropdown-id="#c-selectex-menu-'+pluginId+'">'+
                '           <span class="c-selectex-btn-text"></span>'+
                '           <span class="glyphicon glyphicon-menu-down"></span>'+
                '       </a>'+
                '       <ul id="c-selectex-menu-'+pluginId+'" class="dropdown-menu"> '+
                '       </ul>' +
                '   </div> '+
                '</div>';
            this.container= $(htmlTemplate);
            this.element.before(this.container);
            this.element.hide();

            this.autoUpPosition();
        },
        initElements: function(){
            this.elements= {
                original: this.element,
                viewContainer: $('.c-selectex-btn', this.container),
                view: $('.c-selectex-btn-text', this.container),
                menu: $('.dropdown-menu', this.container)
                // getTab: function(levelIndex){
                //     var tabSelector = '.c-level-tab-' + levelIndex;
                //     return $(tabSelector, context.container);
                // }
            };
            // if (this.element.attr('disabled') == undefined){
            this.dropdown= new s.ui.controls.Dropdown(this.elements.viewContainer, {});
               // this.dropdown2= new s.ui.controls.Dropdown(this.elements.button, {});
            // }

        },
        autoUpPosition: function(){
            var positions = this.settings.position.split(',');
            var topPostion = positions[0];
            var leftPosition = positions[1];
            var dropCss = topPostion==='up'?this.settings.up:this.settings.down;
            $('div',this.container).addClass(dropCss);
            this.settings.isRightPosition = false;
            if(leftPosition === 'right'){
                this.settings.isRightPosition = true;
            }
        },
        autoRightPosition: function(){
            if(this.settings.isRightPosition){
                var viewWidth = this.elements.viewContainer.width();
                var menuWidth = this.elements.menu.width();
                var width = viewWidth - menuWidth;
                this.elements.menu.css('margin-left',width);
            }
        },
        transferAttributes: function(){
            this.elements.view.attr('style', this.element.attr('style'));
            this.elements.view.css('display','block');
            //this.elements.view.show();
            this.elements.view.attr('title', this.element.attr('title'));
            this.elements.view.attr('class', this.elements.view.attr('class') + ' ' + this.element.attr('class') );
            //this.elements.view.css('width', this.element.css('width'));
    
            // if (this.element.attr('disabled')!==undefined){
            //    this.disable();
            // }
            if(this.element.prop('disabled')){
                this.disable();
            }
        },
        bindEvents: function(){
            this.elements.viewContainer.on('click',$.proxy(this.setScrollTop,this));

            this.elements.menu.on('click','li',$.proxy(this.clickItem, this));

            if (typeof (this.settings.onSelected) === 'function') {
                this.elements.view.on('onSelected', $.proxy(this.settings.onSelected, this));
            }
        },
        bindEventsInterface: function(){
            var context=this;
            var element=this.element;

            // element.on('selected.s.select', $.proxy(this.settings.onFooClick, this));
        },
        parse: function () {
            var source = [];
            var i = 0;
            var map = {};
            var showMap = {};
            var key;
            this.element.find('option').each(function() {
                var option = $(this);
                var item = {
                    text:option.text(),
                    value:option.val(),
                    disabled:option.prop('disabled')
                };
                source.push(item);
                key = option.val() || option.text();
                map[key] = item;
                if(option.val())
                    showMap[key] = option.val();
                    // showMap[option.val()] = i;
                i++;
            });
            this.source = source;
            this.map = map;
            this.showMap = showMap;
        },

        updateData: function(data){
            var source = [];
            var map = {};
            var showMap = {};
            var key;
            if(!data)
                return;
            for(var i=0;i < data.length;i++){
                var item = data[i];
                source.push(item);
                key = item.value+'' || item.text;
                map[key] = item;
                if(item.value+'')
                    showMap[key] = item.value;
                    // showMap[item.value] = i;
            }
            this.source = source;
            this.map = map;
            this.showMap = showMap;
        },

        setModelValue: function(value){
            // if(!value){
            //     this.selectId = '';
            //     return;
            // }
            // var i = this.showMap[value];
            // if(!i && i !== 0)
            //     return;
            // this.selectId = i;
            this.selectId = value;
        },

        setValue: function(value){
            this.setModelValue(value);
            this.render();
        },

        getValue: function(isObject){
            var value;
            if(typeof isObject === 'undefined' || isObject === false){
                value = this.selectId;
            }else{
                value = this.map[this.selectId];
            }
            return value;
        },

        getIndex: function(selectId){
            if(!this.source)
                return 0;
            for (var i = 0; i < this.source.length; i++) {
                if(this.source[i].value === selectId)
                    return i;
            }
            return 0;
        },

        setScrollTop: function(e){
            var id = 0;
            if(this.selectId || this.selectId === 0)
                id = this.selectId;
            var distance = this.getIndex(id) - 4;
            if(distance < 0)
                distance = 0;
            distance = distance * 34;
            this.elements.menu.scrollTop(distance);
        },
        
        render: function(){
            var that = this;
            if(!this.source)
                return;
            this.elements.view.text('');
            var menuHtml = '';
            var items = this.source;
            //处理特殊字符代码
            var itemss=$.extend(true,[],items);
                items=this.replaceParseData(itemss);

            /*for (var i = 0; i < items.length; i++) {
                var item = items[i];
                if(item.unselectable || item.disabled)
                    menuHtml += '<li class="disabled"><a title="' + item.text + '" data-id="'+i+'" data-value="'+item.value+'">' + item.text + '</a></li>';
                else
                    menuHtml += '<li><a title="' + item.text + '" data-id="'+i+'" data-value="'+item.value+'">' + item.text + '</a></li>';
            }
             this.elements.menu.html(menuHtml);
             */
            this.elements.menu.html("");
            for (var i = 0; i < items.length; i++) {
                 var item = items[i];
                 var $li=$('<li/>'),$a=$('<a/>');
                 if(item.unselectable || item.disabled){
                        $li.attr("class","disabled");
                       
                 }
                 $a.attr("title",item.text).attr("data-id",""+i).attr("data-value",item.value).html(item.text);
                 $li.html($a);
                 this.elements.menu.append($li);     
            }
           
              
            var $selectItem = this.elements.menu.find('li a[data-value="'+this.selectId+'"]').parent();
            if($selectItem.length){
                $selectItem.addClass('active');
                this.elements.view.text($selectItem.text());
                this.elements.view.attr("title",this.elements.view.text());
            }
            else{
                if( this.elements.menu.find("li").length){
                    if(this.settings.selectFirst){
                        var firstObj = this.elements.menu.find("li").eq(0);
                        var firstValue = firstObj.find("a").attr('data-value');
                        var item = this.map[firstValue]||this.map[firstObj.find("a").text()];
                        this.elements.view.trigger('onSelected', item);
                        
                    }
               }
            }
            this.autoRightPosition();
        },

        clickItem: function(e){
            // this.setScrollTop();
            var target = $(e.target);
            if(target.is('li'))
                target = target.find('a');
            if(target.parent("li").hasClass("disabled"))
                return;
            var value = target.attr('data-value') || target.text();
            this.selectId = value;
            target.parent("li").addClass("active").siblings().removeClass("active");
            var item = this.map[value];
            this.elements.view.text(item.text);
            this.elements.view.attr("title",this.elements.view.text());
            this.elements.view.trigger('onSelected', item);
            // this.unselected();
            this.dropdown.toggle(e);

        },
        // unselected:function(){
        //     if (this.elements.view.text() == this.settings.placeholderValue) {
        //         this.container.addClass("unselected");
        //     }
        //     else
        //     {
        //          this.container.removeClass("unselected");
        //     }
        // },

        // API
        refresh: function(){},
        enable: function(){
            this.elements.viewContainer.removeClass('disabled');
            this.elements.viewContainer.prop('disabled',false);
            // this.elements.button.attr('disabled', false);
        },
        disable: function() {
            //this.elements.view.attr('disabled', true);
            // this.elements.button.attr('disabled', true);
            this.elements.viewContainer.addClass('disabled');
            this.elements.viewContainer.prop('disabled', true);
        },
        destroy: function(){},

         //开放接口
        getApi: function(){
            var context = this;
            var funs = {
                setValue: 'setValue',
                getValue: 'getValue',
                updateData: 'updateData'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });

});


/* select javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'selectex';
    var pluginClass = s.ui.controls.Selectex;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
            //key:'text',
            //当dataSource是数组对象时  以key值为对象的key
            map:{text:'text',value:'value',},   
            multiple:true,//多选or单选  
            itemsType:undefined,//当itemsType为object时 items存放的数据就为object
            selectedItems:[],
            returnValue:[],//只返回数组有的字段，默认为全部。
            click:function(){},
            dblclick:function(){},
            mouseover:function(){}, 
            mouseout:function(){},
    };
    var attributeMap = {
       
    };

    this.Selectlist = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        settings: {},
        
        // 构造函数
        init: function(element, options){
            this.$element =this.element= $(element);
            var self=this;
            this.events={
                click:function(e){
                    var target=$(e.target).is('li') ?  $(e.target)  : $(e.target).parent('li') ;

                    if(!target.is('li')||target.hasClass('c-selectlist-disabled')) return;
                    if(this.settings.multiple){
                            $(target).toggleClass('c-selectlist-active');
                    }
                    else{
                            $(target).siblings('li.c-selectlist-active').removeClass('c-selectlist-active');
                            $(target).addClass('c-selectlist-active');
                    }
                    this.selectedItem();
                    this.settings.click.apply($(e.target),[e,target.data('s-value')]);
                },
                dblclick:function(e){
                    var target=$(e.target);
                    if(!target.is('li')||target.hasClass('c-selectlist-disabled')) 
                        return;
                    this.settings.dblclick.apply($(e.target),[e,target.data('s-value')]);
                },
                mouseover:function(e){
                    var target=$(e.target);
                    if(!target.is('li')||target.hasClass('c-selectlist-disabled')) 
                        return;
                    this.settings.mouseover.apply($(e.target),[e,target.data('s-value')]);
                },
                mouseout:function(e){
                    var target=$(e.target);
                    if(!target.is('li')||target.hasClass('c-selectlist-disabled')) 
                        return;
                    this.settings.mouseout.apply($(e.target),[e,target.data('s-value')]);
                }

        };

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);
            // 获取初始值
            // this.value = this.element.val();

            this.chooseValue = [];
            this.settings.dataSource=this.settings.dataSource||[];
            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            this.bindEvents();
        },

        buildHtml: function(){
            var $el=$('<div class="c-selectlist-container"><ul></ul></div>');
            var $li,
                dataSource=this.settings.dataSource,
                key=this.settings.map.text;
            for(var i=0;i<dataSource.length;i++){
                
                if(typeof dataSource[i]=='object'){
               
                     $li=$('<li onselectstart="return false">'+dataSource[i][key]+'</li>');
                }
                else{
    
                     $li=$('<li onselectstart="return false">'+dataSource[i]+'</li>');   
                }
                $li.data('s-value',dataSource[i]);
                $el.find('ul').append($li);
            }
            this.settings.itemsType=typeof dataSource[0];
           
            this.$element.html($el);
         
        },
        initElements: function(){
            this.elements= {};
        },
        transferAttributes: function(){

        },
        bindEvents: function(){
          for(var event in this.events)
            this.$element.on(event,$.proxy(this.events[event], this));
        },
        bindEventsInterface: function(){
            var context=this;
            var element=this.element;

            // element.on('selected.s.select', $.proxy(this.settings.onFooClick, this));
        },
        setData:function(data){
            this.destroy();
           // this.settings.selectedItems.splice(0,this.settings.selectedItems.length);
            this.settings.selectedItems=[];
            this.settings.dataSource=data;
            this.buildHtml();
            this.bindEvents();
        },
        addItem:function(datas){
            var data,
               key=this.settings.map.text,
                $li;
            if(!Array.isArray(datas)){
                datas=[datas];
            }
            for(var i=0;i<datas.length;i++){
                data=datas[i];
            if(typeof data=='object'){
                $li=$('<li onselectstart="return false">'+data[key]+'</li>');
            }else{
                $li=$('<li onselectstart="return false">'+data+'</li>');
            }
            $li.data('s-value',data);
            this.$element.find('ul').append($li);
            }
        },
        coverItem:function(datas){
            var data,
               key=this.settings.map.text,
                $li;
            if(!Array.isArray(datas)){
                datas=[datas];
            }
            for(var i=0;i<datas.length;i++){
                data=datas[i];
            if(typeof data=='object'){
                $li=$('<li onselectstart="return false">'+data[key]+'</li>');
            }else{
                $li=$('<li onselectstart="return false">'+data+'</li>');
            }
            $li.data('s-value',data);
            this.$element.find('ul').html($li);
            }
        },
        getItems:function(){
            var items=[];
            this.$element.find('li').each(function(index,item){
                  items.push($(item).data('s-value'));
            })
            return items;
        },
        getCheckedItems:function(){
            var items=[];
            this.$element.find('li.c-selectlist-checked').each(function(index,item){
                  items.push($(item).data('s-value'));
            });
            return items;
        },
        selectedItem:function(){
            var arr=[];
            //this.settings.selectedItems.splice(0,this.options.selectedItems.length);
            this.settings.selectedItems=[];
            this.$element.find('li.c-selectlist-active').each(function(index,item){
                arr.push($(item).data('s-value'));
            });     
            this.settings.selectedItems=arr; 
        },
        getSelectedItems:function(){
            var arr=[],
                returnValue=this.settings.returnValue,
                itemsType=this.settings.itemsType,
                selectedItems=this.settings.selectedItems;
            if('object'==itemsType&&returnValue.length>0){
                for(var z=0;z<selectedItems.length;z++){
                    var obj=new Object({});
                    for(var i=0;i<returnValue.length;i++){
                            obj[returnValue[i]]=selectedItems[z][returnValue[i]];
                    }
                    arr.push(obj);
                }
                return arr;
            }
            return this.settings.selectedItems;
        },
        clearCheckedItems:function(){
            this.$element.find('li').removeClass('c-selectlist-checked');
        },
        clearSelectedItems:function(){
            this.$element.find('li').removeClass('c-selectlist-active');
        },
        deleteItem:function(data){
            this.actionType(data,'deleteItem',true);
        },
        setSelectedItem:function(data){ 
            this.actionType(data,'setSelectedItem',true);
        },
        setCheckedItems:function(data){
            this.actionType(data,'setCheckedItems');
        },
        unCheckedItems:function(data){
            this.actionType(data,'unCheckedItems');
        },
        setItemDisabled:function(data){
            this.actionType(data,'setItemDisabled',true);
        },
        actionType:function(datas,type,bol){
            if(!Array.isArray(datas)){
                datas=[datas];
            }
            for(var i=0;i<datas.length;i++){
                if(this.settings.itemsType=='object'){
                    this.isObject(datas[i],type);
                }
                else{
                    this.isString(datas[i],type);
                }
            }
            if(bol)
                this.selectedItem();
        },
        isObject:function(data,type){
            //var key=this.settings.key;
            var value=this.settings.map.value;
            this.$element.find('li').each(function(index,item){
                if($(item).data('s-value')[value]===data[value]){
                    if('setItemDisabled'===type){
                        $(item).addClass('c-selectlist-disabled').removeClass('c-selectlist-active');
                    }else if('setSelectedItem'===type){
                        if($(item).hasClass('c-selectlist-disabled')) return; 
                        $(item).addClass('c-selectlist-active');
                    }else if('deleteItem'===type){
                        $(item).remove();
                    } else if('setCheckedItems'===type){
                        $(item).addClass('c-selectlist-checked');
                    }else if('unCheckedItems'===type){
                        $(item).removeClass('c-selectlist-checked');
                    }
                }
            });
        },
        isString:function(data,type){
            this.$element.find('li').each(function(index,item){
                if($(item).data('s-value')===data){
                    if('setItemDisabled'===type){
                        $(item).addClass('c-selectlist-disabled').removeClass('c-selectlist-active');
                    }else if('setSelectedItem'===type){
                        if($(item).hasClass('c-selectlist-disabled')) return; 
                        $(item).addClass('c-selectlist-active');
                    }else if('deleteItem'===type){
                        $(item).remove();
                    }
                    else if('setCheckedItems'===type){
                        $(item).addClass('c-selectlist-checked');
                    }else if('unCheckedItems'===type){
                        $(item).removeClass('c-selectlist-checked');
                    }
                }
            });
        },
        getApi: function(){
            var context = this;
            var funs = {
                setData: 'setData',
                setSelectedItem: 'setSelectedItem',
                getSelectedItems:'getSelectedItems',
                setItemDisabled:'setItemDisabled',
                addItem:'addItem',
                coverItem:'coverItem',
                getItems:'getItems',
                deleteItem:'deleteItem',
                setCheckedItems:'setCheckedItems',
                unCheckedItems:'unCheckedItems',
                getCheckedItems :'getCheckedItems',

            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        },
        destroy:function(){
            for(var event in this.events)
                this.$element.off(event);
        },
        // API
        refresh: function(){},
        enable: function(){
            
        },
        disable: function() {
           
        },
    });

});


/* select javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'selectlist';
    var pluginClass = s.ui.controls.Selectlist;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
/*
 * metismenu - v1.1.3
 * Easy menu jQuery plugin for Twitter Bootstrap 3
 * https://github.com/onokumus/metisMenu
 *
 * Made by Osman Nuri Okumus
 * Under MIT License
 */
;(function($, window, document, undefined) {

    // 严格模式
    'use strict';

    var pluginName = "sidebar",
        defaults = {
            toggle: true,
            doubleTapToGo: false
        };

    function Plugin(element, options) {
        this.element = $(element);
        this.settings = $.extend({}, defaults, options);
        this._defaults = defaults;
        this._name = pluginName;
        this.init();
    }

    Plugin.prototype = {
        init: function() {

            var $this = this.element,
                $toggle = this.settings.toggle,
                obj = this;

            if (this.isIE() <= 9) {
                $this.find("li.active").has("ul").children("ul").collapse("show");
                $this.find("li").not(".active").has("ul").children("ul").collapse("hide");
            } else {
                $this.find("li.active").has("ul").children("ul").addClass("collapse in");
                $this.find("li").not(".active").has("ul").children("ul").addClass("collapse");
            }

            //add the "doubleTapToGo" class to active items if needed
            if (obj.settings.doubleTapToGo) {
                $this.find("li.active").has("ul").children("a").addClass("doubleTapToGo");
            }

            var content= this;
            $this.find("li").has("ul").children("a").on("click" + "." + pluginName, function(e) {
                e.preventDefault();

                //Do we need to enable the double tap
                if (obj.settings.doubleTapToGo) {

                    //if we hit a second time on the link and the href is valid, navigate to that url
                    if (obj.doubleTapToGo($(this)) && $(this).attr("href") !== "#" && $(this).attr("href") !== "") {
                        e.stopPropagation();
                        document.location = $(this).attr("href");
                        return;
                    }
                }

                var childUL= $(this).parent("li").toggleClass("active").children("ul");
                // childUL.collapse("toggle");
                if(childUL.hasClass('in')){
                    if (content.isIE() <= 9) {
                        childUL.hide();
                    }
                    childUL.removeClass('in');
                }
                else{
                    if (content.isIE() <= 9) {
                        childUL.show();
                    }
                    childUL.addClass('in');
                }

                if ($toggle) {
                    $(this).parent("li").siblings().removeClass("active").children("ul.in").collapse("hide");
                }

            });
        },

        isIE: function() { //https://gist.github.com/padolsey/527683
            var undef,
                v = 3,
                div = document.createElement("div"),
                all = div.getElementsByTagName("i");

            while (
                div.innerHTML = "<!--[if gt IE " + (++v) + "]><i></i><![endif]-->",
                all[0]
            ) {
                return v > 4 ? v : undef;
            }
        },

        //Enable the link on the second click.
        doubleTapToGo: function(elem) {
            var $this = this.element;

            //if the class "doubleTapToGo" exists, remove it and return
            if (elem.hasClass("doubleTapToGo")) {
                elem.removeClass("doubleTapToGo");
                return true;
            }

            //does not exists, add a new class and return false
            if (elem.parent().children("ul").length) {
                 //first remove all other class
                $this.find(".doubleTapToGo").removeClass("doubleTapToGo");
                //add the class on the current element
                elem.addClass("doubleTapToGo");
                return false;
            }
        },

        destroy: function() {
            this.element.off("." + pluginName);
            this.element.removeData(pluginName);
        }

    };

    $.fn[pluginName] = function(options) {
        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new Plugin(this, options));
        });
        return this;
    };

})(jQuery, window, document);
/* spinner javascript jQuery */

/*
 *  Bootstrap TouchSpin - v3.0.1
 *  A mobile and touch friendly input spinner component for Bootstrap 3.
 *  http://www.virtuosoft.eu/code/bootstrap-touchspin/
 *
 *  Made by István Ujj-Mészáros
 *  Under Apache License v2.0 License
 */

Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;

    function _scopedEventName(name, id) {
        return name + '.touchspin_' + id;
    }

    function _scopeEventNames(names, id) {
        return $.map(names, function(name) {
            return _scopedEventName(name, id);
        });
    }

    var defaults = {
        min: 0,
        max: 100,
        //initval: '',
        defaultValue: '', 
        step: 1,
        decimals: 0,
        stepinterval: 100,
        forcestepdivisibility: 'round', // none | floor | round | ceil
        stepintervaldelay: 500,
        //verticalbuttons: false,
        verticalButtons: false,
        verticalupclass: 'glyphicon glyphicon-chevron-up',
        verticaldownclass: 'glyphicon glyphicon-chevron-down',
        prefix: '',
        postfix: '',
        prefix_extraclass: '',
        postfix_extraclass: '',
        booster: true,
        boostat: 10,
        maxboostedstep: false,
        mousewheel: true,
        buttondown_class: 'btn btn-default',
        buttonup_class: 'btn btn-default'
    };

    var attributeMap = {
        min: 'min',
        max: 'max',
        //initval: 'init-val',
        defaultValue: 'default-value',
        step: 'step',
        decimals: 'decimals',
        stepinterval: 'step-interval',
        //verticalbuttons: 'vertical-buttons',
        verticalButtons: 'vertical-buttons',
        verticalupclass: 'vertical-up-class',
        verticaldownclass: 'vertical-down-class',
        forcestepdivisibility: 'force-step-divisibility',
        stepintervaldelay: 'step-interval-delay',
        prefix: 'prefix',
        postfix: 'postfix',
        prefix_extraclass: 'prefix-extra-class',
        postfix_extraclass: 'postfix-extra-class',
        booster: 'booster',
        boostat: 'boostat',
        maxboostedstep: 'max-boosted-step',
        mousewheel: 'mouse-wheel',
        buttondown_class: 'button-down-class',
        buttonup_class: 'button-up-class'
    };

    this.Spinner = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,
        // 构造函数
        init: function(element, options){
            if (options === 'destroy') {
                this.each(function() {
                    var element = $(element),
                            element_data = element.data();
                    $(document).off(_scopeEventNames([
                        'mouseup',
                        'touchend',
                        'touchcancel',
                        'mousemove',
                        'touchmove',
                        'scroll',
                        'scrollstart'], element_data.spinnerid).join(' '));
                });
                return;
            }

            _currentPluginId += 1;

            this.element = $(element);
            this.element_data = this.element.data();
            this.elements;

            this.settings;            
            this.container;
            
            this.value;
            this.downSpinTimer;
            this.upSpinTimer;
            this.downDelayTimeout;
            this.upDelayTimeout;
            this.spincount = 0;
            this.spinning = false;

            if (!this.element.is('input')) {
                console.log('Must be an input.');
                return;
            }

            if (this.element.data('alreadyinitialized')) {
                return;
            }

            this.element.data('alreadyinitialized', true);            
            this.element.data('spinnerid', _currentPluginId);

            this.initSettings();
            this._setInitval();
            this._checkValue();
            this._buildHtml();
            this._initElements();
            this._hideEmptyPrefixPostfix();
            this._bindEvents();
            this._bindEventsInterface();

            this.elements.input.css('display', 'block');
        },

        _setInitval: function () {
            // if (this.settings.initval !== '' && this.element.val() === '') {
            //     this.element.val(this.settings.initval);
            // }
            if (this.settings.defaultValue !== '' && this.element.val() === '') {
                this.element.val(this.settings.defaultValue);
            }
        },

        changeSettings: function (newsettings) {
            this._updateSettings(newsettings);
            this._checkValue();

            var value = this.elements.input.val();

            if (value !== '') {
                value = Number(this.elements.input.val());
                this.elements.input.val(value.toFixed(this.settings.decimals));
            }
        },

        _updateSettings: function (newsettings) {
            this.settings = $.extend({}, this.settings, newsettings);
        },

        _buildHtml: function () {
            var initval = this.element.val(),
                    parentelement = this.element.parent();

            if (initval !== '') {
                initval = Number(initval).toFixed(this.settings.decimals);
            }

            this.element.data('initvalue', initval).val(initval);
            this.element.addClass('form-control');

            if (parentelement.hasClass('input-group')) {
                this._advanceInputGroup(parentelement);
            }
            else {
                this._buildInputGroup();
            }
        },

        _advanceInputGroup: function (parentelement) {
            parentelement.addClass('bootstrap-spinner');

            var prev = this.element.prev(),
                    next = this.element.next();

            var downhtml,
                    uphtml,
                    prefixhtml = '<span class="input-group-addon bootstrap-spinner-prefix">' + this.settings.prefix + '</span>',
                    postfixhtml = '<span class="input-group-addon bootstrap-spinner-postfix">' + this.settings.postfix + '</span>';

            if (prev.hasClass('input-group-btn')) {
                downhtml = '<button class="' + this.settings.buttondown_class + ' bootstrap-spinner-down" type="button">-</button>';
                prev.append(downhtml);
            }
            else {
                downhtml = '<span class="input-group-btn"><button class="' + this.settings.buttondown_class + ' bootstrap-spinner-down" type="button">-</button></span>';
                $(downhtml).insertBefore(this.element);
            }

            if (next.hasClass('input-group-btn')) {
                uphtml = '<button class="' + this.settings.buttonup_class + ' bootstrap-spinner-up" type="button">+</button>';
                next.prepend(uphtml);
            }
            else {
                uphtml = '<span class="input-group-btn"><button class="' + this.settings.buttonup_class + ' bootstrap-spinner-up" type="button">+</button></span>';
                $(uphtml).insertAfter(this.element);
            }

            $(prefixhtml).insertBefore(this.element);
            $(postfixhtml).insertAfter(this.element);

            this.container = parentelement;
        },

        _buildInputGroup: function () {
            var html;

            //if (this.settings.verticalbuttons) {
            if (this.settings.verticalButtons) {
                html = '<div class="input-group bootstrap-spinner"><span class="input-group-addon bootstrap-spinner-prefix">' + 
                    this.settings.prefix + 
                    '</span><span class="input-group-addon bootstrap-spinner-postfix">' + 
                    this.settings.postfix + '</span><span class="input-group-btn-vertical"><button class="' + 
                    this.settings.buttondown_class + ' bootstrap-spinner-up" type="button"><i class="' + 
                    this.settings.verticalupclass + '"></i></button><button class="' + 
                    this.settings.buttonup_class + ' bootstrap-spinner-down" type="button"><i class="' + 
                    this.settings.verticaldownclass + '"></i></button></span></div>';
            }
            else {
                html = '<div class="input-group bootstrap-spinner"><span class="input-group-btn"><button class="' + 
                this.settings.buttondown_class + ' bootstrap-spinner-down" type="button">-</button></span><span class="input-group-addon bootstrap-spinner-prefix">' + 
                this.settings.prefix + '</span><span class="input-group-addon bootstrap-spinner-postfix">' + 
                this.settings.postfix + '</span><span class="input-group-btn"><button class="' + 
                this.settings.buttonup_class + ' bootstrap-spinner-up" type="button">+</button></span></div>';
            }

            this.container = $(html).insertBefore(this.element);

            $('.bootstrap-spinner-prefix', this.container).after(this.element);

            if (this.element.hasClass('input-sm')) {
                this.container.addClass('input-group-sm');
            }
            else if (this.element.hasClass('input-lg')) {
                this.container.addClass('input-group-lg');
            }
        },

        _initElements: function () {
            this.elements = {
                down: $('.bootstrap-spinner-down', this.container),
                up: $('.bootstrap-spinner-up', this.container),
                input: $('input', this.container),
                prefix: $('.bootstrap-spinner-prefix', this.container).addClass(this.settings.prefix_extraclass),
                postfix: $('.bootstrap-spinner-postfix', this.container).addClass(this.settings.postfix_extraclass)
            };
        },

        _hideEmptyPrefixPostfix: function () {
            if (this.settings.prefix === '') {
                this.elements.prefix.hide();
            }

            if (this.settings.postfix === '') {
                this.elements.postfix.hide();
            }
        },

        _bindEvents: function () {
            var context = this;

            this.element.on('keydown', function(ev) {
                var code = ev.keyCode || ev.which;

                if (code === 38) {
                    if (context.spinning !== 'up') {
                        context.upOnce();
                        context.startUpSpin();
                    }
                    ev.preventDefault();
                }
                else if (code === 40) {
                    if (context.spinning !== 'down') {
                        context.downOnce();
                        context.startDownSpin();
                    }
                    ev.preventDefault();
                }
            });

            this.element.on('keyup', function(ev) {
                var code = ev.keyCode || ev.which;

                if (code === 38) {
                    context.stopSpin();
                }
                else if (code === 40) {
                    context.stopSpin();
                }
            });

            this.element.on('blur', function() {
                context._checkValue();
            });

            this.elements.down.on('keydown', function(ev) {
                var code = ev.keyCode || ev.which;

                if (code === 32 || code === 13) {
                    if (context.spinning !== 'down') {
                        context.downOnce();
                        context.startDownSpin();
                    }
                    ev.preventDefault();
                }
            });

            this.elements.down.on('keyup', function(ev) {
                var code = ev.keyCode || ev.which;

                if (code === 32 || code === 13) {
                    context.stopSpin();
                }
            });

            this.elements.up.on('keydown', function(ev) {
                var code = ev.keyCode || ev.which;

                if (code === 32 || code === 13) {
                    if (context.spinning !== 'up') {
                        context.upOnce();
                        context.startUpSpin();
                    }
                    ev.preventDefault();
                }
            });

            this.elements.up.on('keyup', function(ev) {
                var code = ev.keyCode || ev.which;

                if (code === 32 || code === 13) {
                    context.stopSpin();
                }
            });

            this.elements.down.on('mousedown.spinner', function(ev) {
                context.elements.down.off('touchstart.spinner');    // android 4 workaround

                if (context.element.is(':disabled')) {
                    return;
                }

                context.downOnce();
                context.startDownSpin();

                ev.preventDefault();
                ev.stopPropagation();
            });

            this.elements.down.on('touchstart.spinner', function(ev) {
                context.elements.down.off('mousedown.spinner');    // android 4 workaround

                if (this.element.is(':disabled')) {
                    return;
                }

                context._budownOnce();
                context.startDownSpin();

                ev.preventDefault();
                ev.stopPropagation();
            });

            this.elements.up.on('mousedown.spinner', function(ev) {
                context.elements.up.off('touchstart.spinner');    // android 4 workaround

                if (context.element.is(':disabled')) {
                    return;
                }

                context.upOnce();
                context.startUpSpin();

                ev.preventDefault();
                ev.stopPropagation();
            });

            this.elements.up.on('touchstart.spinner', function(ev) {
                context.elements.up.off('mousedown.spinner');    // android 4 workaround

                if (context.element.is(':disabled')) {
                    return;
                }

                context.upOnce();
                context.startUpSpin();

                ev.preventDefault();
                ev.stopPropagation();
            });

            this.elements.up.on('mouseout touchleave touchend touchcancel', function(ev) {
                if (!context.spinning) {
                    return;
                }

                ev.stopPropagation();
                context.stopSpin();
            });

            this.elements.down.on('mouseout touchleave touchend touchcancel', function(ev) {
                if (!context.spinning) {
                    return;
                }

                ev.stopPropagation();
                context.stopSpin();
            });

            this.elements.down.on('mousemove touchmove', function(ev) {
                if (!context.spinning) {
                    return;
                }

                ev.stopPropagation();
                ev.preventDefault();
            });

            this.elements.up.on('mousemove touchmove', function(ev) {
                if (!context.spinning) {
                    return;
                }

                ev.stopPropagation();
                ev.preventDefault();
            });

            $(document).on(_scopeEventNames(['mouseup', 'touchend', 'touchcancel'], _currentPluginId).join(' '), function(ev) {
                if (!context.spinning) {
                    return;
                }

                ev.preventDefault();
                context.stopSpin();
            });

            $(document).on(_scopeEventNames(['mousemove', 'touchmove', 'scroll', 'scrollstart'], _currentPluginId).join(' '), function(ev) {
                if (!context.spinning) {
                    return;
                }

                ev.preventDefault();
                context.stopSpin();
            });

            this.element.on('mousewheel DOMMouseScroll', function(ev) {
                if (!context.settings.mousewheel || !context.element.is(':focus')) {
                    return;
                }

                var delta = ev.originalEvent.wheelDelta || -ev.originalEvent.deltaY || -ev.originalEvent.detail;

                ev.stopPropagation();
                ev.preventDefault();

                if (delta < 0) {
                    context.downOnce();
                }
                else {
                    context.upOnce();
                }
            });
        },

        _bindEventsInterface: function () {
            var context = this;
            this.element.on('spinner.uponce', function() {
                context.stopSpin();
                context.upOnce();
            });

            this.element.on('spinner.downonce', function() {
                context.stopSpin();
                context.downOnce();
            });

            this.element.on('spinner.startupspin', function() {
                context.startUpSpin();
            });

            this.element.on('spinner.startdownspin', function() {
                context.startDownSpin();
            });

            this.element.on('spinner.stopspin', function() {
                context.stopSpin();
            });

            this.element.on('spinner.updatesettings', function(e, newsettings) {
                context.changeSettings(newsettings);
            });
        },

        _forcestepdivisibility: function (value) {
            switch (this.settings.forcestepdivisibility) {
                case 'round':
                    return (Math.round(value / this.settings.step) * this.settings.step).toFixed(this.settings.decimals);
                case 'floor':
                    return (Math.floor(value / this.settings.step) * this.settings.step).toFixed(this.settings.decimals);
                case 'ceil':
                    return (Math.ceil(value / this.settings.step) * this.settings.step).toFixed(this.settings.decimals);
                default:
                    return value;
            }
        },

        _checkValue: function () {
            var val, parsedval, returnval;

            val = this.element.val();

            if (val === '') {
                return;
            }

            if (this.settings.decimals > 0 && val === '.') {
                return;
            }

            parsedval = parseFloat(val);

            if (isNaN(parsedval)) {
                parsedval = 0;
            }

            returnval = parsedval;

            if (parsedval.toString() !== val) {
                returnval = parsedval;
            }

            if (parsedval < this.settings.min) {
                returnval = this.settings.min;
            }

            if (parsedval > this.settings.max) {
                returnval = this.settings.max;
            }

            returnval = this._forcestepdivisibility(returnval);

            if (Number(val).toString() !== returnval.toString()) {
                this.element.val(returnval);
                this.element.trigger('change');
            }
        },

        _getBoostedStep: function () {
            if (!this.settings.booster) {
                return this.settings.step;
            }
            else {
                var boosted = Math.pow(2, Math.floor(this.spincount / this.settings.boostat)) * this.settings.step;

                if (this.settings.maxboostedstep) {
                    if (boosted > this.settings.maxboostedstep) {
                        boosted = this.settings.maxboostedstep;
                        this.value = Math.round((this.value / boosted)) * boosted;
                    }
                }

                return Math.max(this.settings.step, boosted);
            }
        },

        upOnce: function () {
            this._checkValue();

            this.value = parseFloat(this.elements.input.val());
            if (isNaN(this.value)) {
                this.value = 0;
            }

            var initvalue = this.value,
                    boostedstep = this._getBoostedStep();

            this.value = this.value + boostedstep;

            if (this.value > this.settings.max) {
                this.value = this.settings.max;
                this.element.trigger('spinner.on.max');
                this.stopSpin();
            }

            this.elements.input.val(Number(this.value).toFixed(this.settings.decimals));

            if (initvalue !== this.value) {
                this.element.trigger('change');
            }
        },

        downOnce: function () {
            this._checkValue();

            this.value = parseFloat(this.elements.input.val());
            if (isNaN(this.value)) {
                this.value = 0;
            }

            var initvalue = this.value,
                    boostedstep = this._getBoostedStep();

            this.value = this.value - boostedstep;

            if (this.value < this.settings.min) {
                this.value = this.settings.min;
                this.element.trigger('spinner.on.min');
                this.stopSpin();
            }

            this.elements.input.val(this.value.toFixed(this.settings.decimals));

            if (initvalue !== this.value) {
                this.element.trigger('change');
            }
        },

        startDownSpin: function () {
            var context = this;

            this.stopSpin();

            this.spincount = 0;
            this.spinning = 'down';

            this.element.trigger('spinner.on.startspin');
            this.element.trigger('spinner.on.startdownspin');

            this.downDelayTimeout = setTimeout(function() {
                context.downSpinTimer = setInterval(function() {
                    context.spincount++;
                    context.downOnce();
                }, context.settings.stepinterval);
            }, this.settings.stepintervaldelay);
        },

        startUpSpin: function () {
            var context = this;

            this.stopSpin();

            this.spincount = 0;
            this.spinning = 'up';

            this.element.trigger('spinner.on.startspin');
            this.element.trigger('spinner.on.startupspin');

            this.upDelayTimeout = setTimeout(function() {
                context.upSpinTimer = setInterval(function() {
                    context.spincount++;
                    context.upOnce();
                }, context.settings.stepinterval);
            }, this.settings.stepintervaldelay);
        },

        stopSpin: function () {
            clearTimeout(this.downDelayTimeout);
            clearTimeout(this.upDelayTimeout);
            clearInterval(this.downSpinTimer);
            clearInterval(this.upSpinTimer);

            switch (this.spinning) {
                case 'up':
                    this.element.trigger('spinner.on.stopupspin');
                    this.element.trigger('spinner.on.stopspin');
                    break;
                case 'down':
                    this.element.trigger('spinner.on.stopdownspin');
                    this.element.trigger('spinner.on.stopspin');
                    break;
            }

            this.spincount = 0;
            this.spinning = false;
        }

    });

});


/* spinner javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'spinner';
    var pluginClass = s.ui.controls.Spinner;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
            text:'切换至简版,切换至祥版'
    };
    var attributeMap = {
        text:'text'
    };

    this.Switchex = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // 构造函数
        init: function(element, options){
            this.element = $(element);
            
            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);

            var texts=this.settings.text.split(',');
            this.leftText=texts[0];
            this.rightText=texts[1];

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            // this.transferAttributes();
            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            
        },
        buildHtml: function(){
            var htmlTemplate=
                                '<div class="c-switchex-container" id="'+_currentPluginId+'">'
                                +       '<div class="c-switchex-content c-switchex-left">'+this.leftText+'</div>'
                                +       '<div class="c-switchex-circle c-switchex-right"></div>'
                                +'</div>'
            this.container=$(htmlTemplate); 
            this.element.before(this.container);
            this.element.hide();                       
        },
        initElements: function(){
        	this.elements={
        		view: this.container,
                left:$('.c-switchex-left',this.container),
                right:$('.c-switchex-right',this.container)
        	};
        },
        bindEvents: function(){
            this.elements.view.on('click',$.proxy(this.click,this));
        },
        click:function(e){
            var target=$(e.target);
            if(target.hasClass('c-switchex-circle')){
                if(target.hasClass('c-switchex-left')){
                    this.elements.right.addClass('c-switchex-circle').removeClass('c-switchex-content').html("");
                    this.elements.left.removeClass('c-switchex-circle').addClass('c-switchex-content').html(this.leftText);
                }else{
                    this.elements.left.addClass('c-switchex-circle').removeClass('c-switchex-content').html("");
                    this.elements.right.removeClass('c-switchex-circle').addClass('c-switchex-content').html(this.rightText);    
                }                
            }   
            
        },
        render: function(){},
        // API
        refresh: function(){},
        enable: function(){},
        disable: function(){},
        destroy: function(){}
    });

});


/* switchex javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'switchex';
    var pluginClass = s.ui.controls.Switchex;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
/* ========================================================================
 * Bootstrap: tab.js v3.3.5
 * http://getbootstrap.com/javascript/#tabs
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;

    var TRANSITION_DURATION = 150

    var emptyFun= function(){};
    var defaults = {
        // 选项
        // fooOption: true,
        // 覆写 类方法
        // parseData: undefined,
        // 事件
        // onFooSelected: undefined,
        // onFooChange: function(e, data){}
        onSelected: emptyFun
    };
    var attributeMap = {
        // fooOption: 'foo-option'
    };

    this.Tabs = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // 构造函数
        init: function(element, options){
            this.element = $(element);
            this.container=this.element;

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);
            // 获取初始值
            // this.value = this.element.val();

            // 初始化 html DOM 元素
            this.initElements();
            // this.transferAttributes();

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            // this.bindEventsInterface();            
        },
        initElements: function(){
            this.elements={
                original: this.element,
                tabsContainer: $('ul.nav-tabs', this.container),
                tabs: $('ul.nav-tabs .tab', this.container)//,
                // tabPanes: $('div.tab-pane', this.container)
            };
        },
        bindEvents: function(){
            this.elements.tabs.on('click', $.proxy(this.show, this));
            this.elements.tabsContainer.on('onselected', $.proxy(this.settings.onSelected, this));
        },
        bindEventsInterface: function(){
            var context=this;
            var element=this.element;

            // element.on('selected.s.template', $.proxy(this.settings.onFooClick, this));
        },
        setActive: function(index){
            var activeTarget = this.elements.tabsContainer.find('li').eq(index).find('a.tab');
            var eventVar = {   //模仿点击
                preventDefault : emptyFun,
                target: activeTarget
            };
            this.show(eventVar);
        },
        getActiveIndex: function(){
            var activeTarget = this.elements.tabsContainer.find('li.active');
            var index = activeTarget.index();
            return index;
        },
        show: function (e) {
            e.preventDefault();
            // var $this    = this.element
            var $this    = $(e.target);

            // var $ul      = $this.closest('ul:not(.dropdown-menu)')
            var selector = $this.data('s-target');
            if (!selector) {
                selector = $this.attr('href')
                selector = selector && selector.replace(/.*(?=#[^\s]*$)/, '') // strip for ie7
            }

            if ($this.parent('li').hasClass('active')) return

            // var $previous = $ul.find('.active:last a')
            var $previous = this.elements.tabsContainer.find('.active:last .tab');
            var hideEvent = $.Event('hide.bs.tab', {
                relatedTarget: $this[0]
            })
            var showEvent = $.Event('show.bs.tab', {
                relatedTarget: $previous[0]
            })

            $previous.trigger(hideEvent);
            $this.trigger(showEvent)
            if (showEvent.isDefaultPrevented() || hideEvent.isDefaultPrevented()) return

            var $target = $(selector);

            this.activate($this.closest('li'), this.elements.tabsContainer);
            this.activate($target, $target.parent(), function () {
                $previous.trigger({
                    type: 'hidden.bs.tab',
                    relatedTarget: $this[0]
                })
                $this.trigger({
                    type: 'shown.bs.tab',
                    relatedTarget: $previous[0]
                })
            });

            //$this.closest('li').index()
            this.elements.tabsContainer.trigger('onselected',$this.closest('li').index());
        },
        activate: function (element, container, callback) {
            var $active    = container.find('> .active')
            var transition = callback
                && $.support.transition
                && ($active.length && $active.hasClass('fade') || !!container.find('> .fade').length)

            function next() {
                $active
                    .removeClass('active')
                    .find('> .dropdown-menu > .active')
                        .removeClass('active')
                    .end()
                    .find('[data-toggle="tab"]')
                        .attr('aria-expanded', false)

                element
                    .addClass('active')
                    .find('[data-toggle="tab"]')
                        .attr('aria-expanded', true)

                if (transition) {
                    element[0].offsetWidth // reflow for transition
                    element.addClass('in')
                } else {
                    element.removeClass('fade')
                }

                if (element.parent('.dropdown-menu').length) {
                    element
                        .closest('li.dropdown')
                            .addClass('active')
                        .end()
                        .find('[data-toggle="tab"]')
                            .attr('aria-expanded', true)
                }

                callback && callback()
            }

            $active.length && transition ?
                $active
                    .one('bsTransitionEnd', next)
                    .emulateTransitionEnd(TRANSITION_DURATION) :
                next()

            $active.removeClass('in');
        },
        // 事件处理
        // onFooClick: function(e, data){
        //     context= this;
        //  context.element.trigger('selected.s.template');
        // }

        // API
        reflash: function(){},
        enable: function(){},
        disable: function(){},
        destroy: function(){},

        getApi: function(){
            var context = this;
            var funs = {
                setActive: 'setActive',
                getActiveIndex: 'getActiveIndex'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });

});


/* template javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'tabs';
    var pluginClass = s.ui.controls.Tabs;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
	
Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId= 0;
   
    var emptyFun= function(){};
    var defaults = {
        maxLength: 100,
        update: emptyFun,
        tip:'还可以输入'
    };

    var attributeMap = {
        maxLength: 'max-length'
        //tip:'tip'
    };



    this.Textlimit = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,
        // 构造函数
        init: function(element, options){
        	// 区分一个页面中存在多个控件实例
            _currentPluginId += 1;

            this.element= $(element);
        
            //this.container= $(element).parent();
           
            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { return; }
            this.element.data('initialized', true);
            
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);

            this.placeholder = this.element.attr('placeholder');
            this.wordslength = this.settings.maxLength;

             //this.element.placeholder();

            // 初始化数据
            // this.getData();

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            //this.transferAttributes();

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            // this.bindEventsInterface();
        },
        buildHtml:function(){
            var htmlTemplate ='' +  
                '<div class="c-textlimit-wrap"> ' +
                //'   <textarea textlimit class="c-textlimit form-control" data-s-max-length=' + this.wordslength + ' placeholder=' + this.placeholder + '></textarea> ' +
                '   <div class="tip text-right"> ' + 
                '       <span class="word">' + this.settings.tip + '</span> ' + 
                '       <span class="remain"></span>字 ' +
                '   </div>' + 
                '</div>';
            this.container= $(htmlTemplate);
            this.element.before(this.container);
            this.container.prepend(this.element);
            //this.element.hide();
        },
        initElements: function(){
            this.elements={
                original: this.element,
                viewTextarea:$('textarea', this.container),
                remain: $('span.remain', this.container),
                word:$('.word', this.container)
            };
            //this.elements.original.placeholder();
            this.elements.viewTextarea.attr('title', '最多为'+ this.settings.maxLength +'个字');
            this.elements.remain.text(this.settings.maxLength);

        },
        bindEvents: function(){
        	this.elements.viewTextarea
                .on('paste', $.proxy(this.paste, this))
                .on('keyup', $.proxy(this.keyup, this));
        },
        getValue:function(){
        	return  this.trimStr(this.elements.viewTextarea.val());
        },
        setValue: function(value){
            this.elements.viewTextarea.val(value);
        },
        trimStr:function(str){
            return str.replace(/(^\s*) | (\s*$)/g,"");
        },
        refresh: function(){
           
            var textval = this.elements.viewTextarea.val();
            textval = this.trimStr(textval);
            var remain = this.settings.maxLength - textval.length;
            if(remain < 0){
                this.elements.word.text('已经超出');
                this.elements.remain.text(Math.abs(remain));
                this.elements.remain.addClass('warning');
            }
            else
            {
                this.elements.word.text('还可以输入');
                this.elements.remain.text(remain);
                this.elements.remain.removeClass('warning');
            } 
        },
        paste:function(){
            var context= this;
            setTimeout(function(){
                context.refresh();
            },50);
        },
        keyup: function(){
            this.refresh();
        }
    });

});


/* textlimit javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'textlimit';
    var pluginClass = s.ui.controls.Textlimit;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
/**
* @version: 2.1.18
* @author: Dan Grossman http://www.dangrossman.info/
* @copyright: Copyright (c) 2012-2015 Dan Grossman. All rights reserved.
* @license: Licensed under the MIT license. See http://www.opensource.org/licenses/mit-license.php
* @website: https://www.improvely.com/
*/
Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    var emptyFun= function(){};
    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        timePickerIncrement: 5,    //分钟间隔
        format:'HH:mm', //HH:mm:ss.SSS millisecond
        change: emptyFun,
        defaultTime:null,
        minDate:undefined,
        maxDate:undefined,
        timeShowPanel:true
    };
    var attributeMap = {
        timePickerIncrement: 'time-picker-increment',
        format:'format',
        defaultTime:'default-time',
        timeShowPanel:'time-show-panel'
    };

    this.Timepicker = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // 构造函数
        init: function(element, options){
            this.element = $(element);

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);

            this.currentTime = {};

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            this.transferAttributes();

            // 绑定事件
            this.bindEvents();

            this.setValue(this.settings.defaultTime);
            // 获取初始值
            if(this.settings.defaultTime!=null){
                this.elements.view.trigger('changedate',this.settings.defaultTime);
            }
            if (typeof this.settings.minDate === 'string')
                this.settings.minDate = moment(this.settings.minDate, this.settings.format);

            if (typeof this.settings.minDate === 'object')
                this.settings.minDate = moment(this.settings.minDate);

            if (typeof this.settings.maxDate === 'string')
                this.settings.maxDate = moment(this.settings.maxDate, this.settings.format);

            if (typeof this.settings.maxDate === 'object')
                this.settings.maxDate = moment(this.settings.maxDate);

        },

        buildHtml: function(){
            var pluginId= this.element.data('plugin-id');
            var secondHtml =
                    '       <span class="pull-left sepline">:</span>'+
                    '       <div class="second-control dropup">'+
                    '           <input class="timepicker-second" type="text" maxlength="2" data-s-dropdown-id="#c-showsecond-menu-'+pluginId+'">'+
                    '           <div id="c-showsecond-menu-'+pluginId+'" class="dropdown-menu timepicker-seconds">'+
                    '               <p>请选择秒钟</p>'+
                    '               <div class="calendar-seconds">'+
                    '               </div>'+
                    '           </div>'+
                    '       </div>',
                millisecondHtml =
                    '       <span class="pull-left sepline">.</span>'+
                    '       <div class="millisecond-control">'+
                    '           <input class="timepicker-millisecond" type="text" maxlength="3">'+
                    '       </div>';
            millisecondHtml = (this.settings.format == 'HH:mm:ss.SSS' && secondHtml+millisecondHtml) || '';
            secondHtml = (this.settings.format == 'HH:mm:ss' && secondHtml) || '';
            var htmlTemplate=''+
                '<div class="calendar-time">'+
                '   <div class="calendar-time-control">'+
                '       <div class="hour-control dropup">'+
                '           <input class="timepicker-hour" type="text" maxlength="2" data-s-dropdown-id="#c-showhour-menu-'+pluginId+'">'+
                '           <div id="c-showhour-menu-'+pluginId+'" class="dropdown-menu timepicker-hours">'+
                '               <p>请选择小时</p>'+
                '               <div class="calendar-hours">'+
                '               </div>'+
                '           </div>'+
                '       </div>'+
                '       <span class="pull-left sepline">:</span>'+
                '       <div class="minute-control dropup">'+
                '           <input class="timepicker-minute" type="text" maxlength="2" data-s-dropdown-id="#c-showminute-menu-'+pluginId+'">'+
                '           <div id="c-showminute-menu-'+pluginId+'" class="dropdown-menu timepicker-minutes">'+
                '               <p>请选择分钟</p>'+
                '               <div class="calendar-minutes">'+
                '               </div>'+
                '           </div>'+
                '       </div>'+
                        secondHtml+
                        millisecondHtml+
                '       <i class="fa fa-clock-o glyphicon glyphicon-time"></i>'+
                '   </div>'+
                '</div>';

            //this.element.after(this.container);
            this.container= $(htmlTemplate);
            this.element.before(this.container);
            this.element.hide();
        },
        initElements: function() {
            this.elements = {
                view: this.element,
                timepicker: this.container,
                timeControlContainer: $('.calendar-time-control', this.container),
                showHourContainer: $('.hour-control', this.container),
                showMinuteContainer: $('.minute-control', this.container),
                showSecondContainer: $('.second-control', this.container),

                showHour: $('.timepicker-hour', this.container),
                showMinute: $('.timepicker-minute', this.container),
                showSecond: $('.timepicker-second', this.container),
                showMillisecond: $('.timepicker-millisecond', this.container),

                calendarHours: $('.timepicker-hours', this.container),
                calendarMinutes: $('.timepicker-minutes', this.container),
                calendarSeconds: $('.timepicker-seconds', this.container),
                calendarMilliseconds: $('.timepicker-milliseconds', this.container),

                showHours: $('.calendar-hours', this.container),
                showMinutes: $('.calendar-minutes', this.container),
                showSeconds: $('.calendar-seconds', this.container),

                showTimeIcon: $('.glyphicon-time', this.container)
            };
            this.byPassKeys = [96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 37, 39, 8, 48, 49, 50, 51, 52, 53, 54];
            if(this.settings.timeShowPanel) {
                this.dropdown = new s.ui.controls.Dropdown(this.elements.showHour, {});
                this.dropdown1 = new s.ui.controls.Dropdown(this.elements.showMinute, {});
                this.dropdown2 = new s.ui.controls.Dropdown(this.elements.showSecond, {});
            }
        },
        transferAttributes: function(){
            if(this.element.prop('disabled')){
                this.disable();
            }
        },
        bindEvents: function(){
            this.elements.showTimeIcon.on('click',$.proxy(this.showHourPanel, this));

            this.elements.showHour.on('click',$.proxy(this.showPanel, this));
            this.elements.showHour.on('blur',$.proxy(this.confirmHour,this));
            this.elements.showHour.on('keydown',$.proxy(this.inputKeyDown, this));
            this.elements.showHour.on('keyup',$.proxy(this.inputKeyUp, this));

            this.elements.showMinute.on('click',$.proxy(this.showPanel, this));
            this.elements.showMinute.on('blur',$.proxy(this.confirmMinute,this));
            this.elements.showMinute.on('keydown',$.proxy(this.inputKeyDown, this));
            this.elements.showMinute.on('keyup',$.proxy(this.inputKeyUp, this));

            this.elements.showSecond.on('click',$.proxy(this.showPanel, this));
            this.elements.showSecond.on('blur',$.proxy(this.confirmSecond,this));
            this.elements.showSecond.on('keydown',$.proxy(this.inputKeyDown, this));
            this.elements.showSecond.on('keyup',$.proxy(this.inputKeyUp, this));

            this.elements.showMillisecond.on('click',$.proxy(this.showPanel, this));
            this.elements.showMillisecond.on('blur',$.proxy(this.confirmMillisecond,this));
            this.elements.showMillisecond.on('keydown',$.proxy(this.inputKeyDown,this));
            this.elements.showMillisecond.on('keyup',$.proxy(this.inputKeyUp,this));

            this.elements.view.on('changedate',$.proxy(this.settings.change,this));

            this.elements.showHours.on('click', 'span', $.proxy(this.hourChanged, this));
            this.elements.showMinutes.on('click', 'span', $.proxy(this.minuteChanged, this));
            this.elements.showSeconds.on('click', 'span', $.proxy(this.secondChanged, this));

        },
        setMinDate:function(minDate){
            if (typeof minDate === 'string')
                this.settings.minDate = moment(minDate, this.settings.format);
            if (typeof minDate === 'object')
                this.settings.minDate = moment(minDate);
        },
        setMaxDate:function(maxDate){
            if (typeof maxDate === 'string')
                this.settings.maxDate = moment(maxDate, this.settings.format);
            if (typeof maxDate === 'object')
                this.settings.maxDate = moment(maxDate);
        },
        inputKeyDown:function(e){
            this.keyUpFlag = true;
            var currentKey = e.keyCode || e.which,value = $(e.target).val();
            if($.inArray(currentKey, this.byPassKeys) === -1){
                $(e.target).val(value.replace(/[^\d]/g,''));
            }
        },
        inputKeyUp:function(e) {
            if(this.keyUpFlag) {
                var $thisInput = $(e.target),
                    $prevInput = $(e.target).parent().prev().prev().find("input"),
                    $nextInput = $(e.target).parent().next().next().find("input");
                var currentKey = e.keyCode || e.which, value = $thisInput.val();
                if ($.inArray(currentKey, this.byPassKeys) === -1) {
                    $thisInput.val(value.replace(/[^\d]/g, ''));
                    return;
                }
                if ($prevInput.length !== 0) {
                    if ((currentKey === 8 && value.length === 0) || currentKey === 37) {// 退格键 或者 方向键 左
                        $prevInput.trigger("click");
                        return;
                    }
                }
                if ($nextInput.length !== 0) {
                    if (currentKey === 39 || (currentKey!== 37 && value.length == 2)) {// 输入满2位数或方向键 右，跳到下一文本框
                        $nextInput.trigger("click");
                    }
                }
                this.keyUpFlag = false;
            }
        },
        showHourPanel: function(e){
            this.elements.showHour.click();
        },

        confirmHour: function(e){
            var value = this.elements.showHour.val();
            if(value.length > 0){
                var num;
                try {
                    num = value;
                    if((num >= 0 && num <= 23) ){
                        this.currentTime.hour(num);
                    }else{
                        this.elements.showHour.val('00');
                        this.currentTime.hour(0);
                    }
                }catch(err){
                    this.elements.showHour.val('00');
                    this.currentTime.hour(0);
                }
                var date = this.getValue();
                this.setValue(date);
                this.elements.view.trigger('changedate',date);
                this.elements.view.trigger('timeChange',date);
            }else{
                this.elements.view.trigger('changedate',null);
                this.elements.view.trigger('timeChange',null);
            }
        },

        confirmMinute: function(e){
            var value = this.elements.showMinute.val();
            if(value.length > 0){
                var num;
                try {
                    num = value;
                    if(num >= 0 && num <= 59){
                        this.currentTime.minute(num);
                    }else{
                        this.elements.showMinute.val('00');
                        this.currentTime.minute(0);
                    }
                }catch(err){
                    this.elements.showMinute.val('00');
                    this.currentTime.minute(0);
                }
                var date = this.getValue();
                this.setValue(date);
                this.elements.view.trigger('changedate',date);
                this.elements.view.trigger('timeChange',date);
            }else{
                this.elements.view.trigger('changedate',null);
                this.elements.view.trigger('timeChange',null);
            }
        },
        confirmSecond: function(e){
            var value = this.elements.showSecond.val();
            if(value.length > 0){
                var num;
                try {
                    num = value;
                    if(num >= 0 && num <= 59){
                        this.currentTime.second(num);
                    }else{
                        this.elements.showSecond.val('00');
                        this.currentTime.second(0);
                    }
                }catch(err){
                    this.elements.showSecond.val('00');
                    this.currentTime.second(0);
                }
                var date = this.getValue();
                this.setValue(date);
                this.elements.view.trigger('changedate',date);
                this.elements.view.trigger('timeChange',date);
            }else{
                this.elements.view.trigger('changedate',null);
                this.elements.view.trigger('timeChange',null);
            }
        },
        confirmMillisecond: function(e){
            var value = this.elements.showMillisecond.val();
            if(value.length > 0){
                var num;
                try {
                    num = value;
                    if(num >= 0 && num <= 999){
                        this.currentTime.millisecond(num);
                    }else{
                        this.elements.showMillisecond.val('000');
                        this.currentTime.millisecond(0);
                    }
                }catch(err){
                    this.elements.showMillisecond.val('000');
                    this.currentTime.millisecond(0);
                }

                var date = this.getValue();
                this.setValue(date);
                this.elements.view.trigger('changedate',date);
                this.elements.view.trigger('timeChange',date);
            }else{
                this.elements.view.trigger('changedate',null);
                this.elements.view.trigger('timeChange',null);
            }
        },
        focusView:function(){
            this.updateView();
        },

        compareTime:function(){
            if (this.settings.minDate && this.currentTime.isBefore(this.settings.minDate))
                this.currentTime = this.settings.minDate.clone();

            if (this.settings.maxDate && this.currentTime.isAfter(this.settings.maxDate))
                this.currentTime = this.settings.maxDate.clone();
        },
        getValue: function(){
            this.compareTime();
            var date = this.currentTime.clone();
            return date.format(this.settings.format);
        },
        setValue: function(date){
            this.currentTime = moment();
            if(!date){
                if(this.elements){
                    this.elements.showHour.val('');
                    this.elements.showMinute.val('');
                    this.elements.showSecond.val('');
                    this.elements.showMillisecond.val('');
                }
                return;
            }
            var currentTime;
            if(typeof date === 'string'){
                currentTime = moment(date, this.settings.format);
            }
            if(typeof date === 'object'){
                currentTime = moment(date);
            }
            this.currentTime = currentTime.clone();
            if(this.elements){
                var hour = this.currentTime.hour();
                var minute = this.currentTime.minute();
                var second = this.currentTime.second();
                var millisecond = this.currentTime.millisecond();
                hour = hour < 10 ? '0' + hour : hour;
                minute = minute < 10 ? '0' + minute : minute;
                second = second < 10 ? '0' + second : second;
                millisecond = ( millisecond < 10 && '00'+millisecond) || ( millisecond < 100 && '0'+millisecond) || millisecond;
                this.elements.showHour.val(hour);
                this.elements.showMinute.val(minute);
                this.elements.showSecond.val(second);
                this.elements.showMillisecond.val(millisecond);
            }
        },
        updateView: function(e) {
            this.renderTimePicker();
        },

        renderTimePicker: function() {
            var html;
            var calendar = this.currentTime;
            var hour = calendar.hour();
            var minute = calendar.minute();
            var second = calendar.second();
            var minDate = false;
            if (this.settings.minDate)
                minDate = this.settings.minDate.clone();
            var maxDate = false;
            if (this.settings.maxDate)
                maxDate = this.settings.maxDate.clone();
            //
            // hours
            //
            html = '';
            var start = 0 ;
            var end = 23;
            for (var i = start; i <= end; i++) {
                var padded = i < 10 ? '0' + i : i;
                var disabledClass = '';
                if(minDate && i < minDate.hour())
                    disabledClass='disable';
                if(maxDate && i > maxDate.hour())
                    disabledClass='disable';
                html += '<span value="' + i + '" class="'+ disabledClass +
                    (i === hour ? ' selected' : '') +
                    '">' + padded + '</span>';
            }
            this.elements.showHours.html(html);

            //
            // minutes
            //
            html = '';
            for (var i = 0; i < 60; i += this.settings.timePickerIncrement) {
                var padded = i < 10 ? '0' + i : i;
                var disabledClass = '';
                if(minDate && (hour == minDate.hour()) && i < minDate.minute())
                    disabledClass='disable';
                if(maxDate && (hour == maxDate.hour()) && i > maxDate.minute())
                    disabledClass='disable';
                html += '<span value="' + i + '" class="'+ disabledClass +
                    (i === minute ? ' selected' : '') +
                    '">' + padded + '</span>';
            }
            this.elements.showMinutes.html(html);

            //seconds
            html = '';
            for (var i = 0; i < 60; i += this.settings.timePickerIncrement) {
                var padded = i < 10 ? '0' + i : i;
                var disabledClass = '';
                if(minDate && (hour == minDate.hour()) && (minute == minDate.minute()) && i < minDate.second())
                    disabledClass='disable';
                if(maxDate && (hour == maxDate.hour()) && (minute == maxDate.minute()) && i > maxDate.second())
                    disabledClass='disable';
                html += '<span value="' + i + '" class="'+ disabledClass +
                    (i === second ? ' selected' : '') +
                    '">' + padded + '</span>';
            }
            this.elements.showSeconds.html(html);
        },

        showPanel: function(e){
            $(e.target).select();
            this.updateView();
            if($(e.target).hasClass('timepicker-millisecond')){
                this.elements.showSecondContainer.removeClass('open');
            }
        },

        hourChanged: function(e){
            if($(e.target).hasClass('disable')){
                e.stopPropagation();
                return;
            }
            var data = parseInt(e.target.innerHTML);
            this.currentTime.hour(data);
            this.elements.showHour.val(data);
            var date = this.getValue();
            this.setValue(date);
            this.elements.view.trigger('changedate',date);
            this.elements.view.trigger('timeChange',date);
            this.elements.showHourContainer.removeClass('open');
        },

        minuteChanged: function(e){
            if($(e.target).hasClass('disable')){
                e.stopPropagation();
                return;
            }
            var data = parseInt(e.target.innerHTML);
            this.currentTime.minute(data);
            this.elements.showMinute.val(data);
            var date = this.getValue();
            this.setValue(date);
            this.elements.view.trigger('changedate',date);
            this.elements.view.trigger('timeChange',date);
            this.elements.showMinuteContainer.removeClass('open');
        },
        secondChanged: function(e){
            if($(e.target).hasClass('disable')){
                e.stopPropagation();
                return;
            }
            var data = parseInt(e.target.innerHTML);
            this.currentTime.second(data);
            this.elements.showSecond.val(data);
            var date = this.getValue();
            this.setValue(date);
            this.elements.view.trigger('changedate',date);
            this.elements.view.trigger('timeChange',date);
            this.elements.showSecondContainer.removeClass('open');
        },
        timeChanged: function(e) {
            //re-render the time pickers because changing one selection can affect what's enabled in another
            this.renderTimePicker();
        },

        render: function(){},
        refresh: function(){},
        enable: function(){
            this.elements.timeControlContainer.removeClass("disabled");
            this.container.removeClass("disabled");
            this.elements.showHour.removeClass("disabled");
            this.elements.showHour.prop('disabled',false);
            this.elements.showMinute.removeClass("disabled");
            this.elements.showMinute.prop('disabled',false);
            this.elements.showSecond.removeClass("disabled");
            this.elements.showSecond.prop('disabled',false);
            this.elements.showMillisecond.removeClass("disabled");
            this.elements.showMillisecond.prop('disabled',false);
        },
        disable: function() {
            this.elements.timeControlContainer.addClass("disabled");
            this.container.addClass("disabled");
            this.elements.showHour.addClass("disabled");
            this.elements.showHour.prop('disabled', true);
            this.elements.showMinute.addClass("disabled");
            this.elements.showMinute.prop('disabled', true);
            this.elements.showSecond.addClass("disabled");
            this.elements.showSecond.prop('disabled', true);
            this.elements.showMillisecond.addClass("disabled");
            this.elements.showMillisecond.prop('disabled', true);
        },
        destroy: function(){},
        getApi: function(){
            var context = this;
            var funs = {
                setValue:'setValue',
                getValue:'getValue',
                updateView:'updateView',
                render:'renderTimePicker',
                setMinDate:'setMinDate',
                setMaxDate:'setMaxDate'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });
});

/* datepicker javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'timepicker';
    var pluginClass = s.ui.controls.Timepicker;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };
})(jQuery);

Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var defaults = {
        show:true,
        inputWidth:0,
        width:248,
        mateKey:{id:'id'},
        map: '{"text":"text","returnValue":"json"}',
        onblured:undefined,
        typeaheadTemplate:undefined,
        nofound:'<span style="font-size:13px">无法匹配到适合的选项！</span>　<a class="c-userselect-modal" href="#" style="font-size:13px">调出选人控件查询</a>',
        showItemList:false,
        ishide:false,
        showIcon:true,
        items:8,
        typeaheadMethod:{},//这里必须是{} 不能为undefined
        multiSelect:true
    };
    var attributeMap = {
        map:'map',
        showIcon:'show-icon',
        showItemList:'show-item-list',
        items:'items',
        multiSelect:'multi-select'
    };

    this.UserSelect = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // 构造函数
        init: function(element, options){
            this.element = $(element);
            this.textlists=[];
            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { 
                return; 
            }
            this.element.data('initialized', true);
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 获取选项
            this.initSettings(options);
            if(typeof this.settings.map === 'string' && this.settings.map !== '')
                this.settings.map = JSON.parse(this.settings.map);
            if(!this.settings.showIcon){
                this.settings.nofound='<span style="font-size:13px">无法匹配到适合的选项！</span>';
            }
            // 获取初始值
            // this.value = this.element.val();
            this.settings.width=this.element.width()&&this.element.width()>=248 ? this.element.width():this.settings.width;
            this.settings.map=typeof this.settings.map==='string'&&this.settings.map ?  JSON.parse(this.settings.map) : this.settings.map;
            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
			// 给元素绑定属性
            this.transferAttributes();
            // this.transferAttributes();
            this.settings.inputWidth=this.elements.input.width();
            // 获取数据
            // 同步获取数据
            // this.getData();
            var context= this;
                context.render();
            // 绑定事件
            this.bindEvents();
           
            // 绑定事件接口
            // this.bindEventsInterface();            
        },
        buildHtml: function(){
            var html="";
            if(this.settings.showIcon){
                html='<div  id="userSelect" class="c-userselect-container">'
                        + '<div class="c-userselect-view">'
                        +       '<div style="max-width:'+(this.settings.width-90)+'px" class="c-userselect-textlist" ></div>'
                        +       '<input style="width:'+(this.settings.width-28)+'px;border:none;outline:medium;" type="text" class="c-typeahead form-control">'
                        +       '<div class="c-userselect-fr"><span class="glyphicon glyphicon-user"></span></div>'
                        + '</div>'
                        + '<div  class="c-userselect-tips dropdown-menu">'
                        +      '<dl class="c-userselect-list" style=""></dl>'
                        + '<div style="clear:both"></div>' 
                    +'</div>';
            }else{
                html='<div  id="userSelect" class="c-userselect-container">'
                        + '<div class="c-userselect-view">'
                        +       '<div style="max-width:'+(this.settings.width-100)+'px" class="c-userselect-textlist" ></div>'
                        +       '<input style="min-width:100px;width:'+(this.settings.width-10)+'px;border:none;outline:medium;" type="text" class="c-typeahead form-control">'
                       
                        + '</div>'
                        + '<div  class="c-userselect-tips dropdown-menu">'
                        +      '<dl class="c-userselect-list" style=""></dl>'
                        + '<div style="clear:both"></div>' 
                    +'</div>';       
            }
             
            //this.element.html(html);
            this.container= $(html);
            this.element.before(this.container);
            this.element.hide();
            if(this.settings.ishide){
                this.container.hide();
            }else{
                this.container.show();
            }
        },
        initElements: function(){
            var self=this;
        	this.elements={
                container:self.container,
        		view: $('.c-userselect-view', self.container),
                tips: $('.c-userselect-tips', self.container),
                dl:   $('.c-userselect-list',self.container),
                input:$('.c-typeahead.form-control',self.container),
                textlist:$('.c-userselect-textlist',self.container),
                list:$('.c-userselect-list',self.container),
                button:$('.c-userselect-fr',self.container),
        	};
        	var defaultOptions={
                data:self.settings.data,
                map: self.settings.map,
                showItemList:self.settings.showItemList,
                items:self.settings.items,
                template:this.settings.typeaheadMethod.template,
                onSelected:this.settings.typeaheadMethod.onSelected||function(e,value){
                    self.selected.apply(self,arguments);
                },
                matcher: this.settings.typeaheadMethod.matcher||function(item){
                        var matcher = this.matcherReg||new RegExp(this.query,'i');
                        var result = matcher.test(item.name) || matcher.test(item.loginName) || matcher.test(item.spell) || matcher.test(item.spellFirst);
                        return result===true ? 1 : -1;
                 },
                nofound:self.settings.nofound,
                nofoundWidth:this.settings.width-10,
        	}; 
       		self.typeahead=new s.ui.controls.Typeahead(self.elements.input, defaultOptions);
        },
        updateData: function(data){
            this.typeahead.setSource(data);
        },
		transferAttributes: function(){
			this.elements.input.attr('placeholder', this.element.attr('placeholder'));
	
		},
        bindEvents: function(){
           this.elements.input.bind('keydown',$.proxy(this.keydown, this));
           this.elements.input.bind('blur',$.proxy(this.blur, this));
          // this.elements.input.bind('click',$.proxy(this.click, this));
           this.elements.input.bind('focus',$.proxy(this.click, this));
           this.elements.dl.bind('click',$.proxy(this.click, this));
           this.elements.tips.bind('click',$.proxy(this.click, this));
           this.elements.container.bind('mouseleave',$.proxy(this.mouseleave, this));
           this.elements.container.bind('mouseenter',$.proxy(this.mouseenter, this));
           this.elements.list.bind('click',$.proxy(this.clickSpan, this));
           this.elements.button.bind('click',$.proxy(this.clickButton, this));
           this.elements.container.delegate('.c-userselect-modal','click',$.proxy(this.modal, this));
           

        },
        bindEventsInterface: function(){
        	var context=this;
            var element=this.container;
            // element.on('selected.s.template', $.proxy(this.settings.onFooClick, this));
        },
        render: function(){},
        selected:function(e,value){
            var obj=this.typeahead.getValue(value);
            this.elements.tips.show();
            if(obj){
                this.elements.input.val('');
                if(this.contains(this.textlists,obj,this.settings.mateKey)===undefined){
                    if(this.settings.multiSelect){
                        this.textlists.push(obj);
                     }else{
                        this.textlists=[obj]; //只显示一条数据
                        this.elements.list.html("");
                     }
                    this.addEl(this.elements.list,obj);
                    this.automove(this.textlists,this.elements.textlist);
                }
            }  
            this.elements.input.focus();
        },
        disable:function(){
                this.elements.button.prop('disabled', true);
                this.elements.input.prop('disabled', true);
                this.elements.view.addClass('disabled');
        },
        enable:function(){
                this.elements.button.prop('disabled', false);
                this.elements.input.prop('disabled', false);
                this.elements.view.removeClass('disabled');
        },
        changed:function(e,value){
           
        },
        addEl:function($parent,data){
            var name=this.displayText(data);
            var $dd=$('<dd class="c-userselect-item"></dd>');
                $dd.data('value',data);
                $dd.append('<span class="c-userselect-nametext">'+name+'</span><span class="c-userselect-delete">x</span></span>');
                $parent.append($dd);
        },
        removeEl: function($parent,data){
            var istrue=true,
                mateKey=this.settings.mateKey;
            $parent.find('dd').each(function(index,item){
                for(var x in mateKey){
                     if($(item).data('value')[x]!=data[x]){
                           istrue=false;
                     }
                }
                if(istrue) return $(item).remove();
    
                istrue=true;
            });
        },
        modal:function(e){
            e.preventDefault();
            e.stopPropagation();
            this.elements.button.click();
        },
        click:function(e){
            if(this.textlists.length>0)
            //input里面有内容，去掉placeholder值
            	this.elements.input.removeAttr('placeholder');
                 this.elements.tips.show();
                 if(!$(e.target).is('input'))
                    this.elements.input.focus();
        },
        clickButton:function(e){
            this.elements.tips.hide();
            this.settings.show=false;
            if(typeof this.settings.clickButton==='function'){
                this.settings.clickButton.apply(this,[e]);
            }

        },
        clickSpan:function(e){
            var target=$(e.target);
            if(!target.hasClass('c-userselect-delete')) return ;
            var data=target.parent('dd').data('value');
            this.remove(this.textlists,data,this.settings.mateKey);
            this.automove(this.textlists,this.elements.textlist);
            target.parent('dd').remove();

            this.elements.input.focus();
        },
        keydown:function(e){
            var target=$(e.target);
            if(e.which==8){
                if(this.elements.input.val().length<=0){
                    var data=this.textlists.splice(this.textlists.length-1,1);
                        this.automove(this.textlists,this.elements.textlist);
                        this.removeEl(this.elements.list,data[0]);
                }
                // 回车删除input内容时,加上placeholder属性
                if(this.textlists.length<=0){
            			this.elements.input.attr('placeholder',this.element.attr('placeholder'))
                	
                }
            }
        },
        mouseleave:function(){
            this.settings.show=false;
        },
        mouseenter:function(){
            this.settings.show=true;
        },
        blur:function(){
            if(!this.settings.show){
                this.elements.tips.hide();
                if(typeof this.settings.onblured==='function'){
                        this.settings.onblured.apply(this,[this.textlists]);
                }
            }
               
        },
        focus:function(e){
        	
        },
        
        getValue:function(){
            return this.textlists;
        },
        setData:function(data){
            if(!data) return ;
            this.textlists=data;
            this.elements.list.html("");
            for(var i=0;i<data.length;i++){
                this.addEl(this.elements.list,data[i]);
            }
            this.automove(this.textlists,this.elements.textlist);
        },
        getButton:function(){
            return this.elements.button;
        },
        automove:function(datas,element){
            var len=datas.length,
                str="";
            for(var x=0;x<len;x++){
                var text=this.displayText(datas[x]);
                str+=text+' ; ';
            }
            str=str.substring(0,str.length-3);
            element.text(str);
            element.attr('title',str);
            var width=this.elements.textlist.width();
            this.elements.input.width(this.settings.inputWidth-width);
            
        },
        displayText:function(data){
            if(typeof data==='string') return data;

            return data[this.settings.map['text']];
        },
        contains:function(arr,obj,key){
            var istrue=true;
            if(!Array.isArray(arr))
                arr=[arr];
            if(typeof obj !=='object'){ return arr.indexOf(obj);}
            for(var i=0;i<arr.length;i++){
                istrue=true;
                for(var x in key){
                    if(obj[key[x]]!=arr[i][key[x]]){
                        istrue=false;
                    }
                }
                if(istrue) return i;
              
            }
            return undefined;
        },
        remove:function(arr,obj,key){
            var index=this.contains(arr,obj,key);
            if(index>=0)
                arr.splice(index,1);
            return arr;
        },
       /* add:function(arr,obj,key){
            var index=this.contains(arr,obj,key);
            if(index===undefined){
                arr.splice(arr.length,0,obj);
            }
            return arr;
        },*/
        // API
        getApi:function(){
            var context = this;
            var funs = {
                setData: 'setData',
                getValue:'getValue',
                getButton:'getButton',
                updateData:'updateData'

            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        },
        refresh: function(){},
        destroy: function(){}
    });

});


/* template javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'userselect';
    var pluginClass = s.ui.controls.UserSelect;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
/* ========================================================================
 * Bootstrap: carousel.js v3.3.5
 * http://getbootstrap.com/javascript/#carousel
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */

Jx().package("s.ui.controls", function(J){
    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;

    var VERSION  = '3.3.5';

    var TRANSITION_DURATION = 600;

    var defaults = {
        interval: 5000,
        pause: 'hover',
        wrap: true,
        keyboard: true,
        map: '{"url":"url","src":"src","alt":"alt","title":"title","content":"content"}',
    };

    var attributeMap = {
        interval: 'interval',
        pause:'pause',
        wrap:'wrap',
        keyboard:'keyboard',
        map:'map'
    };

    this.Carousel = new J.Class({extend : s.ui.BaseControl}, {
        defaults: defaults,
        attributeMap: attributeMap,

        init: function(element, options){
            this.element = $(element);

            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);
            
            // 初始化选项
            this.initSettings(options);

            this.map = this.settings.map;

            if(typeof this.settings.map === 'string' && this.settings.map !== '')
                this.map = JSON.parse(this.settings.map);

            this.paused      = null
            this.sliding     = null
            this.interval    = null
            this.active     = null
            this.items      = null

            if(this.settings.interval) 
                this.pause().cycle();

            this.buildHtml();
            this.initElements();
            this.bindEvents();
        },
        
        buildHtml: function(){
            var pluginId= this.element.data('plugin-id');
            this.id = this.element.attr("id") || ('carousel'+pluginId);
            var htmlTemplate=''+
                '<ol class="carousel-indicators">'+
                '</ol>'+
                '<div class="carousel-inner" role="listbox">'+
                '</div>'+
                '<a class="left carousel-control" href="'+this.id+'" role="button" data-s-slide="prev">'+
                '    <span class="glyphicon glyphicon-menu-left icon-prev" aria-hidden="true"></span>'+
                '    <span class="sr-only">Previous</span>'+
                '</a>'+
                '<a class="right carousel-control" href="'+this.id+'" role="button" data-s-slide="next">'+
                '    <span class="glyphicon glyphicon-menu-right icon-next" aria-hidden="true"></span>'+
                '    <span class="sr-only">Next</span>'+
                '</a>';
            this.container= $(htmlTemplate);
            this.element.append(this.container);
        },

        updateData: function(data){
            if(!data||data.length===0){
                this.elements.slide.hide();
                return;
            }else{
                this.elements.slide.show();
            }
                
            this.elements.indicators.html('');
            this.elements.inner.html('');
            for (var i = 0; i < data.length; i++) {
                var item = data[i];
                var indicator = '<li data-s-target="'+ this.id +'" data-s-slide-to="'+i+'"></li>';
                indicator = $(indicator);
                if(i === 0)
                    indicator.addClass('active');
                this.elements.indicators.append(indicator);

                var innerItem = ''+
                '    <div class="item">';
                if(item[this.map.url])
                    innerItem += '        <a href="'+item[this.map.url]+'" target="_blank">';
                innerItem += '<img src="'+item[this.map.src]+'" alt="'+item[this.map.src]+'">';
                var title = '';
                if(item[this.map.title] && item[this.map.content]){
                    title = ''+
                        '        <div class="carousel-caption">'+
                        '            <h3 class="title">'+item[this.map.title]+'</h3>'+
                        '            <p class="content">'+item[this.map.content]+'</p>'+
                        '        </div>';
                }else if(item[this.map.title]){
                    title = ''+
                        '        <div class="carousel-caption">'+
                        '            <h3 class="title-only">'+item[this.map.title]+'</h3>'+
                        '        </div>';
                }
                innerItem += title;
                if(item[this.map.url])
                    innerItem += '</a>';
                innerItem += '</div>';
                innerItem = $(innerItem);
                if(i === 0)
                    innerItem.addClass('active');
                this.elements.inner.append(innerItem);
            }

            this.settings.slide = this.element.find('li[data-s-slide-to]');
            var context = this;
            this.settings.slide.on('click',function(e){
                var slideIndex = $(this).attr('data-s-slide-to');
                if (slideIndex) {
                    context.settings.interval = false;
                    context.to(slideIndex);
                }
            });
        },

        initElements:function(){
            this.elements={
                indicators: this.element.find('.carousel-indicators'),
                inner: this.element.find('.carousel-inner'),
                slide: this.element.find('a[data-s-slide]')
                // slideto: this.element.find('li[data-s-slide-to]')
            };
        },

        bindEvents:function(){

            var context=this;

            if(this.settings.keyboard){
                this.element.on('keydown.bs.carousel', $.proxy(this.keydown, this));
            }

            this.elements.slide.on('click', function(e){
                var action=$(this).attr('data-s-slide');
                context[action]();

                e.preventDefault();
            });
        }, 

        keydown: function (e) {
            if (/input|textarea/i.test(e.target.tagName)) return
            switch (e.which) {
                case 37: this.prev(); break
                case 39: this.next(); break
                default: return
            }

            e.preventDefault()
        },

        cycle : function (e) {
            e || (this.paused = false)

            this.interval && clearInterval(this.interval)

            this.settings.interval
                && !this.paused
                && (this.interval = setInterval($.proxy(this.next, this), this.settings.interval))

            return this
        },

        getItemIndex: function (item) {
            this.items = item.parent().children('.item')
            return this.items.index(item || this.active)
        },

        getItemForDirection: function (direction, active) {
            var activeIndex = this.getItemIndex(active)
            var willWrap = (direction == 'prev' && activeIndex === 0)
                                    || (direction == 'next' && activeIndex == (this.items.length - 1))
            if (willWrap && !this.settings.wrap) return active
            var delta = direction == 'prev' ? -1 : 1
            var itemIndex = (activeIndex + delta) % this.items.length
            return this.items.eq(itemIndex)
        },

        to: function (pos) {
            var that        = this
            var activeIndex = this.getItemIndex(this.active = this.element.find('.item.active'))

            if (pos > (this.items.length - 1) || pos < 0) return

            if (this.sliding)       return this.element.one('slid.bs.carousel', function () { that.to(pos) }) // yes, "slid"
            if (activeIndex == pos) return this.pause().cycle()

            return this.slide(pos > activeIndex ? 'next' : 'prev', this.items.eq(pos))
        },

        pause: function (e) {
            e || (this.paused = true)

            if (this.element.find('.next, .prev').length && $.support.transition) {
                this.element.trigger($.support.transition.end)
                this.cycle(true)
            }

            this.interval = clearInterval(this.interval)

            return this
        },

        next: function () {
            if (this.sliding) return
            return this.slide('next')
        },

        prev: function () {
            if (this.sliding) return
            return this.slide('prev')
        },

        slide: function (type, next) {
            var $active   = this.element.find('.item.active')
            var $next     = next || this.getItemForDirection(type, $active)
            var isCycling = this.interval
            var direction = type == 'next' ? 'left' : 'right'
            var that      = this

            if ($next.hasClass('active')) return (this.sliding = false)

            var relatedTarget = $next[0]
            var slideEvent = $.Event('slide.bs.carousel', {
                relatedTarget: relatedTarget,
                direction: direction
            })
            this.element.trigger(slideEvent)
            if (slideEvent.isDefaultPrevented()) return

            this.sliding = true

            isCycling && this.pause()

            if (this.elements.indicators.length) {
                this.elements.indicators.find('.active').removeClass('active')
                var $nextIndicator = $(this.elements.indicators.children()[this.getItemIndex($next)])
                $nextIndicator && $nextIndicator.addClass('active')
            }

            var slidEvent = $.Event('slid.bs.carousel', { relatedTarget: relatedTarget, direction: direction }) // yes, "slid"
            if ($.support.transition && this.element.hasClass('slide')) {
                $next.addClass(type)
                $next[0].offsetWidth // force reflow
                if($active.hasClass('direction')) $active.removeClass(direction);
                if($next.hasClass('direction')) $next.removeClass(direction);
                $active.addClass(direction);
                $next.addClass(direction);
                that.sliding = false;
                $active
                    .one('bsTransitionEnd', function () {
                        $next.removeClass([type, direction].join(' ')).addClass('active')
                        $active.removeClass(['active', direction].join(' '))
                        
                        setTimeout(function () {
                            that.element.trigger(slidEvent)
                        }, 0)
                    })
                    .emulateTransitionEnd(TRANSITION_DURATION)
            } else {
                $active.removeClass('active')
                $next.addClass('active')
                this.sliding = false
                this.element.trigger(slidEvent)
            }

            isCycling && this.cycle()

            return this
        }
    });
});

/* carousel javascript jQuery */


(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = "carousel";

    // 胶水代码
    $.fn[pluginName] = function(options) {
        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
                jqElement.removeData('plugin-ref');
            }
            jqElement.data('plugin-ref', new s.ui.controls.Carousel(this, options));
        });

        return this;

    };

})(jQuery);


/* =========================================================
 * bootstrap-treeview.js v1.2.0
 * =========================================================
 * Copyright 2013 Jonathan Miles
 * Project URL : http://www.jondmiles.com/bootstrap-treeview
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================= */


 /*
    1. removed css (move to tree.less)
    2. removed buildStyleOverride
 */

Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 勾选框是否显示
    var enumShowCheckbox= {
        all: 'all',     // 0 所有显示
        none: 'none',   // 1 不显示(默认)
        leaf: 'leaf'    // 2 叶子节点显示
    };

    var enumCheckedState= {
        checked: 'checked',
        unchecked: 'unchecked',
        partOf: 'partOf'
    };

    // 节点是否可选
    var enumSelectMode= {
        all: 'all',     // 0 所有可选中(默认)
        leaf: 'leaf',   // 1 仅叶子可选中
    };

    // 全局变量、函数、对象
    var emptyFun= function(){};
    var _currentPluginId = 0;
    var defaults = {
        levels: 1,
        dataUrl: '',

        expandIcon: 'glyphicon glyphicon-plus',
        collapseIcon: 'glyphicon glyphicon-minus',
        // expandIcon: 'glyphicon glyphicon-chevron-right',
        // collapseIcon: 'glyphicon glyphicon-chevron-down',
        emptyIcon: 'glyphicon',
        nodeIcon: '',
        selectedIcon: '',
        checkedIcon: 'glyphicon glyphicon-check',
        checkedPartOfIcon: 'glyphicon glyphicon-check partOfchecked',
        uncheckedIcon: 'glyphicon glyphicon-unchecked',

        enableLinks: false,
        enableTitle: false,
        showIcon: true,
        // showCheckbox: false,
        showTags: false,
        multiSelect: false,

        // nodeOptions
        silent: false,
        ignoreChildren: false,

        expandOne:true,

        // extends
        showCheckbox: enumShowCheckbox.none,
        checkRecursive: false,
        selectMode: enumSelectMode.all,
        appendHtml:'',
        parseAppendHtml: undefined, // for angularjs : function(html){ return $compile(html)($scope) ;}

        // states
        selectedNodeIds:'',
        checkedNodeIds:'',

        // Event handlers
        // 节点双击
        onNodeDbclick: emptyFun,
        // 节点勾选
        onNodeChecked: emptyFun,
        // 节点收起
        onNodeCollapsed: emptyFun,
        // 节点禁用
        onNodeDisabled: emptyFun,
        // 节点启用
        onNodeEnabled: emptyFun,
        // 节点展开
        onNodeExpanded: emptyFun,
        // 节点选中
        onNodeSelected: emptyFun,
        // 节点取消勾选
        onNodeUnchecked: emptyFun,
        // 节点取消选中
        onNodeUnselected: emptyFun,
        // 搜索完成
        onSearchComplete: emptyFun,
        // 搜索结果清除
        onSearchCleared: emptyFun
    };
    var attributeMap = {
        expandOne: 'expand-one',
        ignoreChildren: 'ignore-children',
        expandIcon: 'expand-icon',
        collapseIcon: 'collapse-icon',

        showTags: 'show-tags',
        levels: 'levels',
        // enableTitle: 'enable-title',
        dataUrl: 'data-url',

        showCheckbox:'show-checkbox',
        checkRecursive: 'check-recursive',
        selectMode:'select-mode',
        appendHtml:'append-html',

        multiSelect:'multi-select',        
        selectedNodeIds:'selected-node-ids',
        checkedNodeIds:'checked-node-ids'
        
    };

    // var state= jqNode.find('.expand-icon').hasClass(this.settings.expandIcon);
    // fix for multi class in this.settings.expandIcon, like: 'glyphicon glyphicon-chevron-right'
    function hasClasses(jqElement, classes){
        var arrClass= classes.split(' ');
        var has= true;
        for(var i=0; i< arrClass.length; i++){
            if(!jqElement.hasClass(arrClass[i])){
                has= false;
                break;
            }
        }
        return has;
    }

    this.Slowtree = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,


        // 构造函数
        init: function(element, options){
            this.element = $(element);

            // -----------------------------------------------
            // options
            // -----------------------------------------------
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);
            if (typeof (this.settings.parseData) === 'function') {
                this.parseData = this.settings.parseData;
                delete this.settings.parseData;
            }

            // -----------------------------------------------
            // value
            // -----------------------------------------------
            // this.value = this.element.val();

            // -----------------------------------------------
            // data
            // -----------------------------------------------
            this.data = [];
            this.idIndexMap = {};
            this.container = $('<ul class="list-group"></ul>'); // list

            var context= this;
            // context.buildHtml();
            // context.initElements();
            // context.refresh();
            // // context.initStates();

            // // -----------------------------------------------
            // // events
            // // -----------------------------------------------
            // // context.buildObservers();
            // context.bindEvents();
                
            // 初始化数据
            $.when(this.getData())
             .done(function(){
                
                // -----------------------------------------------
                // states
                // -----------------------------------------------
                // this.initStates(jqElement);
                // -----------------------------------------------
                // html
                // -----------------------------------------------
                context.buildHtml();
                context.initElements();
                context.refresh();
                // context.initStates();

                // -----------------------------------------------
                // events
                // -----------------------------------------------
                // context.buildObservers();
                context.bindEvents();
                // context.bindEventsInterface();
            });
        },

        getData: function(){
            var d = $.Deferred();

            if (this.settings.data) {
                var data = $.extend(true, [], this.settings.data);
                this.data= this.parseData(data);
                delete this.settings.data;

                d.resolve();
                return d.promise();
            }

            // var context = this;
            // $.ajax({
            //     dataType: 'json',
            //     url: context.settings.dataUrl,
            //     data: {},
            //     success: function(data){
            //         var innerData= context.parseData(data);
            //         context.data= $.extend(true, [], innerData);
            //         d.resolve();
            //     },
            //     error: function(xmlHttpRequest, status, error){
            //         alert('控件id：' + context.element.attr('id')+'，ajax获取数据失败!');

            //         d.resolve();
            //     }
            // });

            return d.promise();
        },

        parseData: function(data){
            var innerData= [];

            var index= 0;
            var context= this;
            function recurseTree (node, level) {
                if (!node.nodes) return;

                level ++;

                for(var i=0; i<node.nodes.length; i++){
                    var child=node.nodes[i];

                    child.id = child.id || index;
                    child._innerParentId = node.id;
                    child._innerPath = (node._innerPath || 'r') + '|' + child.id;
                    child._innerLevel = level;

                    innerData.push(child);
                    context.idIndexMap['id_' + child.id]= index;

                    index++;

                    if (child.nodes) {
                        recurseTree(child, level);
                    }
                }
            }

            recurseTree({ nodes: data }, 0);

            return innerData;
        },

        updateData: function(data){
            var innerData = $.extend(true, [], data);
            this.data= this.parseData(innerData);
        },

        buildHtml: function(){
            this.element.addClass('c-slowtree');
            this.element
                .empty()
                .append(this.container);
        },

        initElements: function(){
            var context= this;

            this.elements={
                original: context.element,
                getAllNodes: function(){
                    var allNodes= $('li', context.container);
                    return allNodes;
                },
                getNode: function(nodeId){
                    var child= $('li[data-id="'+nodeId+'"]', context.container);
                    return child;
                },
                getChildNodes: function(nodeId){
                    var jqNode= this.getNode(nodeId);
                    var path= jqNode.data('path');
                    var children= $('li[data-path^="'+path+'|"]', context.container);
                    return children;
                },
                getChildNodesChecked: function(nodeId){
                    var jqNode= this.getNode(nodeId);
                    var path= jqNode.data('path');
                    var children= $('li.node-checked[data-path^="'+path+'|"]', context.container);
                    return children;
                },
                getChildNodesCheckedPartOf: function(nodeId){
                    var jqNode= this.getNode(nodeId);
                    var path= jqNode.data('path');
                    var children= $('li.node-checked-partof[data-path^="'+path+'|"]', context.container);
                    return children;
                },
                getLevelNodes: function(level){
                    var child= $('li[data-level="'+level+'"]', context.container);
                    return child;
                },
                getSelectedNodes: function(unselected){
                    var nodeSelector= unselected ? 'li:not(.node-selected)' : 'li.node-selected';
                    var selectedNodes = $(nodeSelector, context.container);
                    return selectedNodes;
                },
                // getCheckedNodes: function(unchecked){
                //     var nodeSelector= unchecked ? 'li:not(.node-checked)' : 'li.node-checked';
                //     var checkedNodes= $(nodeSelector, context.container);
                //     return checkedNodes;
                // },
                getCheckedNodes: function(unchecked,withPartof){
                    var nodeSelector;
                    if(withPartof)
                        nodeSelector = unchecked ? 'li:not(.node-checked):not(.node-checked-partof)' : 'li.node-checked,li.node-checked-partof';
                    else
                        nodeSelector = unchecked ? 'li:not(.node-checked):not(.node-checked-partof)' : 'li.node-checked';
                    var checkedNodes= $(nodeSelector, context.container);
                    return checkedNodes;
                },
                getDisabledNodes: function(disabled){
                    var nodeSelector= disabled ? 'li:not(.node-disabled)' : 'li.node-disabled';
                    var checkedNodes= $(nodeSelector, context.container);
                    return checkedNodes;
                },
                getSearchResultNodes: function(notResult){
                    var nodeSelector= notResult ? 'li:not(.search-result)' : 'li.search-result';
                    var checkedNodes= $(nodeSelector, context.container);
                    return checkedNodes;
                }
            };
        },

        buildTree: function () {
            for(var i=0; i<this.data.length; i++){
                var node= this.data[i];
                var item= this.buildItem(node);
                
                item = $(item);
                this.container.append(item);
                var appendHtml='';
                if(this.settings.appendHtml){
                    if(this.settings.parseAppendHtml){
                        var html = this.settings.parseAppendHtml(this.settings.appendHtml);
                        appendHtml = html;
                    }
                    else
                    {
                        appendHtml= this.settings.appendHtml;
                    }
                    
                     item.find('.appendHtml').append(appendHtml);
                }
               
            }
        },

        buildItem: function(node){
            //var hasChildren = node.nodes && node.nodes.length > 0;
            var hasChildren = false;
            if(node.nodes && node.nodes.length > 0)
                hasChildren = true;

            // indent
            var indent= '';
            for (var j = 0; j < (node._innerLevel - 1); j++) {
                indent+='<span class="indent"></span>';
            }

            // icon
            var cssClassIcon= 'icon';
            if (hasChildren) {
                cssClassIcon += node._innerLevel < this.settings.levels ? ' expand-icon '+this.settings.collapseIcon : ' expand-icon '+this.settings.expandIcon;
            }
            else {
                cssClassIcon += ' ' + this.settings.emptyIcon;
            }
            var icon= '<span class="'+cssClassIcon+'"></span>';

            // node icon
            var nodeIcon= '';
            if (this.settings.showIcon) {
                var cssClassNodeIcon= 'icon node-icon ';
                cssClassNodeIcon += (node.icon || this.settings.nodeIcon);
                nodeIcon= '<span class="'+cssClassNodeIcon+'"></span>'; 
            }

            // Add check / unchecked icon
            var check= '';

            if (this.settings.showCheckbox === enumShowCheckbox.all || (!hasChildren && this.settings.showCheckbox === enumShowCheckbox.leaf)) {
                var cssClassCheck= 'icon check-icon ';
                cssClassCheck += this.settings.uncheckedIcon;
                check= '<span class="'+ cssClassCheck +'"></span>';
            }

            // tags as badges
            var badge= '';
            if (this.settings.showTags && node.tags) {
                for(var k=0; k<node.tags.length; k++){
                    var tag=node.tags[k];
                    badge += '<span class="badge">'+tag+'</span>';  // badge
                }
            }

            // item
            var cssClass= 'list-group-item';
            if(hasChildren){
                // cssClas += ' has-children';
                if(this.settings.selectMode === enumSelectMode.leaf){
                    // 不可选择
                    cssClass += ' node-unselectable';
                }
            }

            var item= ''+
                '<li '+
                '   class="' + cssClass + '" '+
                (this.settings.levels && node._innerLevel>this.settings.levels ? 
                '   style="display: none;"' : '')+  // 隐藏应该折叠的nodes
                (this.settings.enableTitle ? 
                '   title="'+node.text+'"' : '')+
                '   onselectstart = "return false;"'+ //双击时不选中文字
                '   data-id="'+node.id+'"'+
                '   data-hasChildren="'+hasChildren+'"'+
                '   data-level="'+node._innerLevel+'"'+
                '   data-path="'+node._innerPath+'">'+
                '<div class = "item-content">'+
                indent +
                icon +
                nodeIcon +
                check +
                (this.settings.enableLinks ? 
                '   <a class="nodevalue" href="'+node.href+'" style="color:inherit;">'+node.text+'</a>' : '<span class="nodevalue">' + node.text +  '</span>') +
                (this.settings.appendHtml ? 
                '   <span class="appendHtml"></span>' : '') +
                badge +
                '</div></li>';

            return item;
        },

        refresh: function () {
            this.container.empty();
            this.buildTree();
            this.initStates();
        },

        initStates: function(){
            if(this.settings.selectedNodeIds){
                var ids= this.settings.selectedNodeIds;
                if(typeof ids === 'number'){
                    ids= ids + '';
                }
                this.selectNodeByIds(ids);    
            }
            if(this.settings.checkedNodeIds){
                var ids= this.settings.checkedNodeIds;
                if(typeof ids === 'number'){
                    ids= ids + '';
                }
                this.checkNodeByIds(ids);
            }
        },

        // 点击事件处理器
        clickHandler: function (event) {

            if (!this.settings.enableLinks) {
                event.preventDefault();
            }

            var target = $(event.target);
            var jqNode = target.closest('li.list-group-item');
            if (jqNode.hasClass('node-disabled')) {
                return;
            }
            
            var nodeId = jqNode.data('id');
            var node= this.getNode(nodeId);
            if(!node){
                return;
            }
            var classList = target.attr('class') ? target.attr('class').split(' ') : [];
            if ((classList.indexOf('expand-icon') !== -1)) {
                this.toggleExpandedState(node, this.settings.silent, this.settings.ignoreChildren);
                return;
            }

            if ((classList.indexOf('check-icon') !== -1)) {                
                this.toggleCheckedState(node, this.settings.silent);
                return;
            }

            if(!jqNode.hasClass('node-unselectable')){
                this.toggleSelectedState(node, this.settings.silent);
            }
        },

        dblclickHandler: function(event) {
            if (!this.settings.enableLinks) {
                event.preventDefault();
            }

            var target = $(event.target);
            var jqNode = target.closest('li.list-group-item');

            if (jqNode.hasClass('node-disabled') || jqNode.hasClass('node-unselectable')) {
                return;
            }
            // this.container.find("li.list-group-item").
            jqNode.addClass('node-selected').siblings().removeClass('node-selected');
            var nodeId = jqNode.data('id');
            var node= this.getNode(nodeId);

            if (!this.settings.silent) {
                this.elements.original.trigger('nodedbclick', node);
            }
        },

        // -------------------------------------------------------------------
        // 展开 / 折叠
        // -------------------------------------------------------------------

        setOneLevelExpandedState: function (node, state, silent, ignoreChildren) {
            var jqNode= this.elements.getNode(node.id);
            var jqChildren= this.elements.getChildNodes(node.id);

            if (state) {
                jqNode.find('.expand-icon').removeClass(this.settings.expandIcon).addClass(this.settings.collapseIcon);                
                jqChildren.show();

                if (!silent) {
                    this.elements.original.trigger('nodeExpanded', node);
                }
            } else {
                jqNode.find('.expand-icon').removeClass(this.settings.collapseIcon).addClass(this.settings.expandIcon);
                jqChildren.hide();


                if (!silent) {
                    this.elements.original.trigger('nodeCollapsed', node);
                }
            }

            if (jqChildren.length>0) {
                var context= this;
                jqChildren.each(function(){
                    var nodeId= $(this).data('id');
                    var child= context.getNode(nodeId);
                    context.setOneLevelExpandedState(child,false,silent,ignoreChildren);
                });
            }
        },

        setExpandedState: function (node, state, silent, ignoreChildren) {
            if(this.settings.expandOne){
                this.setOneLevelExpandedState(node, state, silent, ignoreChildren);
                return;
            }
            var jqNode= this.elements.getNode(node.id);
            var jqChildren= this.elements.getChildNodes(node.id);

            if (jqChildren.length>0 && !ignoreChildren) {
                var context= this;
                jqChildren.each(function(){
                    var nodeId= $(this).data('id');
                    var child= context.getNode(nodeId);
                    context.setExpandedState(child, state, silent, ignoreChildren);
                });
            }

            if (state) {
                jqNode.find('.expand-icon').removeClass(this.settings.expandIcon).addClass(this.settings.collapseIcon);                
                jqChildren.show();

                if (!silent) {
                    this.elements.original.trigger('nodeExpanded', node);
                }
            } else {
                // if (jqChildren.length>0 && ignoreChildren) {
                //     var context= this;
                //     jqChildren.each(function(){
                //         var nodeId= $(this).data('id');
                //         var child= context.getNode(nodeId);
                //         context.setExpandedState(child, state, silent, ignoreChildren);
                //     });
                // }

                jqNode.find('.expand-icon').removeClass(this.settings.collapseIcon).addClass(this.settings.expandIcon);
                jqChildren.hide();


                if (!silent) {
                    this.elements.original.trigger('nodeCollapsed', node);
                }
            }
        },

        collapseNode: function (nodes, silent, ignoreChildren) {
            for(var i=0; i<nodes.length; i++){
                this.setExpandedState(nodes[i], false, silent, ignoreChildren);
            }
        },

        expandNode: function (nodes, levels, silent, ignoreChildren) {
            var context= this;
            for(var i=0; i<nodes.length; i++){
                var node= nodes[i];

                this.setExpandedState(node, true, silent, ignoreChildren);

                var children= this.elements.getChildNodes(node.id);
                if (children.length>0 && this.settings.levels) {
                    var childNodes= [];
                    children.each(function(){
                        var nodeId= $(this).data('id');
                        var child= context.getNode(nodeId);
                        childNodes.push(child);
                    });
                    this.expandLevels(childNodes, levels-1, silent, ignoreChildren);
                }
            }
        },

        toggleExpandedState: function (node, silent, ignoreChildren) {
            var jqNode= this.elements.getNode(node.id);
            var jqExpand= jqNode.find('.expand-icon');

            var state= hasClasses(jqExpand, this.settings.collapseIcon);

            this.setExpandedState(node, !state, silent, ignoreChildren);
        },

        toggleNodeExpanded: function (nodes, silent, ignoreChildren) {
            for(var i=0; i<nodes.length; i++){
                this.toggleExpandedState(nodes[i], silent, ignoreChildren);
            }
        },

        expandLevels: function (nodes, level, silent, ignoreChildren) {
            var context= this;

            for(var i=0; i<nodes.length; i++){
                var node= nodes[i];
                this.setExpandedState(node, level > 0, silent, ignoreChildren); //  (level > 0) ? true : false
                var children= this.elements.getChildNodes(node.id);
                if(children.length>0){
                    var childNodes= [];
                    children.each(function(){
                        var nodeId= $(this).data('id');
                        var child= context.getNode(nodeId);
                        childNodes.push(child);
                    });

                    this.expandLevels(childNodes, level-1, silent, ignoreChildren);
                }
            }
        },

        expandAll1: function (levels, silent, ignoreChildren) {
            var levels= levels || this.settings.levels;
            if (levels) {
                var nodes=[];
                for(var i=0; i<this.data.length; i++){
                    nodes.push(this.data[i]);
                }

                this.expandLevels(nodes, this.settings.levels, silent, ignoreChildren);   // this.data
            }
            else {
                var nodes= this.elements.getLevelNodes(1);
                var context= this;
                nodes.each(function(){
                    var nodeId= $(this).data('id');
                    var node= context.getNode(nodeId);
                    context.setExpandedState(node, true, silent, ignoreChildren);
                });
            }
        },

        collapseAll2: function (silent, ignoreChildren) {
            var nodes= this.elements.getLevelNodes(1);
            var context= this;
            nodes.each(function(){
                var nodeId= $(this).data('id');
                var node= context.getNode(nodeId);
                context.setExpandedState(node, false, silent, ignoreChildren);
            });
        },   

        expandAll: function(levels, silent, ignoreChildren){
            $(this.element).find('li[data-level!=1]').show();
            $(this.element).find('.expand-icon').removeClass(this.settings.expandIcon).addClass(this.settings.collapseIcon);  
        },

        collapseAll:function(silent, ignoreChildren){
            $(this.element).find('li[data-level!=1]').hide();
            $(this.element).find('.expand-icon').removeClass(this.settings.collapseIcon).addClass(this.settings.expandIcon);  
        },    

        // -------------------------------------------------------------------
        // 选中
        // -------------------------------------------------------------------

        setSelectedState: function (node, state, silent) {
            var jqNode= this.elements.getNode(node.id);

            // 不可选
            if(jqNode.hasClass('node-unselectable')){
                return;
            }

            if (state) {
                if (!this.settings.multiSelect) {
                    this.elements.getSelectedNodes(false).removeClass('node-selected');
                }

                jqNode.addClass('node-selected');
                if (!silent) {
                    this.elements.original.trigger('nodeSelected', node);
                }
            }
            else {
                jqNode.removeClass('node-selected');
                if (!silent) {
                    this.elements.original.trigger('nodeUnselected', node);
                }
            }
        },

        selectNode: function (nodes, silent) {
            for(var i=0; i<nodes.length; i++){
                this.setSelectedState(nodes[i], true, silent);
            }
        },

        unselectNode: function (nodes, silent) {
            for(var i=0; i<nodes.length; i++){
                this.setSelectedState(nodes[i], false, silent);
            }
        },

        toggleSelectedState: function (node, silent) {
            var jqNode= this.elements.getNode(node.id);
            var state= jqNode.hasClass('node-selected');

            this.setSelectedState(node, !state, silent);
        },

        toggleNodeSelected: function (nodes, silent) {
            for(var i=0; i<nodes.length; i++){
                this.toggleSelectedState(nodes[i], silent);
            }
        },

        // -------------------------------------------------------------------
        // checkbox
        // -------------------------------------------------------------------

        setCheckedState1: function(node, state, silent){
            var jqNode= this.elements.getNode(node.id);
            var jqCheck= jqNode.find('.check-icon');

            switch(state){
                case enumCheckedState.checked:{
                    jqNode.removeClass('node-checked-partof');
                    jqNode.addClass('node-checked');

                    jqCheck
                        .removeClass(this.settings.uncheckedIcon)
                        .removeClass(this.settings.checkedPartOfIcon)
                        .addClass(this.settings.checkedIcon);

                    if (!silent) {
                        // this.elements.original.trigger('nodeChecked', $.extend(true, {}, node));
                        this.elements.original.trigger('nodeChecked', node);
                    }
                    break;
                }
                case enumCheckedState.unchecked:{
                    jqNode.removeClass('node-checked-partof');
                    jqNode.removeClass('node-checked');

                    jqCheck
                        .removeClass(this.settings.checkedIcon)
                        .removeClass(this.settings.checkedPartOfIcon)
                        .addClass(this.settings.uncheckedIcon);

                    if (!silent) {
                        this.elements.original.trigger('nodeUnchecked', node);
                    }
                    break;
                }
                case enumCheckedState.partOf:{
                    jqNode.removeClass('node-checked');
                    jqNode.addClass('node-checked-partof');

                    jqCheck
                        .removeClass(this.settings.checkedIcon)
                        .removeClass(this.settings.uncheckedIcon)
                        .addClass(this.settings.checkedPartOfIcon);

                    if (!silent) {
                        this.elements.original.trigger('nodecheckedpartof', node);
                    }
                    break;
                }
            }

            if(this.settings.checkRecursive){
                if(node._innerParentId){
                    var parent= this.getNode(node._innerParentId);
                    while(parent){
                        var parentState;

                        switch(state){
                            case enumCheckedState.checked:
                            case enumCheckedState.unchecked:{
                                // 条件1 state= checked/unchecked
                                // 条件2 Sibling is all checked or has CheckedPartOf// 
                                var isSiblingAllSameCheckedStatus= false;

                                var numberOfSiblingCheckedPartOf= this.elements.getChildNodesCheckedPartOf(parent.id).length;
                                if(numberOfSiblingCheckedPartOf > 0)
                                {
                                    parentState= enumCheckedState.partOf;
                                }
                                else
                                {
                                    var numberOfSiblingChecked= this.elements.getChildNodesChecked(parent.id).length;
                                    if(numberOfSiblingChecked === 0)
                                    {
                                        parentState= enumCheckedState.unchecked;
                                    }
                                    else 
                                    {
                                        // var numberOfSibling= this.elements.getChildNodes(parent.id).length;
                                        // if(numberOfSibling === numberOfSiblingChecked){
                                        parentState= enumCheckedState.checked;     
                                        // }
                                    }
                                }
                                break;
                            }
                            case enumCheckedState.partOf:{
                                parentState= state;
                                break;
                            }
                        }
                        //this.setCheckedState(parent, parentState, silent);
                        this.setNodeCheckedSate(parent, parentState, silent);
                        parent= this.getNode(parent._innerParentId);
                    }
                }

                if(node.nodes && node.nodes.length > 0){
                    for(var i=0; i<node.nodes.length; i++){
                        var child= node.nodes[i];
                        //this.setCheckedState(child, state, silent);
                        this.setNodeCheckedSate(child, state, silent);
                    }
                }
            }
        },

        setCheckedState: function(node,state,silent){
            this._innerSetNodeCheckedSate(node,state,silent);
            if(this.settings.checkRecursive){
                this.setChildrenCheckedState(node,state,silent);
                this.setParentCheckedState(node,state,silent);
            }
        },

        _innerSetNodeCheckedSate: function(node,state,silent){
            var jqNode= this.elements.getNode(node.id);
            var jqCheck= jqNode.find('.check-icon');

            switch(state){
                case enumCheckedState.checked:{
                    jqNode.removeClass('node-checked-partof');
                    jqNode.addClass('node-checked');

                    jqCheck
                        .removeClass(this.settings.uncheckedIcon)
                        .removeClass(this.settings.checkedPartOfIcon)
                        .addClass(this.settings.checkedIcon);

                    if (!silent) {
                        // this.elements.original.trigger('nodeChecked', $.extend(true, {}, node));
                        this.elements.original.trigger('nodeChecked', node);
                    }
                    break;
                }
                case enumCheckedState.unchecked:{
                    jqNode.removeClass('node-checked-partof');
                    jqNode.removeClass('node-checked');

                    jqCheck
                        .removeClass(this.settings.checkedIcon)
                        .removeClass(this.settings.checkedPartOfIcon)
                        .addClass(this.settings.uncheckedIcon);

                    if (!silent) {
                        this.elements.original.trigger('nodeUnchecked', node);
                    }
                    break;
                }
                case enumCheckedState.partOf:{
                    jqNode.removeClass('node-checked');
                    jqNode.addClass('node-checked-partof');

                    jqCheck
                        .removeClass(this.settings.checkedIcon)
                        .removeClass(this.settings.uncheckedIcon)
                        .addClass(this.settings.checkedPartOfIcon);

                    if (!silent) {
                        this.elements.original.trigger('nodecheckedpartof', node);
                    }
                    break;
                }
            }
        },

        setChildrenCheckedState: function(node, state, silent){
            
            if(!node){
                return;
            }
            if(node.nodes && node.nodes.length > 0){
                for(var i=0; i<node.nodes.length; i++){
                    var child= node.nodes[i];
                    this._innerSetNodeCheckedSate(child, state, silent);
                    this.setChildrenCheckedState(child, state, silent);
                }
            }
        },

        setParentCheckedState: function(node,state,silent){
            if(node._innerParentId){
                var parent= this.getNode(node._innerParentId);
                while(parent){
                    var parentState;

                    switch(state){
                        case enumCheckedState.checked:
                        case enumCheckedState.unchecked:{
                            // 条件1 state= checked/unchecked
                            // 条件2 Sibling is all checked or has CheckedPartOf// 
                            var isSiblingAllSameCheckedStatus= false;

                            var numberOfSiblingCheckedPartOf= this.elements.getChildNodesCheckedPartOf(parent.id).length;
                            if(numberOfSiblingCheckedPartOf > 0)
                            {
                                parentState= enumCheckedState.partOf;
                            }
                            else
                            {
                                var numberOfSiblingChecked= this.elements.getChildNodesChecked(parent.id).length;
                                var numberOfSibling= this.elements.getChildNodes(parent.id).length;
                                if(numberOfSiblingChecked === 0){
                                    parentState= enumCheckedState.unchecked;
                                }else if(numberOfSiblingChecked < numberOfSibling){
                                    parentState= enumCheckedState.partOf;
                                }else{
                                    parentState= enumCheckedState.checked; 
                                }
                                // if(numberOfSiblingChecked < numberOfSibling)
                                // {
                                //     parentState= enumCheckedState.unchecked;
                                // }
                                // else 
                                // {
                                //     // var numberOfSibling= this.elements.getChildNodes(parent.id).length;
                                //     // if(numberOfSibling === numberOfSiblingChecked){
                                //     parentState= enumCheckedState.checked;     
                                //     // }
                                // }
                            }
                            break;
                        }
                        case enumCheckedState.partOf:{
                            parentState= state;
                            break;
                        }
                    }
                    this._innerSetNodeCheckedSate(parent, parentState, silent);
                    parent= this.getNode(parent._innerParentId);
                }
            }
        },

        checkNode: function (nodes, silent) {
            for(var i=0; i<nodes.length; i++){
                this.setCheckedState(nodes[i], enumCheckedState.checked, silent);
            }
        },

        uncheckNode: function (nodes, silent) {
            for(var i=0; i<nodes.length; i++){
                this.setCheckedState(nodes[i], enumCheckedState.unchecked, silent);
            }
        },

        toggleCheckedState: function (node, silent) {
            var jqNode= this.elements.getNode(node.id);
            var jqCheck= jqNode.find('.check-icon');

            var state= hasClasses(jqCheck, this.settings.checkedIcon);
            var toggleState;
            if(state)
                toggleState = enumCheckedState.unchecked;
            else
                toggleState = enumCheckedState.checked;
            this.setCheckedState(node, toggleState, silent);
        },

        toggleNodeChecked: function (nodes, silent) {
            for(var i=0; i<nodes.length; i++){
                this.toggleCheckedState(nodes[i], silent);
            }
        },

        checkAll: function (silent) {
            var checkedNodes= this.elements.getCheckedNodes(true);
            var jqCheck= checkedNodes.find('.check-icon');

            checkedNodes.addClass('node-checked');
            jqCheck.removeClass(this.settings.uncheckedIcon).addClass(this.settings.checkedIcon);

            if (!silent) {
                var context= this;
                checkedNodes.each(function(){
                    var nodeId= $(this).data('id');
                    var node= context.getNode(nodeId);
                    context.elements.original.trigger('nodeChecked', node);
                });
            }
        },

        uncheckAll: function (silent) {
            var uncheckedNodes= this.elements.getCheckedNodes(false);
            var jqCheck= uncheckedNodes.find('.check-icon');

            uncheckedNodes.removeClass('node-checked');            
            jqCheck.removeClass(this.settings.checkedIcon).addClass(this.settings.uncheckedIcon);

            if (!silent) {
                var context= this;
                uncheckedNodes.each(function(){
                    var nodeId= $(this).data('id');
                    var node= context.getNode(nodeId);
                    context.elements.original.trigger('nodeUnchecked', node);
                });
            }
        },

        // -------------------------------------------------------------------
        // 启用 / 禁用
        // -------------------------------------------------------------------

        setDisabledState: function (node, state, silent, ignoreChildren) {
            var jqNode= this.elements.getNode(node.id);

            if (state) {
                jqNode.addClass('node-disabled');

                // Disable all other states
                this.setExpandedState(node, false, silent, ignoreChildren);
                this.setSelectedState(node, false, silent);
                this.setCheckedState(node, false, silent);

                if (!silent) {
                    this.elements.original.trigger('nodeDisabled', node);
                }
            }
            else {
                jqNode.removeClass('node-disabled');

                if (!silent) {
                    this.elements.original.trigger('nodeEnabled', node);
                }
            }
        },

        enableNode: function (nodes, silent) {
            for(var i=0; i<nodes.length; i++){
                this.setDisabledState(nodes[i], false, silent);
            }
        },

        disableNode: function (nodes, silent) {
            for(var i=0; i<nodes.length; i++){
                this.setDisabledState(nodes[i], true, silent);
            }
        },

        toggleDisabledState: function (node, silent) {
            var jqNode= this.elements.getNode(node.id);
            var state= jqNode.hasClass('node-disabled');

            this.setDisabledState(node, !state, silent);
        },

        toggleNodeDisabled: function (nodes, silent) {
            for(var i=0; i<nodes.length; i++){
                this.toggleDisabledState(nodes[i], silent);
            }
        },

        disableAll: function (silent, ignoreChildren) {
            var enabledNodes= this.elements.getDisabledNodes(true);
            enabledNodes.addClass('node-disabled');

            var context= this;
            enabledNodes.each(function(){
                var nodeId= $(this).data('id');
                var node= context.getNode(nodeId);

                // Disable all other states
                context.setExpandedState(node, false, silent, ignoreChildren);
                context.setSelectedState(node, false, silent);
                context.setCheckedState(node, false, silent);

                if (!silent) {
                    context.elements.original.trigger('nodeDisabled', node);
                }
            });
        },

        enableAll: function (silent, ignoreChildren) {
            var disabledNodes= this.elements.getDisabledNodes(false);
            disabledNodes.removeClass('node-disabled');

            var context= this;
            disabledNodes.each(function(){
                var nodeId= $(this).data('id');
                var node= context.getNode(nodeId);
                if (!silent) {
                    context.elements.original.trigger('nodeEnabled', node);
                }
            });            
        },

        // -------------------------------------------------------------------
        // 扩展 api
        // -------------------------------------------------------------------

        selectNodeByIds: function(ids,silent){
            var arrId= ids.split(',');
            var nodes= [];
            for(var i=0;i < arrId.length;i++){
                var node =this.getNode(arrId[i]);
                nodes.push(node);
            }
            this.selectNode(nodes, silent);
        },

        unselectNodeByIds: function(ids,silent){
            var arrId= ids.split(',');
            var nodes= [];
            for(var i=0;i < arrId.length;i++){
                var node =this.getNode(arrId[i]);
                nodes.push(node);
            }
            this.unselectNode(nodes, silent);
        },

        checkNodeByIds: function(ids,silent){
            var arrId= ids.split(',');
            var nodes= [];
            for(var i=0;i < arrId.length;i++){
                var node =this.getNode(arrId[i]);
                nodes.push(node);
            }
            this.checkNode(nodes, silent);
        },

        uncheckNodeByIds: function(ids,silent){
            var arrId= ids.split(',');
            var nodes= [];
            for(var i=0;i < arrId.length;i++){
                var node =this.getNode(arrId[i]);
                nodes.push(node);
            }
            this.uncheckNode(nodes, silent);
        },

        getSelectNodes: function(){
            var selectNodes =  this.elements.getSelectedNodes(false);
            var nodes = [];
            var context = this;
            selectNodes.each(function(){
                    var nodeId= $(this).data('id');
                    var node= context.getNode(nodeId);
                    nodes.push(node);
                });
            return nodes;
        },

        getCheckedNodes: function(withPartOf){
            var checkNodes =  this.elements.getCheckedNodes(false,withPartOf);
            var nodes = [];
            var context = this;
            checkNodes.each(function(){
                    var nodeId= $(this).data('id');
                    var node= context.getNode(nodeId);
                    nodes.push(node);
                });
            return nodes;
        },

        getNode: function (nodeId) {
            var nodeIndex= this.idIndexMap['id_'+nodeId];
            return this.data[nodeIndex];
        },

        revealNode: function (nodes, options) {
            for(var i=0; i<nodes.length; i++){
                var path= nodes[i]._innerPath;
                var arrPath= path.split('|');
                for(var j=1; j<arrPath.length-1; j++){
                    var node= this.getNode(arrPath[j]);
                    this.setExpandedState(node, true, options);
                }
            }
        },

        search: function (pattern, options) {
            var searchOptions = {
                ignoreCase: true,
                exactMatch: false,
                revealResults: true
            };

            options = $.extend({}, searchOptions, options);

            this.clearSearch();

            var results = [];
            if (pattern && pattern.length > 0) {

                if (options.exactMatch) {
                    pattern = '^' + pattern + '$';
                }

                var modifier = 'g';
                if (options.ignoreCase) {
                    modifier += 'i';
                }

                results = this.findNodes(pattern, modifier);
            }

            if (options.revealResults) {
                this.revealNode(results);
            }

            this.elements.original.trigger('searchComplete', $.extend(true, {}, results));

            return results;
        },

        clearSearch: function () {
            var searchResults= this.elements.getSearchResultNodes();
            searchResults.removeClass('search-result');
            
            this.elements.original.trigger('searchCleared');    // , $.extend(true, {}, results)
        },

        findNodes: function (pattern, modifier, attribute) {

            modifier = modifier || 'g';
            attribute = attribute || 'text';

            var context = this;
            var allNodes= this.elements.getAllNodes();

            var results= [];
            for(var i=0; i<allNodes.length; i++){
                var jqNode= $(allNodes[i]);
                var val = jqNode.text();
                var isMatched= val.match(new RegExp(pattern, modifier));
                if(isMatched){
                    jqNode.addClass('search-result');
                    var nodeId= jqNode.data('id');
                    var node= this.getNode(nodeId);
                    results.push(node);
                }
            }

            return results;
        },

        destroy: function () {
            this.container.empty();
            this.container = null;
            this.unbindEvents();
        },
        // 取消事件监听
        unbindEvents: function () {
            this.elements.original.off('click');
            this.elements.original.off('dblclick');
            this.elements.original.off('nodedbclicked');
            this.elements.original.off('nodeChecked');
            this.elements.original.off('nodeCollapsed');
            this.elements.original.off('nodeDisabled');
            this.elements.original.off('nodeEnabled');
            this.elements.original.off('nodeExpanded');
            this.elements.original.off('nodeSelected');
            this.elements.original.off('nodeUnchecked');
            this.elements.original.off('nodeUnselected');
            this.elements.original.off('searchComplete');
            this.elements.original.off('searchCleared');
        },
        // 监听事件
        bindEvents: function () {
            this.unbindEvents();

            var time = null;

            var context = this;
            this.elements.original.on('click', $.proxy(function(e) { 
                clearTimeout(time);
                time = setTimeout(function(){
                    context.clickHandler(e);
                },300);
            }, context));

            this.elements.original.on('dblclick', $.proxy(function(e) { 
                clearTimeout(time);
                context.dblclickHandler(e);
            }, context));

            this.elements.original.on('nodedbclick', $.proxy(this.settings.onNodeDbclick, this));
            this.elements.original.on('nodeChecked', $.proxy(this.settings.onNodeChecked, this));
            this.elements.original.on('nodeCollapsed', $.proxy(this.settings.onNodeCollapsed, this));
            this.elements.original.on('nodeDisabled', $.proxy(this.settings.onNodeDisabled, this));
            this.elements.original.on('nodeEnabled', $.proxy(this.settings.onNodeEnabled, this));
            this.elements.original.on('nodeExpanded', $.proxy(this.settings.onNodeExpanded, this));
            this.elements.original.on('nodeSelected', $.proxy(this.settings.onNodeSelected, this));
            this.elements.original.on('nodeUnchecked', $.proxy(this.settings.onNodeUnchecked, this));
            this.elements.original.on('nodeUnselected', $.proxy(this.settings.onNodeUnselected, this));
            this.elements.original.on('searchComplete', $.proxy(this.settings.onSearchComplete, this));
            this.elements.original.on('searchCleared', $.proxy(this.settings.onSearchCleared, this));            
        },

        //开放接口
        getApi: function(){
            var context = this;
            var funs = {
                collapseAll: 'collapseAll',
                expandAll: 'expandAll',
                checkAll:'checkAll',
                uncheckAll:'uncheckAll',
                disableAll:'disableAll',
                enableAll:'enableAll',
                checkNodeByIds:'checkNodeByIds',
                uncheckNodeByIds:'uncheckNodeByIds',
                selectNodeByIds:'selectNodeByIds',
                unselectNodeByIds:'unselectNodeByIds',
                getCheckedNodes:'getCheckedNodes',
                getSelectNodes:'getSelectNodes',
                getNode:'getNode'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });
});

/* tree javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'tree';
    var pluginClass = s.ui.controls.Slowtree;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);


// getApi: function(){
//     var context= this;
//     var api={
//         updateOptions: context.updateOptions,
//         updateData: context.updateData,
//         fresh: context.fresh
//     };
//     return api;
// },
Jx().package('s.ui.controls', function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var emptyFun= function(){};
    var defaults = {
        onConfirm: emptyFun,
        multiSelect:true,
        map:{id:'Id',text:'CNFullName'},
        titleName:'部门',
        inputHide:false,
        renderTreeNode:undefined
    };
    var attributeMap = {
        multiSelect:'multi-select',
        titleName:"title-name"
    };

    this.Departselect = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,
        // data:{},

        // 构造函数
        init:function(element, options){
            this.element = $(element);
            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);
            this.settings._currentPluginId = _currentPluginId;

            // 初始化 html DOM 元素
            this.initElements();
        },
        //disabledNodeIds同时要禁用typeahead下拉选项
        parseTypeaheadData:function(data,ids) {
            ids = ids.split(',');
            // hack 执行parseDataTrees方法后，可获得orgsMap
            this.Treeselect.modal.parseDataTrees(data);
            for (var i = 0; i < ids.length; i++) {
                this.Treeselect.modal.states.orgsMap[ids[i]].unselectable = true;
            }
        },
        updateDataOrgs: function(data,ids) {
            if (ids && ids !== "") {
                this.parseTypeaheadData(data, ids);
            }
            this.Treeselect.modal.updateDataOrgs(data);
            this.Treeselect.userselect.updateData(data);
        },

        initElements:function () {
            var context = this;
           
            this.elements = {
                original: this.element
            };
                 
            this.Treeselect = new s.ui.controls.Treeselect(this.elements.original,{
                onConfirm:this.settings.onConfirm,
                multiSelect:this.settings.multiSelect,
                titleName:this.settings.titleName,
                map:this.settings.map,
                inputHide:this.settings.inputHide,
                renderTreeNode:this.settings.renderTreeNode
            });
           
        },
        setValue: function(value){
           
            this.Treeselect.userselect.setData(value);
        },
        refresh: function(){
            var context= this;

            // $.when(this.d.promise())
            //  .done(function(){
            //     context.modal.refresh();
            //  });
            // this.modal.refresh();
        },
        enable: function(){
            this.Treeselect.element.prop('disabled', false);
            this.Treeselect.userselect.enable();
            // this.elements.view.prop('disabled',false);
            // this.elements.button.prop('disabled',false);
            this.disabled=false;
        },
        disable: function(){
            this.Treeselect.element.prop('disabled', true);
            this.Treeselect.userselect.disable();
            // this.elements.view.prop('disabled', true);
            // this.elements.button.prop('disabled', true);
            this.disabled=true;
        },
        onOpen:function(btn){
             this.Treeselect.userselect.getButton().click();
        },
        disableNodeByIds:function(ids){
             this.Treeselect.modal.controls.orgTree.disableNodeByIds(ids);
        },
        // API
        getApi:function(){
            var context = this;
            var funs = {
                onOpen: 'onOpen',
                disableNodeByIds:'disableNodeByIds'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        },
        destory: function(){}
    });

});

/* orgselect javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'departselect';
    var pluginClass = s.ui.controls.Departselect;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destory();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var emptyFun= function(){};
    var defaults = {
        levels: 2,  // 节点展开的级别
        dataUrl: '',
        showCheckbox: 'none',  // 默认为none,all表示所有显示，leaf表示只有叶子节点显示
        selectMode:'none',   //默认为all，leaf表示为最后一级可选中
        checkRecursive:true,    //默认为all，leaf表示不联动
        multiSelect: false, 
        appendHtml:'',      //在节点上增加元素
        selectedNodeIds:'',
        checkedNodeIds:'',

        // 节点双击
        onNodeDbclick: emptyFun,
        // 节点勾选
        onNodeChecked: emptyFun,
        // 节点收起
        onNodeCollapsed: emptyFun,
        // 节点禁用
        onNodeDisabled: emptyFun,
        // 节点启用
        onNodeEnabled: emptyFun,
        // 节点展开
        onNodeExpanded: emptyFun,
        // 节点选中
        onNodeSelected: emptyFun,
        // 节点取消勾选
        onNodeUnchecked: emptyFun,
        // 节点取消选中
        onNodeUnselected: emptyFun,
        unselect:false
    };
    var attributeMap = {
        dataUrl: 'data-url',
        levels: 'levels',
        showCheckbox:'show-checkbox',
        selectMode:'select-mode',
        checkRecursive:'check-recursive',
        multiSelect:'multi-select',
        selectedNodeIds:'selected-node-ids',
        checkedNodeIds:'checked-node-ids',
        unselect:'unselect'
    };

    this.Combotree = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // 构造函数
        init:function(element, options){
            // 区分一个页面中存在多个控件实例
            _currentPluginId += 1;

            this.element = $(element);
            //this.settings,

            this.container,
            this.elements,

            this.value = this.element.val();

            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            // 防止多次初始化
            if (this.element.data('initialized')) { return; }
            this.element.data('initialized', true);
            
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);

            // 构建html DOM
            this.buildHtml();
            // 初始化 html DOM 元素
            this.initElements();
            this.transferAttributes();

            // // 初始化数据
            // this.getData();
            var context= this;
            this.d= $.Deferred();
            $.when(this.getData(this.d))
             .done(function(){
                context.render();
             })

            // 创建 树型 菜单对象
            //this.menu=new CascadeMenu(this.elements, this.settings);

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            this.bindEventsInterface();
        },
        //data-s-dropdown-id="#c-combotree-menu-'+pluginId+'">
        buildHtml:function () {
            var pluginId= this.element.data('plugin-id');
            var htmlTemplate = ''+
                '<div class="c-combotree-container">'+
                //'   <div>'+
                '       <a class="btn btn-default c-combotree-btn" data-s-dropdown-id="#c-combotree-menu-'+pluginId+'">'+
                '           <span class="c-combotree-btn-text"></span>'+
                '           <span class="glyphicon glyphicon-menu-down"></span>'+
                '       </a>'+
                '       <div id="c-combotree-menu-'+pluginId+'" class="dropdown-menu">'+
                '           <div class="tree-menu"> '+
                '           </div>' +
                '       </div> '+
                //'   </div> '+
                '</div>';

            this.container=$(htmlTemplate);
            this.element.before(this.container);
            this.element.hide();
        },
        initElements:function () {
            this.elements = {
                orginal: this.element,
                viewContainer: $('.c-combotree-btn', this.container),
                view: $('.c-combotree-btn-text', this.container),
                menu: $('.dropdown-menu',this.container),
                tree: $('.tree-menu', this.container)
            };

            this.elements.orginal.hide();

            this.dropdown= new s.ui.controls.Dropdown(this.elements.viewContainer, {});
            this.elements.menu.on('click',function(e){
                e.preventDefault();
                e.stopPropagation();
            });
        },
        getData:function(d){
            if(this.settings.data){
                this.data= $.extend(true, [], this.settings.data);
                delete this.settings.data;

                d.resolve();

                return d.promise();
            }

            // var context = this;
            // $.ajax({
            //     dataType: 'json',
            //     url: this.settings.dataUrl,
            //     data: {},
            //     success: function(data){
            //         context.data= data;

            //         d.resolve();
            //     },
            //     error: function(request, status, error){
            //         alert('控件id: ' + context.element.attr('id')+' , ajax获取数据失败！');

            //         d.resolve();
            //     }
            // });

            return d.promise();
        },
        updateData: function(data){
            this.tree.getApi().setValue(data);
        },
        getCheckedNodes: function(){
            return this.tree.getApi().getCheckedNodes();
        },
        getSelectNodes: function(){
            return this.tree.getApi().getSelectNodes();
        },
        selectNodeByIds: function(ids){
            if(ids)
                this.tree.getApi().selectNodeByIds(ids);
            else
                this._getValues([]);
        },
        unselectNodeByIds: function(ids){
            return this.tree.getApi().unselectNodeByIds(ids);
        },
        checkNodeByIds: function(ids){
            if(ids)
                this.tree.getApi().checkNodeByIds(ids);
            else
                this._getValues([]);
        },

        disableNodeByIds:function(ids){
             if(ids)
                this.tree.getApi().disableNodeByIds(ids);
        },

        uncheckNodeByIds: function(ids){
            return this.tree.getApi().uncheckNodeByIds(ids);
        },

        closeTree:function(){
            this.dropdown.toggle();
        },
        render:function(){
            var context= this;
            this.tree=new s.ui.controls.Tree(this.elements.tree, {
                levels: context.settings.levels,  
                showCheckbox: context.settings.showCheckbox,  
                selectMode:context.settings.selectMode,   
                checkRecursive:context.settings.checkRecursive,    
                multiSelect: context.settings.multiSelect, 
                selectedNodeIds:context.settings.selectedNodeIds,
                checkedNodeIds:context.settings.checkedNodeIds,
                data: [],
                onNodeChecked: function(e, node){
                    var slef=this;
                    //window.setTimeout(function(){
                        var nodes = slef.getCheckedNodes(false);
                        context._getValues(nodes);
                        context.elements.view.trigger('nodeChecked',node);
                    //},100);
                    
                },
                onNodeUnchecked: function(e, node){
                    var slef=this;
                    //window.setTimeout(function(){
                        var nodes = slef.getCheckedNodes(false);
                        context._getValues(nodes);
                        context.elements.view.trigger('nodeUnchecked',node);
                    //},100);
                },
                onNodeSelected: function(e, node){
                    //this.toggleCheckedState(node);
                    var nodes = this.getSelectNodes();
                    if(!context.settings.unselect){
                        context._getValues(nodes);
                    }
                    
                    context.elements.view.trigger('nodeSelected',node);
                    // if(context.settings.multiSelect === false)
                    //     context.elements.tree.hide();
                },
                onNodeUnselected: function(e, node){
                    //this.toggleCheckedState(node);
                    var nodes = this.getSelectNodes();
                    if(!context.settings.unselect){
                        context._getValues(nodes);
                    }
                    context.elements.view.trigger('nodeUnselected',node);
                    // if(context.settings.multiSelect === false)
                    //     context.elements.tree.hide();
                }
            });
        },
        _getValues:function(nodes){
            var viewValues= '';  
            this.values = '';
            for (var i = 0; i < nodes.length; i++) {
                viewValues+= nodes[i].text+',';
                this.values+= nodes[i].id+',';
            }
            viewValues=viewValues.substr(0, viewValues.length-1);
            this.elements.view.text(viewValues);
        },

        unbindEvents: function () {
            this.elements.view.off('nodedbclick');
            this.elements.view.off('nodeChecked');
            this.elements.view.off('nodeCollapsed');
            this.elements.view.off('nodeDisabled');
            this.elements.view.off('nodeEnabled');
            this.elements.view.off('nodeExpanded');
            this.elements.view.off('nodeSelected');
            this.elements.view.off('nodeUnchecked');
            this.elements.view.off('nodeUnselected');
        },

        bindEvents:function(){
            this.unbindEvents();
            this.elements.view.on('nodedbclick', $.proxy(this.settings.onNodeDbclick, this));
            this.elements.view.on('nodeChecked', $.proxy(this.settings.onNodeChecked, this));
            this.elements.view.on('nodeCollapsed', $.proxy(this.settings.onNodeCollapsed, this));
            this.elements.view.on('nodeDisabled', $.proxy(this.settings.onNodeDisabled, this));
            this.elements.view.on('nodeEnabled', $.proxy(this.settings.onNodeEnabled, this));
            this.elements.view.on('nodeExpanded', $.proxy(this.settings.onNodeExpanded, this));
            this.elements.view.on('nodeSelected', $.proxy(this.settings.onNodeSelected, this));
            this.elements.view.on('nodeUnchecked', $.proxy(this.settings.onNodeUnchecked, this));
            this.elements.view.on('nodeUnselected', $.proxy(this.settings.onNodeUnselected, this));
        },

        destroy: function () {
            this.container.empty();
            this.container = null;
            this.unbindEvents();
        },

        bindEventsInterface:function () {},
        transferAttributes: function(){
            if(this.element.prop('disabled')){
                this.disable();
            }
        },

        disable: function() {
            this.element.prop('disabled', true);
            this.elements.viewContainer.addClass('disabled');
        }, 
        enable: function() {
            this.element.prop('disabled', false);
            this.elements.viewContainer.removeClass('disabled');
        },
        getApi: function(){
            var context = this;
            var funs = {
                getCheckedNodes:'getCheckedNodes',
                getSelectNodes:'getSelectNodes',
                disableNodeByIds:'disableNodeByIds',
                selectNodeByIds:'selectNodeByIds',
                unselectNodeByIds:'unselectNodeByIds',
                checkNodeByIds:'checkNodeByIds',
                uncheckNodeByIds:'uncheckNodeByIds',
                closeTree:'closeTree'

            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });

});




/* cascade javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'combotree';
    var pluginClass = s.ui.controls.Combotree;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package('s.ui.controls', function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var emptyFun= function(){};
    var defaults = {
        hasContact: true,   //是否包含联系人组
        hasAddContact: false,  //是否包含修改联系人组
        urlContacts:'', 
        urlContactUsers:'',
        urlAddContacts:'',
        hasJobFilter: false,   //是否有职位筛选
        hasOuterFilter: false, //是否有所内所外筛选  
        urlOrgs:'',
        urlUsers: '',
        multiSelect:true,
        inputHide:false,
        onConfirm: emptyFun,
        confirmHide:true,
        renderTreeNode:undefined,
        titleName:'选择人员',
        typeaheadMethod:{}//这里必须是{} 不能为undefined
    };
    var attributeMap = {
        hasContact:'has-contact',
        hasAddContact:'has-add-contact',
        hasJobFilter:'has-job-filter',
        hasOuterFilter:'has-outer-filter',

        urlContacts: 'url-contacts',
        urlContactUsers: 'url-contact-users',
        urlAddContacts: 'url-add-contacts',
        multiSelect:'multi-select',
        inputHide:'input-hide',
        urlJobs:'url-jobs',
        urlUsers: 'url-users',
        urlOrgs: 'url-orgs',
        confirmHide:'confirm-hide',
        titleName:"title-name"
    };


    var OrgselectModal = new J.Class({
        data:{},
        //init:function(elements, options, data){
        init:function(element, options){
            this.userselect = element;

            // 初始化选项
            //this.initSettings(options);
            // 直接使用地址类实例的设置
            this.settings=options;
            this.orgCount=0;
            this.contactCount=0;
            // 状态
            this.states={
                leftUsers: [],
                rightUsers: [],
                rightUserMap:{},
                filterLeftUsers:[],
                filterStates:'-100'
            };

            // 构建html DOM
            this.buildHtml();
            this.initElements();

            // 初始化数据
            // this.data=data;
            // this.getData();
            // var context= this;
            // context.render();
            // this.d1 = $.Deferred();
            // this.d2 = $.Deferred();
            // $.when(this.getDataUsers(this.d1),this.getDataOrgs(this.d2))
            //  .done(function(){
            //     context.render();
            //  });
            
            //this.transferAttributes();

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            this.bindEventsInterface();
        },
        getDataUsers: function(d){
            if ((this.settings.data !== undefined) && (this.settings.data.users !== undefined) && this.settings.data.users) {
                var users = $.extend(true, [], this.settings.data.users);
                this.data.users = this.parseDataUsers(users);
                delete this.settings.data.users;
                d.resolve();

                return d.promise(); 
            }else{
                var context = this;
                $.ajax({
                    dataType: 'json',
                    // url: this.settings.dataUrl,
                    // url: '/demo/controls/orgselect/users.json',
                    url: context.settings.dataUrlUsers,
                    data: {},
                    success: function(data){
                        // context.data= parseData(data);
                        context.data.users = context.parseDataUsers(data);
                        d.resolve();
                    },
                    error: function(request, status, error){
                        d.resolve();
                    }
                });

                return d.promise();
            }
        },
        getDataOrgs: function(d){
            if ((this.settings.data !== undefined) && (this.settings.data.orgs !== undefined)  && this.settings.data.orgs) {
                var orgs = $.extend(true, [], this.settings.data.orgs);
                this.data.orgs = this.parseDataOrgs(orgs);
                delete this.settings.data.orgs;

                d.resolve();

                return d.promise(); 
            }else{
                var context = this;

                $.support.cors = true;//允许跨域调用Webservice

                $.ajax({
                    // type: 'post',
                    // url: this.settings.dataUrl,
                    // url: 'http://oaapp106.oaad.org.cn/oaframeworksvc/ControlService/SelectControlService.asmx/GetOrgLists',     // serviceURL + '/GetOrgLists',
                    // url: '/demo/controls/orgselect/orgs.json',
                    url: context.settings.dataUrlOrgs,
                    // data: '{'organIds':'' + $this.data('settings').filter.organID + ''}',
                    // data:'{"organIds":""}',
                    dataType: 'json',
                    contentType: 'application/json; charset=utf-8',
                    success: function(data){
                        context.data.orgs = context.parseDataOrgs(data);
                        d.resolve();
                    },
                    error: function(request, status, error){
                        d.resolve();
                    }
                });

                return d.promise();
            }
        },
        parseDataUsers: function(data){
            var users=[];
            this.userMap = {};
            this.parseUserMap = {};
            this.orgUsersMap = {};
            for(var i=0; i<data.length; i++){
                var d=data[i];
                // titleSortNo
                // CoreNonCore
                // var orgPath=d.OrgPath.split('\\u003e').join('>');
                var user = this.tranferOriginalUser(d);
                users[users.length]=user;
                this.userMap[d.Id] = d;
                this.parseUserMap[d.Id] = user;
                var orgPath = user.orgPath;
                if(!orgPath) continue ;
                var parentOrgIds = orgPath.split('>');
                for (var j = 0; j < parentOrgIds.length; j++) {
                    var pid = parentOrgIds[j];
                    if(!this.orgUsersMap[pid])
                        this.orgUsersMap[pid] = [];
                    this.orgUsersMap[pid].push(user);
                }
            }
            for(var key in this.orgMap){
                var orgId = key;
                if(!this.orgUsersMap[orgId])
                    this.orgUsersMap[orgId] = [];
                this.orgMap[key].text += "("+this.orgUsersMap[orgId].length+")";
            }
            return users;
        },
        parseDataOrgs: function(data){
            
            // var users = this.getUsersByOrgId(org.Id);
            // this.orgUsersMap[org.Id] = users;
            this.orgIdPathMap = {};
            this.orgMap = {};
            var org=data[0];
            var node = {
                id:org.Id,
                parentId:org.ParentID,
                text:org.CNFullName, //+'('+users.length+')',
                path: ''+org.Id,
                orgPath: ''+org.Id
            };
            var tree=[node];
            this.orgIdPathMap[org.Id] = tree[0].orgPath;   
            this.orgMap[org.Id] = node;
            for(var i=1; i<data.length; i++){
                org=data[i];
                var node;
                var prevNode=tree[tree.length-1];
                while(org.ParentID !== prevNode.id){
                    if(!prevNode.nodes){
                        break;
                    }
                    prevNode=prevNode.nodes[prevNode.nodes.length-1];
                }

                if(org.ParentID === prevNode.id){
                    prevNode.nodes=prevNode.nodes||[];
                    // users = this.getUsersByOrgId(org.Id);
                    // this.orgUsersMap[org.Id] = users;
                    var orgPath = prevNode.orgPath + '>'+org.Id;
                    var node = {
                        id:org.Id,
                        parentId:org.ParentID,
                        text:org.CNFullName, //+'('+users.length+')',
                        path:prevNode.path+'>'+org.Id,
                        orgPath: orgPath
                    };
                    prevNode.nodes[prevNode.nodes.length]=node;
                    this.orgIdPathMap[org.Id] = orgPath;
                    this.orgMap[org.Id] = node;
                }
            }
            this.orgCount=data.length;
            this.elements.colOrgNumber.text(this.orgCount+this.contactCount);
            return tree;
        },
        parseDataContacts: function(data){
            this.contactUsersMap = {};
            var contact=data[0];
            var users = this.getUsersByContanct(contact);
            this.contactUsersMap[contact.GroupID] = users;
            var tree=[{
                id:contact.GroupID,
                parentId:contact.ParentID,
                text:contact.GroupName+'('+users.length+')',
                path: ''+contact.GroupID
            }];
            var num = 0;
            for(var i=1; i<data.length; i++){
                contact=data[i];
                var node;
                var prevNode=tree[tree.length-1];

                while(contact.ParentID !== prevNode.id){
                    if(!prevNode.nodes){
                        break;
                    }
                    prevNode=prevNode.nodes[prevNode.nodes.length-1];
                }

                if(contact.ParentID === prevNode.id){
                    prevNode.nodes=prevNode.nodes||[];
                    users = this.getUsersByContanct(contact);
                    this.contactUsersMap[contact.GroupID] = users;
                    prevNode.nodes[prevNode.nodes.length]={
                        id:contact.GroupID,
                        parentId:contact.ParentID,
                        text:contact.GroupName+'('+users.length+')',
                        path: ''+contact.GroupID
                    };
                    num += users.length;
                }
            }
            tree[0].text = data[0].GroupName+'('+num+')';
            this.contactCount=num;
            this.elements.colOrgNumber.text(this.orgCount+this.contactCount);
            return tree;
        },

        parseDataSetValueUsers:function(data){
            var users = [];
            for (var i = 0; i < data.length; i++) {
                var d = data[i];
                var user= {
                    id: d.Id,
                    realName: d.RealName,
                    name: d.HasSameName ? (d.RealName+'('+d.LoginName+')'):d.RealName,
                    hasSameName:d.HasSameName,
                    loginName: d.LoginName,
                    spell: d.Spell,
                    spellFirst: d.SpellFirst,
                    title: d.Title||'',
                    orgId: d.OrgID,
                    titleSortNo: d.TitleSortNo,
                    innerOuter: d.CoreNonCore,
                    orgPath: null,
                    oriData:d
                };

                users.push(user);
            }
            return users;
        },

        tranferOriginalUser: function(originalUser){
            var d = originalUser;
            // var orgPath= d.OrgPath ? d.OrgPath.replace(/\\u003e/g,'>') : '';
            var orgPath = this.orgIdPathMap[d.OrgID];
            var user= {
                id: d.Id,
                realName: d.RealName,
                name: d.HasSameName ? (d.RealName+'('+d.LoginName+')'):d.RealName,
                hasSameName:d.HasSameName,
                loginName: d.LoginName,
                spell: d.Spell,
                spellFirst: d.SpellFirst,
                title: d.Title||'',
                orgId: d.OrgID,
                titleSortNo: d.TitleSortNo,
                innerOuter: d.CoreNonCore,
                orgPath: orgPath,// + d.Id
                oriData:originalUser.oriData||originalUser
            };
            return user;
        },

        tranferOriginalUsers: function(originalUsers){
            var users = [];
            for (var i = 0; i < originalUsers.length; i++) {
                users.push(this.tranferOriginalUsers(originalUsers[i]));
            }
            return users;
        },

        tranferInnerUser: function(innerUser){
            var d = innerUser;
            var user = {
                Id: d.id,
                RealName: d.realName,
                LoginName: d.loginName,
                HasSameName: d.hasSameName,
                Spell: d.spell,
                SpellFirst: d.spellFirst,
                Title: d.title||'',
                OrgID: d.orgId,
                TitleSortNo: d.titleSortNo,
                CoreNonCore: d.innerOuter,
                oriData:d.oriData || d
            };
            return user;
        },

        tranferInnerUsers: function(innerUsers){
            var users = [];
            for (var i = 0; i < innerUsers.length; i++) {
                users.push(this.tranferInnerUser(innerUsers[i]));
            }
            return users;
        },

        /**
        * 根据orgId来获取用户
        */
        getUsersByOrgId: function(orgId) {
            var users= $.grep( this.data.users, function(user) {
                var filter=orgId+'>';
                return user.orgPath.indexOf(filter)>-1;
            });
            return users||[];
        },

        getUsersByContanct: function(contact){
            if(!this.parseUserMap)
                return;
            var userIDs = contact.userIDs;
            var users = [];
            if(!userIDs || userIDs.length === 0)
                return users;
            for (var i = 0; i < userIDs.length; i++) {
                var user = this.parseUserMap[userIDs[i]];
                users.push(user);
            }
            return users;
        },

        // updateDataUsers: function(data){
        //     this.data.users = this.parseDataUsers(data);
        //     // this.render();
        // },
        updateDataUsers: function(data){
            this.data.users = data;
            this.render();
        },
        updateDataContacts: function(data){
            this.data.contacts = this.parseDataContacts(data);
            this._renderContactTree();
        },
        updateDataOrgs: function(data){
            this.data.orgs = this.parseDataOrgs(data);
            // this.render();
        },
        updateDataJobFilter: function(data){
            this.data.jobfilter = data;
            this.renderJobFilter();
        },

        buildHtml:function(){


             var htmlTitleFilter=''+
                '                        <select class="joblist">'+
                '                            <option selected="selected" value="-100">请选择职务</option>'+
                '                            <option value="1">理事长</option>'+
                '                            <option value="2">总经理</option>'+
                '                            <option value="3">监事长</option>'+
                '                            <option value="4">副总经理</option>'+
                '                            <option value="5">总经理助理</option>'+
                '                            <option value="6">总监</option>'+
                '                            <option value="7">副总监</option>'+
                '                            <option value="10">首席研究员</option>'+
                '                            <option value="11">主任研究员</option>'+
                '                            <option value="12">高级执行经理</option>'+
                '                            <option value="13">副主任研究员</option>'+
                '                            <option value="14">执行经理</option>'+
                '                            <option value="15">区域首席代表</option>'+
                '                            <option value="16">研究员</option>'+
                '                            <option value="17">高级经理</option>'+
                '                            <option value="18">副研究员</option>'+
                '                            <option value="19">经理</option>'+
                '                            <option value="20">助理经理</option>'+
                '                            <option value="-6">总监以上</option>'+
                '                            <option value="-7">副总监以上</option>'+
                '                            <option value="-17">高级经理以上</option>'+
                '                            <option value="-19">经理以上</option>'+
                '                        </select>';

            var checkAll=this.settings.multiSelect ? '<a class="btn btn-default check-all">全选 <span>&gt;&gt;</span></a>' : "" ; 
            var orgTemplate = ''+
                    '<div class="c-orgselect-container">'+
                    '    <div class="col-org">'+
                    '        <div class="row-query">'+
                    '            <input class="query-typeahead form-control"  placeholder="请输入关键字" />'+
                    '        </div>'+
                    '        <div class="row-header">'+
                    '            <span class="title">部门</span>'+
                    '            <span class="number-container">(<span class="number">0</span>)</span>'+
                    '        </div>'+
                    '        <div class="row-tree">'+
                    '            <div class="contact-tree"></div>'+
                    '            <div class="org-tree"></div>'+
                    '        </div>'+
                    '    </div>'+
                    '    <div class="col-leftselect">'+
                    '        <div class="row-header">'+
                    '            <span class="title">待选</span>'+
                    '            <span class="number-container">(<span class="number">0</span>)</span>'+

                    '        </div>'+
                    '        <div class="row-filter">'+
                   /* '            <div class="joblist-container pull-left"><select class="joblist"></select></div>'+*/
                    '            <div class="joblist-container pull-left">'+htmlTitleFilter+'</div>'+
                    '           <div class="clklist pull-left">'+
                    '               <input type="checkbox" class="c-inner" checked="checked" /><label>所内</label>'+
                    '               <input type="checkbox" class="c-outer" checked="checked" /><label>所外</label>'+
                    '           </div>'+
                    '        </div>'+
                    '        <div class="row-list">'+
                    '            <div class="person-list"></div>'+
                    '        </div>'+
                    '    </div>'+
                    '    <div class="col-buttons">'+
                             checkAll+
                    '        <a class="btn btn-default check-in">添加<span>&gt;</span></a>'+
                    '        <a class="btn btn-default check-out"><span>&lt;</span>移除</a>'+
                    '        <a class="btn btn-default check-clear"><span>&lt;&lt;</span>清除</a>'+
                    '    </div>'+
                    '    <div class="col-rightselect">'+
                    '        <div class="row-header">'+
                    '            <span class="title">已选</span>'+
                    '            <span class="number-container">(<span class="number">0</span>)</span>'+
                    '        </div>'+
                    '        <div class="row-contact">'+
                    '            <div class="btn-control">'+
                    '                <a class="btn btn-default set-contact">设为常用联系组</a>'+
                    '            </div>'+
                    '           <div class="content-control">'+
                    '                <input type="text" class="contact-name" />'+
                    '                <button type="button" class="btn btn-default save-btn">保存</button>'+
                    '            </div>'+
                    '        </div>'+
                    '        <div class="row-list">'+
                    '            <div class="person-list"></div>'+
                    '        </div>'+
                    '    </div>'+
                    '    <div class="clearfix"></div>'+
                    '</div>';

            var htmlTemplate = ''+
                '<div class="modal-content c-orgselect-modal">'+
                '    <div class="modal-header">'+
                '        <button type="button" class="close"><span aria-hidden="true">&times;</span></button>'+
                '        <h4 class="modal-title">' + this.settings.titleName + '</h4>'+
                '    </div>'+
                '    <div class="modal-body">'+
                orgTemplate +
                '    </div>'+
                '    <div class="modal-footer"> '+
                '        <button type="button" class="btn btn-default cancel">取消</button>'+
                '        <button type="button" class="btn btn-primary confirm">确定</button>'+ 
                '    </div>'+
                '</div>';
            this.container = $(htmlTemplate);
        },

        initElements: function(){
            var context = this;
            var dialogId = '#c-orgselect-dialog-'+this.settings._currentPluginId;
            var pop= new s.ui.controls.Modal(this.userselect.getButton(),{
                modalId: dialogId, 
                modalClass: 'c-orgselect-modal',
                content: this.container,
                backdrop: 'static'
            });

            this.elements = {
                colOrgQueryTypeahead: $('.col-org .query-typeahead',this.container),
                colOrgNumber: $('.col-org .number',this.container),
                colOrgContactTree: $('.col-org .contact-tree',this.container),
                colOrgOrgTree: $('.col-org .org-tree',this.container),

                colLeftSelectNumber: $('.col-leftselect .number',this.container),
                colLeftSelectJoblist: $('.col-leftselect .joblist',this.container),
                colLeftSelectInner: $('.col-leftselect .c-inner',this.container),
                colLeftSelectOuter: $('.col-leftselect .c-outer',this.container),
                colLeftSelectLabel: $('.col-leftselect label',this.container),
                colLeftClklist: $('.col-leftselect .clklist',this.container),

                colLeftSelectPersonList: $('.col-leftselect .person-list',this.container),
                colLeftSelectRowFilter: $('.col-leftselect .row-filter',this.container),

                colButtonsCheckAll: $('.col-buttons .check-all',this.container),
                colButtonsCheckIn: $('.col-buttons .check-in',this.container),
                colButtonsCheckOut: $('.col-buttons .check-out',this.container),
                colButtonsCheckClear: $('.col-buttons .check-clear',this.container),

                colRightSelectNumber: $('.col-rightselect .number',this.container),
                colRightSelectSetContact: $('.col-rightselect .set-contact',this.container),
                colRightSelectContactName: $('.col-rightselect .contact-name',this.container),
                colRightSelectSaveBtn: $('.col-rightselect .save-btn',this.container),
                colRightSelectPersonList: $('.col-rightselect .person-list',this.container),
                colRightSelectRowContact: $('.col-rightselect .row-contact',this.container),

                confirm: $('.modal-footer .confirm', this.container),
                cancel: $('.modal-footer .cancel', this.container)
            };

            this.elements.colButtonsCheckIn.css('visibility','hidden');
            this.elements.colButtonsCheckOut.css('visibility','hidden');

            if(!this.settings.hasContact){
                this.elements.colOrgContactTree.remove();
                this.elements.colRightSelectRowContact.remove();
                this.container.addClass('none-contact-container');
            }else if(!this.settings.hasAddContact){
                this.elements.colRightSelectRowContact.remove();
                this.container.addClass('none-contact-container');   
            }

            var flag = true;
            if(!this.settings.hasJobFilter && !this.settings.hasOuterFilter){
                this.elements.colLeftSelectRowFilter.remove();
                this.container.addClass('none-filter-container');
                flag = false;
            }

            if(!this.settings.hasJobFilter && flag){
                this.elements.colLeftSelectJoblist.remove();
            }

            if(!this.settings.hasOuterFilter && flag){
                this.elements.colLeftClklist.remove();
                // this.elements.colLeftSelectLabel.remove();
                // this.elements.colLeftSelectInner.remove();
                // this.elements.colLeftSelectOuter.remove();
            }

            var typeahead= new s.ui.controls.Typeahead(this.elements.colOrgQueryTypeahead, {
                map: '{"text":"name","returnValue":"json"}',
                template:this.settings.typeaheadMethod.template,
                onChanged:this.settings.typeaheadMethod.onChanged|| function(e,value){
                    this.query=this.query;
                    this.matcherReg=new RegExp(value,'i');
                   // context.changed.apply(context,arguments);
                },
                matcher:this.settings.typeaheadMethod.matcher|| function(item){
                    var matcher = this.matcherReg||new RegExp(this.query,'i');
                    var result = matcher.test(item.name) || matcher.test(item.loginName) || matcher.test(item.spell) || matcher.test(item.spellFirst);
                    return result===true ? 1 : -1;
                },

                onSelected:this.settings.typeaheadMethod.onSelected|| function(e,value) {
                    var item = this.getValue(value);
                    context.addUsers([item]);
                    context.checkLeft(context.getRightUsers());
                },

                mouseover:this.settings.typeaheadMethod.mouseover|| function(e,value) {
                    var item = this.getValue(value);
                    if(item.title==="") return;
                    $(e.currentTarget).find('a').html(item.realName+'('+item.title+')');
                },

                mouseout:this.settings.typeaheadMethod.mouseout|| function(e,value){
                    var item = this.getValue(value);
                    $(e.currentTarget).find('a').html(item.name);
                }
            }); 
            
            var contactTree= null, setContactTypeahead = null;
            if(this.settings.hasContact){
                contactTree=new s.ui.controls.Tree(this.elements.colOrgContactTree, {
                    levels: 1,
                    multiSelect: false,
                    onNodeSelected: function(e, data){
                        context.onNodeSelected(e, data,'contact');
                    },
                    renderNode:this.settings.renderTreeNode
                });

                if(this.settings.hasAddContact){
                    setContactTypeahead = new s.ui.controls.Typeahead(this.elements.colRightSelectSetContact, {
                        map: '{"text":"name","returnValue":"json"}',
                        matcher:function(item){
                            if(!this.query){
                                if(item.name.toLowerCase().indexOf(this.query.toLowerCase()) > -1){
                                    return 1;
                                }else if(item.loginName.toLowerCase().indexOf(this.query.toLowerCase()) > -1){
                                    return 1;
                                }else if(item.spell.toLowerCase().indexOf(this.query.toLowerCase()) > -1){
                                    return 1;
                                }else if(item.spellFirst.toLowerCase().indexOf(this.query.toLowerCase()) > -1){
                                    return 1;
                                }
                                return -1;
                            }
                            else{
                                return 1;
                            }
                        },
                        onSelected: function(e,value) {
                            var item = this.getValue(value);
                            //
                            var right=context.controls.rightselect.elements.right;
                            if(right.find('option[value="'+item.id+'"]').length === 0){
                                right.append('<option value="'+item.id+'">'+item.name+'</option>');
                                context._updateSelectedCount();
                            }
                        }
                    });
                } 
            }

            var orgTree=new s.ui.controls.Tree(this.elements.colOrgOrgTree, {
                levels: 1,
                multiSelect: false,
                onNodeSelected: function(e, data){
                    context.onNodeSelected(e, data,'org');
                },
                renderNode:this.settings.renderTreeNode
            });

            var jobFilter = null; 
            if(this.settings.hasJobFilter){
                jobFilter = new s.ui.controls.Selectex(this.elements.colLeftSelectJoblist, {
                    selectFirst:true,
                    onSelected:function(node,e){
                        context.states.filterStates=e.value;
                        context.checkFilter();
                    }
                });
                             
               // jobFilter.updateData(jobFilterData);
              //jobFilter.render();
               jobFilter.setValue("-100");
            }

            var leftSelectPersonList = new s.ui.controls.Selectlist(this.elements.colLeftSelectPersonList,{
                map:{text:'name',value:'id'},
                mouseover: function(e,item) {
                    if(item.title==="") return;
                    $(e.target).html(item.realName+'('+item.title+')');
                },

                mouseout: function(e,item){
                    $(e.target).html(item.name);
                },

                click: function(e,item){
                    var target=$(e.target).is('li') ?  $(e.target)  : $(e.target).parent('li') ;
                    if(target.hasClass('c-selectlist-checked')){
                        context.removeUsers([item]);
                        context.controls.rightSelectPersonList.clearSelectedItems();
                        context.checkLeft(context.getRightUsers());
                    }else{
                        context.addUsers([item]);
                        context.controls.leftSelectPersonList.clearSelectedItems();
                        context.checkLeft(context.getRightUsers());
                    }
                }
            });

            var rightSelectPersonList = new s.ui.controls.Selectlist(this.elements.colRightSelectPersonList,{
                map:{text:'name',value:'id'},
                mouseover: function(e,item) {
                    if(item.title==="") return;
                    $(e.target).html(item.realName+'('+item.title+')');
                },

                mouseout: function(e,item){
                    $(e.target).html(item.name);
                },

                click: function(e,item){
                    context.removeUsers([item]);
                    context.controls.rightSelectPersonList.clearSelectedItems();
                    context.checkLeft(context.getRightUsers());
                }
            });

            this.controls={
                pop: pop,
                typeahead: typeahead,
                contactTree: contactTree,
                orgTree: orgTree,
                jobFilter: jobFilter,
                leftSelectPersonList: leftSelectPersonList,
                rightSelectPersonList: rightSelectPersonList
            };
        },

        bindEvents: function(){
            this.elements.confirm.on('click',$.proxy(this.onConfirm,this));

            if(this.settings.hasOuterFilter){
                this.elements.colLeftSelectInner.on('click',$.proxy(this.checkFilter,this));
                this.elements.colLeftSelectOuter.on('click',$.proxy(this.checkFilter,this));
            }

            this.elements.colButtonsCheckAll.on('click',$.proxy(this.checkAll,this));
            this.elements.colButtonsCheckIn.on('click',$.proxy(this.checkIn,this));
            this.elements.colButtonsCheckOut.on('click',$.proxy(this.checkOut,this));
            this.elements.colButtonsCheckClear.on('click',$.proxy(this.checkClear,this));

            this.elements.confirm.on('onConfirm', $.proxy(this.settings.onConfirm, this));
        },

        bindEventsInterface: function () {},
        //点击部门节点
        onNodeSelected: function(e, node, type){
            var users = null;
            if(type === 'org'){
                users = this.orgUsersMap[node.id];
                this.selectOrgId = node.id;
                if(this.selectContactId){
                    this.controls.contactTree.unselectNodeByIds(this.selectContactId);
                }
                this.selectContactId = null;
            }
            else{
                users = this.contactUsersMap[node.id];
                this.selectContactId = node.id;
                if(this.selectOrgId){
                    this.controls.orgTree.unselectNodeByIds(this.selectOrgId);
                }
                this.selectOrgId = null;
            }
            if(this.settings.hasJobFilter){
                var filterUsers=this.renderLeft(users);
                this.states.filterLeftUsers=filterUsers;
                this.states.leftUsers = users;
                this.controls.leftSelectPersonList.setData(filterUsers);
                this.elements.colLeftSelectNumber.text(filterUsers.length);
                this.checkLeft(this.getRightUsers());
            }
            else{
                this.states.leftUsers = users;
                this.controls.leftSelectPersonList.setData(users);
                this.elements.colLeftSelectNumber.text(users.length);
                this.checkLeft(this.getRightUsers());
            }
        },

        checkAll: function(e){
            if(this.settings.hasJobFilter){
                this.addUsers(this.states.filterLeftUsers);
            }else{
                this.addUsers(this.states.leftUsers);
            }
            this.controls.leftSelectPersonList.clearSelectedItems();
            this.checkLeft(this.getRightUsers());
        },

        checkIn: function(e){
            var users = this.controls.leftSelectPersonList.getSelectedItems();
            this.addUsers(users);
            this.controls.leftSelectPersonList.clearSelectedItems();
            this.checkLeft(this.getRightUsers());
        },

        checkOut: function(){
            var users = this.controls.rightSelectPersonList.getSelectedItems();
            this.removeUsers(users);
            this.controls.leftSelectPersonList.clearSelectedItems();
            this.checkLeft(this.getRightUsers());
        },

        checkClear: function(){
            var users = this.getRightUsers();
            this.removeUsers(users);
            this.controls.leftSelectPersonList.clearSelectedItems();
            this.checkLeft(this.getRightUsers());
        },

        addUsers: function(users){
            var addItem = [];
            var user ;
            
           if(!this.settings.multiSelect){
                    if(Array.isArray(users)) 
                        user=users[users.length-1];
                    else{
                        user=users;
                    }
                    if(!this.states.rightUserMap[user.id]){
                        this.states.rightUserMap={};
                        addItem.push(user);
                        this.states.rightUserMap[user.id] = user;
                    }
                
                this.controls.rightSelectPersonList. coverItem(addItem);
                this.elements.colRightSelectNumber.text(1);
           }else{
                 for (var i = 0; i < users.length; i++) {
                         user = users[i];
                    if(!this.states.rightUserMap[user.id]){
                        addItem.push(user);
                        this.states.rightUserMap[user.id] = user;
                    }
                }
                this.controls.rightSelectPersonList.addItem(addItem);
                var num = Object.keys(this.states.rightUserMap).length;
                this.elements.colRightSelectNumber.text(num);
           }
        },

        removeUsers: function(users){
            for (var i = 0; i < users.length; i++) {
                var user = users[i];
                delete this.states.rightUserMap[user.id];
            }
            this.controls.rightSelectPersonList.deleteItem(users); 
            var num = Object.keys(this.states.rightUserMap).length;
            this.elements.colRightSelectNumber.text(num);
        },

        getRightUsers: function(){
            var users = [];
            for(var key in this.states.rightUserMap){
                users.push(this.states.rightUserMap[key]);
            }
            return users;
        },

        setRightUsers: function(users){
            this.states.rightUserMap = {};
            for (var i = 0; i < users.length; i++) {
                var user = users[i];
                this.states.rightUserMap[user.id] = user;
            }
            this.controls.rightSelectPersonList.setData(users);
        },

        //如果左边的用户已选，则需将用户勾选
        checkLeft: function(users){
            this.controls.leftSelectPersonList.clearCheckedItems();
            this.controls.leftSelectPersonList.setCheckedItems(users);
        },

        onConfirm: function(e){
            this._renderValue();
            var users = this.getRightUsers();
            if(this.settings.confirmHide){
                this.controls.pop.hide();
            }else{
                if(users.length>0){
                    this.controls.pop.hide();
                }
            }
        },
        render: function(){
            this._renderTypeahead();
            this._renderOrgTree();
            // this._renderValue();
        },
        _renderTypeahead: function() {

            this.controls.typeahead.setSource(this.data.users);
        },
        _renderOrgTree: function(){
            //默认展开第一级
            this.controls.orgTree.setValue(this.data.orgs,2);
        },
        _renderContactTree: function(){
            this.controls.contactTree.setValue(this.data.contacts,2);
        },

        setSelectValue: function(data){
            this.setRightUsers(data);
            this.checkLeft(data);
            this.elements.colRightSelectNumber.text(data.length);
        },
        checkFilter:function(){
            var users=this.states.leftUsers||[];
            if(users.length<=0) return ;
            var filterUsers=this.renderLeft(users);
            this.states.filterLeftUsers=filterUsers;
            this.controls.leftSelectPersonList.setData(filterUsers);
            this.elements.colLeftSelectNumber.text(filterUsers.length);
            this.checkLeft(this.getRightUsers());
        },
        renderLeft:function(users){
            var datas=[];
            var title=this.states.filterStates;
            var showInner=this.elements.colLeftSelectInner.is(':checked');
            var showOuter=this.elements.colLeftSelectOuter.is(':checked');
            for(var i=0; i<users.length; i++){
                var user=users[i];
                // 右边有，就不再插入到左边
                // left.append('<option value="'+user.id+'">'+user.name+'</option>');
                if(
                    ((title == "-100") || (title > 0 && user.titleSortNo == title) || (title <= 0 && user.titleSortNo <= -title))// -100: 所有 >0:指定级别 <=0: 大于某级别
                    && ((showInner && user.innerOuter == '内部') || (showOuter && user.innerOuter == '外部'))
                   
                ){
                    datas.push(user);
                }
            }
        // console.log(datas);
           return datas;
        },
        _renderValue: function(){
            var users = this.getRightUsers();
            this.userselect.setData(users);
            this.settings.onConfirm.apply(this,[this.tranferInnerUsers(users)]);
        },
        show: function () {
            this.container.show();
        },
        hide: function(){
            this.container.hide();
            //alert(this.inputElements.original.val());
        }
    });

    this.Orgselect = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,
        // data:{},

        // 构造函数
        init:function(element, options){
            this.element = $(element);
            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);
            this.settings._currentPluginId = _currentPluginId;

            // 初始化 html DOM 元素
            this.initElements();
        },
        updateDataUsers: function(data){
            var users = this.modal.parseDataUsers(data);
            this.userselect.updateData(users);
            this.modal.updateDataUsers(users);
        },
        updateDataOrgs: function(data){
            this.modal.updateDataOrgs(data);
        },
        updateDataContacts: function(data){
            this.modal.updateDataContacts(data);
        },
        initElements:function () {
            var context = this;
            this.elements = {
                original: this.element
            };
            this.userselect = new s.ui.controls.UserSelect(this.elements.original,{
                map:'{"text":"name","returnValue":"json"}',
                typeaheadMethod:this.settings.typeaheadMethod,
                ishide:this.settings.inputHide,
                onblured: function(value){
                    context.settings.onConfirm.apply(context,[context.modal.tranferInnerUsers(value)]);
                }
            });

            this.modal= new OrgselectModal(this.userselect, this.settings);

            var context = this;
            this.userselect.getButton().on('click',function(e){
                context.modal.setSelectValue(context.userselect.getValue());
            });
        },
        setValue: function(value){
            var users = this.modal.parseDataSetValueUsers(value);
            this.userselect.setData(users);
        },
        refresh: function(){
            var context= this;
            // $.when(this.d.promise())
            //  .done(function(){
            //     context.modal.refresh();
            //  });
            // this.modal.refresh();
        },
        enable: function(){
            this.element.prop('disabled', false);
            this.userselect.enable();
            // this.elements.view.prop('disabled',false);
            // this.elements.button.prop('disabled',false);
            this.disabled=false;
        },
        disable: function(){
            this.element.prop('disabled', true);
            this.userselect.disable();
            // this.elements.view.prop('disabled', true);
            // this.elements.button.prop('disabled', true);
            this.disabled=true;
        },
        onOpen:function(btn){
             this.userselect.getButton().click();
        },
        disableNodeByIds:function(ids){
             this.modal.controls.orgTree.disableNodeByIds(ids);
        },
        // API
        getApi:function(){
            var context = this;
            var funs = {
                onOpen: 'onOpen',
                disableNodeByIds:'disableNodeByIds'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        },
        destory: function(){}
    });

});

/* orgselect javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'orgselect';
    var pluginClass = s.ui.controls.Orgselect;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destory();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
/**
 * bootstrap-paginator.js v0.5
 * --
 * Copyright 2013 Yun Lai <lyonlai1984@gmail.com>
 * --
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


 Jx().package('s.ui.controls', function(J){
    // 严格模式
    'use strict';

    var _crrentPluginId = 0;
    var defaults = {
        // 选项
        totalSize: 0,       // 记录数
        pageSize: 10,       // 每页记录数
        pageIndex: 0,       // 当前页
        pageButtons: 5,     // 分页按钮数 必须为奇数 3, 5, 7 ,9 ....
        // totalPages: 1,
        // containerClass: '',
        // size: 'normal',
        // alignment: 'left',
        // listContainerClass: '',
        // 覆写 类方法
        // parseData: undefined,
        onPageIndexChange: undefined,
        //simple | intricacy | normal
        pageType:'intricacy'
    };
    var attributeMap = {
        // fooOption: 'foo-option'
        totalSize: 'total-size',
        pageSize: 'page-size',
        pageIndex: 'page-index',
        pageButtons: 'page-buttons',
        pageType:'page-type'
    };

    this.Paginator = new J.Class({extend: s.ui.BaseControl},{
        defaults: defaults,
        attributeMap: attributeMap,

        settings: {},
        value: '',
        data: {},
        templates: {},

        // 构造函数
        init: function(element, options){
            this.element= $(element);

            if (!this.element.is('ul')) {
                throw 'in Bootstrap version 3 the pagination root item must be an ul element.'
            }

            // // 防止多次初始化
            // if (this.isInitialized()) { 
            //     return this.getRef(); 
            // }
            // this.initialize(element);

            // this.options = $.extend({}, (this.options || $.fn.bootstrapPaginator.defaults), options);
            this.initSettings(options);

            this.inputPage = 1;
            // var id = this.element.attr('id');
            // this.value= this.element.val();
            // parseInt(this.settings.pageSize, 10)

            // this.settings.pageIndex = 0;
            // this.lastPageIndex = 0;
            this.updateOptions(this.settings);


            // this.buildHtml();
            //this.initElements();

            // var context= this;
            // $.when(this.getData())
            //  .done(function(){
            //     context.render();
            //  });

             // this.bindEvents();
             this.bindEventsInterface();
        },

        // // 模板模式 方法
        // getData: function(){
        //     var d = $.Deferred();

        //     if (this.settings.data) {
        //         this.data = $.extend(true, [], this.settings.data);
        //         delete this.settings.data;

        //         d.resolve();

        //         return d.promise();
        //     }

        //     var context = this;
        //     $.ajax({
        //         dataType: 'json',
        //         url: this.settings.dataUrl,
        //         data: {},
        //         success: function(data){
        //             context.data= $.extend(true, [], data);

        //             d.resolve();
        //         },
        //         error: function(xmlHttpRequest, status, error){
        //             alert('控件id：' + context.element.attr('id')+'，ajax获取数据失败!');

        //             d.resolve();
        //         }
        //     });

        //     return d.promise();
        // },
        // parseData: function(data){
        //     var innerData= $.extend(true, [], data);
        //     return innerData;
        // },
        buildHtml: function(){},
        initElements: function(){
            // var context= this;
        },
        // transferAttributes: function(){},

        // bindEvents: function(){
        //     var context= this;
        //     var element= this.element;
        //     // element.on('click', $.proxy(this.onFooClick, this));
        //     this.element.off('page-clicked');
        //     this.element.off('page-changed');// unload the events for the element
        //     if (typeof (this.settings.onPageClicked) === 'function') {
        //         this.element.bind('page-clicked', this.settings.onPageClicked);
        //     }
        //     if (typeof (this.settings.onPageChanged) === 'function') {
        //         this.element.on('page-changed', this.settings.onPageChanged);
        //     }
        //     // this.element.bind('page-clicked', this.onPageClicked);
        //     this.element.on('page-clicked', $.proxy(this.onPageClicked, this));
        // },
        bindEventsInterface: function(){
            var context= this;
            var element= this.element;
            // if(this.settings.onFooSelected){
            //     element.on('page.on.change', $.proxy(this.settings.onFooSelected, this));
            // }
            if(this.settings.onPageIndexChange){
                element.on('paginator.on.pageindexchange', $.proxy(this.settings.onPageIndexChange, this));
            }

        },

        updateOptions: function (options) {
            this.settings = $.extend(true, {}, this.settings, options);
             

            // 总页数
            if(this.settings.totalSize % this.settings.pageSize === 0){
                this.totalPages= this.settings.totalSize / this.settings.pageSize;
            }
            else{
                this.totalPages= Math.ceil(this.settings.totalSize / this.settings.pageSize);
            }
            // 最少有一页
            this.totalPages= this.totalPages < 1 ? 1 : this.totalPages;
            //当只有一页时不显示翻页
            if(this.totalPages == 1){
                this.element.hide();
            }
            else
            {
                this.element.css('display','inline-block');
            }

            // 分页按钮数 必须为奇数 3, 5, 7 ,9 ....
            this.settings.pageButtons= this.settings.pageButtons%2 === 0 ? this.settings.pageButtons+1 : this.settings.pageButtons;     

            this.refresh();

           
        },
        jumpTo: function (pageIndex) {
            this.settings.pageIndex = pageIndex < 0 ? 0 : pageIndex > this.totalPages-1 ? this.totalPages-1 : pageIndex;
            this.refresh();
        },
        // render: function () {},
        _getPageButtons: function () {

            var totalSize = this.settings.totalSize;
            var totalPages= this.totalPages;

            var pageIndexStart= 0;
            var halfItemsCount= Math.floor(this.settings.pageButtons / 2);
            pageIndexStart= this.settings.pageIndex - halfItemsCount;
            // 上限
            pageIndexStart= pageIndexStart < this.totalPages - this.settings.pageButtons ? pageIndexStart : this.totalPages - this.settings.pageButtons;
            // 下限
            pageIndexStart= pageIndexStart < 0 ? 0 : pageIndexStart;
            
            var output = [];
            for (var i = pageIndexStart, counter = 0; counter < this.settings.pageButtons && i < totalPages; i = i + 1, counter = counter + 1) {//fill the pages
                output.push(i);
            }

            // 首页
            if(pageIndexStart>0){
                output.first = 0;
            }

            // 上一段
            if(totalPages >= this.settings.pageButtons+2 && this.settings.pageIndex-halfItemsCount > 1){
                output.prevSection=  this.settings.pageIndex-halfItemsCount-1;
            }

            // 上一页
            if (this.settings.pageIndex > 0) {
                output.prev= this.settings.pageIndex - 1;
            } else {
                output.prev= 0;
            }

            // 下一页
            if (this.settings.pageIndex < totalPages-1) {
                output.next= this.settings.pageIndex + 1;
            } else {
                output.next= totalPages-1;
            }

            // 下一段
            if(totalPages > this.settings.pageButtons && this.settings.pageIndex+halfItemsCount < this.totalPages-2){
                output.nextSection= this.settings.pageIndex+halfItemsCount+1;
            }

            // 尾页
            if(pageIndexStart<totalPages-this.settings.pageButtons){
                output.last= totalPages-1;
            }

            // output.current= this.settings.pageIndex;//mark the current page.
            // output.total= totalPages;
            // output.pageButtons = this.settings.pageButtons;

            return output;
        },

        // 事件
        // onFooSelected: undefined,
        // onFooChange: function(e, data){}
        // onPageClicked: null,
        // onPageChanged: null,
        onPageIndexChange: function(e){
            var pageIndex= parseInt($(e.target).attr('data-pi'), 10);
            if(this.settings.pageIndex == pageIndex){
                return;
            }

            this.element.trigger('paginator.on.pageindexchange', pageIndex);
            this.jumpTo(pageIndex);
        },

        // 事件处理
        // onFooClick: function(e, data){
        //     ;
        // },

        // API
        refresh: function(){
           
            //fetch the container class and add them to the container
            var pages = this._getPageButtons();
            var listContainer = this.element;

         

            // add class
            // this.element.prop('class', '');
            this.element.addClass('pagination');     // pagination-lg / pagination / pagination-sm
            // this.element.addClass(containerClass);

            this.element.empty();

            var htmlItems= '';
            var prev='';
            if(this.settings.pageIndex === 0)
            	prev= '<li class="disabled" data-show="prev"><a data-pi="'+pages.prev+'">&lt;上一页</a><li>';
            else
            	prev= '<li data-show="prev"><a data-pi="'+pages.prev+'">&lt;上一页</a><li>';
            htmlItems+= prev;
            if(this.settings.pageType == 'intricacy'){
                if(typeof pages.first !== 'undefined'){
                    var first = '<li><a data-pi="0">1</a><li>';
                    htmlItems+= first;
                }
                
                if(typeof pages.prevSection !== 'undefined'){
                   var prevSection= '<li class="ellipsed"><a data-pi="'+pages.prevSection+'">...</a><li>';
                   htmlItems+= prevSection;
                }

                for (var i = 0; i < pages.length; i = i + 1) {
                    var itemClass = (pages[i] === this.settings.pageIndex) ? ' class="active"' : '';
                    var item = '<li'+itemClass+'><a data-pi="'+pages[i]+'">'+(pages[i]+1)+'</a><li>';
                    htmlItems+= item;
                }

                if(typeof pages.nextSection !== 'undefined'){
                    var nextSection= '<li class="ellipsed"><a data-pi="'+pages.nextSection+'">...</a><li>';
                    htmlItems+= nextSection;
                   
                }

                if(typeof pages.last !== 'undefined'){
                    var last = '<li><a data-pi="'+pages.last+'">'+(pages.last+1)+'</a><li>'; 
                    htmlItems+= last;
                }

                var next='';
                if(this.settings.pageIndex === (this.totalPages - 1) )
                    next= '<li class="disabled" data-show="next"><a data-pi="'+pages.next+'">下一页&gt;</a><li>';
                else
                    next= '<li data-show="next"><a data-pi="'+pages.next+'">下一页&gt;</a><li>';
                htmlItems+= next;
                listContainer.append(htmlItems);

                $('li a[data-pi="0"]', listContainer).addClass("first");
                $('li a[data-pi="'+ (this.totalPages-1) +'"]', listContainer).addClass("last");



                var go = ''+
                        '<li class="pagination-go intricacy"><div>'+
                        '   <label class="lbl">跳转到：</label>'+
                        '   <div class="input-group">' +
                        '     <input type="text" class="page-cur form-control">'+
                        //'   <span class="separateline">/</span>'+
                       // '   <span class="page-all">'+this.totalPages+'</span>'+
                        '     <span class="page-go-btn input-group-addon">Go!</span>'+
                        '</div></div></li>';


                listContainer.append(go);
            }
            else if(this.settings.pageType == 'simple'){
                 listContainer.append(htmlItems);
                 var go = ''+
                        '<li class="pagination-go simple"><div>'+
                        //'   <label class="lbl">跳转到：</label>'+
                       // '   <div class="input-group">' +
                        '     <input type="text" class="page-cur form-control">'+
                        '   <span class="separateline">/</span>'+
                        '   <span class="page-all">'+this.totalPages+'</span>'+
                        //'     <span class="page-go-btn input-group-addon">Go!</span>'+
                        '</div></li>';
                listContainer.append(go);
                var next='';
                if(this.settings.pageIndex === (this.totalPages - 1) )
                    next= '<li class="disabled" data-show="next"><a data-pi="'+pages.next+'">下一页&gt;</a><li>';
                else
                    next= '<li data-show="next"><a data-pi="'+pages.next+'">下一页&gt;</a><li>';
                //htmlItems+= next;
                listContainer.append(next);
            }

            //.val this.inputPage
			$('input[type=text]', listContainer).val(this.settings.pageIndex + 1);

            // var option = {
            //     mask:'K',
            //     translation: {
            //      'K':{
            //           pattern: new RegExp('^[1-9]\d*$')
            //         }
            //     },
            //     clearifnotmatch: true
            // };
            var option = {
                mask:'99999'
            };
            // if(!this.inputmask)
            //    this.inputmask = new s.ui.controls.Inputmask($('.page-cur', listContainer),option);
            $('.page-cur', listContainer).mask('0000000');
            var context = this;
            // $('.page-go-btn', listContainer).on('click',function(){
            // 	var pageNumber = $('.page-cur',listContainer).val();
            //     if(typeof pageNumber === 'undefined' || pageNumber === '' || isNaN(parseInt(pageNumber))){
            //         pageNumber = 1;
            //     }
            //     pageNumber = parseInt(pageNumber);
            //     if(pageNumber < 1){
            //         pageNumber = 1;
            //     }else if(pageNumber > context.totalPages){
            //         pageNumber = context.totalPages;
            //     }
            //     $('.page-cur',listContainer).val(pageNumber);
            //     pageNumber = parseInt(pageNumber) - 1 ;
            //     context.inputPage = pageNumber + 1;
            //     context.element.trigger('paginator.on.pageindexchange', pageNumber);
            //     context.jumpTo(pageNumber);
            // });

            $('.page-go-btn', listContainer).on('click', $.proxy(this.onPageGo, this));
            $('.page-cur',listContainer).on('keypress', function(e){
                // 回车键事件  
               if(e.which == 13) {  
                    context.onPageGo();
                    return false;
               }  
            });
            $('a', listContainer).on('click', $.proxy(this.onPageIndexChange, this));


        },
        onPageGo:function(){
            var listContainer = this.element;
            var context = this;
            var pageNumber = $('.page-cur',listContainer).val();
                if(typeof pageNumber === 'undefined' || pageNumber === '' || isNaN(parseInt(pageNumber))){
                    pageNumber = 1;
                }
                pageNumber = parseInt(pageNumber);
                if(pageNumber < 1){
                    pageNumber = 1;
                }else if(pageNumber > context.totalPages){
                    pageNumber = context.totalPages;
                }
                $('.page-cur',listContainer).val(pageNumber);
                pageNumber = parseInt(pageNumber) - 1 ;
                context.inputPage = pageNumber + 1;
                context.element.trigger('paginator.on.pageindexchange', pageNumber);
                context.jumpTo(pageNumber);
        },
        checkNumber: function(input){
        	var rule = /^[1-9]+[0-9]*]*$/;
        	var result = -1;
        	if(isNaN(parseInt(input)))
        		return result;
        	input = parseInt(input);
        	if(!rule.test(input))
        		return result;
        	result = 1;
        	return result;
        },

        enable: function(){},
        disable: function(){},
        destroy: function () {
            this.element.off('page-clicked');
            this.element.off('page-changed');
            // this.element.removeData('bootstrapPaginator');
            this.element.empty();
        }
    });
});





/* paginator javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'paginator';
    var pluginClass = s.ui.controls.Paginator;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);
Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 勾选框是否显示
    var enumShowCheckbox= {
        all: 'all',     // 0 所有显示
        none: 'none',   // 1 不显示(默认)
        leaf: 'leaf'    // 2 叶子节点显示
    };

    var enumCheckedState= {
        checked: 'checked',
        unchecked: 'unchecked',
        partOf: 'partOf'
    };

    // 节点是否可选
    var enumSelectMode= {
        all: 'all',     // 0 所有可选中(默认)
        leaf: 'leaf'    // 1 仅叶子可选中
    };

    // 全局变量、函数、对象
    var emptyFun= function(){};
    var _currentPluginId = 0;
    var defaults = {
        levels: 1,
        dataUrl: '',
        expandIcon: 'glyphicon glyphicon-plus',
        collapseIcon: 'glyphicon glyphicon-minus',
        emptyIcon: 'glyphicon',
        nodeIcon: 'empty',
        selectedIcon: '',
        checkedIcon: 'glyphicon glyphicon-check',
        checkedPartOfIcon: 'glyphicon glyphicon-check partOfchecked',
        uncheckedIcon: 'glyphicon glyphicon-unchecked',
        clsNodeText: 'nodeValue',
        clsAppendHtml: 'appendHtml',
        render: undefined,
        renderNode:undefined,
        clsBadge: 'badge',

        clsItemContent: 'item-content',
        clsSwitch:'switch',
        clsNodeUnSelect: 'node-unselectable',
        clsNodeSelected: 'node-selected',

        clsNodeDisabled: 'node-disabled',

        enableLinks: false,
        enableTitle: false,
        showIcon: true,
        showTags: false,
        multiSelect: true,

        // nodeOptions
        silent: false,
        ignoreChildren: false,

        expandOne:true,

        dnd: 'none', //拖拽属性   'drag/drop/dnd/none'

        // extends
        showCheckbox: enumShowCheckbox.none,
        checkRecursive: false,
        selectMode: enumSelectMode.all,
        appendHtml:'',
        parseAppendHtml: undefined,

        // states
        selectedNodeIds:'',
        checkedNodeIds:'',
        disabledNodeIds:'',

        // Event handlers
        // 节点双击
        onNodeDbclick: emptyFun,
        // 节点右击
        onNodeRightclick: emptyFun,
        // 节点勾选
        onNodeChecked: emptyFun,
        // 节点收起
        onNodeCollapsed: emptyFun,
        // 节点禁用
        onNodeDisabled: emptyFun,
        // 节点启用
        onNodeEnabled: emptyFun,
        // 节点展开
        onNodeExpanded: emptyFun,
        // 节点选中
        onNodeSelected: emptyFun,
        // 节点取消勾选
        onNodeUnchecked: emptyFun,
        // 节点取消选中
        onNodeUnselected: emptyFun,
        // 搜索完成
        onSearchComplete: emptyFun,
        // 搜索结果清除
        onSearchCleared: emptyFun
    };
    var attributeMap = {
        expandOne: 'expand-one',
        ignoreChildren: 'ignore-children',
        expandIcon: 'expand-icon',
        collapseIcon: 'collapse-icon',

        showTags: 'show-tags',
        levels: 'levels',
        enableTitle: 'enable-title',
        dataUrl: 'data-url',

        showCheckbox:'show-checkbox',
        checkRecursive: 'check-recursive',
        selectMode:'select-mode',
        appendHtml:'append-html',

        multiSelect:'multi-select',        
        selectedNodeIds:'selected-node-ids',
        checkedNodeIds:'checked-node-ids',

        disabledNodeIds:'disabled-node-ids',
        dnd: 'dnd'
        
    };

    this.Tree = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,

        // 构造函数
        init: function(element, options){
            this.element = $(element);

            // -----------------------------------------------
            // options
            // -----------------------------------------------
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);
            if (typeof (this.settings.parseData) === 'function') {
                this.parseData = this.settings.parseData;
                delete this.settings.parseData;
            }

            // -----------------------------------------------
            // value
            // -----------------------------------------------
            // this.value = this.element.val();

            // -----------------------------------------------
            // data
            // -----------------------------------------------
            
            this.initData();
            this.appendTemplate=this.settings.renderAppend||this.appendTemplate;
            this.render = this.settings.render || this.render;
            //this.appendTemplate();
            this.container = $('<ul></ul>'); // list

            var context= this;
            // 初始化数据
            $.when(this.getData())
             .done(function(){
                context._parseTreeData(context.data);
                context.refresh();
                context.bindEvents();
            });
        },

        getData: function(){
            var d = $.Deferred();
            if (this.settings.data) {
                var data = $.extend(true, [], this.settings.data);
                this.data= {nodes:data};
                delete this.settings.data;

                d.resolve();
                return d.promise();

            }
            d.resolve();
            return d.promise();
        },

        initData: function(){
            this.data = {};
            this.nodeDataMap = {};
            this.nodesMap = {};
            this._currentSelectNode = null;
            this.operateNodes = {
                selectedNodesList:[],
                checkedNodesList:[],
                partOfCheckedList:[]
            };
        },

        setValue: function(data,level){
            this.initData();
            var ndata = $.extend(true, [], data);
            this.data = {
                nodes:ndata
            };
            this._parseTreeData(this.data);
            this.refresh();
            if(typeof level === 'undefined'){
                this.expandLevel(this.settings.levels);
            }else{
                this.expandLevel(level);
            }
        },

        refresh: function(){
            var tree = this._buildTree(this.data);
            this.container.empty();
            this.container.remove();   //删除事件
            this.container.append(tree);
            
            this.element.addClass('c-tree');
            this.element
                .empty()
                .append(this.container);
        },

        // 取消事件监听
        unbindEvents: function () {
            this.element.off('nodedbclick');
            this.element.off('noderightclick');
            this.element.off('nodeChecked');
            this.element.off('nodeCollapsed');
            this.element.off('nodeDisabled');
            this.element.off('nodeEnabled');
            this.element.off('nodeExpanded');
            this.element.off('nodeSelected');
            this.element.off('nodeUnchecked');
            this.element.off('nodeUnselected');
        },

        bindEvents:function(){
            this.unbindEvents();
            this.element.on('nodedbclick', $.proxy(this.settings.onNodeDbclick, this));
            this.element.on('noderightclick', $.proxy(this.settings.onNodeRightclick, this));
            this.element.on('nodeChecked', $.proxy(this.settings.onNodeChecked, this));
            this.element.on('nodeCollapsed', $.proxy(this.settings.onNodeCollapsed, this));
            this.element.on('nodeDisabled', $.proxy(this.settings.onNodeDisabled, this));
            this.element.on('nodeEnabled', $.proxy(this.settings.onNodeEnabled, this));
            this.element.on('nodeExpanded', $.proxy(this.settings.onNodeExpanded, this));
            this.element.on('nodeSelected', $.proxy(this.settings.onNodeSelected, this));
            this.element.on('nodeUnchecked', $.proxy(this.settings.onNodeUnchecked, this));
            this.element.on('nodeUnselected', $.proxy(this.settings.onNodeUnselected, this));
        },
        onNodeDragStart: function(event,ui){
            if(typeof this.settings.onNodeDragStart === 'function'){
                return this.settings.onNodeDragStart.apply(this,[event,ui]);
            }
            return true;
        },

        onNodeDropStart: function(event,ui){
            if(typeof this.settings.onNodeDropStart === 'function'){
                return this.settings.onNodeDropStart.apply(this,[event,ui]);
            }
            return true;
        },

        bindDragForElement: function(e){
            var context = this;
            var option=$.extend(true,{
                revert:false,  //鼠标释放的时候，如目标地不为droppable，则还原
                helper:"clone",    //拖动元素不删除
              /*  appendTo: 'body',*/
                start: function(event,ui){
                    var isDraggable = context.onNodeDragStart(event,ui);
                    if(!isDraggable){
                        e.draggable({ 
                            disabled: true   //阻止拖动
                        });
                    }
                },
                drag: function(event,ui){
                    if(typeof context.settings.onNodeDrag === 'function'){
                        context.settings.onNodeDrag.apply(context,[event,ui]);
                    }
                    
                },
                stop: function(event,ui){
                    //step 2
                    if(typeof context.settings.onNodeDragStop === 'function'){
                        context.settings.onNodeDragStop.apply(context,[event,ui]);
                    }
                }
            },this.settings.draggable);
            e.draggable(option);
        },

        bindDropForElement: function(e){
            var context = this;
            var option=$.extend(true,{
                greedy:true,    //如果设置为true，将在嵌套的droppable的对象中阻止事件的传播
                hoverClass: "node-hover",
                over: function(event,ui){   //此事件会在draggable对象经过这个droppable对象时触发
                    var isDroppable = context.onNodeDropStart(event,ui);
                    if(!isDroppable){
                        e.droppable({
                            disabled: true   //阻止接收
                        });
                    }
                },
                drop: function(event,ui){
                    //step 1
                    if(typeof context.settings.onNodeDrop === 'function'){
                        context.settings.onNodeDrop.apply(context,[event,ui]);
                    }
                }
            },this.settings.droppable);
            e.droppable(option);
        },

        bindDnDForElement: function(e){
            if(this.settings.dnd === 'drag'){
                this.bindDragForElement(e);
            }else if(this.settings.dnd === 'drop'){
                this.bindDropForElement(e);
            }else if(this.settings.dnd === 'dnd'){
                this.bindDragForElement(e);
                this.bindDropForElement(e);
            }
        },

        destroy: function () {
            this.container.empty();
            this.container = null;
            this.unbindEvents();
        },

        _replaceClass:function(el,newClass,oldClass){
            if(el){
                el.removeClass(oldClass).addClass(newClass);
            }
            
        },

        _parseTreeData: function(node,level){

           // var node={id:-1,nodes:[{id:4,nodes:[],text:'4'}]};
            var index = 0,
                indexData = null;
            var parentId = node.id || -1;
            if(typeof level === 'undefined'){
                level = 1;
            }
            if(!node || !node.nodes)
                return;
            var length = node.nodes.length;
            while(index < length){
                indexData = node.nodes[index];

                indexData._innerParentId = parentId;
                indexData._innerLevel = level;
                //indexData._isDisabled = false;

                if(indexData.nodes && indexData.nodes.length > 0){
                    indexData._isExpand = false;
                    indexData._hasChildren = true;
                }else{
                    indexData._hasChildren = false;
                }
                index++;
                this.nodeDataMap[indexData.id] = indexData;
                this._parseTreeData(indexData,level+1);
            }
            return node;
        },
        offsetHeight:function(el){
            if(el instanceof jQuery){
                el=el[0]
            }
           return el.getBoundingClientRect();
        },
        _getElPosition:function(el){
            if(el instanceof jQuery){
                el=el[0]
            }
            return el.getBoundingClientRect();
        },
        _buildTree: function(node,level,fg){
            if(typeof level == 'undefined'){
                level = 1;
            }

            var fragment    =   fg || $(document.createDocumentFragment()),
                nodeData    =   node,
                indexData   =   null,
                index       =   0;
            if(!nodeData || !nodeData.nodes)
                return;
            var length = nodeData.nodes.length;   
            while(index < length){
                indexData = nodeData.nodes[index];
                
                var object = {
                        node            :   $('<li/>')
                                            .attr('data-id', indexData.id)
                                            .attr('data-level', level)
                                            .attr('data-hasChildren', indexData._hasChildren)
                                            .attr('data-disabled', indexData._isDisabled)
                                            .data('node-data', indexData),
                        nodeIcon        :   $('<span/>')
                                            .addClass(this.settings.clsSwitch)
                                            .addClass(this.settings.nodeIcon),
                        checkIcon       :   $('<span/>').addClass(this.settings.uncheckedIcon),
                        nodeValue       :   $('<span/>').addClass(this.settings.clsNodeText).text(indexData.text)
                                            .attr('onselectstart','return false'),
                        appendHtml      :   $('<span/>').addClass(this.settings.clsAppendHtml),
                        wrapper         :   $('<div/>')
                                            .addClass(this.settings.clsItemContent),
                        childrenNode    :   null
                };
                if(this.settings.enableTitle){

                    object.node.attr('title',indexData.text);
                }

                //设置expand
                if(indexData._hasChildren){
                    object.nodeIcon.on('click',$.proxy(this.toggleExpandNode,this));
                    this._replaceClass(object.nodeIcon,this.settings.expandIcon,this.settings.nodeIcon);
                    //mouseover:Function
                }
                
                //设置select   
                if(indexData._hasChildren && this.settings.selectMode === enumSelectMode.leaf){
                   object.wrapper.addClass(this.settings.clsNodeUnSelect);
                }else {
                    if (indexData._isSelected) {
                        object.wrapper.addClass(this.settings.clsNodeSelected);
                        this._addSelectedNode(indexData);
                        this._currentSelectNode = object.wrapper;
                    }
                    object.nodeValue.on('click', $.proxy(this.toggleSelectNode, this));
                }
                object.nodeValue.on('dblclick',$.proxy(this.onNodeDbclick,this));
                object.nodeValue.on('mousedown',$.proxy(this.onNodeRightclick,this));
                object.wrapper.append(object.nodeIcon);

                //设置checkbox
                if(this.settings.showCheckbox === enumShowCheckbox.all || (!indexData._hasChildren && this.settings.showCheckbox === enumShowCheckbox.leaf)) {
                    object.wrapper.append(object.checkIcon);
                    object.node.data('node-check',object.checkIcon);
                    object.checkIcon.on('click',$.proxy(this.toggleCheckNode,this));
                    if(typeof (indexData._checkSate) === 'undefined'){
                        indexData._checkSate = enumCheckedState.unchecked;
                    }else{
                        this._replaceClass(object.checkIcon,this._getCheckIconByState(indexData._checkSate),this.settings.uncheckedIcon);
                        //this.checkNode(indexData,object.node);
                        this._singleCheckSateNode(object.node,indexData,indexData._checkSate );
                    }
                }
                object.wrapper.append(object.nodeValue);

                //设置disabled
                if(indexData._isDisabled){
                    object.wrapper.addClass(this.settings.clsNodeDisabled);
                    this.disableNodeByIds(indexData.id);
                }

                //设置拖拽
                this.bindDnDForElement(object.node);

                //渲染text
                var renderNodeText='';
                if(this.settings.renderNode){
                    renderNodeText = this.settings.renderNode(indexData);
                    object.nodeValue.html("");
                    object.nodeValue.append(renderNodeText);
                    object.wrapper.append(object.nodeValue);
                }

                //appendHtml
                var appendHtml='';
                if(this.settings.appendHtml){
                    if(this.settings.parseAppendHtml){
                        var html = this.settings.parseAppendHtml(this.settings.appendHtml);
                        appendHtml = html;
                    }
                    else{
                        appendHtml= this.settings.appendHtml;
                    }
                    
                    object.appendHtml.append(appendHtml);
                    object.wrapper.append(object.appendHtml);
                }

                //渲染追加
                var renderHtml='';
                if(this.settings.render){
                    renderHtml = this.settings.render(indexData);
                    renderHtml =  this.settings.parseAppendHtml(renderHtml);
                    object.appendHtml.html("");
                    object.appendHtml.append(renderHtml);
                    object.wrapper.append(object.appendHtml);
                }

                if (this.settings.showTags && indexData.tags){
                    for (var i = 0; i < indexData.tags.length; i++) {
                        var badge = $('<span/>').addClass(this.settings.clsBadge).text(indexData.tags[i]);
                        object.wrapper.append(badge);
                    }
                }
                object.node.append(object.wrapper);
                this.nodesMap[indexData.id] = object.node;
                fragment.append(object.node); 
                index++;          
            }
            return fragment;
        },

        /* 点击select事件
         *
         */
        toggleSelectNode: function(event){
            var currentTarget   =   $(event.currentTarget),
                currentParent   =   currentTarget.closest('li'),
                currentWrapper  =   currentTarget.closest('div'),
                nodeData        =   currentParent.data('node-data');
            if(currentWrapper.hasClass(this.settings.clsNodeDisabled))
                return;
            this.selectNode(nodeData,currentParent);
            event.preventDefault();
            event.stopPropagation();
        },

        onNodeDbclick: function(event){
            var currentTarget   =   $(event.currentTarget),
                currentParent   =   currentTarget.closest('li'),
                currentWrapper  =   currentTarget.closest('div'),
                nodeData        =   currentParent.data('node-data');
            if(currentWrapper.hasClass(this.settings.clsNodeDisabled))
                return;
            if(nodeData._hasChildren){
                this.toggleExpandNode(event);
            }
            this.element.trigger('nodedbclick',nodeData);
        },
         //右击事件
         onNodeRightclick: function(event) {
             if (event.button == 2) {
                 var currentTarget = $(event.currentTarget),
                     currentParent = currentTarget.closest('li'),
                     currentWrapper = currentTarget.closest('div'),
                     nodeData = currentParent.data('node-data');
                 if (currentWrapper.hasClass(this.settings.clsNodeDisabled))
                     return;
                 this.element.trigger('noderightclick', nodeData);
             }

         },

        /* 点击expand事件
         *
         */
        toggleExpandNode: function(event){
            var currentTarget   =   $(event.currentTarget),
                currentParent   =   currentTarget.closest('li'),
                currentWrapper  =   currentTarget.closest('div'),
                nodeData        =   currentParent.data('node-data');
                //禁用还是可以展开收缩
           /* if(currentWrapper.hasClass(this.settings.clsNodeDisabled))
                return;*/
            if(nodeData._isExpand){  //展开时
                this._collapseSingleNode(nodeData,currentParent);
            }else{
                this._expandSingleNode(nodeData,currentParent);
            }
            event.preventDefault();
            event.stopPropagation();
        },

        /* 点击checkbox触发事件
         *
         */
        toggleCheckNode: function(event){
            event.preventDefault();
            event.stopPropagation();
            var currentTarget   =   $(event.currentTarget),
                currentParent   =   currentTarget.closest('li'),
                currentWrapper  =   currentTarget.closest('div'),
                nodeData        =   currentParent.data('node-data');
            if(currentWrapper.hasClass(this.settings.clsNodeDisabled))
                return;
            this.checkNode(nodeData,currentParent);
            
        },

        checkNode: function(node,nodeTarget){
            if(typeof nodeTarget === 'undefined'){
                nodeTarget = this.nodesMap[node.id];
            }
            if(!nodeTarget)  //未创建节点不能展开
                return;
            this._singleCheckNode(nodeTarget,node);
            if(this.settings.checkRecursive){ //级联
                this._setChildrenCheckedState(nodeTarget,node);
                this._setParentCheckedState(nodeTarget,node);
            }
        },

        /* 点击checkbox，对一个checkbox的作用
         *
         */
        _singleCheckNode: function(object,nodeData){
            var state;
            if(nodeData._checkSate === enumCheckedState.unchecked){
                state = enumCheckedState.checked;
            }else{
                state = enumCheckedState.unchecked;
            }
            this._singleCheckSateNode(object,nodeData,state);
        },

        /* 根据状态改变checkbox
         *
         */
        _singleCheckSateNode: function(object,nodeData,state){
            if(object){
                this._replaceClass(object.data('node-check'),this._getCheckIconByState(state),this._getCheckIconByState(nodeData._checkSate));
            }
            nodeData._checkSate = state;
            if(state === enumCheckedState.checked){
                this._addCheckedNode(nodeData);
                if(!this.settings.silent){
                    this.element.trigger('nodeChecked', nodeData);
                }
            }else if(state === enumCheckedState.unchecked){
                this._removeCheckedNode(nodeData);
                if(!this.settings.silent){
                    this.element.trigger('nodeUnchecked', nodeData);
                }
            }else if(state === enumCheckedState.partOf){
                this._removeCheckedNode(nodeData);
            }
        },

        /* checkbox: 设置子节点check状态
         *
         */
        _setChildrenCheckedState: function(object,nodeData){
            if(nodeData._hasChildren){
                for (var i = 0; i < nodeData.nodes.length; i++) {
                    var childNodeData = nodeData.nodes[i];
                    var childNode = this.nodesMap[childNodeData.id];
                    this._singleCheckSateNode(childNode,childNodeData,nodeData._checkSate);
                    this._setChildrenCheckedState(childNode,childNodeData);
                }
            }
        },

        /* checkbox: 设置父节点check状态
         *
         */
        _setParentCheckedState: function(object,nodeData){
            if(nodeData._innerParentId){
                var parentNode = this.nodesMap[nodeData._innerParentId];
                while(parentNode){
                    var parentState,uncheckedNum = 0,checkedNum = 0;
                    var parentData = parentNode.data('node-data');
                    for (var i = 0; i < parentData.nodes.length; i++) {
                        var childData = parentData.nodes[i];
                        if(childData._checkSate === enumCheckedState.unchecked){
                            uncheckedNum++;
                        }else if(childData._checkSate === enumCheckedState.checked){
                            checkedNum++;
                        }
                    }
                    if(uncheckedNum === parentData.nodes.length){
                        parentState = enumCheckedState.unchecked;
                    }else if(checkedNum === parentData.nodes.length){
                        parentState = enumCheckedState.checked;
                    }else{
                        parentState = enumCheckedState.partOf;
                    }
                    this._singleCheckSateNode(parentNode,parentData,parentState);
                    parentNode = this.nodesMap[parentData._innerParentId];
                }
            }
        },

        /* 根据状态获取checkbox图标
         *
         */
        _getCheckIconByState: function(state){
            var nodeCheckIcon;
            if(state === enumCheckedState.unchecked){
                nodeCheckIcon = this.settings.uncheckedIcon;
            }else if(state === enumCheckedState.checked){
                nodeCheckIcon = this.settings.checkedIcon;
            }else{
                nodeCheckIcon = this.settings.checkedPartOfIcon;
            }
            return nodeCheckIcon;
        },

        /* 数组中添加元素
         *
         */
        _addItem: function(item,array){
            for (var i = 0; i < array.length; i++) {
                if(array[i].id === item.id)
                    return;
            }
            array.push(item);
        },

        /* 数组中删除元素
         *
         */
        _removeItem: function(item,array){
            var flag = -1;
            for (var i = 0; i < array.length; i++) {
                if(array[i].id === item.id){
                    flag = i;
                    break;
                }
            }
            if(flag >= 0){
                array.splice(flag,1);
            }
        },

        _addSelectedNode: function(node){

            this._addItem(node,this.operateNodes.selectedNodesList);
        },

        _removeSelectedNode: function(node){

            this._removeItem(node,this.operateNodes.selectedNodesList);
        },

        _addCheckedNode: function(node){

            this._addItem(node,this.operateNodes.checkedNodesList);
        },

        _removeCheckedNode: function(node){

            this._removeItem(node,this.operateNodes.checkedNodesList);
        },

        /* expand单个节点
         *
         */
        _expandSingleNode: function(node,nodeTarget){
            if(node._isExpand || !node._hasChildren)
                return;
            if(typeof nodeTarget === 'undefined'){
                nodeTarget = this.nodesMap[node.id];
            }
            if(!nodeTarget)  //未创建节点不能展开
                return;
            var children =  nodeTarget.find('ul');
            var nodeIcon = $(nodeTarget.find('div>span').get(0));
            if(children.get(0)){        //已经有子节点，展开
                $(children.get(0)).show();
            }else{  //还没有初始化
                var childrenNode     =   $('<ul/>');
                nodeTarget.append(this._buildTree(node,node._innerLevel + 1,childrenNode));
                
                childrenNode.show();
            }
            this._replaceClass(nodeIcon,this.settings.collapseIcon,this.settings.expandIcon);
            node._isExpand = true;
            if(!this.settings.silent){
                this.element.trigger('nodeExpanded', node);
            }
        },

        /* collapse单个节点
         *
         */
        _collapseSingleNode: function(node,nodeTarget){
            if(!node._isExpand)
                return;
            if(typeof nodeTarget === 'undefined'){
                nodeTarget = this.nodesMap[node.id];
            }
            if(!nodeTarget)  //未创建节点不能展开
                return;
            var children =  nodeTarget.find('ul');
            var nodeIcon = $(nodeTarget.find('div>span').get(0));
            $(children.get(0)).hide();
            node._isExpand = false;
            this._replaceClass(nodeIcon,this.settings.expandIcon,this.settings.collapseIcon);
            if(!this.settings.silent){
                this.element.trigger('nodeCollapsed', node);
            }
        },

        /* expand节点的所有父节点
         *
         */
        expandPathNode: function(node){
            var parents = [];
            var tmp = node;
            parents.push(tmp);
            while(tmp._innerParentId !== -1){
                var parentId = tmp._innerParentId;
                tmp =  this.nodeDataMap[parentId];
                parents.push(tmp);
            }
            for (var i = parents.length - 1; i >= 0; i--) {
                var parentNode = parents[i];
                this._expandSingleNode(parentNode);
            }
        },

        /* expand所有节点
         *
         */
        expandAll: function(){
            var context = this;
            if(!this.data)
                return;
            expandOneLevel(this.data);
            function expandOneLevel(node){
                var index = 0;
                if(!node.nodes)
                    return;
                while(index < node.nodes.length){
                    var indexData = node.nodes[index];
                    context._expandSingleNode(indexData);
                    if(indexData.nodes){
                        expandOneLevel(indexData);
                    }
                    index++;
                }
            }
        },
        expandNode: function(data){
            var context = this;
            if(!data)
                return;
            expandOneLevel(data);
            function expandOneLevel(node){
                var index = 0;
                if(!node.nodes)
                    return;
                while(index < node.nodes.length){
                    var indexData = node.nodes[index];
                    context._expandSingleNode(indexData);
                    if(indexData.nodes){
                        expandOneLevel(indexData);
                    }
                    index++;
                }
            }
        },

        /* expand 所有level层以内的节点
         *
         */
        expandLevel: function(levels){
            var context = this;
            if(!this.data)
                return;
            expandOneLevel(this.data);
            function expandOneLevel(node){
                var level = 1;
                if(typeof node._innerLevel !== 'undefined')
                    level = node._innerLevel;
                if(level >= levels)
                    return;
                var index = 0;
                if(!node.nodes)
                    return;
                while(index < node.nodes.length){
                    var indexData = node.nodes[index];
                    if(indexData._innerLevel < levels){
                        context._expandSingleNode(indexData);
                        if(indexData.nodes){
                            expandOneLevel(indexData);
                        }
                    }
                    index++;
                }
            }
        },

        collapseAll: function(){
            if(!this.data)
                return;
            for (var i = 0; i < this.data.nodes.length; i++) {
                this. _collapseSingleNode(this.data.nodes[i]);
            }
        },
        collapseNode: function(data){
            if(!this.data || !data)
                return;
            if(data.length===1){
                this. _collapseSingleNode(data);
            }else {
                for (var i = 0; i < data.nodes.length; i++) {
                    this._collapseSingleNode(data.nodes[i]);
                }
            }
        },
        setCheckedNode: function(node){
            this.expandPathNode(node);
            if(node._checkSate === enumCheckedState.checked)
                return;
            this.checkNode(node);
        },

        setUncheckedNode: function(node){
            if(node._checkSate === enumCheckedState.unchecked)
                return;
            this.checkNode(node);
        },

        checkNodeByIds: function(ids){
            var arrId= ids.split(',');
            for(var i=0;i < arrId.length;i++){
                var node =this.nodeDataMap[arrId[i]];
                if(node){
                    this.setCheckedNode(node);
                }
            }
        },


        uncheckNodeByIds: function(ids){
            var arrId= ids.split(',');
            for(var i=0;i < arrId.length;i++){
                var node =this.nodeDataMap[arrId[i]];
                if(node){
                    this.setUncheckedNode(node);
                }
            }
        },

        checkAll: function(){
            var context = this;
            if(!this.data)
                return;
            checkOneLevel(this.data);
            function checkOneLevel(node){
                var index = 0;
                if(!node.nodes)
                    return;
                while(index < node.nodes.length){
                    var indexData = node.nodes[index];
                    if(indexData._checkSate !== enumCheckedState.checked){
                        var object = context.nodesMap[indexData.id];
                        context._singleCheckSateNode(object,indexData,enumCheckedState.checked);
                    }
                    if(indexData.nodes){
                        checkOneLevel(indexData);
                    }
                    index++;
                }
            }
        },

        uncheckAll: function(){
            var context = this;
            if(!this.data)
                return;
            uncheckOneLevel(this.data);
            function uncheckOneLevel(node){
                var index = 0;
                if(!node.nodes)
                    return;
                while(index < node.nodes.length){
                    var indexData = node.nodes[index];
                    if(indexData._checkSate !== enumCheckedState.unchecked){
                        var object = context.nodesMap[indexData.id];
                        context._singleCheckSateNode(object,indexData,enumCheckedState.unchecked);
                    }
                    if(indexData.nodes){
                        uncheckOneLevel(indexData);
                    }
                    index++;
                }
            }
        },

        _singleDisabledNode: function(object,node){
            if(object && !object.hasClass(this.settings.clsNodeDisabled)){
                object.addClass(this.settings.clsNodeDisabled);
                if(!this.settings.silent){
                    this.element.trigger('nodeDisabled', node);
                }
            }
        },

        setDisabledNode: function(obj,node){
            //this.expandPathNode(node);
            this._singleDisabledNode(obj,node);
        },
        setEnabledNode: function(obj,node){
            //this.expandPathNode(node);
            this._singleEnabledNode(obj,node);
        },
        //disablednode
        disableNodeByIds: function(ids){
            var arrId= ids.split(',');
            for(var i=0;i < arrId.length;i++){
                var node = this.nodeDataMap[arrId[i]];
                var obj = this.nodesMap[node.id];
                node._isDisabled = true;
                if(obj){
                    var currentWrapper  =  obj.children('div');
                    if(node){
                        this.setDisabledNode(currentWrapper,node);
                    }
                } 
                
                
            }
        },
        //enableNodeByIds
        enableNodeByIds: function(ids){
            var arrId= ids.split(',');
            for(var i=0;i < arrId.length;i++){
                var node = this.nodeDataMap[arrId[i]];
                var obj = this.nodesMap[node.id];
                node._isDisabled = false;
                if(obj){
                    var currentWrapper  =  obj.children('div');
                    if(node){
                        this.setEnabledNode(currentWrapper,node);
                    }
                } 
                
                
            }
        },
       
        _singleEnabledNode: function(object,node){
            if(object && object.hasClass(this.settings.clsNodeDisabled)){
               //禁用后 还是能展开，收缩
                object.removeClass(this.settings.clsNodeDisabled);
                if(!this.settings.silent){
                    this.element.trigger('nodeEnabled', node);
                }
            }
        },

        disableAll: function(){
            var context = this;
            if(!this.data)
                return;
            disableOneLevel(this.data);
            function disableOneLevel(node){
                var index = 0;
                if(!node.nodes)
                    return;
                while(index < node.nodes.length){
                    var indexData = node.nodes[index];
                    indexData._isDisabled = true;
                    var object = context.nodesMap[indexData.id];
                    if(object){
                        var currentWrapper  =  object.children('div');
                        context._singleDisabledNode(currentWrapper,indexData);
                    }
                    if(indexData.nodes){
                        disableOneLevel(indexData);
                    }
                    index++;
                }
            }
        },
      

        enableAll: function(){
            var context = this;
            if(!this.data)
                return;
            disableOneLevel(this.data);
            function disableOneLevel(node){
                var index = 0;
                if(!node.nodes)
                    return;
                while(index < node.nodes.length){
                    var indexData = node.nodes[index];
                    indexData._isDisabled = false;
                    var object = context.nodesMap[indexData.id];
                    if(object){
                        var currentWrapper = object.children('div');
                        context._singleEnabledNode(currentWrapper,indexData);
                    }
                    if(indexData.nodes){
                        disableOneLevel(indexData);
                    }
                    index++;
                }
            }
        },

        selectNode: function(node,nodeTarget){
            if(typeof nodeTarget === 'undefined'){
                nodeTarget = this.nodesMap[node.id];
            }
            if(!nodeTarget)  //未创建节点不能展开
                return;
            var currentWrapper  =   nodeTarget.children('div');
            if(currentWrapper.hasClass(this.settings.clsNodeUnSelect))
                return;
            if(node._isSelected){
                node._isSelected = false;
                currentWrapper.removeClass(this.settings.clsNodeSelected);
                if(!this.settings.multiSelect)
                    this._currentSelectNode = null;
                this._removeSelectedNode(node);
                if(!this.settings.silent){
                    this.element.trigger('nodeUnselected', node);
                }
            }else{
                this._addSelectedNode(node);
                currentWrapper.addClass(this.settings.clsNodeSelected);
                node._isSelected = true;
                if(!this.settings.multiSelect){
                    if(this._currentSelectNode){
                        this._currentSelectNode.removeClass(this.settings.clsNodeSelected);
                        this._currentSelectNode.closest('li').data('node-data')._isSelected = false;
                        this._removeSelectedNode(this._currentSelectNode.closest('li').data('node-data'));
                        if(!this.settings.silent){
                            this.element.trigger('nodeUnselected', this._currentSelectNode.closest('li').data('node-data'));
                        }
                    }
                    this._currentSelectNode = currentWrapper;
                }
                if(!this.settings.silent){
                    this.element.trigger('nodeSelected', node);
                }
            }
        },

        setSelectedNode: function(node){
            this.expandPathNode(node);
            if(node._isSelected)
                return;
            this.selectNode(node);
        },

        setUnselectedNode: function(node){
            if(!node._isSelected)
                return;
            this.selectNode(node);
        },

        selectNodeByIds: function(ids){
            var arrId= ids.split(',');
            for(var i=0;i < arrId.length;i++){
                var node =this.nodeDataMap[arrId[i]];
                if(node){
                    this.setSelectedNode(node);
                }
            }
        },

        unselectNodeByIds: function(ids){
            var arrId= ids.split(',');
            for(var i=0;i < arrId.length;i++){
                var node =this.nodeDataMap[arrId[i]];
                if(node){
                    this.setUnselectedNode(node);
                }
            }
        },

        getCheckedNodes: function(isPartOf){ 
            if(isPartOf) return this.getCheckedNodesPartOf();
            return this.operateNodes.checkedNodesList;
        },
        getCheckedNodesPartOf:function(){
            var checkedNodesPartOf=[];
             var map=   this.nodeDataMap;
               for(var m in map){
                if(map[m]._checkSate==='partOf'||map[m]._checkSate==='checked')
                       checkedNodesPartOf.push(map[m]);
               }
            return checkedNodesPartOf;
        },
        getSelectNodes: function(){
            return this.operateNodes.selectedNodesList;
        },

        getNode: function(nodeId){
            return this.nodeDataMap[nodeId];
        },
        _beforeOrAfterNode:function(idNode,nodeData,nodeTarget,type){
            var node={
                id:idNode._innerParentId,
                _innerLevel: idNode._innerLevel,
                nodes:[nodeData]
            };
            var parentNode = this.getNode(idNode._innerParentId);
            var index= 0,temp=0;
            if(type==='after') temp+=1;
            if(parentNode){
                //parentNode.nodes.push(nodeData);
                index = $(parentNode.nodes).index(idNode);
                parentNode.nodes.splice(index+temp,0,nodeData);
            }
            this._parseTreeData(node,node._innerLevel+1);
            nodeData._innerLevel=idNode._innerLevel;
            if(idNode._innerParentId===-1){
                //this.data.nodes.push(nodeData);
                index = $(this.data.nodes).index(idNode);
                this.data.nodes.splice(index+temp,0,nodeData);
            }
            if(type==='after'){
                nodeTarget.after(this._buildTree(node,node._innerLevel));
            }else if(type==='before'){
                nodeTarget.before(this._buildTree(node,node._innerLevel));
            }
        },
        addNode: function(nodeId,nodeData,type){
            var idNode = this.getNode(nodeId);
            if(!idNode)
                return;
            var node = {
                id : idNode.id,
                _innerLevel: idNode._innerLevel,
                nodes:[nodeData]
            };
            var nodeTarget = this.nodesMap[idNode.id];
            if(!nodeTarget)  //未创建节点则不用添加
                return;
            if(type==='before' || type=='after'){
                this.container.find("li[data-id='"+nodeData.id+"']").remove();
                this._beforeOrAfterNode(idNode,nodeData,nodeTarget,type);
            } else {
                var isFirstChild = false;
                if(idNode._hasChildren){
                    idNode.nodes.push(nodeData);
                }else{
                    idNode.nodes = [nodeData];
                    idNode._hasChildren = true;
                    isFirstChild = true;
                }
                this._parseTreeData(node,node._innerLevel+1);
                if(isFirstChild){  //开始没有子节点，添加后需要重新初始化该节点
                    var pNode = {
                        id : idNode._innerParentId,
                        _innerLevel : idNode._innerLevel - 1,
                        nodes:[idNode]
                    };
                    nodeTarget.after(this._buildTree(pNode,pNode._innerLevel + 1));
                    nodeTarget.remove();
                }else{
                    var children =  nodeTarget.find('ul');
                    if(children.get(0)){        //已经有子节点，展开
                        $(children.get(0)).append(this._buildTree(node,node._innerLevel + 1));
                    }
                }
            }
        },

        removeNode: function(nodeId){
            var idNode = this.getNode(nodeId);
            if(!idNode)
                return;
            var parentId = idNode._innerParentId;
            var nodeTarget = this.nodesMap[idNode.id];
            if(parentId === -1){
                this._removeItem(idNode,this.data.nodes);
                nodeTarget.remove();
            }else{
                var pNode = this.getNode(parentId);
                if(pNode.nodes.length > 1){
                    this._removeItem(idNode,pNode.nodes);
                    if(nodeTarget)
                        nodeTarget.remove();
                }else{
                    this._removeItem(idNode,pNode.nodes);
                    pNode._hasChildren = false;
                    pNode._isExpand = null;
                    var tempNode = {
                        id : pNode._innerParentId,
                        _innerLevel : pNode._innerLevel - 1,
                        nodes:[pNode]
                    };
                    var pnodeTarget = this.nodesMap[pNode.id];
                    if(pnodeTarget){
                        pnodeTarget.after(this._buildTree(tempNode,tempNode._innerLevel + 1));
                        pnodeTarget.remove(); 
                    }
                } 
            }
            this._removeIdNode(idNode);
        },
        /*
        * 修改节点
        * */
        updateNode: function(nodeId,text){
            var idNode = this.getNode(nodeId);
            if(!idNode)
                return;
            var nodeTarget = this.nodesMap[nodeId];
            idNode.text = text;
            var nodeText =  $(nodeTarget).find('.nodeValue');
            if(nodeText.length>0){  //已经生成过子节点
                nodeText = nodeText[0];
            }
            $(nodeText).html(text);
        },

        _removeIdNode: function(idNode){
            this.nodeDataMap[idNode.id] = null;
            this.nodesMap[idNode.id] = null;
            if(this._currentSelectNode){

                //解决：找不到li.data('node-data') 会报错的问题
                if(!this._currentSelectNode.closest('li').data('node-data') || this._currentSelectNode.closest('li').data('node-data').id === idNode.id)

                    this._currentSelectNode = null;
            }
            this._removeSelectedNode(idNode);
            this._removeCheckedNode(idNode);
            if(idNode._hasChildren){
               for (var i = 0; i < idNode.nodes.length; i++) {
                    var tnode = idNode.nodes[i];
                    this._removeIdNode(tnode);
                } 
            }
        },
        getTreeData:function(){
            return this.data.nodes;
        },
        //
        appendTemplate:function(html,node){
            //var html = this.settings.parseAppendHtml(html);
            return html;
        },
        render:function(node){
            //var html = this.settings.parseAppendHtml(html);
            return node;
        },
        renderNode:function(node){
            //var html = this.settings.parseAppendHtml(html);
            return node;
        },  
        getParentNode:function(node){
            var count=0;
            if(typeof node ==='object'){
                while(node._innerParentId&&-1!==node._innerParentId&&count<10){
                    node=this.nodeDataMap[node._innerParentId];
                    count++;
                }   
            }
           
            return node;
        },
        renderPointNode:function(ids,func,callback){
            var context=this;
            if(Array.isArray(ids)){
                ids.forEach(function(id){
                    var data=context.nodeDataMap[id];
                    var innerParentId=-1;
                    context.expandNode({nodes:[context.getParentNode(data)]});
                    var $point=context.container.find('li[data-id="'+id+'"] div:first span.nodeValue')
                    var template=func(data)||{};
                    $point.prop("outerHTML",template);
                    $point=context.container.find('li[data-id="'+id+'"] div:first span.nodeValue')
                    $point.on('click', $.proxy(context.toggleSelectNode, context));
                    $point.on('dblclick',$.proxy(context.onNodeDbclick,context));
                    $point.on('mousedown',$.proxy(context.onNodeRightclick,context));
                    (typeof callback==='function')&&callback(data);
               })
            }
        },
        //开放接口
        getApi: function(){
            var context = this;
            var funs = {
                collapseAll: 'collapseAll',
                collapseNode: 'collapseNode',
                expandAll: 'expandAll',
                expandNode:'expandNode',
                expandLevel:'expandLevel',
                checkAll:'checkAll',
                uncheckAll:'uncheckAll',
                disableAll:'disableAll',
                disableNodeByIds:'disableNodeByIds',
                enableNodeByIds:'enableNodeByIds',
                enableAll:'enableAll',
                checkNodeByIds:'checkNodeByIds',
                uncheckNodeByIds:'uncheckNodeByIds',
                selectNodeByIds:'selectNodeByIds',
                unselectNodeByIds:'unselectNodeByIds',
                getCheckedNodes:'getCheckedNodes',
                getSelectNodes:'getSelectNodes',
                getNode:'getNode',
                setValue:'setValue',
                addNode:'addNode',
                removeNode:'removeNode',
                updateNode:'updateNode',
                getTreeData:'getTreeData',
                renderPointNode:'renderPointNode'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });
});

/* tree javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'tree';
    var pluginClass = s.ui.controls.Tree;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);

/* =========================================================
 * bootstrap-treeview.js v1.2.0
 * =========================================================
 * Copyright 2013 Jonathan Miles
 * Project URL : http://www.jondmiles.com/bootstrap-treeview
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================= */


 /*
    1. removed css (move to tree.less)
    2. removed buildStyleOverride
 */

Jx().package("s.ui.controls", function(J){

    // 严格模式
    'use strict';

    // 勾选框是否显示
    var enumShowCheckbox= {
        all: 'all',     // 0 所有显示
        none: 'none',   // 1 不显示(默认)
        leaf: 'leaf'    // 2 叶子节点显示
    };

    var enumCheckedState= {
        checked: 'checked',
        unchecked: 'unchecked',
        partOf: 'partOf'
    };

    // 节点是否可选
    var enumSelectMode= {
        all: 'all',     // 0 所有可选中(默认)
        leaf: 'leaf',   // 1 仅叶子可选中
    };

    // 全局变量、函数、对象
    var emptyFun= function(){};
    var _currentPluginId = 0;
    var defaults = {
        levels: 1,
        dataUrl: '',

        expandIcon: 'glyphicon glyphicon-plus',
        collapseIcon: 'glyphicon glyphicon-minus',
        // expandIcon: 'glyphicon glyphicon-chevron-right',
        // collapseIcon: 'glyphicon glyphicon-chevron-down',
        emptyIcon: 'glyphicon',
        nodeIcon: '',
        selectedIcon: '',
        checkedIcon: 'glyphicon glyphicon-check',
        checkedPartOfIcon: 'glyphicon glyphicon-check partOfchecked',
        uncheckedIcon: 'glyphicon glyphicon-unchecked',

        enableLinks: false,
        enableTitle: false,
        showIcon: true,
        // showCheckbox: false,
        showTags: false,
        multiSelect: false,

        // nodeOptions
        silent: false,
        ignoreChildren: false,

        expandOne:true,

        isAsyc: true,

        // extends
        showCheckbox: enumShowCheckbox.none,
        checkRecursive: false,
        selectMode: enumSelectMode.all,
        appendHtml:'',
        parseAppendHtml: undefined, // for angularjs : function(html){ return $compile(html)($scope) ;}

        // states
        selectedNodeIds:'',
        checkedNodeIds:'',

        // Event handlers
        // 节点勾选
        onNodeChecked: emptyFun,
        // 节点收起
        onNodeCollapsed: emptyFun,
        // 节点禁用
        onNodeDisabled: emptyFun,
        // 节点启用
        onNodeEnabled: emptyFun,
        // 节点展开
        onNodeExpanded: emptyFun,
        // 节点选中
        onNodeSelected: emptyFun,
        // 节点取消勾选
        onNodeUnchecked: emptyFun,
        // 节点取消选中
        onNodeUnselected: emptyFun,
        // 搜索完成
        onSearchComplete: emptyFun,
        // 搜索结果清除
        onSearchCleared: emptyFun
    };
    var attributeMap = {
        isAsyc: 'is-asyc',
        expandOne: 'expand-one',
        ignoreChildren: 'ignore-children',
        expandIcon: 'expand-icon',
        collapseIcon: 'collapse-icon',

        showTags: 'show-tags',
        levels: 'levels',
        // enableTitle: 'enable-title',
        dataUrl: 'data-url',

        showCheckbox:'show-checkbox',
        checkRecursive: 'check-recursive',
        selectMode:'select-mode',
        appendHtml:'append-html',

        multiSelect:'multi-select',        
        selectedNodeIds:'selected-node-ids',
        checkedNodeIds:'checked-node-ids'
        
    };

    // var state= jqNode.find('.expand-icon').hasClass(this.settings.expandIcon);
    // fix for multi class in this.settings.expandIcon, like: 'glyphicon glyphicon-chevron-right'
    function hasClasses(jqElement, classes){
        var arrClass= classes.split(' ');
        var has= true;
        for(var i=0; i< arrClass.length; i++){
            if(!jqElement.hasClass(arrClass[i])){
                has= false;
                break;
            }
        }
        return has;
    }

    this.Treeasyc = new J.Class({extend : s.ui.controls.Slowtree}, {
        defaults : defaults,
        attributeMap : attributeMap,


        // 构造函数
        init: function(element, options){
            this.element = $(element);

            // -----------------------------------------------
            // options
            // -----------------------------------------------
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);
            if (typeof (this.settings.parseData) === 'function') {
                this.parseData = this.settings.parseData;
                delete this.settings.parseData;
            }

            // -----------------------------------------------
            // value
            // -----------------------------------------------
            // this.value = this.element.val();

            // -----------------------------------------------
            // data
            // -----------------------------------------------
            this.data = [];
            this.idIndexMap = {};
            this.container = $('<ul class="list-group"></ul>'); // list

            var context= this;
            // context.buildHtml();
            // context.initElements();
            // context.refresh();
            // // context.initStates();

            // // -----------------------------------------------
            // // events
            // // -----------------------------------------------
            // // context.buildObservers();
            // context.bindEvents();
                
            // 初始化数据
            $.when(this.getData())
             .done(function(){
                
                // -----------------------------------------------
                // states
                // -----------------------------------------------
                // this.initStates(jqElement);
                // -----------------------------------------------
                // html
                // -----------------------------------------------
                context.buildHtml();
                context.initElements();
                context.refresh();
                // context.initStates();

                // -----------------------------------------------
                // events
                // -----------------------------------------------
                // context.buildObservers();
                context.bindEvents();
                // context.bindEventsInterface();
            });
        },

        parseData: function(node){
            var innerData= [];
            var index = 0,level = 0;
            if(this.index)
                index= this.index;
            if(node._innerLevel)
                level = node._innerLevel; 
            var context= this;
            function recurseTree (node, level) {
                if (!node.nodes) return;

                level ++;

                for(var i=0; i<node.nodes.length; i++){
                    var child=node.nodes[i];

                    child.id = child.id || index;
                    child._innerParentId = node.id;
                    child._innerPath = (node._innerPath || 'r') + '|' + child.id;
                    child._innerLevel = level;

                    innerData.push(child);
                    context.idIndexMap['id_' + child.id]= index;

                    index++;

                    if (child.nodes) {
                        recurseTree(child, level);
                    }
                }
            }

            recurseTree(node, level);

            this.index = index;

            return innerData;
        },

        updateData: function(data){
            var innerData = $.extend(true, [], data);
            this.level = 0;
            this.index = 0;
            var node = {nodes: innerData};
            this.data= this.parseData(node);
        },

        addNodeData: function(tnode,data){
            var childrenData = $.extend(true, [], data);
            var node = tnode;
            if(!node)
                node = {};
            node.nodes = childrenData;
            this.childData = this.parseData(node);
            if(this.childData && this.childData.length > 0){
                for(var i=0; i<this.childData.length; i++){
                    var temp= this.childData[i];
                    this.data.push(temp);
                }
                this.buildChildTree(node);
                this.setOneLevelExpandedState(node, true, true, this.settings.ignoreChildren);
            }
        },

        addChildrenData: function(node,data){
            var childrenData = $.extend(true, [], data);
            node.nodes = childrenData;
            this.childData = this.parseData(node);
            if(!this.childData || this.childData.length === 0){
                var jqNode= this.elements.getNode(node.id);
                jqNode.find('.expand-icon').removeClass(this.settings.expandIcon);
                jqNode.find('.expand-icon').removeClass('expand-icon');
                jqNode.attr('data-hasChildren','false');
                return;
            }else{
                for(var i=0; i<this.childData.length; i++){
                    var temp= this.childData[i];
                    this.data.push(temp);
                }
                this.buildChildTree(node);
                this.setOneLevelExpandedState(node, true, true, this.settings.ignoreChildren);
            }
        },

        buildHtml: function(){
            this.element.addClass('c-slowtree');
            this.element
                .empty()
                .append(this.container);
        },

        buildChildTree: function (parentNode) {
            if(!this.childData)
                return;
            for(var i=0; i<this.childData.length; i++){
                var node= this.childData[i];
                var item= this.buildItem(node);
                item = $(item);
                if(node._innerParentId || node._innerParentId === 0){
                    var jqNode= this.elements.getNode(node._innerParentId);
                    item.insertAfter(jqNode);
                }else{
                    this.container.append(item);
                }

                var appendHtml='';
                if(this.settings.appendHtml){
                    if(this.settings.parseAppendHtml){
                        var html = this.settings.parseAppendHtml(this.settings.appendHtml);
                        appendHtml = html;
                    }
                    else
                    {
                        appendHtml= this.settings.appendHtml;
                    }
                    
                     item.find('.appendHtml').append(appendHtml);
                }
            }
        },

        buildItem: function(node){
            //var hasChildren = node.nodes && node.nodes.length > 0;
            var hasChildren = false;
            if(this.settings.isAsyc)
                hasChildren = true;
            else if(!this.settings.isAsyc && node.nodes && node.nodes.length > 0)
                hasChildren = true;

            // indent
            var indent= '';
            for (var j = 0; j < (node._innerLevel - 1); j++) {
                indent+='<span class="indent"></span>';
            }

            // icon
            var cssClassIcon= 'icon';
            if (hasChildren) {
                if(this.settings.isAsyc)
                    cssClassIcon += ' expand-icon '+this.settings.expandIcon;
                else
                    cssClassIcon += node._innerLevel < this.settings.levels ? ' expand-icon '+this.settings.collapseIcon : ' expand-icon '+this.settings.expandIcon;
            }
            else {
                cssClassIcon += ' ' + this.settings.emptyIcon;
            }
            var icon= '<span class="'+cssClassIcon+'"></span>';

            // node icon
            var nodeIcon= '';
            if (this.settings.showIcon) {
                var cssClassNodeIcon= 'icon node-icon ';
                cssClassNodeIcon += (node.icon || this.settings.nodeIcon);
                nodeIcon= '<span class="'+cssClassNodeIcon+'"></span>'; 
            }

            // Add check / unchecked icon
            var check= '';

            if (this.settings.showCheckbox === enumShowCheckbox.all || (!hasChildren && this.settings.showCheckbox === enumShowCheckbox.leaf)) {
                var cssClassCheck= 'icon check-icon ';
                cssClassCheck += this.settings.uncheckedIcon;
                check= '<span class="'+ cssClassCheck +'"></span>';
            }

            // tags as badges
            var badge= '';
            if (this.settings.showTags && node.tags) {
                for(var k=0; k<node.tags.length; k++){
                    var tag=node.tags[k];
                    badge += '<span class="badge">'+tag+'</span>';  // badge
                }
            }

            // item
            var cssClass= 'list-group-item';
            if(hasChildren){
                // cssClas += ' has-children';
                if(this.settings.selectMode === enumSelectMode.leaf){
                    // 不可选择
                    cssClass += ' node-unselectable';
                }
            }

            var item= ''+
                '<li '+
                '   class="' + cssClass + '" '+
                (this.settings.levels && node._innerLevel>this.settings.levels ? 
                '   style="display: none;"' : '')+  // 隐藏应该折叠的nodes
                (this.settings.enableTitle ? 
                '   title="'+node.text+'"' : '')+
                '   data-id="'+node.id+'"'+
                '   data-hasChildren="'+hasChildren+'"'+
                '   data-level="'+node._innerLevel+'"'+
                '   data-path="'+node._innerPath+'">'+
                '<div class = "item-content">'+
                indent +
                icon +
                nodeIcon +
                check +
                (this.settings.enableLinks ? 
                '   <a class="nodevalue" href="'+node.href+'" style="color:inherit;">'+node.text+'</a>' : '<span class="nodevalue">' + node.text +  '</span>') +
        
                (this.settings.appendHtml ? 
                '   <span class="appendHtml"></span>' : '') +
                badge +
                '</div></li>';

            return item;
        },

        // -------------------------------------------------------------------
        // 展开 / 折叠
        // -------------------------------------------------------------------

        setOneLevelExpandedState: function (node, state, silent, ignoreChildren) {
            if(this.settings.isAsyc && node && !node.nodes){
                if (!silent) {
                    this.elements.original.trigger('nodeExpanded', node);
                }
                return;
            }
            var jqNode= this.elements.getNode(node.id);
            var jqChildren= this.elements.getChildNodes(node.id);


            if (state) {
                jqNode.find('.expand-icon').removeClass(this.settings.expandIcon).addClass(this.settings.collapseIcon);                
                jqChildren.show();

                if (!silent) {
                    this.elements.original.trigger('nodeExpanded', node);
                }
            } else {
                jqNode.find('.expand-icon').removeClass(this.settings.collapseIcon).addClass(this.settings.expandIcon);
                jqChildren.hide();


                if (!silent) {
                    this.elements.original.trigger('nodeCollapsed', node);
                }
            }

            if (jqChildren.length>0) {
                var context= this;
                jqChildren.each(function(){
                    var nodeId= $(this).data('id');
                    var child= context.elements.getNode(nodeId);
                    var children= context.elements.getChildNodes(nodeId);
                    child.find('.expand-icon').removeClass(context.settings.collapseIcon).addClass(context.settings.expandIcon);
                    children.hide();
                });
            }
        },

        //开放接口
        getApi: function(){
            var context = this;
            var funs = {
                collapseAll: 'collapseAll',
                expandAll: 'expandAll',
                checkAll:'checkAll',
                uncheckAll:'uncheckAll',
                disableAll:'disableAll',
                enableAll:'enableAll',
                checkNodeByIds:'checkNodeByIds',
                uncheckNodeByIds:'uncheckNodeByIds',
                selectNodeByIds:'selectNodeByIds',
                unselectNodeByIds:'unselectNodeByIds',
                getCheckedNodes:'getCheckedNodes',
                getSelectNodes:'getSelectNodes',
                getNode:'getNode',
                addChildrenData:'addChildrenData',
                addNodeData:'addNodeData'
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        }
    });
});

/* tree javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'treeasyc';
    var pluginClass = s.ui.controls.Treeasyc;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destroy();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);


// getApi: function(){
//     var context= this;
//     var api={
//         updateOptions: context.updateOptions,
//         updateData: context.updateData,
//         fresh: context.fresh
//     };
//     return api;
// },
Jx().package('s.ui.controls', function(J){

    // 严格模式
    'use strict';

    // 全局变量、函数、对象
    var _currentPluginId = 0;
    var emptyFun= function(){};
    var defaults = {
        onConfirm: emptyFun,
        map:{id:'id',text:'text'},
        titleName:'title',
        tranferOriginalOrgs:undefined,
        tranferInnerOrgs:undefined,
        parseDataTrees:undefined,
        multiSelect:true,
        inputHide:false,
        renderTreeNode:undefined
    };
    var attributeMap = {
        titleName:"title-name",
        multiSelect:'multi-select',
        inputHide:'input-hide',
        map:"map"
    };

    var TreeselectModal = new J.Class({
        data:{},
        //init:function(elements, options, data){
        init:function(element, options){
            this.userselect = element;

            // 初始化选项
            //this.initSettings(options);
            // 直接使用地址类实例的设置
            this.settings=options;
            if(typeof this.settings.map === 'string' && this.settings.map !== '')
                this.settings.map = JSON.parse(this.settings.map);
            // 状态
            this.states={
                rightOrg: [],
                rightOrgMap:{},
                orgsMap:{},
            };

            // 构建html DOM
            this.buildHtml();
            this.initElements();

            // 绑定事件
            this.bindEvents();
            // 绑定事件接口
            this.bindEventsInterface();
        },
        parseDataTrees: function(data){
            this.states.orgsMap={};
            var org=data[0];
            var mateId=this.settings.map.id;
            var mateText=this.settings.map.text;
            this.states.orgsMap[org[mateId]]=org;
            //如果parseDataTrees为空 就执行下面
            if(typeof this.settings.parseDataTrees==='function'){
                return this.settings.parseDataTrees();
            }
           // var users = this.getUsersByOrgId(org.Id);
           // this.orgUsersMap[org.Id] = users;
            var tree=[{
                id:org[mateId],
                parentId:org.ParentID,
                text:org[mateText],
                path: ''+org[mateId]
            }];
            for(var i=1; i<data.length; i++){
                org=data[i];
                var node;
                var prevNode=tree[tree.length-1];

                while(org.ParentID !== prevNode.id){
                    if(!prevNode.nodes){
                        break;
                    }
                    prevNode=prevNode.nodes[prevNode.nodes.length-1];
                }

                if(org.ParentID === prevNode.id){
                    prevNode.nodes=prevNode.nodes||[];
                    //users = this.getUsersByOrgId(org.Id);
                   // this.orgUsersMap[org.Id] = users;
                    prevNode.nodes[prevNode.nodes.length]={
                        id:org[mateId],
                        parentId:org.ParentID,
                        text:org[mateText],
                        path:prevNode.path+'>'+org[mateId]
                    }
                }
                this.states.orgsMap[org[mateId]]=org;
            }
           this.elements.colOrgNumber.text(data.length);
           return tree;
           
        },
        updateDataOrgs: function(data){
            this.data.orgs = this.parseDataTrees(data);
            this.typeheadSrouces=data;
            //this.
            this.render();
        },
        buildHtml:function(){
            var orgTemplate = ''+
                    '<div class="c-treeselect-container">'+
                    '    <div class="col-org">'+
                    '        <div class="row-query">'+
                    '            <input class="query-typeahead form-control"  placeholder="请输入关键字" />'+
                    '        </div>'+
                    '        <div class="row-header">'+
                    '            <span class="title">'+this.settings.titleName+'</span>'+
                    '            <span class="number-container">(<span class="number">0</span>)</span>'+
                    '        </div>'+
                    '        <div class="row-tree">'+
                    '            <div class="contact-tree"></div>'+
                    '            <div class="org-tree"></div>'+
                    '        </div>'+
                    '    </div>'+
                    '    <div class="col-buttons">'+
                  //  '        <a class="btn btn-default check-all">全选 <span>&gt;&gt;</span></a>'+
                    '        <a class="btn btn-default check-in">添加<span>&gt;</span></a>'+
                    '        <a class="btn btn-default check-out"><span>&lt;</span>移除</a>'+
                    '        <a class="btn btn-default check-clear"><span>&lt;&lt;</span>清除</a>'+
                    '    </div>'+
                    '    <div class="col-rightselect">'+
                    '        <div class="row-header">'+
                    '            <span class="title">已选</span>'+
                    '            <span class="number-container">(<span class="number">0</span>)</span>'+
                    '        </div>'+
                    '        <div class="row-list">'+
                    '            <div class="person-list"></div>'+
                    '        </div>'+
                    '    </div>'+
                    '    <div class="clearfix"></div>'+
                    '</div>';

            var htmlTemplate = ''+
                '<div class="modal-content c-treeselect-modal">'+
                '    <div class="modal-header">'+
                '        <button type="button" class="close"><span aria-hidden="true">&times;</span></button>'+
                '        <h4 class="modal-title">选择'+this.settings.titleName+'</h4>'+
                '    </div>'+
                '    <div class="modal-body">'+
                orgTemplate +
                '    </div>'+
                '    <div class="modal-footer"> '+
                '        <button type="button" class="btn btn-default cancel">取消</button>'+
                '        <button type="button" class="btn btn-primary confirm">确定</button>'+ 
                '    </div>'+
                '</div>';
            this.container = $(htmlTemplate);
        },

        initElements: function(){
            var context = this;
            var dialogId = '#c-treeselect-dialog-'+this.settings._currentPluginId;
            var pop= new s.ui.controls.Modal(this.userselect.getButton(),{
                modalId: dialogId, 
                modalClass: 'c-treeselect-modal',
                content: this.container,
                backdrop: 'static'
            });

            this.elements = {
                colOrgQueryTypeahead: $('.col-org .query-typeahead',this.container),
                colOrgNumber: $('.col-org .number',this.container),
                colOrgContactTree: $('.col-org .contact-tree',this.container),
                colOrgOrgTree: $('.col-org .org-tree',this.container),

                colButtonsCheckAll: $('.col-buttons .check-all',this.container),
                colButtonsCheckIn: $('.col-buttons .check-in',this.container),
                colButtonsCheckOut: $('.col-buttons .check-out',this.container),
                colButtonsCheckClear: $('.col-buttons .check-clear',this.container),

                colRightSelectNumber: $('.col-rightselect .number',this.container),
                colRightSelectSetContact: $('.col-rightselect .set-contact',this.container),
                colRightSelectContactName: $('.col-rightselect .contact-name',this.container),
                colRightSelectSaveBtn: $('.col-rightselect .save-btn',this.container),
                colRightSelectPersonList: $('.col-rightselect .person-list',this.container),
                colRightSelectRowContact: $('.col-rightselect .row-contact',this.container),

                confirm: $('.modal-footer .confirm', this.container),
                cancel: $('.modal-footer .cancel', this.container)
            };

            var typeahead= new s.ui.controls.Typeahead(this.elements.colOrgQueryTypeahead, {
                map: '{"text":"'+context.settings.map.text+'","returnValue":"json"}',
                onChanged: function(e,value){
                    this.query=this.query;
                    this.matcherReg=new RegExp(value,'i');
                   // context.changed.apply(context,arguments);
                },
                matcher: function(item){
                    var matcher = this.matcherReg||new RegExp(this.query,'i');
                    var result = matcher.test(item[context.settings.map.text]);
                    return result===true ? 1 : -1;
                },
                onSelected: function(e,value) {
                    var item = this.getValue(value);
                        item=context.tranferInnerOrgs([item]);
                        context.addOrgs(item);
                        context.addNodeClass(item);
                   // context.checkLeft(context.getRightUsers());
                },
            }); 
            
            var orgTree=new s.ui.controls.Tree(this.elements.colOrgOrgTree, {
                levels: 1,
                multiSelect: this.settings.multiSelect,
                onNodeDbclick:function(e, data){
                     context.onNodeDbclick(e, data);
                },
                renderNode:this.settings.renderTreeNode
            });
            var rightSelectPersonList = new s.ui.controls.Selectlist(this.elements.colRightSelectPersonList,{
                map:{text:'text',value:'id'},
                dblclick: function(e,item){
                    context.removeOrgs([item]);
                    context.removeNodeClass([item]);
                    context.removeOrgs([item]);
                }
            });

            this.controls={
                pop: pop,
                typeahead: typeahead,
                orgTree: orgTree,
                rightSelectPersonList: rightSelectPersonList
            };
        },

        bindEvents: function(){
            this.elements.confirm.on('click',$.proxy(this.onConfirm,this));
            this.elements.colButtonsCheckAll.on('click',$.proxy(this.checkAll,this));
            this.elements.colButtonsCheckIn.on('click',$.proxy(this.checkIn,this));
            this.elements.colButtonsCheckOut.on('click',$.proxy(this.checkOut,this));
            this.elements.colButtonsCheckClear.on('click',$.proxy(this.checkClear,this));
            this.elements.confirm.on('onConfirm', $.proxy(this.settings.onConfirm, this));
        },

        bindEventsInterface: function () {},
        tranferOriginalOrgs: function(datas){
            var orgs = [];
            var mateId=this.settings.map.id;
            //如果tranferOriginalOrgs为空 就执行下面
            if(typeof this.settings.tranferOriginalOrgs==='function'){
                return this.settings.tranferOriginalOrgs();
            }
            for(var i = 0; i < datas.length; i++) {
                var id=datas[i][mateId] ? datas[i][mateId] : datas[i].id;
                orgs.push(this.states.orgsMap[id]);
            }
            return orgs;
        },

        tranferInnerOrgs: function(innerOrgs){
            var orgs = [];
            var mateId=this.settings.map.id;
            var mateText=this.settings.map.text;
            //如果tranferInnerOrgs为空 就执行下面
            if(typeof this.settings.tranferInnerOrgs==='function'){
                return this.settings.tranferInnerOrgs();
            }

            for (var i = 0; i < innerOrgs.length; i++) {
                    var org = {
                        id:innerOrgs[i][mateId],
                        parentId:innerOrgs[i].ParentID,
                        text:innerOrgs[i][mateText],
                        path:''+innerOrgs[i][mateId]
                     };
                orgs.push(org);
            }
            return orgs;
        },
        setRightOrgs: function(orgs){
            this.states.rightOrgMap = {};
            for (var i = 0; i < orgs.length; i++) {
                var org = orgs[i];
                this.states.rightOrgMap[org.id] = org;
            }
            var num = Object.keys(this.states.rightOrgMap).length;
            this.elements.colRightSelectNumber.text(num);
            this.controls.rightSelectPersonList.setData(orgs);
        },
        addOrgs:function(datas){
            if(datas.length===0) return;
            var addItem = [];
            var data;
            if(!this.settings.multiSelect){
                if(Array.isArray(datas))
                    data=datas[datas.length-1];
                else{
                    data=datas;
                }

                if(!this.states.rightOrgMap[data.id]){
                    this.states.rightOrgMap={};
                    addItem.push(data);
                    this.states.rightOrgMap[data.id] = data;
                }

                this.controls.rightSelectPersonList.coverItem(addItem);
                this.elements.colRightSelectNumber.text(1);
            }else{
                for (var i = 0; i < datas.length; i++) {
                    var data = datas[i];
                    if(!this.states.rightOrgMap[data.id]){
                        addItem.push(data);
                        this.states.rightOrgMap[data.id] = data;
                    }
                }
                var num = Object.keys(this.states.rightOrgMap).length;
                this.controls.rightSelectPersonList.addItem(addItem);
                this.elements.colRightSelectNumber.text(num);
            }


           /* var addItem = [];
            for (var i = 0; i < datas.length; i++) {
                var data = datas[i];
                if(!this.states.rightOrgMap[data.id]){
                    addItem.push(data);
                    this.states.rightOrgMap[data.id] = data;
                }
            }
            var num = Object.keys(this.states.rightOrgMap).length;
            this.controls.rightSelectPersonList.addItem(addItem);
            this.elements.colRightSelectNumber.text(num);*/
        },
        removeOrgs: function(datas){
            for (var i = 0; i < datas.length; i++) {
                var data = datas[i];
                delete this.states.rightOrgMap[data.id];
            }
            this.controls.rightSelectPersonList.deleteItem(datas); 

            var num = Object.keys(this.states.rightOrgMap).length;
            this.elements.colRightSelectNumber.text(num);
        },
        onNodeDbclick:function(e,org){
            this.addOrgs([org]);
            this.addNodeClass([org]);
            this.selectedClear([org]);
        },
         checkAll: function(e){
           
            //this.addOrgs(this.states.leftUsers);
         
        },
        checkIn: function(e){
            var orgs=this.controls.orgTree.getSelectNodes();
            this.addOrgs(orgs);
            this.addNodeClass(orgs);
            this.selectedClear(orgs);
        },

        checkOut: function(){
            var orgs = this.controls.rightSelectPersonList.getSelectedItems();
            this.removeNodeClass(orgs);
            this.removeOrgs(orgs);
        },

        checkClear: function(){
            var orgs= this.getRightOrgs();
            this.removeOrgs(orgs);
            this.removeNodeClass();
            this.selectedClear(orgs);
        },

        onConfirm: function(e){
            this._renderValue();
            this.controls.pop.hide();
        },
        getRightOrgs:function(){
            var orgs=[];
            for(var key in this.states.rightOrgMap){
                 orgs.push(this.states.rightOrgMap[key]);
            }
            return orgs
        },
        selectedClear:function(datas){
            var ids="";
            for(var i=0;i<datas.length;i++){
                var id=datas[i]['id'];
                ids+=id+",";
            }
            ids=ids.substring(0,ids.length-1);
            this.controls.orgTree.unselectNodeByIds(ids);
        },
        
        addNodeClass:function(datas){
             if(!this.settings.multiSelect){
                this.container.find('li[data-id]').removeClass('checked');
            }
           for(var i=0;i<datas.length;i++){
            var id=datas[i]['id'];
                this.container.find('li[data-id='+id+']').addClass('checked');
           }
        },
        removeNodeClass:function(datas){

            if(datas){
                    for(var i=0;i<datas.length;i++){
                        var id=datas[i]['id'];
                        this.container.find('li[data-id='+id+']').removeClass('checked');
                    }
            }else{
                this.container.find('li[data-id]').removeClass('checked');
            }

        },
        render: function(){
            this._renderTypeahead();
            this._renderOrgTree();
            // this._renderValue();
        },
        _renderTypeahead: function() {

            this.controls.typeahead.setSource(this.typeheadSrouces);
        },
        _renderOrgTree: function(){
            //默认展开第一级
            this.controls.orgTree.setValue(this.data.orgs,2);
        },
        _renderContactTree: function(){
            this.controls.contactTree.setValue(this.data.contacts,2);
        },

        setSelectValue: function(data){
            this.setRightOrgs(data);
            this.removeNodeClass();
            this.addNodeClass(data);
        },
        _renderValue: function(){
            var orgs = this.tranferOriginalOrgs(this.getRightOrgs());
            this.userselect.setData(orgs);
            this.settings.onConfirm.apply(this,[orgs]);
        },
        show: function () {
            this.container.show();
        },
        hide: function(){
            this.container.hide();
            //alert(this.inputElements.original.val());
        }
    });

    this.Treeselect = new J.Class({extend : s.ui.BaseControl}, {
        defaults : defaults,
        attributeMap : attributeMap,
        // data:{},

        // 构造函数
        init:function(element, options){
            this.element = $(element);
            /*
                initialized 和 plugin-id 属于控件的内部属性， 保存在 element.data 中，不能在 defalut 中暴露给外界。
                也不在 parseAttributes 中解析，同理不加 data-s 前缀
            */
            _currentPluginId += 1;
            this.element.data('plugin-id', _currentPluginId);

            // 初始化选项
            this.initSettings(options);
            this.settings._currentPluginId = _currentPluginId;

            // 初始化 html DOM 元素
            this.initElements();
        },
        updateDataOrgs: function(data){
            this.modal.updateDataOrgs(data);
            this.userselect.updateData(data);
        },

        initElements:function () {
            var context = this;
           
            this.elements = {
                original: this.element
            };

            this.userselect = new s.ui.controls.UserSelect(this.elements.original,{
                showItemList:true,
                ishide:this.settings.inputHide,
                mateKey:{id:context.settings.map.id},
                map:'{"text":"'+context.settings.map.text+'","returnValue":"json"}',
                onblured: function(value){
                    context.settings.onConfirm.apply(context,[context.modal.tranferOriginalOrgs(value)]);
                }
            });
            this.modal= new TreeselectModal(this.userselect, this.settings);
            this.userselect.getButton().on('click',function(e){
                context.modal.setSelectValue(context.modal.tranferInnerOrgs(context.userselect.getValue()));
            });
        },
        setValue: function(value){
          
            this.userselect.setData(value);
        },
        refresh: function(){
            var context= this;

            // $.when(this.d.promise())
            //  .done(function(){
            //     context.modal.refresh();
            //  });
            // this.modal.refresh();
        },
        enable: function(){
            this.element.prop('disabled', false);
            this.userselect.enable();
            this.disabled=false;
        },
        disable: function(){
            this.element.prop('disabled', true);
            this.userselect.disable();
            this.disabled=true;
        },
        onOpen:function(btn){
             this.userselect.getButton().click();
        },
        disableNodeByIds:function(ids){
            this.modal.controls.orgTree.disableNodeByIds(ids);
        },
        // API
        getApi:function(){
            var context = this;
            var funs = {
                onOpen: 'onOpen',
                disableNodeByIds:'disableNodeByIds'
                
            };
            var api=s.utilities.buildApi(context, funs);
            return api;
        },
        destory: function(){}
    });

});

/* orgselect javascript jQuery */

(function($) {
    // 严格模式
    'use strict';

    // 控件类名
    var pluginName = 'treeselect';
    var pluginClass = s.ui.controls.Treeselect;

        // 胶水代码
    $.fn[pluginName] = function(options) {

        this.each(function () {
            var jqElement = $(this);
            if (jqElement.data('plugin-ref')) {
                jqElement.data('plugin-ref').destory();
            }
            jqElement.data('plugin-ref', new pluginClass(this, $.extend(true, {}, options)));
        });

        return this;

    };

})(jQuery);